@@ -22,11 +22,14 @@
 #define _ALLJOYN_C_BUSATTACHMENT_H
 
 #include <alljoyn_c/AjAPI.h>
+#include <alljoyn_c/ApplicationStateListener.h>
 #include <alljoyn_c/KeyStoreListener.h>
 #include <alljoyn_c/AuthListener.h>
 #include <alljoyn_c/BusListener.h>
 #include <alljoyn_c/BusObject.h>
 #include <alljoyn_c/ProxyBusObject.h>
+#include <alljoyn_c/PermissionConfigurationListener.h>
+#include <alljoyn_c/PermissionConfigurator.h>
 #include <alljoyn_c/InterfaceDescription.h>
 #include <alljoyn_c/Session.h>
 #include <alljoyn_c/SessionListener.h>
@@ -293,8 +296,39 @@ extern AJ_API const char* AJ_CALL alljoyn_busattachment_getconnectspec(alljoyn_b
  * Allow the currently executing method/signal handler to enable concurrent callbacks
  * during the scope of the handler's execution.
  *
- * @param bus    The alljoyn_busattachment to enable concurrent callbacks on
+ * This member function can ONLY be called from within the body of a signal
+ * handler, method handler or other AllJoyn callback. It allows AllJoyn to
+ * dispatch a single (additional) callback while the current one is still
+ * executing. This method is typically used when a method, signal handler or
+ * other AllJoyn callback needs to execute for a long period of time or when
+ * the callback needs to make any kind of blocking call.
  *
+ * This function MUST be called prior to making any non-asynchronous AllJoyn
+ * remote procedure calls from within an AllJoyn callback. This includes
+ * calls such as alljoyn_busattachment_joinsession(),
+ * alljoyn_busattachment_advertisename(),
+ * alljoyn_busattachment_canceladvertisename(),
+ * alljoyn_busattachment_findadvertisedname(),
+ * alljoyn_busattachment_canceladvertisename(),
+ * alljoyn_busattachment_setlinktimeout(), etc.
+ *
+ * alljoyn_busattachment_enableconcurrentcallbacks doesn't take effect when a
+ * alljoyn_busattachment is created with just one thread. If the
+ * alljoyn_busattachment is created with just one thread,
+ * i.e.`alljoyn_busattachment_create_concurrency(appName, true, 1)`, and the
+ * application developer attempts to make a blocking method call in a callback
+ * after invoking alljoyn_busattachment_enableconcurrentcallbacks(), the
+ * application will deadlock.
+ *
+ * For the same reason that alljoyn_busattachment_enableconcurrentcallbacks
+ * cannot be used with just one thread, the maximum number of concurrent
+ * callbacks is limited to the value specified when creating the BusAttachment.
+ * If no concurrency value was chosen the default is 4. It is the application
+ * developers responsibility to make sure the maximum number of concurrent
+ * callbacks is not exceeded. If the maximum number is exceeded the application
+ * will deadlock.
+ *
+ * @param bus    The alljoyn_busattachment to enable concurrent callbacks on
  */
 extern AJ_API void AJ_CALL alljoyn_busattachment_enableconcurrentcallbacks(alljoyn_busattachment bus);
 
@@ -656,7 +690,7 @@ extern AJ_API QStatus AJ_CALL alljoyn_busattachment_unbindsessionport(alljoyn_bu
 
 /**
  * Enable peer-to-peer security. This function must be called by applications that
- * want to use authentication and encryption . The bus must have been started by calling
+ * want to use authentication and encryption. The bus must have been started by calling
  * alljoyn_busattachment_start() before this function is called. If the application
  * is providing its own key store implementation it must have already called
  * alljoyn_busattachment_registerkeystorelistener() before calling this function.
@@ -670,7 +704,9 @@ extern AJ_API QStatus AJ_CALL alljoyn_busattachment_unbindsessionport(alljoyn_bu
  * @param authMechanisms   The authentication mechanism(s) to use for peer-to-peer authentication.
  *                         If this parameter is NULL peer-to-peer authentication is disabled.
  *                         This is a space separated list of any of the following values: ALLJOYN_SRP_LOGON,
- *                         ALLJOYN_SRP_KEYX, ALLJOYN_ECDHE_NULL, ALLJOYN_ECDHE_PSK, ALLJOYN_ECDHE_ECDSA, GSSAPI.
+ *                         ALLJOYN_SRP_KEYX, ALLJOYN_ECDHE_NULL, ALLJOYN_ECDHE_PSK, ALLJOYN_ECDHE_ECDSA, GSSAPI,
+ *                         ALLJOYN_ECDHE_SPEKE. Note that the SRP and PSK mechanisms are deprecated and will be
+ *                         removed in a future release.
  *
  * @param listener         Passes password and other authentication related requests to the application.
  *
@@ -679,10 +715,7 @@ extern AJ_API QStatus AJ_CALL alljoyn_busattachment_unbindsessionport(alljoyn_bu
  *                         Note that this parameter is only meaningful when using the default
  *                         key store implementation.
  *
- * @param isShared         Optional parameter that indicates if the key store is shared between multiple
- *                         applications. It is generally harmless to set this to true even when the
- *                         key store is not shared but it adds some unnecessary calls to the key store
- *                         listener to load and store the key store in this case.
+ * @param isShared         This parameter is not used as of 16.04. It is ignored internally (always shared).
  *
  * @return
  *      - #ER_OK if peer security was enabled.
@@ -692,6 +725,51 @@ extern AJ_API QStatus AJ_CALL alljoyn_busattachment_enablepeersecurity(alljoyn_b
                                                                        alljoyn_authlistener listener, const char* keyStoreFileName,
                                                                        QCC_BOOL isShared);
 
+/**
+ * Enable peer-to-peer security. This function must be called by applications that
+ * want to use authentication and encryption. The bus must have been started by calling
+ * alljoyn_busattachment_start() before this function is called. If the application
+ * is providing its own key store implementation it must have already called
+ * alljoyn_busattachment_registerkeystorelistener() before calling this function.
+ *
+ * Once peer security has been enabled it is not possible to change the authMechanism set without
+ * clearing it first (setting authMechanism to NULL). This is true regardless of whether the BusAttachment
+ * has been disconnected or not.
+ *
+ * @param bus                   The bus on which to enable security.
+ *
+ * @param authMechanisms        The authentication mechanism(s) to use for peer-to-peer authentication.
+ *                              If this parameter is NULL peer-to-peer authentication is disabled.
+ *                              This is a space separated list of any of the following values: ALLJOYN_SRP_LOGON,
+ *                              ALLJOYN_SRP_KEYX, ALLJOYN_ECDHE_NULL, ALLJOYN_ECDHE_PSK, ALLJOYN_ECDHE_ECDSA, GSSAPI,
+ *                              ALLJOYN_ECDHE_SPEKE. Note that the SRP and PSK mechanisms are deprecated and will be
+ *                              removed in a future release.
+ *
+ * @param authListener          Passes password and other authentication related requests to the application.
+ *
+ * @param keyStoreFileName      Optional parameter to specify the filename of the default key store. The
+ *                              default value is the applicationName parameter of alljoyn_busattachment_create().
+ *                              Note that this parameter is only meaningful when using the default
+ *                              key store implementation.
+ *
+ * @param isShared              Optional parameter that indicates if the key store is shared between multiple
+ *                              applications. It is generally harmless to set this to true even when the
+ *                              key store is not shared but it adds some unnecessary calls to the key store
+ *                              listener to load and store the key store in this case.
+ *
+ * @param permissionConfigurationListener    Passes security 2.0 callbacks to the application.
+ *
+ * @return
+ *      - #ER_OK if peer security was enabled.
+ *      - #ER_BUS_BUS_NOT_STARTED alljoyn_busattachment_start has not been called
+ */
+extern AJ_API QStatus AJ_CALL alljoyn_busattachment_enablepeersecuritywithpermissionconfigurationlistener(alljoyn_busattachment bus,
+                                                                                                          const char* authMechanisms,
+                                                                                                          alljoyn_authlistener authListener,
+                                                                                                          const char* keyStoreFileName,
+                                                                                                          QCC_BOOL isShared,
+                                                                                                          alljoyn_permissionconfigurationlistener permissionConfigurationListener);
+
 /**
  * Check is peer security has been enabled for this bus attachment.
  *
@@ -908,22 +986,6 @@ extern AJ_API QStatus AJ_CALL alljoyn_busattachment_unregistersignalhandlerwithr
                                                                                     const alljoyn_interfacedescription_member member,
                                                                                     const char* matchRule);
 
-/**
- * Unregister a signal handler with a filter rule.
- *
- * Remove the signal handler that was registered with the given parameters.
- *
- * @param bus             The alljoyn_busattachment to unregister the signal handler with
- * @param signal_handler  The signal handler method.
- * @param member          The interface/member of the signal.
- * @param matchRule       The filter rule.
- * @return #ER_OK
- */
-extern AJ_API QStatus AJ_CALL alljoyn_busattachment_unregistersignalhandlerwithrule(alljoyn_busattachment bus,
-                                                                                    alljoyn_messagereceiver_signalhandler_ptr signal_handler,
-                                                                                    const alljoyn_interfacedescription_member member,
-                                                                                    const char* matchRule);
-
 /**
  * Unregister all signal and reply handlers for the specified alljoyn_busattachment.
  *
@@ -941,7 +1003,7 @@ extern AJ_API QStatus AJ_CALL alljoyn_busattachment_unregisterallhandlers(alljoy
  * @return
  *      - #ER_OK if the key store listener was set
  *      - #ER_BUS_LISTENER_ALREADY_SET if a listener has been set by this function or because
- *         EnablePeerSecurity has been called.
+ *         alljoyn_busattachment_enablepeersecurity has been called.
  */
 extern AJ_API QStatus AJ_CALL alljoyn_busattachment_registerkeystorelistener(alljoyn_busattachment bus, alljoyn_keystorelistener listener);
 
@@ -1089,6 +1151,52 @@ extern AJ_API QStatus AJ_CALL alljoyn_busattachment_setsessionlistener(alljoyn_b
  */
 extern AJ_API QStatus AJ_CALL alljoyn_busattachment_leavesession(alljoyn_busattachment bus, alljoyn_sessionid sessionId);
 
+/**
+ * Explicitly secure the connection to the remote peer. Peer-to-peer
+ * connections can only be secured if alljoyn_busattachment_enablepeersecurity() was previously called on the bus
+ * attachment. If the peer-to-peer connection is already secure this
+ * function does nothing. Note that peer-to-peer connections are automatically secured when a
+ * method call requiring encryption is sent.
+ *
+ * This call causes messages to be sent on the bus, therefore it cannot be called within AllJoyn
+ * callbacks (method/signal/reply handlers or ObjectRegistered callbacks, etc.)
+ *
+ * @param[in]  bus        The busattachment on which to secure the connection.
+ * @param[in]  name       The unique name of the remote peer or NULL to secure the connections to all peers
+ *                        this alljoyn_busattachment is in session with.
+ * @param[in]  forceAuth  If true, forces re-authentication even if the peer connection is already
+ *                        authenticated.
+ *
+ * @return
+ *          - #ER_OK if the connection was secured or an error status indicating that the
+ *            connection could not be secured.
+ *          - #ER_BUS_NO_AUTHENTICATION_MECHANISM if alljoyn_busattachment_enablepeersecurity() has not been called.
+ *          - #ER_AUTH_FAIL if the attempt(s) to authenticate the peer failed.
+ *          - Other error status codes indicating a failure.
+ */
+extern AJ_API QStatus AJ_CALL alljoyn_busattachment_secureconnection(alljoyn_busattachment bus, const char* name, QCC_BOOL forceAuth);
+
+/**
+ * Asynchronously secure the connection to the remote peer. Peer-to-peer connections can only
+ * be secured if alljoyn_busattachment_enablepeersecurity() was previously called on the bus attachment.
+ * If the peer-to-peer connection is already secure this function does nothing.
+ * Note that peer-to-peer connections are automatically secured when a
+ * method call requiring encryption is sent.
+ *
+ * Notification of success or failure is via the alljoyn_authlistener passed to alljoyn_busattachment_enablepeersecurity().
+ *
+ * @param[in]  bus        The busattachment on which to secure the connection.
+ * @param[in]  name       The unique name of the remote peer or NULL to secure the connections to all peers
+ *                        this alljoyn_busattachment is in session with.
+ * @param[in]  forceAuth  If true, forces re-authentication even if the peer connection is already
+ *                        authenticated.
+ * @return
+ *          - #ER_OK if securing could begin.
+ *          - #ER_BUS_NO_AUTHENTICATION_MECHANISM if alljoyn_busattachment_enablepeersecurity() has not been called.
+ *          - Other error status codes indicating a failure.
+ */
+extern AJ_API QStatus AJ_CALL alljoyn_busattachment_secureconnectionasync(alljoyn_busattachment bus, const char* name, QCC_BOOL forceAuth);
+
 /**
  * Remove a member from an existing multipoint session.
  * This function may be called by the binder of the session to forcefully remove a member from a session.
@@ -1282,9 +1390,6 @@ extern AJ_API QStatus AJ_CALL alljoyn_busattachment_ping(alljoyn_busattachment b
  * The handler is only called if a call to alljoyn_busattachment_whoimplements_*
  * has been has been called.
  *
- * This function is experimental, and as such has not been fully tested.
- * Please help make it more robust by contributing fixes if you find problems.
- *
  * Important: the alljoyn_aboutlistener should be registered before calling
  * alljoyn_busattachment_whoimplments_*
  *
@@ -1297,9 +1402,6 @@ extern AJ_API void AJ_CALL alljoyn_busattachment_registeraboutlistener(alljoyn_b
 /**
  * Unregisters the AnnounceHandler from receiving the org.alljoyn.about Announce signal.
  *
- * This function is experimental, and as such has not been fully tested.
- * Please help make it more robust by contributing fixes if you find problems.
- *
  * @param[in] bus           alljoyn_busattachment this call is made for
  * @param[in] aboutListener alljoyn_aboutlistener to be unregistered
  */
@@ -1309,9 +1411,6 @@ extern AJ_API void AJ_CALL alljoyn_busattachment_unregisteraboutlistener(alljoyn
 /**
  * Unregisters all AboutListeners from receiving any org.alljoyn.about Announce signal
  *
- * This function is experimental, and as such has not been fully tested.
- * Please help make it more robust by contributing fixes if you find problems.
- *
  * @param[in] bus alljoyn_busattachment this call is made for
  */
 extern AJ_API void AJ_CALL alljoyn_busattachment_unregisterallaboutlisteners(alljoyn_busattachment bus);
@@ -1333,9 +1432,6 @@ extern AJ_API void AJ_CALL alljoyn_busattachment_unregisterallaboutlisteners(all
  * significant impact on network performance and should be avoided unless
  * all announcements are needed.
  *
- * This function is experimental, and as such has not been fully tested.
- * Please help make it more robust by contributing fixes if you find problems.
- *
  * @param[in] bus                  alljoyn_busattachment this call is made for
  * @param[in] implementsInterfaces a list of interfaces that the Announce signal
  *                                 reports as implemented. NULL to receive all
@@ -1360,9 +1456,6 @@ extern AJ_API QStatus AJ_CALL alljoyn_busattachment_whoimplements_interfaces(all
  * impact on network performance and should be avoided unless all
  * announcements are needed.
  *
- * This function is experimental, and as such has not been fully tested.
- * Please help make it more robust by contributing fixes if you find problems.
- *
  * @see alljoyn_busattachment_whoimplements_interfaces(const char**, size_t)
  * @param[in] bus                 alljoyn_busattachment this call is made for
  * @param[in] implementsInterface interface that must be implemented in order
@@ -1381,9 +1474,6 @@ extern AJ_API QStatus AJ_CALL alljoyn_busattachment_whoimplements_interface(allj
  * Note if alljoyn_busattachment_whoimplements_interfaces has been called multiple
  * times the announce signal will still be received for any interfaces that still remain.
  *
- * This function is experimental, and as such has not been fully tested.
- * Please help make it more robust by contributing fixes if you find problems.
- *
  * @param[in] bus                  alljoyn_busattachment this call is made for
  * @param[in] implementsInterfaces a list of interfaces. The list must match the
  *                                 list previously passed to the WhoImplements
@@ -1407,9 +1497,6 @@ extern AJ_API QStatus AJ_CALL alljoyn_busattachment_cancelwhoimplements_interfac
  * This is identical to alljoyn_busattachment_cancelwhoimplements_interfaces
  * except this is specialized for a single interface not several interfaces.
  *
- * This function is experimental, and as such has not been fully tested.
- * Please help make it more robust by contributing fixes if you find problems.
- *
  * @see alljoyn_busattachment_cancelwhoimplements(const char**, size_t)
  * @param[in] bus                 alljoyn_busattachment this call is made for
  * @param[in] implementsInterface interface that must be implement in order to
@@ -1424,6 +1511,44 @@ extern AJ_API QStatus AJ_CALL alljoyn_busattachment_cancelwhoimplements_interfac
 extern AJ_API QStatus AJ_CALL alljoyn_busattachment_cancelwhoimplements_interface(alljoyn_busattachment bus,
                                                                                   const char* implementsInterface);
 
+/**
+ * Retrieves the alljoyn_permissionconfigurator associated with this alljoyn_busattachment.
+ *
+ * @param[in] bus The alljoyn_busattachment containing the alljoyn_permissionconfigurator.
+ *
+ * @return
+ *      - The alljoyn_permissionconfigurator
+ */
+extern AJ_API const alljoyn_permissionconfigurator AJ_CALL alljoyn_busattachment_getpermissionconfigurator(alljoyn_busattachment bus);
+
+/**
+ * Registers a handler to receive the org.alljoyn.Bus.Application State signal.
+ *
+ * @param[in] bus       The alljoyn_busattachment which registers the alljoyn_applicationstatelistener.
+ * @param[in] listener  Listener instance that will receive state event notifications.
+ *
+ * @return
+ *    - #ER_OK on success
+ *    - #ER_APPLICATION_STATE_LISTENER_ALREADY_EXISTS when the same listener has already been registered.
+ *    - Different error status otherwise
+ */
+extern AJ_API QStatus AJ_CALL alljoyn_busattachment_registerapplicationstatelistener(alljoyn_busattachment bus,
+                                                                                     alljoyn_applicationstatelistener listener);
+
+/**
+ * Unregisters the listener that was registered with alljoyn_busattachment_registerapplicationstatelistener
+ *
+ * @param[in] bus       The alljoyn_busattachment from which to detach the alljoyn_applicationstatelistener.
+ * @param[in] listener  Object instance to unregister as a listener.
+ *
+ * @return
+ *    - #ER_OK on success
+ *    - #ER_APPLICATION_STATE_LISTENER_NO_SUCH_LISTENER when this listener has not been registered.
+ *    - Different error status otherwise
+ */
+extern AJ_API QStatus AJ_CALL alljoyn_busattachment_unregisterapplicationstatelistener(alljoyn_busattachment bus,
+                                                                                       alljoyn_applicationstatelistener listener);
+
 #ifdef __cplusplus
 } /* extern "C" */
 #endif
