@@ -29,9 +29,13 @@
 
 #include <signal.h>
 #include <stdio.h>
-#include <assert.h>
 #include <stdlib.h>
 
+/**
+ * Returns the size of a statically allocated array
+ */
+#define ArraySize(a)  (sizeof(a) / sizeof(a[0]))
+
 using namespace ajn;
 
 static volatile sig_atomic_t s_interrupt = QCC_FALSE;
@@ -107,7 +111,7 @@ static alljoyn_busobject create_my_alljoyn_busobject(alljoyn_busattachment bus,
     result = alljoyn_busobject_create(path, QCC_FALSE, NULL, NULL);
 
     alljoyn_interfacedescription iface = alljoyn_busattachment_getinterface(bus, INTERFACE_NAME);
-    assert(iface != NULL);
+    QCC_ASSERT(iface != NULL);
 
     status = alljoyn_busobject_addinterface(result, iface);
     alljoyn_busobject_setannounceflag(result, iface, ANNOUNCED);
@@ -229,14 +233,21 @@ int CDECL_CALL main(void)
     }
 
     char interface[256] = { 0 };
-    snprintf(interface, 256, "<node>"                                           \
-             "<interface name='%s'>"                                  \
-             "  <method name='Echo'>"                                 \
-             "    <arg name='out_arg' type='s' direction='in' />"     \
-             "    <arg name='return_arg' type='s' direction='out' />" \
-             "  </method>"                                            \
-             "</interface>"                                           \
-             "</node>", INTERFACE_NAME);
+
+#if defined(QCC_OS_GROUP_WINDOWS)
+    _snprintf(
+#else
+    snprintf(
+#endif
+        interface, ArraySize(interface), "<node>"                \
+        "<interface name='%s'>"                                  \
+        "  <method name='Echo'>"                                 \
+        "    <arg name='out_arg' type='s' direction='in' />"     \
+        "    <arg name='return_arg' type='s' direction='out' />" \
+        "  </method>"                                            \
+        "</interface>"                                           \
+        "</node>", INTERFACE_NAME);
+    interface[ArraySize(interface) - 1] = '\0';
 
     printf("Interface = %s\n", interface);
     status = alljoyn_busattachment_createinterfacesfromxml(bus, interface);
@@ -297,5 +308,5 @@ int CDECL_CALL main(void)
     alljoyn_routershutdown();
 #endif
     alljoyn_shutdown();
-    return 0;
+                     return 0;
 }
