@@ -18,7 +18,6 @@
  *    ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
  *    OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  ******************************************************************************/
-#include <assert.h>
 
 #include <alljoyn_c/BusAttachment.h>
 #include <alljoyn/InterfaceDescription.h>
@@ -31,14 +30,14 @@ struct _alljoyn_busattachment_handle {
     /* Empty by design, this is just to allow the type restrictions to save coders from themselves */
 };
 
-alljoyn_busattachment AJ_CALL alljoyn_busattachment_create(const char* applicationName, QCC_BOOL allowRemoteMessages)
+alljoyn_busattachment AJ_CALL alljoyn_busattachment_create(AJ_PCSTR applicationName, QCC_BOOL allowRemoteMessages)
 {
     QCC_DbgTrace(("%s", __FUNCTION__));
     bool allowRemoteMessagesBool = (allowRemoteMessages == QCC_TRUE ? true : false);
     return ((alljoyn_busattachment) new ajn::BusAttachmentC(applicationName, allowRemoteMessagesBool));
 }
 
-alljoyn_busattachment AJ_CALL alljoyn_busattachment_create_concurrency(const char* applicationName, QCC_BOOL allowRemoteMessages, uint32_t concurrency)
+alljoyn_busattachment AJ_CALL alljoyn_busattachment_create_concurrency(AJ_PCSTR applicationName, QCC_BOOL allowRemoteMessages, uint32_t concurrency)
 {
     QCC_DbgTrace(("%s", __FUNCTION__));
     bool allowRemoteMessagesBool = (allowRemoteMessages == QCC_TRUE ? true : false);
@@ -49,7 +48,7 @@ alljoyn_busattachment AJ_CALL alljoyn_busattachment_create_concurrency(const cha
 void AJ_CALL alljoyn_busattachment_destroy(alljoyn_busattachment bus)
 {
     QCC_DbgTrace(("%s", __FUNCTION__));
-    assert(bus != NULL && "NULL parameter passed to alljoyn_destroy_busattachment.");
+    QCC_ASSERT(bus != NULL && "NULL parameter passed to alljoyn_destroy_busattachment.");
 
     delete (ajn::BusAttachmentC*)bus;
 }
@@ -78,10 +77,10 @@ uint32_t AJ_CALL alljoyn_busattachment_getconcurrency(alljoyn_busattachment bus)
     return ((ajn::BusAttachmentC*)bus)->GetConcurrency();
 }
 
-const char* AJ_CALL alljoyn_busattachment_getconnectspec(alljoyn_busattachment bus)
+AJ_PCSTR AJ_CALL alljoyn_busattachment_getconnectspec(alljoyn_busattachment bus)
 {
     QCC_DbgTrace(("%s", __FUNCTION__));
-    return ((ajn::BusAttachmentC*)bus)->GetConnectSpec().c_str();
+    return ((ajn::BusAttachmentC*)bus)->GetConnectSpec();
 }
 
 void AJ_CALL alljoyn_busattachment_enableconcurrentcallbacks(alljoyn_busattachment bus)
@@ -91,7 +90,7 @@ void AJ_CALL alljoyn_busattachment_enableconcurrentcallbacks(alljoyn_busattachme
 }
 
 QStatus AJ_CALL alljoyn_busattachment_createinterface(alljoyn_busattachment bus,
-                                                      const char* name,
+                                                      AJ_PCSTR name,
                                                       alljoyn_interfacedescription* iface)
 {
     QCC_DbgTrace(("%s", __FUNCTION__));
@@ -103,7 +102,7 @@ QStatus AJ_CALL alljoyn_busattachment_createinterface(alljoyn_busattachment bus,
 }
 
 QStatus AJ_CALL alljoyn_busattachment_createinterface_secure(alljoyn_busattachment bus,
-                                                             const char* name,
+                                                             AJ_PCSTR name,
                                                              alljoyn_interfacedescription* iface,
                                                              alljoyn_interfacedescription_securitypolicy secPolicy)
 {
@@ -115,7 +114,7 @@ QStatus AJ_CALL alljoyn_busattachment_createinterface_secure(alljoyn_busattachme
     return ret;
 }
 
-QStatus AJ_CALL alljoyn_busattachment_connect(alljoyn_busattachment bus, const char* connectSpec)
+QStatus AJ_CALL alljoyn_busattachment_connect(alljoyn_busattachment bus, AJ_PCSTR connectSpec)
 {
     QCC_DbgTrace(("%s", __FUNCTION__));
     if (connectSpec == NULL) {
@@ -137,37 +136,37 @@ void AJ_CALL alljoyn_busattachment_unregisterbuslistener(alljoyn_busattachment b
     ((ajn::BusAttachmentC*)bus)->UnregisterBusListener((*(ajn::BusListener*)listener));
 }
 
-QStatus AJ_CALL alljoyn_busattachment_findadvertisedname(alljoyn_busattachment bus, const char* namePrefix)
+QStatus AJ_CALL alljoyn_busattachment_findadvertisedname(alljoyn_busattachment bus, AJ_PCSTR namePrefix)
 {
     QCC_DbgTrace(("%s", __FUNCTION__));
     return ((ajn::BusAttachmentC*)bus)->FindAdvertisedName(namePrefix);
 }
 
-QStatus AJ_CALL alljoyn_busattachment_findadvertisednamebytransport(alljoyn_busattachment bus, const char* namePrefix, alljoyn_transportmask transports)
+QStatus AJ_CALL alljoyn_busattachment_findadvertisednamebytransport(alljoyn_busattachment bus, AJ_PCSTR namePrefix, alljoyn_transportmask transports)
 {
     QCC_DbgTrace(("%s", __FUNCTION__));
     return ((ajn::BusAttachmentC*)bus)->FindAdvertisedNameByTransport(namePrefix, transports);
 }
 
-QStatus AJ_CALL alljoyn_busattachment_cancelfindadvertisedname(alljoyn_busattachment bus, const char* namePrefix)
+QStatus AJ_CALL alljoyn_busattachment_cancelfindadvertisedname(alljoyn_busattachment bus, AJ_PCSTR namePrefix)
 {
     QCC_DbgTrace(("%s", __FUNCTION__));
     return ((ajn::BusAttachmentC*)bus)->CancelFindAdvertisedName(namePrefix);
 }
 
-QStatus AJ_CALL alljoyn_busattachment_cancelfindadvertisednamebytransport(alljoyn_busattachment bus, const char* namePrefix, alljoyn_transportmask transports)
+QStatus AJ_CALL alljoyn_busattachment_cancelfindadvertisednamebytransport(alljoyn_busattachment bus, AJ_PCSTR namePrefix, alljoyn_transportmask transports)
 {
     QCC_DbgTrace(("%s", __FUNCTION__));
     return ((ajn::BusAttachmentC*)bus)->CancelFindAdvertisedNameByTransport(namePrefix, transports);
 }
 
-const alljoyn_interfacedescription AJ_CALL alljoyn_busattachment_getinterface(alljoyn_busattachment bus, const char* name)
+const alljoyn_interfacedescription AJ_CALL alljoyn_busattachment_getinterface(alljoyn_busattachment bus, AJ_PCSTR name)
 {
     QCC_DbgTrace(("%s", __FUNCTION__));
     return (alljoyn_interfacedescription)((ajn::BusAttachmentC*)bus)->GetInterface(name);
 }
 
-QStatus AJ_CALL alljoyn_busattachment_joinsession(alljoyn_busattachment bus, const char* sessionHost,
+QStatus AJ_CALL alljoyn_busattachment_joinsession(alljoyn_busattachment bus, AJ_PCSTR sessionHost,
                                                   alljoyn_sessionport sessionPort, alljoyn_sessionlistener listener,
                                                   alljoyn_sessionid* sessionId, alljoyn_sessionopts opts)
 {
@@ -178,7 +177,7 @@ QStatus AJ_CALL alljoyn_busattachment_joinsession(alljoyn_busattachment bus, con
 }
 
 QStatus AJ_CALL alljoyn_busattachment_joinsessionasync(alljoyn_busattachment bus,
-                                                       const char* sessionHost,
+                                                       AJ_PCSTR sessionHost,
                                                        alljoyn_sessionport sessionPort,
                                                        alljoyn_sessionlistener listener,
                                                        const alljoyn_sessionopts opts,
@@ -215,7 +214,7 @@ void AJ_CALL alljoyn_busattachment_unregisterbusobject(alljoyn_busattachment bus
     ((ajn::BusAttachmentC*)bus)->UnregisterBusObject(*((ajn::BusObject*)object));
 }
 
-QStatus AJ_CALL alljoyn_busattachment_requestname(alljoyn_busattachment bus, const char* requestedName, uint32_t flags)
+QStatus AJ_CALL alljoyn_busattachment_requestname(alljoyn_busattachment bus, AJ_PCSTR requestedName, uint32_t flags)
 {
     QCC_DbgTrace(("%s", __FUNCTION__));
     return ((ajn::BusAttachmentC*)bus)->RequestName(requestedName, flags);
@@ -236,20 +235,20 @@ QStatus AJ_CALL alljoyn_busattachment_unbindsessionport(alljoyn_busattachment bu
     return ((ajn::BusAttachmentC*)bus)->UnbindSessionPort(sessionPort);
 }
 
-QStatus AJ_CALL alljoyn_busattachment_advertisename(alljoyn_busattachment bus, const char* name, alljoyn_transportmask transports)
+QStatus AJ_CALL alljoyn_busattachment_advertisename(alljoyn_busattachment bus, AJ_PCSTR name, alljoyn_transportmask transports)
 {
     QCC_DbgTrace(("%s", __FUNCTION__));
     return ((ajn::BusAttachmentC*)bus)->AdvertiseName(name, transports);
 }
 
-QStatus AJ_CALL alljoyn_busattachment_canceladvertisename(alljoyn_busattachment bus, const char* name, alljoyn_transportmask transports)
+QStatus AJ_CALL alljoyn_busattachment_canceladvertisename(alljoyn_busattachment bus, AJ_PCSTR name, alljoyn_transportmask transports)
 {
     QCC_DbgTrace(("%s", __FUNCTION__));
     return ((ajn::BusAttachmentC*)bus)->CancelAdvertiseName(name, transports);
 }
 
-QStatus AJ_CALL alljoyn_busattachment_enablepeersecurity(alljoyn_busattachment bus, const char* authMechanisms,
-                                                         alljoyn_authlistener listener, const char* keyStoreFileName,
+QStatus AJ_CALL alljoyn_busattachment_enablepeersecurity(alljoyn_busattachment bus, AJ_PCSTR authMechanisms,
+                                                         alljoyn_authlistener listener, AJ_PCSTR keyStoreFileName,
                                                          QCC_BOOL isShared)
 {
     QCC_DbgTrace(("%s", __FUNCTION__));
@@ -258,9 +257,9 @@ QStatus AJ_CALL alljoyn_busattachment_enablepeersecurity(alljoyn_busattachment b
 }
 
 QStatus AJ_CALL alljoyn_busattachment_enablepeersecuritywithpermissionconfigurationlistener(alljoyn_busattachment bus,
-                                                                                            const char* authMechanisms,
+                                                                                            AJ_PCSTR authMechanisms,
                                                                                             alljoyn_authlistener authListener,
-                                                                                            const char* keyStoreFileName,
+                                                                                            AJ_PCSTR keyStoreFileName,
                                                                                             QCC_BOOL isShared,
                                                                                             alljoyn_permissionconfigurationlistener permissionConfigurationListener)
 {
@@ -278,7 +277,7 @@ QCC_BOOL AJ_CALL alljoyn_busattachment_ispeersecurityenabled(alljoyn_busattachme
     return (((ajn::BusAttachmentC*)bus)->IsPeerSecurityEnabled() == true ? QCC_TRUE : QCC_FALSE);
 }
 
-QStatus AJ_CALL alljoyn_busattachment_createinterfacesfromxml(alljoyn_busattachment bus, const char* xml)
+QStatus AJ_CALL alljoyn_busattachment_createinterfacesfromxml(alljoyn_busattachment bus, AJ_PCSTR xml)
 {
     QCC_DbgTrace(("%s", __FUNCTION__));
     return ((ajn::BusAttachmentC*)bus)->CreateInterfacesFromXml(xml);
@@ -315,7 +314,7 @@ QCC_BOOL AJ_CALL alljoyn_busattachment_isconnected(const alljoyn_busattachment b
     return (((const ajn::BusAttachmentC*)bus)->IsConnected() == true ? QCC_TRUE : QCC_FALSE);
 }
 
-QStatus AJ_CALL alljoyn_busattachment_disconnect(alljoyn_busattachment bus, const char* unused)
+QStatus AJ_CALL alljoyn_busattachment_disconnect(alljoyn_busattachment bus, AJ_PCSTR unused)
 {
     QCC_UNUSED(unused);
     QCC_DbgTrace(("%s", __FUNCTION__));
@@ -340,13 +339,13 @@ const alljoyn_proxybusobject AJ_CALL alljoyn_busattachment_getalljoyndebugobj(al
     return (const alljoyn_proxybusobject)(&((ajn::BusAttachmentC*)bus)->GetAllJoynDebugObj());
 }
 
-const char* AJ_CALL alljoyn_busattachment_getuniquename(const alljoyn_busattachment bus)
+AJ_PCSTR AJ_CALL alljoyn_busattachment_getuniquename(const alljoyn_busattachment bus)
 {
     QCC_DbgTrace(("%s", __FUNCTION__));
-    return ((const ajn::BusAttachmentC*)bus)->GetUniqueName().c_str();
+    return ((ajn::BusAttachmentC*)bus)->GetUniqueName();
 }
 
-const char* AJ_CALL alljoyn_busattachment_getglobalguidstring(const alljoyn_busattachment bus)
+AJ_PCSTR AJ_CALL alljoyn_busattachment_getglobalguidstring(const alljoyn_busattachment bus)
 {
     QCC_DbgTrace(("%s", __FUNCTION__));
     return ((const ajn::BusAttachmentC*)bus)->GetGlobalGUIDString().c_str();
@@ -370,44 +369,44 @@ void AJ_CALL alljoyn_busattachment_clearkeystore(alljoyn_busattachment bus)
     ((ajn::BusAttachmentC*)bus)->ClearKeyStore();
 }
 
-QStatus AJ_CALL alljoyn_busattachment_clearkeys(alljoyn_busattachment bus, const char* guid)
+QStatus AJ_CALL alljoyn_busattachment_clearkeys(alljoyn_busattachment bus, AJ_PCSTR guid)
 {
     QCC_DbgTrace(("%s", __FUNCTION__));
     return ((ajn::BusAttachmentC*)bus)->ClearKeys(guid);
 }
 
-QStatus AJ_CALL alljoyn_busattachment_setkeyexpiration(alljoyn_busattachment bus, const char* guid, uint32_t timeout)
+QStatus AJ_CALL alljoyn_busattachment_setkeyexpiration(alljoyn_busattachment bus, AJ_PCSTR guid, uint32_t timeout)
 {
     QCC_DbgTrace(("%s", __FUNCTION__));
     return ((ajn::BusAttachmentC*)bus)->SetKeyExpiration(guid, timeout);
 }
 
-QStatus AJ_CALL alljoyn_busattachment_getkeyexpiration(alljoyn_busattachment bus, const char* guid, uint32_t* timeout)
+QStatus AJ_CALL alljoyn_busattachment_getkeyexpiration(alljoyn_busattachment bus, AJ_PCSTR guid, uint32_t* timeout)
 {
     QCC_DbgTrace(("%s", __FUNCTION__));
     return ((ajn::BusAttachmentC*)bus)->GetKeyExpiration(guid, *timeout);
 }
 
-QStatus AJ_CALL alljoyn_busattachment_addlogonentry(alljoyn_busattachment bus, const char* authMechanism,
-                                                    const char* userName, const char* password)
+QStatus AJ_CALL alljoyn_busattachment_addlogonentry(alljoyn_busattachment bus, AJ_PCSTR authMechanism,
+                                                    AJ_PCSTR userName, AJ_PCSTR password)
 {
     QCC_DbgTrace(("%s", __FUNCTION__));
     return ((ajn::BusAttachmentC*)bus)->AddLogonEntry(authMechanism, userName, password);
 }
 
-QStatus AJ_CALL alljoyn_busattachment_releasename(alljoyn_busattachment bus, const char* name)
+QStatus AJ_CALL alljoyn_busattachment_releasename(alljoyn_busattachment bus, AJ_PCSTR name)
 {
     QCC_DbgTrace(("%s", __FUNCTION__));
     return ((ajn::BusAttachmentC*)bus)->ReleaseName(name);
 }
 
-QStatus AJ_CALL alljoyn_busattachment_addmatch(alljoyn_busattachment bus, const char* rule)
+QStatus AJ_CALL alljoyn_busattachment_addmatch(alljoyn_busattachment bus, AJ_PCSTR rule)
 {
     QCC_DbgTrace(("%s", __FUNCTION__));
     return ((ajn::BusAttachmentC*)bus)->AddMatch(rule);
 }
 
-QStatus AJ_CALL alljoyn_busattachment_removematch(alljoyn_busattachment bus, const char* rule)
+QStatus AJ_CALL alljoyn_busattachment_removematch(alljoyn_busattachment bus, AJ_PCSTR rule)
 {
     QCC_DbgTrace(("%s", __FUNCTION__));
     return ((ajn::BusAttachmentC*)bus)->RemoveMatch(rule);
@@ -426,7 +425,7 @@ QStatus AJ_CALL alljoyn_busattachment_leavesession(alljoyn_busattachment bus, al
     return ((ajn::BusAttachmentC*)bus)->LeaveSession(sessionId);
 }
 
-QStatus AJ_CALL alljoyn_busattachment_removesessionmember(alljoyn_busattachment bus, alljoyn_sessionid sessionId, const char* memberName)
+QStatus AJ_CALL alljoyn_busattachment_removesessionmember(alljoyn_busattachment bus, alljoyn_sessionid sessionId, AJ_PCSTR memberName)
 {
     QCC_DbgTrace(("%s", __FUNCTION__));
     return ((ajn::BusAttachmentC*)bus)->RemoveSessionMember(sessionId, memberName);
@@ -454,19 +453,19 @@ QStatus AJ_CALL alljoyn_busattachment_setlinktimeoutasync(alljoyn_busattachment
                                                             (void*)new ajn::SetLinkTimeoutContext(callback, context));
 }
 
-QStatus AJ_CALL alljoyn_busattachment_secureconnection(alljoyn_busattachment bus, const char* name, QCC_BOOL forceAuth)
+QStatus AJ_CALL alljoyn_busattachment_secureconnection(alljoyn_busattachment bus, AJ_PCSTR name, QCC_BOOL forceAuth)
 {
     QCC_DbgTrace(("%s", __FUNCTION__));
     return ((ajn::BusAttachmentC*)bus)->SecureConnection(name, forceAuth);
 }
 
-QStatus AJ_CALL alljoyn_busattachment_secureconnectionasync(alljoyn_busattachment bus, const char* name, QCC_BOOL forceAuth)
+QStatus AJ_CALL alljoyn_busattachment_secureconnectionasync(alljoyn_busattachment bus, AJ_PCSTR name, QCC_BOOL forceAuth)
 {
     QCC_DbgTrace(("%s", __FUNCTION__));
     return ((ajn::BusAttachmentC*)bus)->SecureConnectionAsync(name, forceAuth);
 }
 
-QStatus AJ_CALL alljoyn_busattachment_namehasowner(alljoyn_busattachment bus, const char* name, QCC_BOOL* hasOwner)
+QStatus AJ_CALL alljoyn_busattachment_namehasowner(alljoyn_busattachment bus, AJ_PCSTR name, QCC_BOOL* hasOwner)
 {
     QCC_DbgTrace(("%s", __FUNCTION__));
     bool result;
@@ -475,7 +474,7 @@ QStatus AJ_CALL alljoyn_busattachment_namehasowner(alljoyn_busattachment bus, co
     return ret;
 }
 
-QStatus AJ_CALL alljoyn_busattachment_getpeerguid(alljoyn_busattachment bus, const char* name, char* guid, size_t* guidSz)
+QStatus AJ_CALL alljoyn_busattachment_getpeerguid(alljoyn_busattachment bus, AJ_PCSTR name, AJ_PSTR guid, size_t* guidSz)
 {
     QCC_DbgTrace(("%s", __FUNCTION__));
     qcc::String guidStr;
@@ -492,7 +491,7 @@ QStatus AJ_CALL alljoyn_busattachment_getpeerguid(alljoyn_busattachment bus, con
 QStatus AJ_CALL alljoyn_busattachment_registersignalhandler(alljoyn_busattachment bus,
                                                             alljoyn_messagereceiver_signalhandler_ptr signal_handler,
                                                             const alljoyn_interfacedescription_member member,
-                                                            const char* srcPath)
+                                                            AJ_PCSTR srcPath)
 {
     QCC_DbgTrace(("%s", __FUNCTION__));
     return ((ajn::BusAttachmentC*)bus)->RegisterSignalHandlerC(signal_handler,
@@ -503,7 +502,7 @@ QStatus AJ_CALL alljoyn_busattachment_registersignalhandler(alljoyn_busattachmen
 QStatus AJ_CALL alljoyn_busattachment_registersignalhandlerwithrule(alljoyn_busattachment bus,
                                                                     alljoyn_messagereceiver_signalhandler_ptr signal_handler,
                                                                     const alljoyn_interfacedescription_member member,
-                                                                    const char* matchRule)
+                                                                    AJ_PCSTR matchRule)
 {
     QCC_DbgTrace(("%s", __FUNCTION__));
     return ((ajn::BusAttachmentC*)bus)->RegisterSignalHandlerWithRuleC(signal_handler,
@@ -514,7 +513,7 @@ QStatus AJ_CALL alljoyn_busattachment_registersignalhandlerwithrule(alljoyn_busa
 QStatus AJ_CALL alljoyn_busattachment_unregistersignalhandler(alljoyn_busattachment bus,
                                                               alljoyn_messagereceiver_signalhandler_ptr signal_handler,
                                                               const alljoyn_interfacedescription_member member,
-                                                              const char* srcPath)
+                                                              AJ_PCSTR srcPath)
 {
     QCC_DbgTrace(("%s", __FUNCTION__));
     return ((ajn::BusAttachmentC*)bus)->UnregisterSignalHandlerC(signal_handler,
@@ -525,7 +524,7 @@ QStatus AJ_CALL alljoyn_busattachment_unregistersignalhandler(alljoyn_busattachm
 QStatus AJ_CALL alljoyn_busattachment_unregistersignalhandlerwithrule(alljoyn_busattachment bus,
                                                                       alljoyn_messagereceiver_signalhandler_ptr signal_handler,
                                                                       const alljoyn_interfacedescription_member member,
-                                                                      const char* matchRule)
+                                                                      AJ_PCSTR matchRule)
 {
     QCC_DbgTrace(("%s", __FUNCTION__));
     return ((ajn::BusAttachmentC*)bus)->UnregisterSignalHandlerWithRuleC(signal_handler,
@@ -539,7 +538,7 @@ QStatus AJ_CALL alljoyn_busattachment_unregisterallhandlers(alljoyn_busattachmen
     return ((ajn::BusAttachmentC*)bus)->UnregisterAllHandlersC();
 }
 
-QStatus AJ_CALL alljoyn_busattachment_setdaemondebug(alljoyn_busattachment bus, const char* module, uint32_t level)
+QStatus AJ_CALL alljoyn_busattachment_setdaemondebug(alljoyn_busattachment bus, AJ_PCSTR module, uint32_t level)
 {
     QCC_DbgTrace(("%s", __FUNCTION__));
     return ((ajn::BusAttachmentC*)bus)->SetDaemonDebug(module, level);
@@ -551,7 +550,7 @@ uint32_t AJ_CALL alljoyn_busattachment_gettimestamp()
     return ajn::BusAttachmentC::GetTimestamp();
 }
 
-QStatus AJ_CALL alljoyn_busattachment_ping(alljoyn_busattachment bus, const char* name, uint32_t timeout)
+QStatus AJ_CALL alljoyn_busattachment_ping(alljoyn_busattachment bus, AJ_PCSTR name, uint32_t timeout)
 {
     QCC_DbgTrace(("%s", __FUNCTION__));
     return ((ajn::BusAttachmentC*)bus)->Ping(name, timeout);
@@ -578,7 +577,7 @@ void AJ_CALL alljoyn_busattachment_unregisterallaboutlisteners(alljoyn_busattach
 }
 
 QStatus AJ_CALL alljoyn_busattachment_whoimplements_interfaces(alljoyn_busattachment bus,
-                                                               const char** implementsInterfaces,
+                                                               AJ_PCSTR* implementsInterfaces,
                                                                size_t numberInterfaces)
 {
     QCC_DbgTrace(("%s", __FUNCTION__));
@@ -586,14 +585,14 @@ QStatus AJ_CALL alljoyn_busattachment_whoimplements_interfaces(alljoyn_busattach
 }
 
 QStatus AJ_CALL alljoyn_busattachment_whoimplements_interface(alljoyn_busattachment bus,
-                                                              const char* implementsInterface)
+                                                              AJ_PCSTR implementsInterface)
 {
     QCC_DbgTrace(("%s", __FUNCTION__));
     return ((ajn::BusAttachmentC*)bus)->WhoImplements(implementsInterface);
 }
 
 QStatus AJ_CALL alljoyn_busattachment_cancelwhoimplements_interfaces(alljoyn_busattachment bus,
-                                                                     const char** implementsInterfaces,
+                                                                     AJ_PCSTR* implementsInterfaces,
                                                                      size_t numberInterfaces)
 {
     QCC_DbgTrace(("%s", __FUNCTION__));
@@ -601,9 +600,26 @@ QStatus AJ_CALL alljoyn_busattachment_cancelwhoimplements_interfaces(alljoyn_bus
 }
 
 QStatus AJ_CALL alljoyn_busattachment_cancelwhoimplements_interface(alljoyn_busattachment bus,
-                                                                    const char* implementsInterface)
+                                                                    AJ_PCSTR implementsInterface)
 {
     QCC_DbgTrace(("%s", __FUNCTION__));
     return ((ajn::BusAttachmentC*)bus)->CancelWhoImplements(implementsInterface);
 }
 
+const alljoyn_permissionconfigurator AJ_CALL alljoyn_busattachment_getpermissionconfigurator(alljoyn_busattachment bus)
+{
+    QCC_DbgTrace(("%s", __FUNCTION__));
+    return (const alljoyn_permissionconfigurator)(&((ajn::BusAttachmentC*)bus)->GetPermissionConfigurator());
+}
+
+QStatus AJ_CALL alljoyn_busattachment_registerapplicationstatelistener(alljoyn_busattachment bus, alljoyn_applicationstatelistener listener)
+{
+    QCC_DbgTrace(("%s", __FUNCTION__));
+    return ((ajn::BusAttachmentC*)bus)->RegisterApplicationStateListener((*(ajn::ApplicationStateListener*)listener));
+}
+
+QStatus AJ_CALL alljoyn_busattachment_unregisterapplicationstatelistener(alljoyn_busattachment bus, alljoyn_applicationstatelistener listener)
+{
+    QCC_DbgTrace(("%s", __FUNCTION__));
+    return ((ajn::BusAttachmentC*)bus)->UnregisterApplicationStateListener((*(ajn::ApplicationStateListener*)listener));
+}
