@@ -81,12 +81,14 @@ class ObjectSecurityTest : public testing::Test {
 
         /* set up the service bus */
         servicebus = alljoyn_busattachment_create("AuthListenerTestService", false);
+        EXPECT_EQ(ER_OK, DeleteDefaultKeyStoreFileCTest("AuthListenerTestService"));
         status = alljoyn_busattachment_start(servicebus);
         EXPECT_EQ(ER_OK, status) << "  Actual Status: " << QCC_StatusText(status);
         status = alljoyn_busattachment_connect(servicebus, ajn::getConnectArg().c_str());
         EXPECT_EQ(ER_OK, status) << "  Actual Status: " << QCC_StatusText(status);
 
         clientbus = alljoyn_busattachment_create("AuthListenerTestClient", false);
+        EXPECT_EQ(ER_OK, DeleteDefaultKeyStoreFileCTest("AuthListenerTestClient"));
         status = alljoyn_busattachment_start(clientbus);
         EXPECT_EQ(ER_OK, status) << "  Actual Status: " << QCC_StatusText(status);
         status = alljoyn_busattachment_connect(clientbus, ajn::getConnectArg().c_str());
