@@ -223,6 +223,33 @@ class BusAttachment : public MessageReceiver {
     /**
      * Allow the currently executing method/signal handler to enable concurrent callbacks
      * during the scope of the handler's execution.
+     *
+     * This member function can ONLY be called from within the body of a signal
+     * handler, method handler or other AllJoyn callback. It allows AllJoyn to
+     * dispatch a single (additional) callback while the current one is still
+     * executing. This method is typically used when a method, signal handler or
+     * other AllJoyn callback needs to execute for a long period of time or when
+     * the callback needs to make any kind of blocking call.
+     *
+     * This method MUST be called prior to making any non-asynchronous AllJoyn
+     * remote procedure calls from within an AllJoyn callback. This includes
+     * calls such as JoinSession(), AdvertiseName(), CancelAdvertisedName(),
+     * FindAdvertisedName(), CancelFindAdvertisedName(), SetLinkTimeout(), etc.
+     *
+     * EnableConcurrentCallbacks doesn't take effect when a BusAttachment is
+     * created with just one thread. If the BusAttachment is created with just
+     * one thread, i.e. `ajn::BusAttachment busAttachment(appName, true, 1)` and
+     * the application developer attempts to make a blocking method call in a
+     * callback after invoking EnableConcurrentCallbacks(), the application will
+     * deadlock.
+     *
+     * For the same reason that EnableConcurrentCallbacks cannot be used with
+     * just one thread, the maximum number of concurrent callbacks is limited
+     * to the value specified when creating the BusAttachment. If no concurrency
+     * value was chosen the default is 4. It is the application developers
+     * responsibility to make sure the maximum number of concurrent callbacks is
+     * not exceeded. If the maximum number is exceeded the application will
+     * deadlock.
      */
     void EnableConcurrentCallbacks();
 
@@ -775,7 +802,8 @@ class BusAttachment : public MessageReceiver {
      *                             If this parameter is NULL peer-to-peer authentication is disabled.  This is a
      *                             space separated list of any of the following values:
      *                             ALLJOYN_SRP_LOGON, ALLJOYN_SRP_KEYX, ALLJOYN_ECDHE_NULL, ALLJOYN_ECDHE_PSK,
-     *                             ALLJOYN_ECDHE_ECDSA, GSSAPI.
+     *                             ALLJOYN_ECDHE_ECDSA, GSSAPI, ALLJOYN_ECDHE_SPEKE. Note that the SRP and PSK
+     *                             mechanisms are deprecated and will be removed in a future release.
      *
      * @param authListener         Passes password and other authentication related requests to the application.
      *
@@ -784,10 +812,7 @@ class BusAttachment : public MessageReceiver {
      *                             Note that this parameter is only meaningful when using the default
      *                             key store implementation.
      *
-     * @param isShared             optional parameter that indicates if the key store is shared between multiple
-     *                             applications. It is generally harmless to set this to true even when the
-     *                             key store is not shared but it adds some unnecessary calls to the key store
-     *                             listener to load and store the key store in this case.
+     * @param isShared             This parameter is not used as of 16.04. It is ignored internally (always shared).
      * @param permissionConfigurationListener   Passes security 2.0 callbacks to the application.
      *
      * @return
@@ -797,7 +822,7 @@ class BusAttachment : public MessageReceiver {
     QStatus EnablePeerSecurity(const char* authMechanisms,
                                AuthListener* authListener,
                                const char* keyStoreFileName = NULL,
-                               bool isShared = false,
+                               bool isShared = true,
                                PermissionConfigurationListener* permissionConfigurationListener = NULL);
 
     /**
@@ -1878,8 +1903,13 @@ class BusAttachment : public MessageReceiver {
      * State signal.
      *
      * @param[in] applicationStateListener reference to an ApplicationStateListener
+     *
+     * @return
+     *    - #ER_OK on success
+     *    - #ER_APPLICATION_STATE_LISTENER_ALREADY_EXISTS when the same listener has already been registered.
+     *    - Different error status otherwise
      */
-    void RegisterApplicationStateListener(ApplicationStateListener& applicationStateListener);
+    QStatus RegisterApplicationStateListener(ApplicationStateListener& applicationStateListener);
 
     /**
      * Unregisters the ApplicationStateListener from receiving the
@@ -1888,34 +1918,13 @@ class BusAttachment : public MessageReceiver {
      * @param[in] applicationStateListener reference to an
      *                                     ApplicationStateListener to
      *                                     unregister
-     */
-    void UnregisterApplicationStateListener(ApplicationStateListener& applicationStateListener);
-
-    /**
-     * This is a helper function that will add the match rule responsible for
-     * receiving the org.alljoyn.Bus.Application State signal.
-     *
-     * The ApplicationStateListener should be registered before calling this method.
-     *
-     * This will only call AddMatch for the State signal.
      *
      * @return
      *    - #ER_OK on success
-     *    - An error status otherwise
+     *    - #ER_APPLICATION_STATE_LISTENER_NO_SUCH_LISTENER when this listener has not been registered.
+     *    - Different error status otherwise
      */
-    QStatus AddApplicationStateRule();
-
-    /**
-     * This is a helper function that will remove the match rule responsible for
-     * receiving the org.alljoyn.Bus.Application State signal.
-     *
-     * This will only call RemoveMatch for the State signal.
-     *
-     * @return
-     *    - #ER_OK on success
-     *    - An error status otherwise
-     */
-    QStatus RemoveApplicationStateRule();
+    QStatus UnregisterApplicationStateListener(ApplicationStateListener& applicationStateListener);
 
     /// @cond ALLJOYN_DEV
     /**
