@@ -46,8 +46,9 @@ static const uint8_t PROP_ACCESS_RW    = 3; /**< Read-Write Access type */
 // @}
 /** @name Property Annotation flags */
 // @{
-static const uint8_t PROP_ANNOTATE_EMIT_CHANGED_SIGNAL             = 1; /**< EmitChangedSignal annotate flag. */
-static const uint8_t PROP_ANNOTATE_EMIT_CHANGED_SIGNAL_INVALIDATES = 2; /**< EmitChangedSignal annotate flag (for notifying invalidation of property instead of value. */
+static const uint8_t PROP_ANNOTATE_EMIT_CHANGED_SIGNAL             = 1; /**< EmitsChangedSignal annotate flag. */
+static const uint8_t PROP_ANNOTATE_EMIT_CHANGED_SIGNAL_INVALIDATES = 2; /**< EmitsChangedSignal annotate flag for notifying invalidation of property instead of value. */
+static const uint8_t PROP_ANNOTATE_EMIT_CHANGED_SIGNAL_CONST       = 4; /**< EmitsChangedSignal annotate flag for const property. */
 // @}
 /** @name Method/Signal Annotation flags */
 // @{
@@ -103,6 +104,9 @@ class InterfaceDescription {
     /** map containing description tables per argument name */
     class ArgumentDescriptions;
 
+    /** map containing annotations per argument name */
+    class ArgumentAnnotations;
+
   public:
 
     class AnnotationsMap; /**< A map to store string annotations */
@@ -125,6 +129,7 @@ class InterfaceDescription {
         bool isSessionlessSignal;                   /**< True if this is described as a sessionless signal */
         bool isUnicastSignal;                       /**< True if this is described as a unicast signal */
         bool isGlobalBroadcastSignal;               /**< True if this is described as a global broadcast signal */
+        ArgumentAnnotations* argumentAnnotations;   /**< Map of argument annotations */
 
         /**
          * %Member constructor.
@@ -175,17 +180,38 @@ class InterfaceDescription {
          *
          * @param        annotationName   name of the annotation to look for
          * @param[out]   value            The value of the annotation, if found
-         * @return                        true iff annotations[name] == value
+         * @return                        true if annotations[name] == value
          */
         bool GetAnnotation(const qcc::String& annotationName, qcc::String& value) const;
 
+        /**
+         * Get the names and values of argument annotations.
+         *
+         * @param[in] argName   Argument name of the member from which to get annotations.
+         * @param[out] names    Annotation names. Either NULL or the caller must allocate enough memory for the number of annotations.
+         * @param[out] values   Annotation values. Either NULL or the caller must allocate enough memory for the number of annotations.
+         * @param[in]  size     Number of annotations to get.
+         * @return              The number of annotations returned or the total number of annotations if names or values is NULL.
+         */
+        size_t GetArgAnnotations(const char* argName, qcc::String* names = NULL, qcc::String* values = NULL, size_t size = 0) const;
+
+        /**
+         * Get this member's argument annotation value
+         *
+         * @param[in]    argName          Argument name of the member from which to get the annotation.
+         * @param[in]    annotationName   Annotation name to look for.
+         * @param[out]   value            The value of the annotation, if found.
+         * @return                        true if annotations[name] == value.
+         */
+        bool GetArgAnnotation(const char* argName, const qcc::String& annotationName, qcc::String& value) const;
+
         /**
          * Equality. Two members are defined to be equal if their members are
          * equal except for iface which is ignored for equality.
          *
          * @param o   Member to compare against this member.
          *
-         * @return    true iff o == this member.
+         * @return    true if o == this member.
          */
         bool operator==(const Member& o) const;
     };
@@ -241,7 +267,7 @@ class InterfaceDescription {
          * Get this property's annotation value
          * @param annotationName   name of the annotation to look for
          * @param[out]             value  The value of the annotation, if found
-         * @return                 true iff annotations[name] == value
+         * @return                 true if annotations[name] == value
          */
         bool GetAnnotation(const qcc::String& annotationName, qcc::String& value) const;
 
@@ -533,7 +559,7 @@ class InterfaceDescription {
      * Returns a description of the interface in introspection XML format
      * @return The interface description in introspection XML format.
      *
-     * @param indent   Number of space chars to use in XML indentation.
+     * @param indent   Number of characters to indent the XML
      * @param languageTag Specifies the description language or NULL to omit descriptions
      * @param translator Translator instance to use for introspection descriptions
      * @return The XML introspection data.
@@ -683,12 +709,40 @@ class InterfaceDescription {
      */
     bool HasDescription() const;
 
+    /**
+     * Add an annotation to an existing argument.
+     *
+     * @param member     Name of member.
+     * @param arg        Name of the argument.
+     * @param name       Name of annotation.
+     * @param value      Value for the annotation.
+     *
+     * @return
+     *      - #ER_OK if successful
+     *      - #ER_BUS_ANNOTATION_ALREADY_EXISTS if annotation already exists
+     */
+    QStatus AddArgAnnotation(const char* member, const char* arg, const qcc::String& name, const qcc::String& value);
+
+    /**
+     * Get annotation from an existing argument.
+     *
+     * @param member     Name of member.
+     * @param arg        Name of the argument.
+     * @param name       Name of annotation.
+     * @param value      Output value for the annotation.
+     *
+     * @return
+     *      - true if found
+     *      - false if annotation not found
+     */
+    bool GetArgAnnotation(const char* member, const char* arg, const qcc::String& name, qcc::String& value) const;
+
   private:
 
     /**
      * Default constructor is private
      */
-    InterfaceDescription() { }
+    InterfaceDescription();
 
     /**
      * Construct an interface with no methods or properties
@@ -708,16 +762,28 @@ class InterfaceDescription {
      */
     InterfaceDescription& operator=(const InterfaceDescription& other);
 
-    void AppendDescriptionXml(qcc::String& xml, const char* language,
-                              const char* localDescription, Translator* translator, qcc::String const& indent) const;
+    void AppendDescriptionToAnnotations(AnnotationsMap& annotations, const char* description, Translator* translator) const;
 
-    qcc::String NextArg(const char*& signature, qcc::String& argNames, bool inOut,
-                        size_t indent, Member const& member, bool withDescriptions,
-                        const char* langTag, Translator* translator) const;
+    void AppendDescriptionToArgAnnotations(ArgumentAnnotations& argAnnotations, const char* argName, const char* description, Translator* translator) const;
+
+    void AppendDescriptionXml(qcc::String& xml,
+                              const char* language,
+                              const char* localDescription,
+                              Translator* translator,
+                              const char* languageDescription,
+                              qcc::String const& indent) const;
+
+    qcc::String NextArg(const char*& signature,
+                        qcc::String& argNames,
+                        bool inOut,
+                        size_t indent,
+                        Member const& member,
+                        bool withDescriptions,
+                        const char* langTag,
+                        Translator* translator) const;
 
     const char* Translate(const char* toLanguage, const char* text, qcc::String& buffer, Translator* translator) const;
 
-
     struct Definitions;
     Definitions* defs;   /**< The definitions for this interface */
 
