@@ -179,11 +179,11 @@ typedef qcc::ManagedObj<_Message> Message;
 
 
 /**
- * This class implements the functionality underlying the #Message class. Instances
+ * This class implements the functionality underlying the Message class. Instances
  * of #_Message should not be declared directly by applications. Rather applications
- * create instances of the class #Message which handles reference counting for the
+ * create instances of the class Message which handles reference counting for the
  * underlying #_Message instance. The members of #_Message are always accessed
- * indirectly via #Message.
+ * indirectly via Message.
  */
 class _Message {
 
@@ -203,6 +203,7 @@ class _Message {
     friend class Crypto;
     friend class _PeerState;
     friend class PermissionMgmtObj;
+    friend class _Manifest;
     friend struct Rule;
 
   public:
@@ -934,6 +935,22 @@ class _Message {
      */
     QStatus Unmarshal(qcc::String& endpointName, bool handlePassing, bool checkSender, bool pedantic = true, uint32_t timeout = 0);
 
+    /**
+     * @internal
+     * Unmarshals a message. Only the message header is unmarshaled at this
+     * time.
+     *
+     * @param peerStateTable The peer state table used when unmarshalling encrypted arguments.
+     * @param endpointName   The uinique name of the endpoint that this message came from.
+     * @param handlePassing  True if handle passing is allowed.
+     * @param checkSender    True if message's sender field should be validated against the endpoint's unique name.
+     * @param pedantic       Perform detailed checks on the header fields.
+     * @return
+     *      - #ER_OK if successful
+     *      - An error status otherwise
+     */
+    QStatus Unmarshal(PeerStateTable* peerStateTable, qcc::String& endpointName, bool handlePassing, bool checkSender, bool pedantic);
+
     /**
      * @internal
      * Deliver a marshaled message to a remote endpoint.
@@ -972,6 +989,14 @@ class _Message {
      */
     void SetSerialNumber();
 
+    /**
+     * @internal
+     * Sets the serial number for this message.
+     *
+     * @param serialNumber  New serial number for this message.
+     */
+    void SetSerialNumber(uint32_t serialNumber);
+
     /// @endcond
 
   private:
