@@ -44,7 +44,7 @@ class SecurityApplicationProxy : public ProxyBusObject {
      * @param[in] busName Unique or well-known name of remote AllJoyn bus
      * @param[in] sessionId the session received after joining AllJoyn session
      */
-    SecurityApplicationProxy(BusAttachment& bus, const char* busName, SessionId sessionId = 0);
+    SecurityApplicationProxy(BusAttachment& bus, AJ_PCSTR busName, SessionId sessionId = 0);
 
     /**
      * ClaimableApplicationProxy destructor
@@ -66,17 +66,14 @@ class SecurityApplicationProxy : public ProxyBusObject {
      * GetApplicationState get An enumeration representing the current state of
      * the application.  The list of valid values:
      *
-     * | Value | Description                                                       |
-     * |-------|-------------------------------------------------------------------|
-     * | 0     | NotClaimable.  The application is not claimed and not accepting   |
-     * |       | claim requests.                                                   |
-     * | 1     | Claimable.  The application is not claimed and is accepting claim |
-     * |       | requests.                                                         |
-     * | 2     | Claimed. The application is claimed and can be configured.        |
-     * | 3     | NeedUpdate. The application is claimed, but requires a            |
-     * |       | configuration update (after a software upgrade).                  |
+     * | Value         | Description                                                      |
+     * |---------------|------------------------------------------------------------------|
+     * | NOT_CLAIMABLE | The application is not claimed and not accepting claim requests. |
+     * | CLAIMABLE     | The application is not claimed and is accepting claim requests.  |
+     * | CLAIMED       | The application is claimed and can be configured.                |
+     * | NEED_UPDATE   | The application is claimed, but requires a configuration update. |
      *
-     * @param[out] version of the service.
+     * @param[out] applicationState of the service.
      *
      * @return
      *  - #ER_OK if successful
@@ -140,17 +137,30 @@ class SecurityApplicationProxy : public ProxyBusObject {
      */
     QStatus GetManifestTemplate(MsgArg& rules);
 
+    /**
+     * Returns the XML version of the manifest template. The returned value
+     * is managed by the caller and has to be freed using delete[].
+     *
+     * @param[out] manifestTemplate The manifest temaplte in XML format.
+     *
+     * @return
+     *  - #ER_OK if successful.
+     *  - An error status indicating failure.
+     */
+    QStatus GetManifestTemplate(AJ_PSTR* manifestTemplateXml);
+
     /**
      * The authentication mechanisms the application supports for the claim process.
      * It is a bit mask.
      *
-     * | Mask  | Description                                                   |
-     * |-------|---------------------------------------------------------------|
-     * | 0x1   | claiming via ECDHE_NULL                                       |
-     * | 0x2   | claiming via ECDHE_PSK                                        |
-     * | 0x4   | claiming via ECDHE_ECDSA                                      |
+     * | Mask                  | Description              |
+     * |-----------------------|--------------------------|
+     * | CAPABLE_ECDHE_NULL    | claiming via ECDHE_NULL  |
+     * | CAPABLE_ECDHE_PSK     | claiming via ECDHE_PSK   |
+     * | CAPABLE_ECDHE_SPEKE   | claiming via ECDHE_SPEKE |
+     * | CAPABLE_ECDHE_ECDSA   | claiming via ECDHE_ECDSA |
      *
-     * @param[out] claimCapabilities The authentication mechanisms the application
+     * @param[out] claimCapabilities The authentication mechanisms the application supports
      *
      * @return
      *  - #ER_OK if successful
@@ -162,12 +172,12 @@ class SecurityApplicationProxy : public ProxyBusObject {
      * The additional information information on the claim capabilities.
      * It is a bit mask.
      *
-     * | Mask  | Description                                                   |
-     * |-------|---------------------------------------------------------------|
-     * | 0x1   | PSK generated by Security Manager                             |
-     * | 0x2   | PSK generated by application                                  |
+     * | Mask                              | Description                                   |
+     * |-----------------------------------|-----------------------------------------------|
+     * | PSK_GENERATED_BY_SECURITY_MANAGER | PSK or password generated by Security Manager |
+     * | PSK_GENERATED_BY_APPLICATION      | PSK or password generated by application      |
      *
-     * @param[out] claimCapabilities The authentication mechanisms the application
+     * @param[out] claimCapabilitiesAdditionalInfo The additional information
      *
      * @return
      *  - #ER_OK if successful
@@ -179,8 +189,13 @@ class SecurityApplicationProxy : public ProxyBusObject {
      * Claim the app. This will make the claimer the admin and certificate
      * authority. The KeyInfo object description is shown below.
      *
+     * It is highly recommended that element 0 of identityCertificateChain, the peer's
+     * end entity certificate, be of type qcc::IdentityCertificate, so that the correct
+     * Extended Key Usage (EKU) is set. The remaining certificates in the chain can be
+     * of this or the base CertificateX509 type.
+     *
      * Access restriction: None if the app is not yet claimed. An error will be
-     * raised if the app already has an admin and it is not the caller.
+     * raised if the app has already been claimed.
      *
      * @param[in] certificateAuthority   a KeyInfo object representing the
      *                                   public key of the certificate authority
@@ -189,10 +204,10 @@ class SecurityApplicationProxy : public ProxyBusObject {
      *                                   security group authority
      * @param[in] identityCertChain      the identity certificate chain for the
      *                                   claimed app.  The leaf cert is listed first
-     * @param[in] identityCertChainSize  the size of the identity certificate
+     * @param[in] identityCertChainCount the number of the identity certificates in the
      *                                   chain.
-     * @param[in] manifest               the manifest of the application
-     * @param[in] manifestSize           the number of rules in the manifest
+     * @param[in] manifests              the signed manifests to install on the application
+     * @param[in] manifestCount          the number of manifests
      * @return
      *  - #ER_OK if successful
      *  - #ER_PERMISSION_DENIED Error raised when the application is not claimable
@@ -200,15 +215,54 @@ class SecurityApplicationProxy : public ProxyBusObject {
      *                            chain is not valid
      *  - #ER_INVALID_CERTIFICATE_USAGE Error raised when the Extended Key Usage
      *                                  is not AllJoyn specific
-     *  - #ER_DIGEST_MISMATCH Error raised when the digest of the manifest does
-     *                        not match the digested listed in the identity certificate
+     *  - #ER_DIGEST_MISMATCH Error raised when the manifest is not accepted by the application.
      *  - an error status indicating failure
      */
     QStatus Claim(const qcc::KeyInfoNISTP256& certificateAuthority,
                   const qcc::GUID128& adminGroupId,
                   const qcc::KeyInfoNISTP256& adminGroup,
-                  const qcc::IdentityCertificate* identityCertChain, size_t identityCertChainSize,
-                  const PermissionPolicy::Rule* manifest, size_t manifestSize);
+                  const qcc::CertificateX509* identityCertChain, size_t identityCertChainCount,
+                  const Manifest* manifests, size_t manifestCount);
+
+    /**
+     * Claim the app using a manifests in XML format. This will make
+     * the claimer the admin and certificate authority. The KeyInfo
+     * object description is shown below.
+     *
+     * Access restriction: None if the app is not yet claimed. An error will be
+     * raised if the app has already been claimed.
+     *
+     * @param[in]   certificateAuthority    A KeyInfo object representing the
+     *                                      public key of the certificate authority.
+     * @param[in]   adminGroupId            The admin group Id.
+     * @param[in]   adminGroup              A KeyInfo object representing the admin
+     *                                      security group authority.
+     * @param[in]   identityCertChain       The identity certificate chain for the
+     *                                      claimed app.  The leaf cert is listed first
+     *                                      followed by each intermediate Certificate
+     *                                      Authority's certificate, ending in the trusted
+     *                                      root's certificate.
+     * @param[in]   identityCertChainCount  The count of the identity certificate
+     *                                      chain elements.
+     * @param[in]   manifestsXmls           An array of null-terminated strings containing
+     *                                      the application's manifests in XML format.
+     * @param[in]   manifestsCount          Count of "manifestsXmls" array elements.
+     * @return
+     *  - #ER_OK if successful
+     *  - #ER_PERMISSION_DENIED Error raised when the application is not claimable
+     *  - #ER_INVALID_CERTIFICATE Error raised when the identity certificate
+     *                            chain is not valid
+     *  - #ER_INVALID_CERTIFICATE_USAGE Error raised when the Extended Key Usage
+     *                                  is not AllJoyn specific.
+     *  - #ER_DIGEST_MISMATCH Error raised when none of the provided signed manifests are
+     *                                         valid for the given identity certificate.
+     *  - an error status indicating failure
+     */
+    QStatus Claim(const qcc::KeyInfoNISTP256& certificateAuthority,
+                  const qcc::GUID128& adminGroupId,
+                  const qcc::KeyInfoNISTP256& adminGroup,
+                  const qcc::CertificateX509* identityCertChain, size_t identityCertChainCount,
+                  AJ_PCSTR* manifestsXmls, size_t manifestsCount);
 
     /**
      * The ClaimableApplication version
@@ -240,7 +294,47 @@ class SecurityApplicationProxy : public ProxyBusObject {
 
     /**
      * This method allows an admin to update the application's identity certificate
-     * chain and its manifest.
+     * chain and its manifests using the Manifest objects.
+     *
+     * After having a new identity certificate installed, the target bus clears
+     * out all of its peer's secret and session keys, so the next call will get
+     * security violation. After calling UpdateIdentity, SecureConnection(true)
+     * should be called to force the peers to create a new set of secret and
+     * session keys.
+     *
+     * It is highly recommended that element 0 of identityCertificateChain, the peer's
+     * end entity certificate, be of type qcc::IdentityCertificate. Other certs can be
+     * of this or the base CertificateX509 type.
+     *
+     * The target peer also clears all manifests it has already stored, and so all
+     * manifests the peer needs must be sent again. Use GetManifests to retrieve
+     * the currently-installed manifests before calling UpdateIdentity to reuse them.
+     *
+     * Manifests must already be signed by the authority that issued the identity
+     * certificate chain.
+     *
+     * @see ProxyBusObject.SecureConnection(bool)
+     * @see SecurityApplicationProxy.GetManifests(std::vector<Manifest>&)
+     *
+     * @param[in] identityCertificateChain             the identity certificate
+     * @param[in] identityCertificateChainCount        the number of identity certificates
+     * @param[in] manifests                            the signed manifests to install on the application
+     * @param[in] manifestCount                        the number of manifests
+     *
+     * @return
+     *  - #ER_OK if successful
+     *  - #ER_PERMISSION_DENIED Error raised when the caller does not have permission
+     *  - #ER_INVALID_CERTIFICATE Error raised when the identity certificate chain is not valid
+     *  - #ER_INVALID_CERTIFICATE_USAGE Error raised when the Extended Key Usage is not AllJoyn specific
+     *  - #ER_DIGEST_MISMATCH Error raised when the digest is not accepted
+     *  - an error status indicating failure
+     */
+    QStatus UpdateIdentity(const qcc::CertificateX509* identityCertificateChain, size_t identityCertificateChainCount,
+                           const Manifest* manifest, size_t manifestCount);
+
+    /**
+     * This method allows an admin to update the application's identity certificate
+     * chain and its manifests using manifests in XML format.
      *
      * After having a new identity certificate installed, the target bus clears
      * out all of its peer's secret and session keys, so the next call will get
@@ -250,21 +344,23 @@ class SecurityApplicationProxy : public ProxyBusObject {
      *
      * @see ProxyBusObject.SecureConnection(bool)
      *
-     * @param[in] identityCertificateChain             the identity certificate
-     * @param[in] identityCertificateChainSize         the number of identity certificates
-     * @param[in] manifest                             the manifest
-     * @param[in] manifestSize                         the number of rules in the manifest
+     * @param[in] identityCertificateChain       the identity certificate
+     * @param[in] identityCertificateChainCount  the number of identity certificates
+     * @param[in] manifestsXmls                  An array of null-terminated strings containing the application's
+     *                                           manifests in XML format.
+     * @param[in] manifestsCount                 Count of "manifestsXmls" array elements.
      *
      * @return
-     *  - #ER_OK if successful
-     *  - #ER_PERMISSION_DENIED Error raised when the caller does not have permission
-     *  - #ER_INVALID_CERTIFICATE Error raised when the identity certificate chain is not valid
-     *  - #ER_INVALID_CERTIFICATE_USAGE Error raised when the Extended Key Usage is not AllJoyn specific
-     *  - #ER_DIGEST_MISMATCH Error raised when the digest of the not have permission
-     *  - an error status indicating failure
+     *  - #ER_OK                        If successful.
+     *  - #ER_PERMISSION_DENIED         Error raised when the caller does not have permission.
+     *  - #ER_INVALID_CERTIFICATE       Error raised when the identity certificate chain is not valid.
+     *  - #ER_INVALID_CERTIFICATE_USAGE Error raised when the Extended Key Usage is not AllJoyn specific.
+     *  - #ER_DIGEST_MISMATCH           Error raised when none of the provided signed manifests are
+     *                                  valid for the given identity certificate.
+     *  - An error status indicating failure
      */
-    QStatus UpdateIdentity(const qcc::IdentityCertificate* identityCertificateChain, size_t identityCertificateChainSize,
-                           const PermissionPolicy::Rule* manifest, size_t manifestSize);
+    QStatus UpdateIdentity(const qcc::CertificateX509* identityCertificateChain, size_t identityCertificateChainCount,
+                           AJ_PCSTR* manifestsXmls, size_t manifestsCount);
 
     /**
      * This method allows an admin to install the permission policy to the
@@ -276,6 +372,10 @@ class SecurityApplicationProxy : public ProxyBusObject {
      * violation. After calling UpdatePolicy, SecureConnection(true) should be
      * called to force the peers to create a new set of secret and session keys.
      *
+     * Until ASACORE-2755 is fixed the caller must include all default policies
+     * (containing information about the trust anchors) with each call, so they
+     * would not be removed.
+     *
      * @see ProxyBusObject.SecureConnection(bool)
      *
      * @param[in] policy the new policy.
@@ -289,6 +389,36 @@ class SecurityApplicationProxy : public ProxyBusObject {
      */
     QStatus UpdatePolicy(const PermissionPolicy& policy);
 
+    /**
+     * This method allows an admin to install the permission policy to the
+     * application using an XML version of the policy.
+     * Any existing policy will be replaced if the new policy version
+     * number is greater than the existing policy's version number.
+     *
+     * After having a new policy installed, the target bus clears out all of
+     * its peer's secret and session keys, so the next call will get security
+     * violation. After calling UpdatePolicy, SecureConnection(true) should be
+     * called to force the peers to create a new set of secret and session keys.
+     *
+     * Until ASACORE-2755 is fixed the caller must include all default policies
+     * (containing information about the trust anchors) with each call, so they
+     * would not be removed.
+     *
+     * @see ProxyBusObject.SecureConnection(bool)
+     *
+     * @param[in] policy    The new policy in XML format. For the policy XSD refer to
+     *                      alljoyn_core/docs/policy.xsd.
+     *
+     * @return
+     *  - #ER_OK if successful.
+     *  - #ER_PERMISSION_DENIED Error raised when the caller does not have permission.
+     *  - #ER_POLICY_NOT_NEWER  Error raised when the new policy does not have a
+     *                          greater version number than the existing policy.
+     *  - #ER_XML_MALFORMED     If the policy was not in the valid XML format.
+     *  - an error status indicating failure.
+     */
+    QStatus UpdatePolicy(AJ_PCSTR policyXml);
+
     /**
      * This method allows an admin to remove the currently installed policy.  The
      * application reverts back to the default policy generated during the claiming
@@ -302,13 +432,18 @@ class SecurityApplicationProxy : public ProxyBusObject {
     QStatus ResetPolicy();
 
     /**
-     * This method allows the amdin to install a membership cert chain to the
+     * This method allows the admin to install a membership cert chain to the
      * application.
      *
-     * @param[in] certificateChain the membership certificate chain. It can be a
-     *                             single certificate if it is issued by the security
-     *                             group authority.
-     * @param[in] certificateChainSize the number of certificates in the certificate chain
+     * It is highly recommended that element 0 of certificateChain, the peer's
+     * end entity certificate, be of type qcc::MembershipCertificate, so that the correct
+     * Extended Key Usage (EKU) is set. The remaining certificates in the chain can be
+     * of this or the base CertificateX509 type.
+     *
+     * @param[in] certificateChain      The membership certificate chain. It can be a
+     *                                  single certificate if it is issued by the security
+     *                                  group authority.
+     * @param[in] certificateChainCount The number of certificates in the certificate chain.
      *
      * @return
      *  - #ER_OK if successful
@@ -318,7 +453,7 @@ class SecurityApplicationProxy : public ProxyBusObject {
      *  - #ER_INVALID_CERTIFICATE Error raised when the membership certificate is not valid.
      *  - an error status indicating failure
      */
-    QStatus InstallMembership(const qcc::MembershipCertificate* certificateChain, size_t certificateChainSize);
+    QStatus InstallMembership(const qcc::CertificateX509* certificateChain, size_t certificateChainSize);
 
     /**
      * This method allows an admin to remove a membership certificate chain from the
@@ -335,6 +470,20 @@ class SecurityApplicationProxy : public ProxyBusObject {
      */
     QStatus RemoveMembership(const qcc::String& serial, const qcc::KeyInfoNISTP256& issuerKeyInfo);
 
+    /**
+     * This method allows an admin to install one or more additional signed manifests to the application.
+     *
+     * @param[in] manifests array of signed manifests to append to the application's existing set of manifests
+     * @param[in] manifestCount number of manifests in the array
+     *
+     * @return
+     *  - #ER_OK if successful
+     *  - #ER_PERMISSION_DENIED Error raised when the caller does not have permission
+     *  - #ER_DIGEST_MISMATCH Error raised when none of the manifests are accepted by the application
+     *  - an error status indicating failure
+     */
+    QStatus InstallManifests(const Manifest* manifests, size_t manifestCount);
+
     /**
      * Get the ManagedApplication version
      *
@@ -358,15 +507,15 @@ class SecurityApplicationProxy : public ProxyBusObject {
     QStatus GetIdentity(MsgArg& identityCertificate);
 
     /**
-     * Get the manifest
+     * Get the manifests
      *
-     * @param[out] manifest the manifest
+     * @param[out] manifests the manifest
      *
      * @return
      *  - #ER_OK if successful
      *  - an error status indicating failure
      */
-    QStatus GetManifest(MsgArg& manifest);
+    QStatus GetManifests(std::vector<Manifest>& manifest);
 
     /**
      * Get the serial number and issuer of the currently installed identity certificate.
@@ -427,6 +576,28 @@ class SecurityApplicationProxy : public ProxyBusObject {
      */
     QStatus GetMembershipSummaries(MsgArg& membershipSummaries);
 
+    /**
+     * This method notifies the application about the fact that the Security Manager
+     * will start to make changes to the application's security settings.
+     *
+     * @return
+     *  - #ER_OK if successful
+     *  - #ER_PERMISSION_DENIED Error raised when the caller does not have permission
+     *  - some other error status indicating failure
+     */
+    QStatus StartManagement();
+
+    /**
+     * This method notifies the application about the fact that the Security Manager
+     * has finished making changes to the application's security settings.
+     *
+     * @return
+     *  - #ER_OK if successful
+     *  - #ER_PERMISSION_DENIED Error raised when the caller does not have permission
+     *  - some other error status indicating failure
+     */
+    QStatus EndManagement();
+
     /**
      * Populate the array of identity certificates with data from the msg arg
      * @param arg the message arg with signature a(yay)
@@ -436,7 +607,7 @@ class SecurityApplicationProxy : public ProxyBusObject {
      *  - #ER_OK if successful
      *  - an error status indicating failure
      */
-    static QStatus MsgArgToIdentityCertChain(const MsgArg& arg, qcc::IdentityCertificate* certs, size_t expectedSize);
+    static QStatus MsgArgToIdentityCertChain(const MsgArg& arg, qcc::CertificateX509* certs, size_t expectedSize);
 
     /**
      * Populate the array of serial numbers and issuer key infos with data from
@@ -462,6 +633,22 @@ class SecurityApplicationProxy : public ProxyBusObject {
      */
     static QStatus MsgArgToRules(const MsgArg& arg, PermissionPolicy::Rule* rules, size_t expectedSize);
 
+  private:
+
+    /**
+     * Extracts a vector of Manifest objects from an array of signed
+     * manifests in XML format.
+     *
+     * @param[in]    manifestsXmls   An array of signed manifests in XML format.
+     * @param[in]    manifestsCount  Count of the "manifestsXmls" array elements.
+     * @param[out]   manifests       Output vector containing the Manifest objects.
+     *
+     * @return
+     *  - #ER_OK             If successful
+     *  - #ER_XML_MALFORMED  If one of the signed manifests was in an invalid format.
+     *  - Some other error status indicating failure.
+     */
+    QStatus ExtractManifests(AJ_PCSTR* manifestsXmls, size_t manifestsCount, std::vector<Manifest>& manifests);
 };
 }
 
