@@ -20,7 +20,6 @@
  ******************************************************************************/
 #include <qcc/platform.h>
 
-#include <assert.h>
 #include <errno.h>
 #include <sys/types.h>
 #if !defined(QCC_OS_GROUP_WINDOWS)
@@ -85,12 +84,12 @@ static String ExpandPath(const String& path, const String& srcFileName = String:
          */
         size_t position = path.find_first_of(PATH_SEP);
         String user = path.substr(1, position - 1);
-        String home;
+        char* home;
         struct passwd* pwd = NULL;
         if (user.empty()) {
             home = getenv("HOME");
             /* If HOME is not set get the user's home directory from the /etc/passwd */
-            if (home.empty()) {
+            if (!home) {
                 pwd = getpwuid(getuid());
             }
         } else {
@@ -99,14 +98,14 @@ static String ExpandPath(const String& path, const String& srcFileName = String:
         if (pwd) {
             home = pwd->pw_dir;
         }
-        return home + path.substr(position);
+        return String(home) + path.substr(position);
 #endif
     } else if (srcFileName.empty()) {
         // Use path as is.
         return path;
     } else {
         // A path relative to the currently processed file
-        return srcFileName.substr(0, srcFileName.find_last_of(PATH_SEP) + 1) + path;
+        return srcFileName.substr(0, srcFileName.find_last_of_std(PATH_SEP) + 1) + path;
     }
 
 #undef PATH_SEP
@@ -133,7 +132,7 @@ void ConfigDB::NameOwnerChanged(const String& alias,
 ConfigDB::ConfigDB(const String defaultXml, const String srcFileName) :
     defaultXml(defaultXml), fileName(srcFileName), db(new DB()), stopping(false)
 {
-    assert(!singleton);
+    QCC_ASSERT(!singleton);
     if (!singleton) {
         singleton = this;
     }
