@@ -22,13 +22,13 @@
 
 #include <qcc/platform.h>
 
-#include <assert.h>
 
 #include <qcc/Debug.h>
 #include <qcc/Logger.h>
 #include <qcc/String.h>
 #include <qcc/Util.h>
 #include <qcc/atomic.h>
+#include <qcc/LockLevel.h>
 
 #include <alljoyn/AllJoynStd.h>
 #include <alljoyn/Status.h>
@@ -52,7 +52,9 @@ namespace ajn {
 
 #define SESSION_SELF_JOIN 0x02
 
-DaemonRouter::DaemonRouter() : ruleTable(), nameTable(), busController(NULL), alljoynObj(NULL), sessionlessObj(NULL)
+DaemonRouter::DaemonRouter()
+    : ruleTable(), nameTable(), busController(NULL), alljoynObj(NULL), sessionlessObj(NULL),
+    m_Lock(LOCK_LEVEL_DAEMONROUTER_MLOCK)
 {
 #ifdef ENABLE_POLICYDB
     AddBusNameListener(ConfigDB::GetConfigDB());
@@ -208,7 +210,7 @@ QStatus DaemonRouter::PushMessage(Message& msg, BusEndpoint& src)
                   msg->IsSessionless() ? "sessionless " : "",
                   msg->Description().c_str(), msg->GetCallSerial(), src->GetUniqueName().c_str()));
 
-    assert(src->GetEndpointType() != ENDPOINT_TYPE_VIRTUAL);
+    QCC_ASSERT(src->GetEndpointType() != ENDPOINT_TYPE_VIRTUAL);
     /*
      * Since asserts are compiled out in release code, we return an error here.
      */
@@ -566,7 +568,7 @@ QStatus DaemonRouter::PushMessage(Message& msg, BusEndpoint& src)
 #endif
     }
 
-    assert(status != ER_NONE);
+    QCC_ASSERT(status != ER_NONE);
 
     // ASACORE-1632: Why are autogenerated error replies not sent when the sender is a B2B endpoint?
     if ((status != ER_OK) && replyIsExpected && !srcIsB2b) {
