@@ -383,7 +383,7 @@ class TCPTransport : public Transport, public _RemoteEndpoint::EndpointListener,
      * @internal
      * @brief Manage the list of endpoints for the transport.
      */
-    void ManageEndpoints(qcc::Timespec authTimeout, qcc::Timespec sessionSetupTimeout);
+    void ManageEndpoints(uint32_t authTimeout, uint32_t sessionSetupTimeout);
 
     /**
      * @internal
@@ -570,7 +570,7 @@ class TCPTransport : public Transport, public _RemoteEndpoint::EndpointListener,
      * in connection establishment, there should be few connections in this
      * state, so we default to a quite low number.
      */
-    static const uint32_t ALLJOYN_MAX_INCOMPLETE_CONNECTIONS_TCP_DEFAULT = 10;
+    static const uint32_t ALLJOYN_MAX_INCOMPLETE_CONNECTIONS_TCP_DEFAULT = 48;
 
     /**
      * @brief The default value for the maximum number of TCP connections
@@ -589,7 +589,7 @@ class TCPTransport : public Transport, public _RemoteEndpoint::EndpointListener,
      * connections will be accepted.  This is because we are defending against
      * attacks from "abroad" and trust ourselves implicitly.
      */
-    static const uint32_t ALLJOYN_MAX_COMPLETED_CONNECTIONS_TCP_DEFAULT = 50;
+    static const uint32_t ALLJOYN_MAX_COMPLETED_CONNECTIONS_TCP_DEFAULT = 64;
 
     /**
      * @brief The default value for the maximum number of remote clients using tcp.
@@ -600,7 +600,7 @@ class TCPTransport : public Transport, public _RemoteEndpoint::EndpointListener,
      * @warning This maximum is enforced on incoming connections from untrusted clients over tcp.
      * This is to limit the amount of resources being used by untrusted clients over tcp.
      */
-    static const uint32_t ALLJOYN_MAX_REMOTE_CLIENTS_TCP_DEFAULT = 16;
+    static const uint32_t ALLJOYN_MAX_REMOTE_CLIENTS_TCP_DEFAULT = 48;
 
     /**
      * @brief The default value for the router advertisement prefix that untrusted thin clients
