@@ -420,7 +420,7 @@ class UDPTransport : public Transport, public _RemoteEndpoint::EndpointListener,
      * @internal
      * @brief Manage the list of endpoints for the transport.
      */
-    void ManageEndpoints(qcc::Timespec authTimeout, qcc::Timespec sessionSetupTimeout);
+    void ManageEndpoints(uint32_t authTimeout, uint32_t sessionSetupTimeout);
 
     /**
      * @internal
@@ -606,7 +606,7 @@ class UDPTransport : public Transport, public _RemoteEndpoint::EndpointListener,
      * in connection establishment, there should be few connections in this
      * state, so we default to a quite low number.
      */
-    static const uint32_t ALLJOYN_MAX_INCOMPLETE_CONNECTIONS_UDP_DEFAULT = 10;
+    static const uint32_t ALLJOYN_MAX_INCOMPLETE_CONNECTIONS_UDP_DEFAULT = 48;
 
     /**
      * @brief The default value for the maximum number of UDP connections
@@ -625,7 +625,7 @@ class UDPTransport : public Transport, public _RemoteEndpoint::EndpointListener,
      * connections will be accepted.  This is because we are defending against
      * attacks from "abroad" and trust ourselves implicitly.
      */
-    static const uint32_t ALLJOYN_MAX_COMPLETED_CONNECTIONS_UDP_DEFAULT = 50;
+    static const uint32_t ALLJOYN_MAX_COMPLETED_CONNECTIONS_UDP_DEFAULT = 64;
 
     /**
      * @brief The default value for the maximum number of remote clients over udp.
@@ -636,7 +636,7 @@ class UDPTransport : public Transport, public _RemoteEndpoint::EndpointListener,
      * @warning This maximum is enforced on incoming connections from untrusted clients over udp.
      * This is to limit the amount of resources being used by untrusted clients over udp.
      */
-    static const uint32_t ALLJOYN_MAX_REMOTE_CLIENTS_UDP_DEFAULT = 16;
+    static const uint32_t ALLJOYN_MAX_REMOTE_CLIENTS_UDP_DEFAULT = 48;
 
     /**
      * @brief The default value for the router advertisement prefix that untrusted thin clients
@@ -823,14 +823,14 @@ class UDPTransport : public Transport, public _RemoteEndpoint::EndpointListener,
      * mess about while they should be authenticating.  If they take longer
      * than this time, we feel free to disconnect them as deniers of service.
      */
-    qcc::Timespec m_authTimeout;
+    uint32_t m_authTimeout;
 
     /**
      * sessionSetupTimeout is the maximum amount of time we allow incoming connections to
      * mess about while they should be sending messages to set up the session routes.
      * If they take longer than this time, we feel free to disconnect them as deniers of service.
      */
-    qcc::Timespec m_sessionSetupTimeout;
+    uint32_t m_sessionSetupTimeout;
 
     /**
      * m_maxAuth is the maximum number of incoming connections that can be in
