@@ -43,6 +43,7 @@ namespace ajn {
 _VirtualEndpoint::_VirtualEndpoint(const String& uniqueName, RemoteEndpoint& b2bEp) :
     _BusEndpoint(ENDPOINT_TYPE_VIRTUAL),
     m_uniqueName(uniqueName),
+    m_b2bEndpointsLock(LOCK_LEVEL_VIRTUALENDPOINT_MB2BENDPOINTSLOCK),
     m_hasRefs(false),
     m_epState(EP_STARTED)
 {
@@ -86,14 +87,14 @@ QStatus _VirtualEndpoint::PushMessage(Message& msg, SessionId id)
         String thisShortGuid = GetRemoteGUIDShortString();
         for (multimap<SessionId, RemoteEndpoint>::iterator it = m_b2bEndpoints.begin(); (it != m_b2bEndpoints.end()) && (id == it->first); ++it) {
             RemoteEndpoint ep = it->second;
-            String epShortGuid = ep->GetRemoteName().substr(1, ep->GetRemoteName().find_last_of('.') - 1);
+            String epShortGuid = ep->GetRemoteName().substr(1, ep->GetRemoteName().find_last_of_std('.') - 1);
             if (thisShortGuid == epShortGuid) {
                 tryEndpoints.push_back(ep);
             }
         }
         for (multimap<SessionId, RemoteEndpoint>::iterator it = m_b2bEndpoints.begin(); (it != m_b2bEndpoints.end()) && (id == it->first); ++it) {
             RemoteEndpoint ep = it->second;
-            String epShortGuid = ep->GetRemoteName().substr(1, ep->GetRemoteName().find_last_of('.') - 1);
+            String epShortGuid = ep->GetRemoteName().substr(1, ep->GetRemoteName().find_last_of_std('.') - 1);
             if (thisShortGuid != epShortGuid) {
                 tryEndpoints.push_back(ep);
             }
@@ -148,7 +149,7 @@ bool _VirtualEndpoint::AddBusToBusEndpoint(RemoteEndpoint& endpoint)
     m_b2bEndpointsLock.Lock(MUTEX_CONTEXT);
 
     /* Sanity check */
-    assert(m_epState == EP_STARTED);
+    QCC_ASSERT(m_epState == EP_STARTED);
     multimap<SessionId, RemoteEndpoint>::iterator it = m_b2bEndpoints.begin();
     bool found = false;
     while ((it != m_b2bEndpoints.end()) && (it->first == 0)) {
@@ -257,7 +258,7 @@ QStatus _VirtualEndpoint::AddSessionRef(SessionId id, RemoteEndpoint& b2bEp)
 {
     QCC_DbgTrace(("_VirtualEndpoint::AddSessionRef(this=%s [%x], id=%u, b2b=%s)", GetUniqueName().c_str(), this, id, b2bEp->GetUniqueName().c_str()));
 
-    assert(id != 0);
+    QCC_ASSERT(id != 0);
 
     m_b2bEndpointsLock.Lock(MUTEX_CONTEXT);
 
@@ -277,7 +278,7 @@ QStatus _VirtualEndpoint::AddSessionRef(SessionId id, RemoteEndpoint& b2bEp)
 void _VirtualEndpoint::RemoveSessionRef(SessionId id)
 {
     QCC_DbgTrace(("_VirtualEndpoint::RemoveSessionRef(this=%s [%x], id=%u)", GetUniqueName().c_str(), this, id));
-    assert(id != 0);
+    QCC_ASSERT(id != 0);
     m_b2bEndpointsLock.Lock(MUTEX_CONTEXT);
     multimap<SessionId, RemoteEndpoint>::iterator it = m_b2bEndpoints.find(id);
     if (it != m_b2bEndpoints.end()) {
