@@ -57,10 +57,10 @@ static const char bundledConfig[] =
     "  <listen>tcp:iface=*,port=0</listen>"
     "  <listen>udp:iface=*,port=0</listen>"
     "  <limit name=\"auth_timeout\">20000</limit>"
-    "  <limit name=\"max_incomplete_connections\">4</limit>"
-    "  <limit name=\"max_completed_connections\">16</limit>"
-    "  <limit name=\"max_remote_clients_tcp\">8</limit>"
-    "  <limit name=\"max_remote_clients_udp\">8</limit>"
+    "  <limit name=\"max_incomplete_connections\">48</limit>"
+    "  <limit name=\"max_completed_connections\">64</limit>"
+    "  <limit name=\"max_remote_clients_tcp\">48</limit>"
+    "  <limit name=\"max_remote_clients_udp\">48</limit>"
     "  <property name=\"router_power_source\">Battery powered and chargeable</property>"
     "  <property name=\"router_mobility\">Intermediate mobility</property>"
     "  <property name=\"router_availability\">3-6 hr</property>"
@@ -239,7 +239,7 @@ QStatus BundledRouter::Start(NullTransport* nullTransport)
     lock.Lock(MUTEX_CONTEXT);
     while (stopping) {
         if (!transports.empty()) {
-            assert(transports.empty());
+            QCC_ASSERT(transports.empty());
         }
         lock.Unlock(MUTEX_CONTEXT);
         qcc::Sleep(5);
