@@ -125,7 +125,7 @@ StringID _PolicyDB::UpdateDictionary(const String& key)
         if (it == dictionary.end()) {
             /* New string found; assign it an ID. */
             id = dictionary.size();
-            pair<StringMapKey, StringID> p(key, id);
+            pair<std::string, StringID> p(key, id);
             dictionary.insert(p);
         } else {
             /* The string already has an ID. */
@@ -156,8 +156,8 @@ StringID _PolicyDB::LookupStringID(const char* key) const
 
 const _PolicyDB::IDSet _PolicyDB::LookupStringIDPrefix(const char* idStr, char sep) const
 {
-    assert(idStr);
-    assert(sep != '\0');
+    QCC_ASSERT(idStr);
+    QCC_ASSERT(sep != '\0');
     IDSet ret;
     char* prefix = strdup(idStr); // duplicate idStr since we are modifying it
 
@@ -255,9 +255,9 @@ bool _PolicyDB::AddRule(PolicyRuleList& ownList,
 
         QCC_DEBUG_ONLY(rule.ruleString += " " + attrStr + "=\"" + attrVal);
 
-        if (attrStr.compare(0, sizeof("send_") - 1, "send_") == 0) {
+        if (attrStr.compare_std(0, sizeof("send_") - 1, "send_") == 0) {
             policyGroup = RULE_SEND;
-        } else if (attrStr.compare(0, sizeof("receive_") - 1, "receive_") == 0) {
+        } else if (attrStr.compare_std(0, sizeof("receive_") - 1, "receive_") == 0) {
             policyGroup = RULE_RECEIVE;
         }
 
@@ -364,7 +364,7 @@ bool _PolicyDB::AddRule(PolicyRuleList& ownList,
     QCC_DEBUG_ONLY(rule.ruleString += "/>");
 
     if (success && !skip) {
-        assert(policyGroup != RULE_UNKNOWN);
+        QCC_ASSERT(policyGroup != RULE_UNKNOWN);
 
         if (policyGroup & RULE_SEND) {
             sendList.push_back(rule);
@@ -475,7 +475,7 @@ void _PolicyDB::AddAlias(const String& alias, const String& name)
         QCC_DbgPrintf(("Add %s{%d} to table for %s", alias.c_str(), nameID, name.c_str()));
         bnids->insert(nameID);
     }
-    pair<StringMapKey, IDSet> p(alias, bnids);
+    pair<std::string, IDSet> p(alias, bnids);
     busNameIDMap.insert(p);
 }
 
@@ -540,7 +540,7 @@ void _PolicyDB::Finalize(Bus* bus)
         QCC_DbgPrintf(("    \"%s\" = %u", it->first.c_str(), it->second));
     }
     QCC_DbgPrintf(("Name Table:"));
-    for (unordered_map<qcc::StringMapKey, IDSet>::const_iterator it = busNameIDMap.begin(); it != busNameIDMap.end(); ++it) {
+    for (unordered_map<std::string, IDSet>::const_iterator it = busNameIDMap.begin(); it != busNameIDMap.end(); ++it) {
         QCC_DbgPrintf(("    \"%s\" = {%s}", it->first.c_str(), IDSet2String(it->second).c_str()));
     }
 #endif
@@ -615,14 +615,14 @@ void _PolicyDB::NameOwnerChanged(const String& alias,
         IDSet bnids;
         BusNameIDMap::iterator it = busNameIDMap.find(*newOwner);
         if (it != busNameIDMap.end()) {
-            assert(alias != *newOwner);
+            QCC_ASSERT(alias != *newOwner);
             bnids = it->second;
         }
         if ((aliasID != ID_NOT_FOUND) && (aliasID != WILDCARD)) {
             QCC_DbgPrintf(("Add %s{%d} to table for %s", alias.c_str(), aliasID, newOwner->c_str()));
             bnids->insert(aliasID);
         }
-        pair<StringMapKey, IDSet> p(alias, bnids);
+        pair<std::string, IDSet> p(alias, bnids);
         busNameIDMap.insert(p);
     }
 
