@@ -64,9 +64,12 @@
 #if !defined(ROUTER_LIB)
 
 #if defined(QCC_OS_LINUX)
+
 #include <sys/prctl.h>
 #include <sys/capability.h>
+
 #elif defined(QCC_OS_ANDROID)
+
 #include <linux/capability.h>
 extern "C" {
 extern int capset(cap_user_header_t hdrp, const cap_user_data_t datap);
@@ -100,10 +103,10 @@ static volatile sig_atomic_t quit;
 static const char defaultConfig[] =
     "<busconfig>"
     "  <limit name=\"auth_timeout\">20000</limit>"
-    "  <limit name=\"max_incomplete_connections\">16</limit>"
-    "  <limit name=\"max_completed_connections\">32</limit>"
-    "  <limit name=\"max_remote_clients_tcp\">0</limit>"
-    "  <limit name=\"max_remote_clients_udp\">0</limit>"
+    "  <limit name=\"max_incomplete_connections\">48</limit>"
+    "  <limit name=\"max_completed_connections\">64</limit>"
+    "  <limit name=\"max_remote_clients_tcp\">48</limit>"
+    "  <limit name=\"max_remote_clients_udp\">48</limit>"
     "  <property name=\"router_power_source\">Battery powered and chargeable</property>"
     "  <property name=\"router_mobility\">Intermediate mobility</property>"
     "  <property name=\"router_availability\">3-6 hr</property>"
@@ -113,10 +116,10 @@ static const char defaultConfig[] =
 static const char defaultConfig[] =
     "<busconfig>"
     "  <limit name=\"auth_timeout\">20000</limit>"
-    "  <limit name=\"max_incomplete_connections\">16</limit>"
-    "  <limit name=\"max_completed_connections\">32</limit>"
-    "  <limit name=\"max_remote_clients_tcp\">16</limit>"
-    "  <limit name=\"max_remote_clients_udp\">16</limit>"
+    "  <limit name=\"max_incomplete_connections\">48</limit>"
+    "  <limit name=\"max_completed_connections\">64</limit>"
+    "  <limit name=\"max_remote_clients_tcp\">48</limit>"
+    "  <limit name=\"max_remote_clients_udp\">48</limit>"
     "</busconfig>";
 #endif
 
@@ -292,7 +295,7 @@ class OptParse {
 
 void OptParse::PrintUsage() {
     String cmd = argv[0];
-    cmd = cmd.substr(cmd.find_last_of('/') + 1);
+    cmd = cmd.substr(cmd.find_last_of_std('/') + 1);
 
     fprintf(
         stderr,
@@ -419,7 +422,7 @@ OptParse::ParseResultCode OptParse::ParseResult()
                 goto exit;
             }
             configFile = argv[i];
-        } else if (arg.compare(0, sizeof("--config-file") - 1, "--config-file") == 0) {
+        } else if (arg.compare_std(0, sizeof("--config-file") - 1, "--config-file") == 0) {
             if (!configFile.empty() || internal) {
                 result = PR_OPTION_CONFLICT;
                 goto exit;
@@ -433,14 +436,11 @@ OptParse::ParseResultCode OptParse::ParseResult()
             }
             configService = true;
 #endif
-        } else if (arg.compare(0, sizeof("--print-address") - 1,
-                               "--print-address") == 0) {
-            if (arg[sizeof("--print-address") - 1] == '=') {
-                printAddressFd = StringToI32(arg.substr(
-                                                 sizeof("--print-address")), 10, -2);
+        } else if (arg.compare_std(0, sizeof("--print-address") - 1, "--print-address") == 0) {
+            if ((arg.size() > sizeof("--print-address") - 1) && (arg[sizeof("--print-address") - 1] == '=')) {
+                printAddressFd = StringToI32(arg.substr(sizeof("--print-address")), 10, -2);
             } else {
-                if (((i + 1) == argc) || ((argv[i + 1][0] == '-') && (argv[i
-                                                                           + 1][1] == '-'))) {
+                if (((i + 1) == argc) || ((argv[i + 1][0] == '-') && (argv[i + 1][1] == '-'))) {
                     printAddressFd = STDERR_FILENO;
                 } else {
                     ++i;
@@ -451,14 +451,12 @@ OptParse::ParseResultCode OptParse::ParseResult()
                 result = PR_INVALID_OPTION;
                 goto exit;
             }
-        } else if (arg.substr(0, sizeof("--print-pid") - 1).compare(
-                       "--print-pid") == 0) {
-            if (arg[sizeof("--print-pid") - 1] == '=') {
-                printPidFd = StringToI32(arg.substr(sizeof("--print-pid")), 10,
-                                         -2);
+        } else if (arg.compare_std(0, sizeof("--print-pid") - 1, "--print-pid") == 0) {
+            if ((arg.size() > sizeof("--print-pid") - 1) &&
+                (arg[sizeof("--print-pid") - 1] == '=')) {
+                printPidFd = StringToI32(arg.substr(sizeof("--print-pid")), 10, -2);
             } else {
-                if (((i + 1) == argc) || ((argv[i + 1][0] == '-') && (argv[i
-                                                                           + 1][1] == '-'))) {
+                if (((i + 1) == argc) || ((argv[i + 1][0] == '-') && (argv[i + 1][1] == '-'))) {
                     printPidFd = STDERR_FILENO;
                 } else {
                     ++i;
@@ -561,8 +559,8 @@ int daemon(OptParse& opts) {
     for (ConfigDB::_ListenList::const_iterator it = listenList->begin(); it != listenList->end(); ++it) {
         String addrStr = *it;
         bool skip = false;
-        if (addrStr.compare(0, sizeof("unix:") - 1, "unix:") == 0) {
-            if (addrStr.compare(sizeof("unix:") - 1, sizeof("tmpdir=") - 1, "tmpdir=") == 0) {
+        if (addrStr.compare_std(0, sizeof("unix:") - 1, "unix:") == 0) {
+            if (addrStr.compare_std(sizeof("unix:") - 1, sizeof("tmpdir=") - 1, "tmpdir=") == 0) {
                 // Process tmpdir specially.
                 String randStr = addrStr.substr(sizeof("unix:tmpdir=") - 1) + "/alljoyn-";
                 addrStr = ("unix:abstract=" + RandomString(randStr.c_str()));
@@ -575,16 +573,16 @@ int daemon(OptParse& opts) {
             }
 
 #if defined(QCC_OS_DARWIN)
-        } else if (addrStr.compare(0, sizeof("launchd:") - 1, "launchd:") == 0) {
+        } else if (addrStr.compare_std(0, sizeof("launchd:") - 1, "launchd:") == 0) {
             skip = opts.GetNoLaunchd();
 #endif
 
-        } else if (addrStr.compare(0, sizeof("tcp:") - 1, "tcp:") == 0) {
+        } else if (addrStr.compare_std(0, sizeof("tcp:") - 1, "tcp:") == 0) {
             skip = opts.GetNoTCP();
-        } else if (addrStr.compare(0, sizeof("udp:") - 1, "udp:") == 0) {
+        } else if (addrStr.compare_std(0, sizeof("udp:") - 1, "udp:") == 0) {
             skip = opts.GetNoUDP();
 
-        } else if (addrStr.compare(0, sizeof("slap:") - 1, "slap:") == 0) {
+        } else if (addrStr.compare_std(0, sizeof("slap:") - 1, "slap:") == 0) {
             skip = opts.GetNoSLAP();
 
         } else {
@@ -597,7 +595,7 @@ int daemon(OptParse& opts) {
         } else {
             Log(LOG_INFO, "Setting up transport for address: %s\n", addrStr.c_str());
             if (!listenSpecs.empty()) {
-                listenSpecs.append(';');
+                listenSpecs.append(1, ';');
             }
             listenSpecs.append(addrStr);
         }
@@ -754,13 +752,13 @@ int CDECL_CALL main(int argc, char** argv, char** env)
 
     config = new ConfigDB(configStr, opts.GetConfigFile());
     if (!config->LoadConfig()) {
-        const char* errsrc;
+        String errsrc;
         if (opts.GetInternalConfig()) {
             errsrc = "internal default config";
         } else {
-            errsrc = opts.GetConfigFile().c_str();
+            errsrc = opts.GetConfigFile();
         }
-        Log(LOG_ERR, "Failed to load the configuration - problem with %s.\n", errsrc);
+        Log(LOG_ERR, "Failed to load the configuration - problem with %s.\n", errsrc.c_str());
         ret = DAEMON_EXIT_CONFIG_ERROR;
         goto exit;
     }
