@@ -61,9 +61,10 @@ static const char daemonConfig[] =
     "<busconfig>"
     "  <type>alljoyn</type>"
     "  <limit name=\"auth_timeout\">20000</limit>"
-    "  <limit name=\"max_incomplete_connections\">16</limit>"
-    "  <limit name=\"max_completed_connections\">32</limit>"
-    "  <limit name=\"max_untrusted_clients\">16</limit>"
+    "  <limit name=\"max_incomplete_connections\">48</limit>"
+    "  <limit name=\"max_completed_connections\">64</limit>"
+    "  <limit name=\"max_remote_clients_tcp\">48</limit>"
+    "  <limit name=\"max_remote_clients_udp\">48</limit>"
     "</busconfig>";
 
 /** Static top level message bus object */
@@ -105,7 +106,7 @@ class LocalTestObject : public BusObject {
 
         /* Add the test interface to this object */
         const InterfaceDescription* regTestIntf = bus.GetInterface(::org::alljoyn::alljoyn_test::InterfaceName);
-        assert(regTestIntf);
+        QCC_ASSERT(regTestIntf);
         AddInterface(*regTestIntf);
         /* Add the values interface to this object */
         const InterfaceDescription* valuesIntf = bus.GetInterface(::org::alljoyn::alljoyn_test::values::InterfaceName);
@@ -133,7 +134,7 @@ class LocalTestObject : public BusObject {
 
     void ObjectRegistered(void) {
         BusObject::ObjectRegistered();
-        assert(bus);
+        QCC_ASSERT(bus);
 
         /* Request a well-known name */
         /* Note that you cannot make a blocking method call here */
