@@ -70,7 +70,7 @@ using namespace std;
 static const char defaultConfig[] =
     "<busconfig>"
     "  <limit name=\"auth_timeout\">20000</limit>"
-    "  <limit name=\"max_incomplete_connections\">16</limit>"
+    "  <limit name=\"max_incomplete_connections\">48</limit>"
     "  <limit name=\"max_completed_connections\">64</limit>"
     "  <limit name=\"max_remote_clients_tcp\">48</limit>"
     "  <limit name=\"max_remote_clients_udp\">0</limit>"
@@ -214,7 +214,7 @@ OptParse::ParseResultCode OptParse::ParseResult()
             }
             configFile = argv[i];
             useInternalConfig = false;
-        } else if (arg.compare(0, sizeof("--config-file") - 1, "--config-file") == 0) {
+        } else if (arg.compare_std(0, sizeof("--config-file") - 1, "--config-file") == 0) {
             if (!configFile.empty()) {
                 result = PR_OPTION_CONFLICT;
                 goto exit;
@@ -273,9 +273,9 @@ int daemon(OptParse& opts)
     for (ConfigDB::_ListenList::const_iterator it = listenList->begin(); it != listenList->end(); ++it) {
         String addrStr = *it;
         bool skip = false;
-        if (addrStr.compare(0, sizeof("tcp:") - 1, "tcp:") == 0) {
+        if (addrStr.compare_std(0, sizeof("tcp:") - 1, "tcp:") == 0) {
             // No special processing needed for TCP.
-        } else if (addrStr.compare(0, sizeof("udp:") - 1, "udp:") == 0) {
+        } else if (addrStr.compare_std(0, sizeof("udp:") - 1, "udp:") == 0) {
             // No special processing needed for UDP.
         } else {
             Log(LOG_ERR, "Unsupported listen address: %s (ignoring)\n", it->c_str());
@@ -287,7 +287,7 @@ int daemon(OptParse& opts)
         } else {
             Log(LOG_INFO, "Setting up transport for address: %s\n", it->c_str());
             if (!listenSpecs.empty()) {
-                listenSpecs.append(';');
+                listenSpecs.append(1, ';');
             }
             listenSpecs.append(*it);
         }
@@ -394,13 +394,13 @@ DAEMONLIBRARY_API int LoadDaemon(int argc, char** argv)
 
     ConfigDB config(configStr, opts.GetConfigFile());
     if (!config.LoadConfig()) {
-        const char* errsrc;
+        String errsrc;
         if (opts.UseInternalConfig()) {
             errsrc = "internal default config";
         } else {
-            errsrc = opts.GetConfigFile().c_str();
+            errsrc = opts.GetConfigFile();
         }
-        Log(LOG_ERR, "Failed to load the configuration - problem with %s.\n", errsrc);
+        Log(LOG_ERR, "Failed to load the configuration - problem with %s.\n", errsrc.c_str());
         return DAEMON_EXIT_CONFIG_ERROR;
     }
 
