@@ -23,7 +23,6 @@
 #include <qcc/StringSource.h>
 
 #include <stdio.h>
-#include <assert.h>
 
 #include "AboutDataInternal.h"
 
@@ -177,7 +176,7 @@ QStatus AboutData::CreateFromXml(const qcc::String& aboutDataXml)
                         continue;
                     }
                 } else {
-                    assert(aboutDataInternal->aboutFields[it->first].signature == "s");
+                    QCC_ASSERT(aboutDataInternal->aboutFields[it->first].signature == "s");
                     arg.Set("s", root->GetChild(it->first)->GetContent().c_str());
                     status = SetField(it->first.c_str(), arg);
                     if (status != ER_OK) {
@@ -580,38 +579,69 @@ QStatus AboutData::SetSupportedLanguage(const char* language)
     //     tags conform to this RFC
     bool added;
     QStatus status = aboutDataInternal->translator->AddTargetLanguage(language, &added);
-    if ((status == ER_OK) && added) {
-        // A new language has been added. Rebuild the MsgArg and update the field.
+
+    if (status == ER_OK) {
         size_t supportedLangsNum = aboutDataInternal->translator->NumTargetLanguages();
-        const char** supportedLangs = new const char*[supportedLangsNum];
-        for (size_t count = 0; count < supportedLangsNum; count++) {
-            qcc::String lang;
-            aboutDataInternal->translator->GetTargetLanguage(count, lang);
-            supportedLangs[count] = lang.c_str();
+        if (!added) {
+            // Verify the language numbers in translator and about field are the same
+            size_t size = 0;
+            MsgArg* arg;
+            QStatus status = GetField(SUPPORTED_LANGUAGES, arg);
+            if (ER_OK == status) {
+                MsgArg* langs;
+                status = arg->Get(aboutDataInternal->aboutFields[SUPPORTED_LANGUAGES].signature.c_str(), &size, &langs);
+                added = (size != supportedLangsNum);    // Numbers don't match. Need to update the field
+            }
         }
-        MsgArg arg;
-        status = arg.Set(aboutDataInternal->aboutFields[SUPPORTED_LANGUAGES].signature.c_str(), supportedLangsNum, supportedLangs);
-        if (status != ER_OK) {
-            return status;
+
+        if (added) {
+            // A new language has been added. Rebuild the MsgArg and update the field.
+            char** supportedLangs = new char*[supportedLangsNum];
+
+            for (size_t count = 0; count < supportedLangsNum; count++) {
+                qcc::String lang;
+                aboutDataInternal->translator->GetTargetLanguage(count, lang);
+                supportedLangs[count] = new char[lang.length() + 1];
+                strcpy(supportedLangs[count], lang.c_str());
+            }
+
+            MsgArg arg;
+            status = arg.Set(aboutDataInternal->aboutFields[SUPPORTED_LANGUAGES].signature.c_str(), supportedLangsNum, supportedLangs);
+            if (status == ER_OK) {
+                status = SetField(SUPPORTED_LANGUAGES, arg);
+            }
+
+            for (size_t count = 0; count < supportedLangsNum; count++) {
+                delete[](supportedLangs[count]);
+            }
+            delete[] supportedLangs;
         }
-        status = SetField(SUPPORTED_LANGUAGES, arg);
-        delete [] supportedLangs;
     }
+
     return status;
 }
 
 size_t AboutData::GetSupportedLanguages(const char** languageTags, size_t num)
 {
-    size_t numTargetLanguages = aboutDataInternal->translator->NumTargetLanguages();
-    if (!languageTags) {
-        return numTargetLanguages;
+    size_t size = 0;
+    MsgArg* arg = nullptr;
+    MsgArg* args = nullptr;
+    QStatus status = GetField(SUPPORTED_LANGUAGES, arg);
+    if (ER_OK == status) {
+        status = arg->Get(aboutDataInternal->aboutFields[SUPPORTED_LANGUAGES].signature.c_str(), &size, &args);
     }
-    qcc::String language;
-    size_t count;
-    for (count = 0; (count < numTargetLanguages) && (count < num); count++) {
-        aboutDataInternal->translator->GetTargetLanguage(count, language);
-        languageTags[count] = language.c_str();
+
+    if (languageTags == nullptr) {
+        return size;
     }
+
+    size_t count = 0;
+    if ((ER_OK == status) && (args != nullptr)) {
+        for (count = 0; (count < size) && (count < num); count++) {
+            args[count].Get("s", &(languageTags[count]));
+        }
+    }
+
     return count;
 }
 
@@ -626,6 +656,7 @@ QStatus AboutData::SetDescription(const char* description, const char* language)
     status = SetField(DESCRIPTION, arg, language);
     return status;
 }
+
 QStatus AboutData::GetDescription(char** description, const char* language)
 {
     QStatus status;
@@ -838,7 +869,7 @@ QStatus AboutData::GetAboutData(MsgArg* msgArg, const char* language)
     char* defaultLanguage = NULL;
     GetDefaultLanguage(&defaultLanguage);
     // A default language must exist or IsValid would have been false above.
-    assert(defaultLanguage != NULL);
+    QCC_ASSERT(defaultLanguage != NULL);
 
     qcc::String defaultLang(defaultLanguage);
     qcc::String bestLanguage;
@@ -846,7 +877,7 @@ QStatus AboutData::GetAboutData(MsgArg* msgArg, const char* language)
 
     // At least a default language must exist or IsValid would have been
     // false above.
-    assert(bestLanguage.c_str() != NULL);
+    QCC_ASSERT(bestLanguage.c_str() != NULL);
 
     size_t dictionarySize = 0;
     typedef std::map<qcc::String, FieldDetails>::iterator it_aboutFields;
@@ -901,7 +932,7 @@ QStatus AboutData::GetAboutData(MsgArg* msgArg, const char* language)
             return status;
         }
     }
-    assert(dictionarySize == count);
+    QCC_ASSERT(dictionarySize == count);
 
     msgArg->Set("a{sv}", dictionarySize, aboutDictionary);
     msgArg->Stabilize();
@@ -978,7 +1009,7 @@ QStatus AboutData::GetAnnouncedAboutData(MsgArg* msgArg)
         }
     }
 
-    assert(dictionarySize == count);
+    QCC_ASSERT(dictionarySize == count);
 
     msgArg->Set("a{sv}", dictionarySize, announceDictionary);
     msgArg->Stabilize();
