@@ -119,10 +119,12 @@ class AllJoynPeerObj : public BusObject, public BusListener, public qcc::AlarmLi
     /**
      * Authenticate the connection to a remote peer. Authentication establishes a session key with a remote peer.
      *
+     * @param msgType   Message type we're trying to send.
      * @param busName   The bus name of the remote peer we are securing.
      * @param wait      If true the function will block if there is an authentication already in
      *                  progress with the peer on a separate thread. If false, the function will
      *                  return an ER_WOULD_BLOCK status instead of waiting.
+     * @param msg       If available, the message to be sent to determine manifests to send.
      *
      * @return
      *      - ER_OK if successful
@@ -130,7 +132,7 @@ class AllJoynPeerObj : public BusObject, public BusListener, public qcc::AlarmLi
      *        and the wait parameter was false.
      *      - An error status otherwise
      */
-    QStatus AuthenticatePeer(AllJoynMessageType msgType,  const qcc::String& busName, bool wait = true);
+    QStatus AuthenticatePeer(AllJoynMessageType msgType,  const qcc::String& busName, bool wait = true, Message* msg = nullptr);
 
     /**
      * Authenticate the connection to a remote peer asynchronously. Authentication establishes a session key with
@@ -356,7 +358,7 @@ class AllJoynPeerObj : public BusObject, public BusListener, public qcc::AlarmLi
      * @param seed       A seed string negotiated by the peers.
      * @param verifier   A verifier string that used to verify the session key.
      */
-    QStatus KeyGen(PeerState& peerState, qcc::String seed, qcc::String& verifier, qcc::KeyBlob::Role role);
+    QStatus KeyGen(PeerState& peerState, const std::vector<uint8_t, qcc::SecureAllocator<uint8_t> >& seed, qcc::String& verifier, qcc::KeyBlob::Role role);
 
     /**
      * Get a property from this object
@@ -438,15 +440,7 @@ class AllJoynPeerObj : public BusObject, public BusListener, public qcc::AlarmLi
      * @param member  The member that was called
      * @param msg     The method call message
      */
-    void HandleSendManifest(const InterfaceDescription::Member* member, Message& msg);
-    /**
-     * Send the manifest to the peer
-     * @param remotePeerObj  The remote peer
-     * @param ifc     The interface object
-     * @param peerState the peer state object
-     * @return ER_OK if successful; otherwise, an error code.
-     */
-    QStatus SendManifest(ProxyBusObject& remotePeerObj, const InterfaceDescription* ifc, PeerState& peerState);
+    void HandleSendManifests(const InterfaceDescription::Member* member, Message& msg);
 
     /**
      * Send an SendMembership to the peer
@@ -469,10 +463,20 @@ class AllJoynPeerObj : public BusObject, public BusListener, public qcc::AlarmLi
      *  conversation hash state. The authentication protocol version is also
      *  hashed (before the GUIDs).
      *
+     *  @param initiatorFlag   The desired conversation hash to use
      *  @param peerState   The state of the peer.
      *  @param localFirst  If true compute Hash(local||remote), if false compute Hash(remote||local).
      */
-    void HashGUIDs(PeerState& peerState, bool localFirst);
+    void HashGUIDs(bool initiatorFlag, PeerState& peerState, bool localFirst);
+
+    /**
+     * determine whether the local peer needs to yield to the remote peer in a
+     * concurrent key exchange.  This call needs to protected for thread safe
+     * access.
+     *  @param initiatorFlag   is the key exchange started as an initiator or responder
+     *  @param peerState   The state of the peer.
+     */
+    bool NeedToYieldToPeer(bool initiatorFlag, PeerState& peerState);
 
     /**
      * The peer-to-peer authentication mechanisms available to this object
