@@ -74,12 +74,12 @@ const char* org::alljoyn::Daemon::WellKnownName = "org.alljoyn.Daemon";
 const char* org::alljoyn::Daemon::Debug::ObjectPath = "/org/alljoyn/Debug";
 const char* org::alljoyn::Daemon::Debug::InterfaceName = "org.alljoyn.Debug";
 
-/** org.allseen.Introsoectable interface definitions */
+/** org.allseen.Introspectable interface definitions */
 const char* org::allseen::Introspectable::InterfaceName = "org.allseen.Introspectable";
 const char* org::allseen::Introspectable::IntrospectDocType =
     "<!DOCTYPE"
-    " node PUBLIC \"-//allseen//DTD ALLJOYN Object Introspection 1.0//EN\"\n"
-    "\"http://www.allseen.org/alljoyn/introspect-1.0.dtd\""
+    " node PUBLIC \"-//allseen//DTD ALLJOYN Object Introspection 1.1//EN\"\n"
+    "\"http://www.allseen.org/alljoyn/introspect-1.1.dtd\""
     ">\n";
 
 QStatus org::alljoyn::CreateInterfaces(BusAttachment& bus)
@@ -175,7 +175,7 @@ QStatus org::alljoyn::CreateInterfaces(BusAttachment& bus)
             return status;
         }
         ifc->AddProperty("Version", "q", PROP_ACCESS_READ);
-        ifc->AddPropertyAnnotation("Version", org::freedesktop::DBus::AnnotateEmitsChanged, "false");
+        ifc->AddPropertyAnnotation("Version", org::freedesktop::DBus::AnnotateEmitsChanged, "const");
         ifc->AddSignal("State", "(yyayay)q", "publicKey,state", 0);
 
         ifc->Activate();
@@ -241,7 +241,7 @@ QStatus org::alljoyn::CreateInterfaces(BusAttachment& bus)
         ifc->AddMethod("ExchangeSuites",     "au",   "au",  "localAuthList,remoteAuthList");
         ifc->AddMethod("KeyExchange",     "uv",   "uv",  "localAuthMask,localPublicKey, remoteAuthMask, remotePublicKey");
         ifc->AddMethod("KeyAuthentication",     "v",   "v",  "localVerifier,remoteVerifier");
-        ifc->AddMethod("SendManifest",     "a(ssa(syy))",   "a(ssa(syy))", "manifest,manifest");
+        ifc->AddMethod("SendManifests",     "a(ua(ssa(syy))saysay)",   "a(ua(ssa(syy))saysay)", "manifests,manifests");
         ifc->AddMethod("SendMemberships",     "ya(yay)",   "ya(yay)", "sendCode,memberships,sendCode,memberships");
         ifc->AddProperty("Mechanisms",  "s", PROP_ACCESS_READ);
         ifc->AddProperty("Version",     "u", PROP_ACCESS_READ);
@@ -269,7 +269,7 @@ QStatus org::alljoyn::CreateInterfaces(BusAttachment& bus)
         }
         ifc->AddAnnotation(org::alljoyn::Bus::Secure, "true");
         ifc->AddProperty("Version", "q", PROP_ACCESS_READ);
-        ifc->AddPropertyAnnotation("Version", org::freedesktop::DBus::AnnotateEmitsChanged, "false");
+        ifc->AddPropertyAnnotation("Version", org::freedesktop::DBus::AnnotateEmitsChanged, "const");
         ifc->AddProperty("ApplicationState", "q", PROP_ACCESS_READ);
         ifc->AddPropertyAnnotation("ApplicationState", org::freedesktop::DBus::AnnotateEmitsChanged, "false");
         ifc->AddProperty("ManifestTemplateDigest", "(yay)", PROP_ACCESS_READ);
@@ -297,9 +297,9 @@ QStatus org::alljoyn::CreateInterfaces(BusAttachment& bus)
         }
         ifc->AddAnnotation(org::alljoyn::Bus::Secure, "true");
         ifc->AddProperty("Version", "q", PROP_ACCESS_READ);
-        ifc->AddPropertyAnnotation("Version", org::freedesktop::DBus::AnnotateEmitsChanged, "false");
-        ifc->AddMethod("Claim", "(yyayay)ayay(yyayay)aya(yay)a(ssa(syy))", "",
-                       "certificateAuthority,authorityKeyIdentifier,adminSecurityGroupId,adminSecurityGroupAuthority,adminGroupAuthorityKeyIdentifier,identityCertificateChain,manifest");
+        ifc->AddPropertyAnnotation("Version", org::freedesktop::DBus::AnnotateEmitsChanged, "const");
+        ifc->AddMethod("Claim", "(yyayay)ayay(yyayay)aya(yay)a(ua(ssa(syy))saysay)", "",
+                       "certificateAuthority,authorityKeyIdentifier,adminSecurityGroupId,adminSecurityGroupAuthority,adminGroupAuthorityKeyIdentifier,identityCertificateChain,manifests");
 
         ifc->Activate();
     }
@@ -313,12 +313,12 @@ QStatus org::alljoyn::CreateInterfaces(BusAttachment& bus)
         }
         ifc->AddAnnotation(org::alljoyn::Bus::Secure, "true");
         ifc->AddProperty("Version", "q", PROP_ACCESS_READ);
-        ifc->AddPropertyAnnotation("Version", org::freedesktop::DBus::AnnotateEmitsChanged, "false");
+        ifc->AddPropertyAnnotation("Version", org::freedesktop::DBus::AnnotateEmitsChanged, "const");
         ifc->AddMethod("Reset", "", "", "");
-        ifc->AddMethod("UpdateIdentity", "a(yay)a(ssa(syy))", "", "certificateChain,manifest");
+        ifc->AddMethod("UpdateIdentity", "a(yay)a(ua(ssa(syy))saysay)", "", "certificateChain,manifests");
         ifc->AddProperty("Identity", "a(yay)", PROP_ACCESS_READ);
         ifc->AddPropertyAnnotation("Identity", org::freedesktop::DBus::AnnotateEmitsChanged, "false");
-        ifc->AddProperty("Manifest", "a(ssa(syy))", PROP_ACCESS_READ);
+        ifc->AddProperty("Manifests", "a(ua(ssa(syy))saysay)", PROP_ACCESS_READ);
         ifc->AddPropertyAnnotation("Manifest", org::freedesktop::DBus::AnnotateEmitsChanged, "false");
         ifc->AddProperty("IdentityCertificateId", "(ayay(yyayay))", PROP_ACCESS_READ);
         ifc->AddPropertyAnnotation("IdentityCertificateId", org::freedesktop::DBus::AnnotateEmitsChanged, "false");
@@ -330,10 +330,13 @@ QStatus org::alljoyn::CreateInterfaces(BusAttachment& bus)
         ifc->AddProperty("DefaultPolicy", "(qua(a(ya(yyayayay)ay)a(ssa(syy))))", PROP_ACCESS_READ);
         ifc->AddPropertyAnnotation("DefaultPolicy", org::freedesktop::DBus::AnnotateEmitsChanged, "false");
         ifc->AddMethod("ResetPolicy", "", "", "");
-        ifc->AddMethod("InstallMembership", "a(yay)", "", "certificateChain,");
+        ifc->AddMethod("InstallMembership", "a(yay)", "", "certificateChain");
         ifc->AddMethod("RemoveMembership", "(ayay(yyayay))", "", "certificateId");
         ifc->AddProperty("MembershipSummaries", "a(ayay(yyayay))", PROP_ACCESS_READ);
         ifc->AddPropertyAnnotation("MembershipSummaries", org::freedesktop::DBus::AnnotateEmitsChanged, "false");
+        ifc->AddMethod("StartManagement", "", "", "");
+        ifc->AddMethod("EndManagement", "", "", "");
+        ifc->AddMethod("InstallManifests", "a(ua(ssa(syy))saysay)", "", "manifests");
 
         ifc->Activate();
     }
