@@ -22,7 +22,6 @@
 
 #include <qcc/platform.h>
 
-#include <assert.h>
 
 #include <qcc/Crypto.h>
 #include <qcc/Util.h>
@@ -79,7 +78,11 @@ void AuthMechLogon::ComputeMS()
     /*
      * Use the PRF function to compute the master secret.
      */
-    Crypto_PseudorandomFunction(pms, label, clientRandom + serverRandom, keymatter, sizeof(keymatter));
+    vector<uint8_t, SecureAllocator<uint8_t> > seed;
+    seed.reserve(clientRandom.size() + serverRandom.size());
+    AppendStringToSecureVector(clientRandom, seed);
+    AppendStringToSecureVector(serverRandom, seed);
+    Crypto_PseudorandomFunction(pms, label, seed, keymatter, sizeof(keymatter));
     masterSecret.Set(keymatter, sizeof(keymatter), KeyBlob::GENERIC);
     masterSecret.SetExpiration(expiration);
 }
@@ -96,7 +99,9 @@ qcc::String AuthMechLogon::ComputeVerifier(const char* labelStr)
      * Snapshot msg hash and compute the verifier string.
      */
     msgHash.GetDigest(digest, true);
-    qcc::String seed((const char*)digest, sizeof(digest));
+    vector<uint8_t, SecureAllocator<uint8_t> > seed;
+    seed.reserve(sizeof(digest));
+    seed.insert(seed.end(), digest, digest + sizeof(digest));
     Crypto_PseudorandomFunction(masterSecret, labelStr, seed, verifier, sizeof(verifier));
     QCC_DbgHLPrintf(("Verifier:  %s", BytesToHexString(verifier, sizeof(verifier)).c_str()));
     return BytesToHexString(verifier, sizeof(verifier));
@@ -207,7 +212,7 @@ static void UserNameToGuid(qcc::GUID128& guid, qcc::String userName)
     static const char labelStr[] = "SRP Logon Verifier";
     Crypto_SHA1 sha1;
     uint8_t digest[Crypto_SHA1::DIGEST_SIZE];
-    assert(Crypto_SHA1::DIGEST_SIZE >= qcc::GUID128::SIZE);
+    QCC_ASSERT(Crypto_SHA1::DIGEST_SIZE >= qcc::GUID128::SIZE);
     /*
      * The label makes the generated Guid unique for this authentication mechanism.
      */
