@@ -22,8 +22,6 @@
 
 #include <qcc/platform.h>
 
-#include <assert.h>
-
 #include <qcc/Crypto.h>
 #include <qcc/Util.h>
 #include <qcc/Debug.h>
@@ -79,7 +77,11 @@ void AuthMechSRP::ComputeMS()
     /*
      * Use the PRF function to compute the master secret.
      */
-    Crypto_PseudorandomFunction(pms, label, clientRandom + serverRandom, keymatter, sizeof(keymatter));
+    vector<uint8_t, SecureAllocator<uint8_t> > seed;
+    seed.reserve(clientRandom.size() + serverRandom.size());
+    AppendStringToSecureVector(clientRandom, seed);
+    AppendStringToSecureVector(serverRandom, seed);
+    Crypto_PseudorandomFunction(pms, label, seed, keymatter, sizeof(keymatter));
     masterSecret.Set(keymatter, sizeof(keymatter), KeyBlob::GENERIC);
     QCC_DbgHLPrintf(("MasterSecret:  %s", BytesToHexString(masterSecret.GetData(), masterSecret.GetSize()).c_str()));
     masterSecret.SetExpiration(expiration);
@@ -98,7 +100,9 @@ qcc::String AuthMechSRP::ComputeVerifier(const char* labelStr)
      * Snapshot msg hash and compute the verifier string.
      */
     msgHash.GetDigest(digest, true);
-    qcc::String seed((const char*)digest, sizeof(digest));
+    vector<uint8_t, SecureAllocator<uint8_t> > seed;
+    seed.reserve(sizeof(digest));
+    seed.insert(seed.end(), digest, digest + sizeof(digest));
     Crypto_PseudorandomFunction(masterSecret, labelStr, seed, verifier, sizeof(verifier));
     QCC_DbgHLPrintf(("Verifier(%s):  %s", labelStr, BytesToHexString(verifier, sizeof(verifier)).c_str()));
     return BytesToHexString(verifier, sizeof(verifier));
