@@ -18,10 +18,10 @@
 #include <alljoyn/BusAttachment.h>
 #include <qcc/Thread.h>
 #include <qcc/time.h>
+#include <qcc/LockLevel.h>
 #include <algorithm>
 #include <memory>
 #include <set>
-#include <cassert>
 
 #define PING_TIMEOUT 5000
 
@@ -158,7 +158,7 @@ static AutoPingAsyncCB* pingCallback = NULL;
 void AutoPingerInternal::Init()
 {
     ctxs = new std::set<PingAsyncContext*>();
-    globalPingerLock = new qcc::Mutex();
+    globalPingerLock = new qcc::Mutex(qcc::LOCK_LEVEL_AUTOPINGERINTERNAL_GLOBALPINGERLOCK);
     pingCallback = new AutoPingAsyncCB();
 }
 
@@ -276,7 +276,7 @@ void AutoPingerInternal::Resume()
 {
     globalPingerLock->Lock(MUTEX_CONTEXT);
 
-    assert(timer.IsRunning());
+    QCC_ASSERT(timer.IsRunning());
     if (true == pausing) {
         // re-add all Alarm objects
         std::map<qcc::String, PingGroup*>::const_iterator it = pingGroups.begin();
