@@ -22,7 +22,6 @@
 
 #include <qcc/platform.h>
 
-#include <assert.h>
 
 #include <algorithm>
 #include <map>
@@ -71,10 +70,15 @@ static bool operator==(const MethodContext& a, const MethodContext& b)
 }
 
 struct BusObject::Components {
+    /** Constructor */
+    Components() : counterLock(LOCK_LEVEL_BUSOBJECT_COMPONENTS_COUNTERLOCK), inUseCounter(0) { }
+
     /** The interfaces this object implements */
     vector<pair<const InterfaceDescription*, bool> > ifaces;
+
     /** The method handlers for this object */
     vector<MethodContext> methodContexts;
+
     /** Child objects of this object */
     vector<BusObject*> children;
 
@@ -171,7 +175,7 @@ qcc::String BusObject::GetName()
 {
     if (!path.empty()) {
         qcc::String name = path;
-        size_t pos = name.find_last_of('/');
+        size_t pos = name.find_last_of_std('/');
         if (pos == 0) {
             if (name.length() > 1) {
                 name.erase(0, 1);
@@ -195,6 +199,7 @@ qcc::String BusObject::GenerateIntrospection(const char* requestedLanguageTag, b
     qcc::String in(indent, ' ');
     qcc::String xml;
     qcc::String buffer;
+    bool unifiedFormat = (requestedLanguageTag == NULL);
 
     /* Iterate over child nodes */
     vector<BusObject*>::const_iterator iter = components->children.begin();
@@ -202,15 +207,19 @@ qcc::String BusObject::GenerateIntrospection(const char* requestedLanguageTag, b
         BusObject* child = *iter++;
         xml += in + "<node name=\"" + child->GetName() + "\"";
 
-        const char* nodeDesc = NULL;
-        if (requestedLanguageTag) {
-            nodeDesc = child->GetDescription(requestedLanguageTag, buffer);
-        }
-
+        const char* nodeDesc = child->GetDescription(requestedLanguageTag, buffer);
         if (deep || nodeDesc) {
             xml += ">\n";
             if (nodeDesc) {
-                xml += in + "  <description>" + XmlElement::EscapeXml(nodeDesc) + "</description>";
+                if (unifiedFormat) {
+                    xml += in + "  <annotation name=\"org.alljoyn.Bus.DocString";
+                    if (!languageTag.empty()) {
+                        xml += "." + languageTag;
+                    }
+                    xml += "\" value=\"" + XmlElement::EscapeXml(nodeDesc) + "\" />";
+                } else {
+                    xml += in + "  <description>" + XmlElement::EscapeXml(nodeDesc) + "</description>";
+                }
             }
             if (deep) {
                 xml += child->GenerateIntrospection(requestedLanguageTag, deep, indent + 2);
@@ -295,7 +304,7 @@ void BusObject::EmitPropChanged(const char* ifcName, const char* propName, MsgAr
 {
     QCC_DbgTrace(("BusObject::EmitPropChanged(ifcName = \"%s\", propName = \"%s\", val = <>, id = %u)",
                   ifcName, propName, id));
-    assert(bus);
+    QCC_ASSERT(bus);
     const InterfaceDescription* ifc = bus->GetInterface(ifcName);
 
     qcc::String emitsChanged;
@@ -342,7 +351,7 @@ void BusObject::EmitPropChanged(const char* ifcName, const char* propName, MsgAr
 
 QStatus BusObject::EmitPropChanged(const char* ifcName, const char** propNames, size_t numProps, SessionId id, uint8_t flags)
 {
-    assert(bus);
+    QCC_ASSERT(bus);
     qcc::String emitsChanged;
     QStatus status = ER_OK;
 
@@ -393,9 +402,9 @@ QStatus BusObject::EmitPropChanged(const char* ifcName, const char** propNames,
         }
         if (status == ER_OK) {
             const InterfaceDescription* bus_ifc = bus->GetInterface(org::freedesktop::DBus::Properties::InterfaceName);
-            assert(bus_ifc);
+            QCC_ASSERT(bus_ifc);
             const InterfaceDescription::Member* propChanged = bus_ifc->GetMember("PropertiesChanged");
-            assert(propChanged);
+            QCC_ASSERT(propChanged);
 
             MsgArg args[3];
             args[0].Set("s", ifcName);
@@ -501,20 +510,24 @@ void BusObject::GetAllProps(const InterfaceDescription::Member* member, Message&
                     }
                 }
             }
-            MsgArg* dict = new MsgArg[readable];
-            MsgArg* entry = dict;
-            /* Get readable properties */
-            for (size_t i = 0; i < numProps; i++) {
-                if ((props[i]->access & PROP_ACCESS_READ) && allowed[i]) {
-                    MsgArg* val = new MsgArg();
-                    status = Get(iface->v_string.str, props[i]->name.c_str(), *val);
-                    if (status != ER_OK) {
-                        delete val;
-                        break;
+
+            MsgArg* dict = NULL;
+            if (readable > 0) {
+                dict = new MsgArg[readable];
+                MsgArg* entry = dict;
+                /* Get readable properties */
+                for (size_t i = 0; i < numProps; i++) {
+                    if ((props[i]->access & PROP_ACCESS_READ) && allowed[i]) {
+                        MsgArg* val = new MsgArg();
+                        status = Get(iface->v_string.str, props[i]->name.c_str(), *val);
+                        if (status != ER_OK) {
+                            delete val;
+                            break;
+                        }
+                        entry->Set("{sv}", props[i]->name.c_str(), val);
+                        entry->v_dictEntry.val->SetOwnershipFlags(MsgArg::OwnsArgs, false);
+                        entry++;
                     }
-                    entry->Set("{sv}", props[i]->name.c_str(), val);
-                    entry->v_dictEntry.val->SetOwnershipFlags(MsgArg::OwnsArgs, false);
-                    entry++;
                 }
             }
             vals.Set("a{sv}", readable, dict);
@@ -539,10 +552,37 @@ void BusObject::Introspect(const InterfaceDescription::Member* member, Message&
 
     qcc::String xml = org::freedesktop::DBus::Introspectable::IntrospectDocType;
     xml += "<node>\n";
+
+    /* Append descriptions in all target languages */
+    if (!description.empty()) {
+        Translator* myTranslator = translator;
+        if (!myTranslator && bus) {
+            myTranslator = bus->GetDescriptionTranslator();
+        }
+        if (myTranslator != NULL) {
+            qcc::String language;
+            size_t size = myTranslator->NumTargetLanguages();
+            for (size_t index = 0; index < size; index++) {
+                myTranslator->GetTargetLanguage(index, language);
+                if (!language.empty()) {
+                    qcc::String buffer;
+                    qcc::String bestLanguage;
+                    myTranslator->GetBestLanguage(language.c_str(), languageTag, bestLanguage);
+                    const char* d = myTranslator->Translate(languageTag.c_str(), bestLanguage.c_str(), description.c_str(), buffer);
+                    if ((d == NULL) || (d[0] == '\0')) {
+                        continue;
+                    }
+                    xml += qcc::String("  <annotation name=\"org.alljoyn.Bus.DocString.") + language + "\" value=\"" + XmlElement::EscapeXml(d) + "\"/>\n";
+                }
+            }
+        } else if (!languageTag.empty()) {
+            xml += qcc::String("  <annotation name=\"org.alljoyn.Bus.DocString.") + languageTag + "\" value=\"" + description + "\"/>\n";
+        }
+    }
     if (isSecure) {
         xml += "  <annotation name=\"org.alljoyn.Bus.Secure\" value=\"true\"/>\n";
     }
-    xml += GenerateIntrospection(false, 2);
+    xml += GenerateIntrospection(NULL, false, 2);
     xml += "</node>\n";
     MsgArg arg("s", xml.c_str());
     QStatus status = MethodReply(msg, &arg, 1);
@@ -657,14 +697,14 @@ QStatus BusObject::DoRegistration(BusAttachment& busAttachment)
 
     /* Add the standard DBus interfaces */
     const InterfaceDescription* introspectable = bus->GetInterface(org::freedesktop::DBus::Introspectable::InterfaceName);
-    assert(introspectable);
+    QCC_ASSERT(introspectable);
     pair<const InterfaceDescription*, bool> comp = make_pair(introspectable, false);
     if (find(components->ifaces.begin(), components->ifaces.end(), comp) == components->ifaces.end()) {
         components->ifaces.push_back(comp);
     }
 
     const InterfaceDescription* allseenIntrospectable = bus->GetInterface(org::allseen::Introspectable::InterfaceName);
-    assert(allseenIntrospectable);
+    QCC_ASSERT(allseenIntrospectable);
     comp = make_pair(allseenIntrospectable, false);
     if (find(components->ifaces.begin(), components->ifaces.end(), comp) == components->ifaces.end()) {
         components->ifaces.push_back(comp);
@@ -686,7 +726,7 @@ QStatus BusObject::DoRegistration(BusAttachment& busAttachment)
 
             /* Add the org::freedesktop::DBus::Properties interface to this list of interfaces implemented by this obj */
             const InterfaceDescription* propIntf = bus->GetInterface(org::freedesktop::DBus::Properties::InterfaceName);
-            assert(propIntf);
+            QCC_ASSERT(propIntf);
             comp = make_pair(propIntf, false);
             if (find(components->ifaces.begin(), components->ifaces.end(), comp) == components->ifaces.end()) {
                 components->ifaces.push_back(comp);
@@ -710,6 +750,11 @@ QStatus BusObject::DoRegistration(BusAttachment& busAttachment)
     return status;
 }
 
+static bool IsBroadcastSignal(AJ_PCSTR destination, SessionId sessionId)
+{
+    return (((destination == nullptr) || (strlen(destination) == 0)) && (sessionId == 0));
+}
+
 QStatus BusObject::SignalInternal(const char* destination,
                                   SessionId sessionId,
                                   const InterfaceDescription::Member& signalMember,
@@ -815,6 +860,23 @@ QStatus BusObject::SignalInternal(const char* destination,
                                          numArgs,
                                          flags,
                                          timeToLive);
+
+        if ((aStatus == ER_OK) && !IsBroadcastSignal(destination, sessionId)) {
+            /* If not a broadcast signal, exchange manifests before doing authorization check. */
+            aStatus = bus->GetInternal().GetPermissionManager().GetPermissionMgmtObj()->SendManifests(nullptr, &msg);
+            if (ER_BUS_BLOCKING_CALL_NOT_ALLOWED == aStatus) {
+                /* We're sending a signal from inside a callback handler. We need to enable
+                 * concurrent callbacks to send needed manifests.
+                 */
+                bus->EnableConcurrentCallbacks();
+                aStatus = bus->GetInternal().GetPermissionManager().GetPermissionMgmtObj()->SendManifests(nullptr, &msg);
+            }
+            if (aStatus != ER_OK) {
+                QCC_LogError(aStatus, ("Failed to SendManifests"));
+            }
+            QCC_ASSERT(ER_OK == aStatus);
+        }
+
         if (aStatus == ER_OK) {
             if (msg->IsEncrypted()) {
                 if (authorizationCallback != NULL) {
@@ -823,6 +885,9 @@ QStatus BusObject::SignalInternal(const char* destination,
                      * not need to authorize the message again.
                      */
                     msg->authorizationChecked = true;
+                } else if (IsBroadcastSignal(destination, sessionId)) {
+                    /* broadcast signal */
+                    msg->authorizationChecked = true; /* skip authorization check */
                 } else if (!bus->GetInternal().GetRouter().IsDaemon()) {
                     /* do an earlier permission authorization to make sure this
                      * signal is allowed to send to the router for delivery.
@@ -984,7 +1049,7 @@ QStatus BusObject::MethodReply(const Message& msg, QStatus status, Message* repl
         } else {
             Message error(*bus);
             QStatus result = error->ErrorMsg(msg, status);
-            assert(ER_OK == result);
+            QCC_ASSERT(ER_OK == result);
             QCC_UNUSED(result);
             if (NULL != replyMsg) {
                 *replyMsg = error;
@@ -1007,7 +1072,7 @@ QStatus BusObject::RemoveChild(BusObject& child)
     QStatus status = ER_BUS_NO_SUCH_OBJECT;
     vector<BusObject*>::iterator it = find(components->children.begin(), components->children.end(), &child);
     if (it != components->children.end()) {
-        assert(*it == &child);
+        QCC_ASSERT(*it == &child);
         child.parent = NULL;
         QCC_DbgPrintf(("RemoveChild %s from object with path = \"%s\"", child.GetPath(), GetPath()));
         components->children.erase(it);
@@ -1075,7 +1140,6 @@ BusObject::BusObject(BusAttachment& bus, const char* path, bool isPlaceholder) :
     description(),
     translator(NULL)
 {
-    components->inUseCounter = 0;
 }
 
 BusObject::BusObject(const char* path, bool isPlaceholder) :
@@ -1090,7 +1154,6 @@ BusObject::BusObject(const char* path, bool isPlaceholder) :
     description(),
     translator(NULL)
 {
-    components->inUseCounter = 0;
 }
 
 BusObject::~BusObject()
@@ -1104,12 +1167,14 @@ BusObject::~BusObject()
     components->counterLock.Unlock(MUTEX_CONTEXT);
 
     QCC_DbgPrintf(("BusObject destructor for object with path = \"%s\"", GetPath()));
+
     /*
      * If this object has a parent it has not been unregistered so do so now.
      */
-    if (bus && parent) {
+    if (bus && (parent || isRegistered)) {
         bus->GetInternal().GetLocalEndpoint()->UnregisterBusObject(*this);
     }
+
     delete components;
 }
 
