@@ -22,7 +22,6 @@
 
 #include <qcc/platform.h>
 #include <qcc/String.h>
-#include <qcc/StringMapKey.h>
 #include <qcc/XmlElement.h>
 #include <map>
 #include <alljoyn/AllJoynStd.h>
@@ -65,6 +64,9 @@ class InterfaceDescription::AnnotationsMap : public std::map<qcc::String, qcc::S
 class InterfaceDescription::ArgumentDescriptions : public std::map<qcc::String, qcc::String> {
 };
 
+class InterfaceDescription::ArgumentAnnotations : public std::map<std::pair<qcc::String, qcc::String>, qcc::String> {
+};
+
 qcc::String InterfaceDescription::NextArg(const char*& signature, qcc::String& argNames, bool inOut,
                                           size_t indent, Member const& member, bool withDescriptions,
                                           const char* langTag, Translator* translator) const
@@ -91,30 +93,57 @@ qcc::String InterfaceDescription::NextArg(const char*& signature, qcc::String& a
     arg += " type=\"" + argType + "\" direction=\"";
     arg += inOut ? "in\"" : "out\"";
 
+    qcc::String childNodesXml;
     const char* myDesc = NULL;
-    if (withDescriptions) {
-        ArgumentDescriptions::const_iterator search = member.argumentDescriptions->find(argName);
-        if (member.argumentDescriptions->end() != search) {
-            myDesc = search->second.c_str();
+    ArgumentDescriptions::const_iterator search = member.argumentDescriptions->find(argName);
+    if (member.argumentDescriptions->end() != search) {
+        myDesc = search->second.c_str();
+    }
+    if (myDesc || member.argumentAnnotations->size() > 0) {
+        if (myDesc) {
+            if (withDescriptions) {
+                qcc::String value;
+                member.GetArgAnnotation(argName.c_str(), qcc::String("org.alljoyn.Bus.DocString.") + langTag, value);
+                AppendDescriptionXml(childNodesXml, langTag, myDesc, translator, value.c_str(), in);
+            }
+            if (langTag == NULL) {
+                AppendDescriptionToArgAnnotations(*member.argumentAnnotations, argName.c_str(), myDesc, translator);
+            }
+        }
+
+        /* Add annotations that belongs to this argument */
+        ArgumentAnnotations::const_iterator ait = member.argumentAnnotations->begin();
+        for (; ait != member.argumentAnnotations->end(); ++ait) {
+            if (ait->first.first == argName) {
+                childNodesXml += in + "  <annotation name=\"" + ait->first.second.c_str() + "\" value=\"" + ait->second + "\"/>\n";
+            }
         }
     }
-    if (myDesc) {
-        arg += ">\n";
-        AppendDescriptionXml(arg, langTag, myDesc, translator, in);
-        arg += in + "</arg>\n";
+    if (!childNodesXml.empty()) {
+        arg += ">\n" + childNodesXml + in + "</arg>\n";
     } else {
         arg += "/>\n";
     }
+
     return arg;
 }
 
 void InterfaceDescription::AppendDescriptionXml(qcc::String& xml, const char* language,
-                                                const char* localDescription, Translator* translator, qcc::String const& indent) const
+                                                const char* localDescription, Translator* translator,
+                                                const char* languageDescription,
+                                                qcc::String const& indent) const
 {
     qcc::String buffer;
-    const char* d = Translate(language, localDescription, buffer, translator);
-    if (!d || d[0] == '\0') {
-        return;
+    const char* d;
+    if ((languageDescription != nullptr) && (languageDescription[0] != '\0') && ((translator == nullptr) || (localDescription == nullptr))) {
+        // Language description is from DocString annotation.
+        // Use it only when the local description cannot be translated
+        d = languageDescription;
+    } else {
+        d = Translate(language, localDescription, buffer, translator);
+        if (!d || d[0] == '\0') {
+            return;
+        }
     }
     xml += indent + "  <description>" + XmlElement::EscapeXml(d) + "</description>\n";
 }
@@ -140,7 +169,8 @@ InterfaceDescription::Member::Member(const InterfaceDescription* iface,
     isSessioncastSignal(false),
     isSessionlessSignal(false),
     isUnicastSignal(false),
-    isGlobalBroadcastSignal(false)
+    isGlobalBroadcastSignal(false),
+    argumentAnnotations(new ArgumentAnnotations())
 {
     if (annotation & MEMBER_ANNOTATE_DEPRECATED) {
         (*annotations)[org::freedesktop::DBus::AnnotateDeprecated] = "true";
@@ -181,7 +211,8 @@ InterfaceDescription::Member::Member(const Member& other)
     isSessioncastSignal(other.isSessioncastSignal),
     isSessionlessSignal(other.isSessionlessSignal),
     isUnicastSignal(other.isUnicastSignal),
-    isGlobalBroadcastSignal(other.isGlobalBroadcastSignal)
+    isGlobalBroadcastSignal(other.isGlobalBroadcastSignal),
+    argumentAnnotations(new ArgumentAnnotations(*(other.argumentAnnotations)))
 {
 }
 
@@ -198,7 +229,10 @@ InterfaceDescription::Member& InterfaceDescription::Member::operator=(const Memb
         annotations = new AnnotationsMap(*(other.annotations));
         accessPerms = other.accessPerms;
         description = other.description;
-        *argumentDescriptions = *(other.argumentDescriptions);
+        delete argumentDescriptions;
+        argumentDescriptions = new ArgumentDescriptions(*(other.argumentDescriptions));
+        delete argumentAnnotations;
+        argumentAnnotations = new ArgumentAnnotations(*(other.argumentAnnotations));
         isSessioncastSignal = other.isSessioncastSignal;
         isSessionlessSignal = other.isSessionlessSignal;
         isUnicastSignal = other.isUnicastSignal;
@@ -211,6 +245,7 @@ InterfaceDescription::Member::~Member()
 {
     delete annotations;
     delete argumentDescriptions;
+    delete argumentAnnotations;
 }
 
 size_t InterfaceDescription::Member::GetAnnotations(qcc::String* names, qcc::String* values, size_t size) const
@@ -218,10 +253,30 @@ size_t InterfaceDescription::Member::GetAnnotations(qcc::String* names, qcc::Str
     return GetAnnotationsWithValues(*annotations, names, values, size);
 }
 
+size_t InterfaceDescription::Member::GetArgAnnotations(const char* argName, qcc::String* names, qcc::String* values, size_t size) const
+{
+    size_t count = argumentAnnotations->size();
+
+    if ((names != NULL) && (values != NULL)) {
+        ArgumentAnnotations::const_iterator ait = argumentAnnotations->begin();
+        count = std::min(count, size);
+        for (size_t i = 0; (i < count) && (ait != argumentAnnotations->end()); ++ait) {
+            if (ait->first.first == argName) {
+                names[i] = ait->first.second;
+                values[i] = ait->second;
+                ++i;
+            }
+        }
+    }
+
+    return count;
+}
+
 bool InterfaceDescription::Member::operator==(const Member& o) const {
     return ((memberType == o.memberType) && (name == o.name) && (signature == o.signature)
             && (returnSignature == o.returnSignature) && (*annotations == *(o.annotations))
             && (description == o.description) && (*argumentDescriptions == *(o.argumentDescriptions))
+            && (*argumentAnnotations == *(o.argumentAnnotations))
             && (isSessioncastSignal == o.isSessioncastSignal)
             && (isSessionlessSignal == o.isSessionlessSignal)
             && (isUnicastSignal == o.isUnicastSignal)
@@ -252,6 +307,10 @@ InterfaceDescription::Property::Property(const char* name, const char* signature
         (*annotations)[org::freedesktop::DBus::AnnotateEmitsChanged] = "true";
         cacheable = true;
     }
+    if (annotation & PROP_ANNOTATE_EMIT_CHANGED_SIGNAL_CONST) {
+        (*annotations)[org::freedesktop::DBus::AnnotateEmitsChanged] = "const";
+        cacheable = true;
+    }
     if (annotation & PROP_ANNOTATE_EMIT_CHANGED_SIGNAL_INVALIDATES) {
         (*annotations)[org::freedesktop::DBus::AnnotateEmitsChanged] = "invalidates";
         cacheable = true;
@@ -298,10 +357,9 @@ bool InterfaceDescription::Property::operator==(const Property& o) const {
             && description == o.description);
 }
 
-
 struct InterfaceDescription::Definitions {
-    typedef std::map<qcc::StringMapKey, Member> MemberMap;
-    typedef std::map<qcc::StringMapKey, Property> PropertyMap;
+    typedef std::map<std::string, Member> MemberMap;
+    typedef std::map<std::string, Property> PropertyMap;
 
     MemberMap members;              /**< Interface members */
     PropertyMap properties;         /**< Interface properties */
@@ -312,23 +370,42 @@ struct InterfaceDescription::Definitions {
     Translator* translator;
     bool hasDescription;
 
-
     Definitions() :
         translator(&stringTableTranslator), hasDescription(false)
     { }
 
-    Definitions(const MemberMap& m, const PropertyMap& p, const AnnotationsMap& a,
-                const qcc::String& langTag, const qcc::String& desc, Translator* dt, bool isDefaultTranslator, bool hd) :
-        members(m), properties(p), annotations(a),
-        languageTag(langTag), description(desc), hasDescription(hd)
+    Definitions(const Definitions& other) :
+        members(other.members), properties(other.properties), annotations(other.annotations),
+        languageTag(other.languageTag), description(other.description), hasDescription(other.hasDescription)
     {
-        if (isDefaultTranslator) {
+        bool usingDefaultTranslator = (other.translator == &other.stringTableTranslator);
+        if (usingDefaultTranslator) {
             translator = &stringTableTranslator;
-            *translator = *dt;
+            *translator = *other.translator;
         } else {
-            translator = dt;
+            translator = other.translator;
         }
     }
+
+    Definitions& operator=(const Definitions& other)
+    {
+        if (this != &other) {
+            members = other.members;
+            properties = other.properties;
+            annotations = other.annotations;
+            languageTag = other.languageTag;
+            description = other.description;
+            bool usingDefaultTranslator = (other.translator == &other.stringTableTranslator);
+            if (usingDefaultTranslator) {
+                translator = &stringTableTranslator;
+                *translator = *other.translator;
+            } else {
+                translator = other.translator;
+            }
+            hasDescription = other.hasDescription;
+        }
+        return *this;
+    }
 };
 
 bool InterfaceDescription::Member::GetAnnotation(const qcc::String& annotationName, qcc::String& value) const
@@ -337,6 +414,13 @@ bool InterfaceDescription::Member::GetAnnotation(const qcc::String& annotationNa
     return (it != annotations->end() ? value = it->second, true : false);
 }
 
+bool InterfaceDescription::Member::GetArgAnnotation(const char* argName, const qcc::String& annotationName, qcc::String& value) const
+{
+    pair<qcc::String, qcc::String> item(qcc::String(argName), annotationName);
+    ArgumentAnnotations::const_iterator ait = argumentAnnotations->find(item);
+    return (ait != argumentAnnotations->end() ? value = ait->second, true : false);
+}
+
 bool InterfaceDescription::Property::GetAnnotation(const qcc::String& annotationName, qcc::String& value) const
 {
     AnnotationsMap::const_iterator it = annotations->find(annotationName);
@@ -367,9 +451,7 @@ InterfaceDescription::~InterfaceDescription()
 }
 
 InterfaceDescription::InterfaceDescription(const InterfaceDescription& other) :
-    defs(new Definitions(other.defs->members, other.defs->properties, other.defs->annotations,
-                         other.defs->languageTag, other.defs->description, other.defs->translator,
-                         other.defs->translator == &other.defs->stringTableTranslator, other.defs->hasDescription)),
+    defs(new Definitions(*other.defs)),
     name(other.name),
     isActivated(false),
     secPolicy(other.secPolicy)
@@ -384,17 +466,11 @@ InterfaceDescription::InterfaceDescription(const InterfaceDescription& other) :
 InterfaceDescription& InterfaceDescription::operator=(const InterfaceDescription& other)
 {
     if (this != &other) {
+        *defs = *other.defs;
         name = other.name;
-        secPolicy = other.secPolicy;
         isActivated = false;
-        defs->members = other.defs->members;
-        defs->properties = other.defs->properties;
-        defs->annotations = other.defs->annotations;
-        defs->languageTag = other.defs->languageTag;
-        defs->description = other.defs->description;
-        defs->translator = other.defs->translator;
-
-        /* Update the iface pointer in each member */
+        secPolicy = other.secPolicy;
+        /* Update the iface pointer in each definitions member */
         Definitions::MemberMap::iterator mit = defs->members.begin();
         while (mit != defs->members.end()) {
             mit++->second.iface = this;
@@ -416,12 +492,18 @@ qcc::String InterfaceDescription::Introspect(size_t indent, const char* language
     }
 
     bool withDescriptions = languageTag && defs->hasDescription;
+    bool unifiedFormat = (languageTag == NULL);
 
     qcc::String xml = in + "<interface name=\"";
     xml += name + close;
 
     if (withDescriptions) {
-        AppendDescriptionXml(xml, languageTag, defs->description.c_str(), myTranslator, in);
+        qcc::String value;
+        GetAnnotation(qcc::String("org.alljoyn.Bus.DocString.") + languageTag, value);
+        AppendDescriptionXml(xml, languageTag, defs->description.c_str(), myTranslator, value.c_str(), in);
+    }
+    if (unifiedFormat && defs->hasDescription) {
+        AppendDescriptionToAnnotations(defs->annotations, defs->description.c_str(), myTranslator);
     }
 
     /*
@@ -453,8 +535,28 @@ qcc::String InterfaceDescription::Introspect(size_t indent, const char* language
         }
         xml     += close;
 
+        if (unifiedFormat && !isMethod) {
+            if (member.isSessioncastSignal) {
+                (*member.annotations)["org.alljoyn.Bus.Signal.Sessioncast"] = "true";
+            }
+            if (member.isSessionlessSignal) {
+                (*member.annotations)["org.alljoyn.Bus.Signal.Sessionless"] = "true";
+            }
+            if (member.isUnicastSignal) {
+                (*member.annotations)["org.alljoyn.Bus.Signal.Unicast"] = "true";
+            }
+            if (member.isGlobalBroadcastSignal) {
+                (*member.annotations)["org.alljoyn.Bus.Signal.GlobalBroadcast"] = "true";
+            }
+        }
+
         if (withDescriptions) {
-            AppendDescriptionXml(xml, languageTag, member.description.c_str(), myTranslator, in + "  ");
+            qcc::String value;
+            member.GetAnnotation(qcc::String("org.alljoyn.Bus.DocString.") + languageTag, value);
+            AppendDescriptionXml(xml, languageTag, member.description.c_str(), myTranslator, value.c_str(), in + "  ");
+        }
+        if (unifiedFormat) {
+            AppendDescriptionToAnnotations(*member.annotations, member.description.c_str(), myTranslator);
         }
 
         /* Iterate over IN arguments */
@@ -494,12 +596,11 @@ qcc::String InterfaceDescription::Introspect(size_t indent, const char* language
             xml += " access=\"readwrite\"";
         }
 
-        // Does this property have a description? Only if (a) we're doing descriptions,
-        // (b) the property has some description text, (c) that text is not a
+        // Does this property have a description? Only if
+        // (a) the property has some description text, (b) that text is not a
         // lookup key (empty language tag) with no Translator to produce a description string,
-        // and (d) the translator has a description in the requested language.
-        bool propWithDescription = withDescriptions &&
-                                   !property.description.empty() &&
+        // and (c) the translator has a description in the requested language.
+        bool propWithDescription = !property.description.empty() &&
                                    !(defs->languageTag.empty() && !myTranslator);
         if (propWithDescription) {
             qcc::String buffer;
@@ -513,6 +614,15 @@ qcc::String InterfaceDescription::Introspect(size_t indent, const char* language
         if (property.annotations->size() || propWithDescription) {
             xml += ">\n";
 
+            if (withDescriptions) {
+                qcc::String value;
+                property.GetAnnotation(qcc::String("org.alljoyn.Bus.DocString.") + languageTag, value);
+                AppendDescriptionXml(xml, languageTag, property.description.c_str(), myTranslator, value.c_str(), in + "  ");
+            }
+            if (unifiedFormat) {
+                AppendDescriptionToAnnotations(*property.annotations, property.description.c_str(), myTranslator);
+            }
+
             if (property.annotations->size()) {
                 // add annotations
                 AnnotationsMap::const_iterator ait = property.annotations->begin();
@@ -520,10 +630,6 @@ qcc::String InterfaceDescription::Introspect(size_t indent, const char* language
                     xml += in + "    <annotation name=\"" + ait->first.c_str() + "\" value=\"" + ait->second + "\"/>\n";
                 }
             }
-
-            if (propWithDescription) {
-                AppendDescriptionXml(xml, languageTag, property.description.c_str(), myTranslator, in + "  ");
-            }
             xml += in + "  </property>\n";
         } else {
             xml += "/>\n";
@@ -554,9 +660,9 @@ QStatus InterfaceDescription::AddMember(AllJoynMessageType type,
         return ER_BUS_INTERFACE_ACTIVATED;
     }
 
-    StringMapKey key = qcc::String(memberName);
+    std::string key = qcc::String(memberName);
     Member member(this, type, memberName, inSig, outSig, argNames, annotation, accessPerms);
-    pair<StringMapKey, Member> item(key, member);
+    pair<std::string, Member> item(key, member);
     pair<Definitions::MemberMap::iterator, bool> ret = defs->members.insert(item);
     return ret.second ? ER_OK : ER_BUS_MEMBER_ALREADY_EXISTS;
 }
@@ -567,7 +673,7 @@ QStatus InterfaceDescription::AddMemberAnnotation(const char* member, const qcc:
         return ER_BUS_INTERFACE_ACTIVATED;
     }
 
-    Definitions::MemberMap::iterator it = defs->members.find(StringMapKey(member));
+    Definitions::MemberMap::iterator it = defs->members.find(std::string(member));
     if (it == defs->members.end()) {
         return ER_BUS_INTERFACE_NO_SUCH_MEMBER;
     }
@@ -579,7 +685,7 @@ QStatus InterfaceDescription::AddMemberAnnotation(const char* member, const qcc:
 
 bool InterfaceDescription::GetMemberAnnotation(const char* member, const qcc::String& annotationName, qcc::String& value) const
 {
-    Definitions::MemberMap::const_iterator mit = defs->members.find(StringMapKey(member));
+    Definitions::MemberMap::const_iterator mit = defs->members.find(std::string(member));
     if (mit == defs->members.end()) {
         return false;
     }
@@ -596,9 +702,9 @@ QStatus InterfaceDescription::AddProperty(const char* propertyName, const char*
         return ER_BUS_INTERFACE_ACTIVATED;
     }
 
-    StringMapKey key = qcc::String(propertyName);
+    std::string key = qcc::String(propertyName);
     Property prop(propertyName, signature, access);
-    pair<StringMapKey, Property> item(key, prop);
+    pair<std::string, Property> item(key, prop);
     pair<Definitions::PropertyMap::iterator, bool> ret = defs->properties.insert(item);
     return ret.second ? ER_OK : ER_BUS_PROPERTY_ALREADY_EXISTS;
 }
@@ -609,7 +715,7 @@ QStatus InterfaceDescription::AddPropertyAnnotation(const qcc::String& p_name, c
         return ER_BUS_INTERFACE_ACTIVATED;
     }
 
-    Definitions::PropertyMap::iterator pit = defs->properties.find(StringMapKey(p_name));
+    Definitions::PropertyMap::iterator pit = defs->properties.find(std::string(p_name));
     if (pit == defs->properties.end()) {
         return ER_BUS_NO_SUCH_PROPERTY;
     }
@@ -617,15 +723,24 @@ QStatus InterfaceDescription::AddPropertyAnnotation(const qcc::String& p_name, c
     Property& property = pit->second;
     std::pair<AnnotationsMap::iterator, bool> ret = property.annotations->insert(AnnotationsMap::value_type(annotationName, value));
     QStatus status = (ret.second || (ret.first->first == annotationName && ret.first->second == value)) ? ER_OK : ER_BUS_ANNOTATION_ALREADY_EXISTS;
-    if (status == ER_OK && annotationName == org::freedesktop::DBus::AnnotateEmitsChanged && value != "false") {
-        property.cacheable = true;
+    if (status == ER_OK) {
+        if ((annotationName == org::freedesktop::DBus::AnnotateEmitsChanged) && (value != "false")) {
+            property.cacheable = true;
+        } else if ((annotationName == "org.alljoyn.Bus.Type.Min") ||
+                   (annotationName == "org.alljoyn.Bus.Type.Max") ||
+                   (annotationName == "org.alljoyn.Bus.Type.Units") ||
+                   (annotationName == "org.alljoyn.Bus.Type.Default") ||
+                   (annotationName == "org.alljoyn.Bus.Type.Reference") ||
+                   (annotationName == "org.alljoyn.Bus.Type.DisplayHint")) {
+            property.cacheable = true;
+        }
     }
     return status;
 }
 
 bool InterfaceDescription::GetPropertyAnnotation(const qcc::String& p_name, const qcc::String& annotationName, qcc::String& value) const
 {
-    Definitions::PropertyMap::const_iterator pit = defs->properties.find(StringMapKey(p_name));
+    Definitions::PropertyMap::const_iterator pit = defs->properties.find(std::string(p_name));
     if (pit == defs->properties.end()) {
         return false;
     }
@@ -697,7 +812,7 @@ size_t InterfaceDescription::GetProperties(const Property** props, size_t numPro
 
 const InterfaceDescription::Property* InterfaceDescription::GetProperty(const char* propertyName) const
 {
-    Definitions::PropertyMap::const_iterator pit = defs->properties.find(qcc::StringMapKey(propertyName));
+    Definitions::PropertyMap::const_iterator pit = defs->properties.find(std::string(propertyName));
     return (pit == defs->properties.end()) ? NULL : &(pit->second);
 }
 
@@ -716,7 +831,7 @@ size_t InterfaceDescription::GetMembers(const Member** members, size_t numMember
 
 const InterfaceDescription::Member* InterfaceDescription::GetMember(const char* memberName) const
 {
-    Definitions::MemberMap::const_iterator mit = defs->members.find(qcc::StringMapKey(memberName));
+    Definitions::MemberMap::const_iterator mit = defs->members.find(std::string(memberName));
     return (mit == defs->members.end()) ? NULL : &(mit->second);
 }
 
@@ -761,7 +876,7 @@ QStatus InterfaceDescription::SetMemberDescription(const char* member, const cha
         return ER_BUS_INTERFACE_ACTIVATED;
     }
 
-    Definitions::MemberMap::iterator it = defs->members.find(StringMapKey(member));
+    Definitions::MemberMap::iterator it = defs->members.find(std::string(member));
     if (it == defs->members.end()) {
         return ER_BUS_INTERFACE_NO_SUCH_MEMBER;
     }
@@ -780,7 +895,7 @@ QStatus InterfaceDescription::SetMemberDescription(const char* member, const cha
         return ER_BUS_INTERFACE_ACTIVATED;
     }
 
-    Definitions::MemberMap::iterator it = defs->members.find(StringMapKey(member));
+    Definitions::MemberMap::iterator it = defs->members.find(std::string(member));
     if (it == defs->members.end()) {
         return ER_BUS_INTERFACE_NO_SUCH_MEMBER;
     }
@@ -812,7 +927,7 @@ QStatus InterfaceDescription::SetArgDescription(const char* member, const char*
         return ER_BUS_INTERFACE_ACTIVATED;
     }
 
-    Definitions::MemberMap::iterator it = defs->members.find(StringMapKey(member));
+    Definitions::MemberMap::iterator it = defs->members.find(std::string(member));
     if (it == defs->members.end()) {
         return ER_BUS_INTERFACE_NO_SUCH_MEMBER;
     }
@@ -830,7 +945,7 @@ QStatus InterfaceDescription::SetPropertyDescription(const char* propName, const
         return ER_BUS_INTERFACE_ACTIVATED;
     }
 
-    Definitions::PropertyMap::iterator pit = defs->properties.find(StringMapKey(propName));
+    Definitions::PropertyMap::iterator pit = defs->properties.find(std::string(propName));
     if (pit == defs->properties.end()) {
         return ER_BUS_NO_SUCH_PROPERTY;
     }
@@ -847,6 +962,34 @@ bool InterfaceDescription::HasDescription() const
     return defs->hasDescription;
 }
 
+QStatus InterfaceDescription::AddArgAnnotation(const char* member, const char* arg, const qcc::String& name, const qcc::String& value)
+{
+    if (isActivated) {
+        return ER_BUS_INTERFACE_ACTIVATED;
+    }
+
+    Definitions::MemberMap::iterator it = defs->members.find(std::string(member));
+    if (it == defs->members.end()) {
+        return ER_BUS_INTERFACE_NO_SUCH_MEMBER;
+    }
+
+    Member& m = it->second;
+    pair<qcc::String, qcc::String> item(qcc::String(arg), name);
+    std::pair<ArgumentAnnotations::iterator, bool> ret = m.argumentAnnotations->insert(ArgumentAnnotations::value_type(item, value));
+    return (ret.second || ((ret.first->first.first.compare(arg) == 0) && (ret.first->first.second == name))) ? ER_OK : ER_BUS_ANNOTATION_ALREADY_EXISTS;
+}
+
+bool InterfaceDescription::GetArgAnnotation(const char* member, const char* arg, const qcc::String& name, qcc::String& value) const
+{
+    Definitions::MemberMap::iterator it = defs->members.find(std::string(member));
+    if (it == defs->members.end()) {
+        return false;
+    }
+
+    Member& m = it->second;
+    return m.GetArgAnnotation(arg, name, value);
+}
+
 const char* InterfaceDescription::Translate(const char* toLanguage, const char* text, qcc::String& buffer, Translator* translator) const
 {
     if (NULL == text) {
@@ -862,7 +1005,7 @@ const char* InterfaceDescription::Translate(const char* toLanguage, const char*
         }
     }
 
-    if (text && text[0] != '\0' && !defs->languageTag.empty()) {
+    if (text && text[0] != '\0') {
         return text;
     }
 
@@ -878,4 +1021,100 @@ Translator* InterfaceDescription::GetDescriptionTranslator() const
     return defs->translator;
 }
 
+void InterfaceDescription::AppendDescriptionToAnnotations(AnnotationsMap& annotations, const char* description, Translator* translator) const
+{
+    if ((description == NULL) || (description[0] == '\0')) {
+        return;
+    }
+
+    qcc::String docString("org.alljoyn.Bus.DocString");
+    qcc::String localDescription(description);
+
+    /* Append description of all languages */
+    qcc::String language;
+    if (translator != NULL) {
+        size_t size = translator->NumTargetLanguages();
+        for (size_t index = 0; index < size; index++) {
+            translator->GetTargetLanguage(index, language);
+
+            if (!language.empty()) {
+                qcc::String buffer;
+                const char* d = Translate(language.c_str(), description, buffer, translator);
+                if ((d == NULL) || (d[0] == '\0')) {
+                    continue;
+                }
+                localDescription = XmlElement::EscapeXml(d);
+                qcc::String name = docString + "." + language;
+                annotations[name] = localDescription;
+            }
+        }
+    } else {
+        /* Append with default language tag only if it hasn't been annotated */
+        bool found = false;
+        AnnotationsMap::const_iterator mit = annotations.begin();
+        for (; mit != annotations.end(); ++mit) {
+            if (localDescription == mit->second) {
+                found = true;
+                break;
+            }
+        }
+        if (!found) {
+            qcc::String name = docString;
+            if (!defs->languageTag.empty()) {
+                name += "." + defs->languageTag;
+            }
+            annotations[name] = localDescription;
+        }
+    }
+}
+
+void InterfaceDescription::AppendDescriptionToArgAnnotations(ArgumentAnnotations& argAnnotations, const char* argName, const char* description, Translator* translator) const
+{
+    if ((description == NULL) || (description[0] == '\0')) {
+        return;
+    }
+
+    qcc::String docString("org.alljoyn.Bus.DocString");
+    qcc::String localDescription(description);
+
+    /* Append description of all languages */
+    qcc::String language;
+    if (translator != NULL) {
+        size_t size = translator->NumTargetLanguages();
+        for (size_t index = 0; index < size; index++) {
+            translator->GetTargetLanguage(index, language);
+
+            if (!language.empty()) {
+                qcc::String buffer;
+                const char* d = Translate(language.c_str(), description, buffer, translator);
+                if ((d == NULL) || (d[0] == '\0')) {
+                    continue;
+                }
+                localDescription = XmlElement::EscapeXml(d);
+                qcc::String name = docString + "." + language;
+
+                pair<qcc::String, qcc::String> item(qcc::String(argName), name);
+                argAnnotations.insert(ArgumentAnnotations::value_type(item, localDescription));
+            }
+        }
+    } else {
+        /* Append with default language tag only if it hasn't been annotated */
+        bool found = false;
+        ArgumentAnnotations::const_iterator ait = argAnnotations.begin();
+        for (; ait != argAnnotations.end(); ++ait) {
+            if (localDescription == ait->second) {
+                found = true;
+                break;
+            }
+        }
+        if (!found) {
+            qcc::String name = docString;
+            if (!defs->languageTag.empty()) {
+                name += "." + defs->languageTag;
+            }
+            pair<qcc::String, qcc::String> item(qcc::String(argName), name);
+            argAnnotations.insert(ArgumentAnnotations::value_type(item, localDescription));
+        }
+    }
+}
 }
