@@ -39,7 +39,6 @@
 #include <qcc/time.h>
 
 #include <alljoyn/KeyStoreListener.h>
-#include "ProtectedKeyStoreListener.h"
 
 #include <alljoyn/Status.h>
 
@@ -48,6 +47,7 @@ namespace ajn {
 
 /* forward decl */
 class KeyStoreKeyEventListener;
+class ProtectedKeyStoreListener;
 
 /**
  * The %KeyStore class manages the storing and loading of key blobs from
@@ -161,6 +161,16 @@ class KeyStore {
         qcc::GUID128 guid;
     };
 
+    /**
+     * Static initialization routine called by AllJoynInit.
+     */
+    static void Init();
+
+    /**
+     * Static cleanup routine called by AllJoynShutdown.
+     */
+    static void Shutdown();
+
     /**
      * KeyStore constructor
      */
@@ -176,16 +186,13 @@ class KeyStore {
      *
      * @param fileName  The filename to be used by the default key store if the default key store is being used.
      *                  This overrides the value in the applicationName parameter passed into the constructor.
-     *
-     * @param isShared  Indicates if the key store is being shared between multiple applications.
+     * @param isShared  This parameter is not used as of 16.04. It is ignored internally (always shared).
+     * @return
+     *      - ER_OK if successful
+     *      - An error status otherwise
      */
     QStatus Init(const char* fileName, bool isShared);
 
-    /**
-     * Requests the key store listener to store the contents of the key store
-     */
-    QStatus Store();
-
     /**
      * Re-read keys from the key store. This is a no-op unless the key store is shared.
      * If the key store is shared the key store is reloaded merging any changes made by
@@ -233,7 +240,9 @@ class KeyStore {
      * @param key         The unique identifier for the key
      * @param keyBlob     The key blob to add
      * @param accessRights The access rights associated with the key
-     * @return ER_OK
+     * @return
+     *      - ER_OK if successful
+     *      - An error status otherwise
      */
     QStatus AddKey(const Key& key, const qcc::KeyBlob& keyBlob, const uint8_t accessRights[4]);
 
@@ -242,7 +251,9 @@ class KeyStore {
      *
      * @param key         The unique identifier for the key
      * @param keyBlob     The key blob to add
-     * @return ER_OK
+     * @return
+     *      - ER_OK if successful
+     *      - An error status otherwise
      */
     QStatus AddKey(const Key& key, const qcc::KeyBlob& keyBlob)
     {
@@ -254,27 +265,35 @@ class KeyStore {
      * Remove a key blob from the key store
      *
      * @param key  The unique identifier for the key
-     * return ER_OK
+     * @param includeAssociatedKeys  True to search for associated keys and also delete them
+     * @param exclusiveLockAlreadyHeld  True to indicate that DelKey() should not try to acquire the exclusive lock
+     * @return
+     *      - ER_OK if successful
+     *      - ER_BUS_KEY_STORE_NOT_LOADED if the GUID is not available
      */
-    QStatus DelKey(const Key& key);
+    QStatus DelKey(const Key& key, bool includeAssociatedKeys = false, bool exclusiveLockAlreadyHeld = false);
 
     /**
      * Set expiration time on a key blob from the key store
      *
      * @param key        The unique identifier for the key
      * @param expiration  Time to expire the key
-     * return ER_OK
+     * @return
+     *      - ER_OK if successful
+     *      - ER_BUS_KEY_STORE_NOT_LOADED if the GUID is not available
      */
-    QStatus SetKeyExpiration(const Key& key, const qcc::Timespec& expiration);
+    QStatus SetKeyExpiration(const Key& key, const qcc::Timespec<qcc::EpochTime>& expiration);
 
     /**
      * Get expiration time on a key blob from the key store
      *
      * @param key        The unique identifier for the key
      * @param expiration  Time the key will expire
-     * return ER_OK
+     * @return
+     *      - ER_OK if successful
+     *      - ER_BUS_KEY_STORE_NOT_LOADED if the GUID is not available
      */
-    QStatus GetKeyExpiration(const Key& key, qcc::Timespec& expiration);
+    QStatus GetKeyExpiration(const Key& key, qcc::Timespec<qcc::EpochTime>& expiration);
 
     /**
      * Test is there is a requested key blob in the key store
@@ -318,19 +337,28 @@ class KeyStore {
      *
      * @param listener  The listener that will override the default listener. This function
      *                  must be called before the key store is initialized.
+     * @return
+     *      - ER_OK on success
+     *      - ER_BUS_KEY_STORE_NOT_LOADED if the GUID is not available
      */
     QStatus SetListener(KeyStoreListener& listener);
 
     /**
-     * Setup the key event listener.
+     * Setup the key event listener. This function must be called before the key store is initialized.
      *
      * @param listener  The listener that will listen to key event.
+     * @return
+     *      - ER_OK on success
+     *      - ER_BUS_KEY_STORE_NOT_LOADED if the GUID is not available
      */
     QStatus SetKeyEventListener(KeyStoreKeyEventListener* listener);
 
     /**
      * Restores the default listener
      *
+     * @return
+     *      - ER_OK on success
+     *      - ER_BUS_KEY_STORE_NOT_LOADED if the GUID is not available
      */
     QStatus SetDefaultListener();
 
@@ -376,23 +404,24 @@ class KeyStore {
      */
     QStatus Pull(qcc::Source& source, const qcc::String& password);
 
+    /**
+     * This is similar to Pull(), but used for initializing in-memory keys (since the file is still empty).
+     *
+     * @param password  The password required to decrypt the key store
+     *
+     */
+    void LoadFromEmptyFile(const qcc::String& password);
+
     /**
      * Push the current keys from the key store into a sink
      *
-     * @param sink The sink to write the keys to.
+     * @param sink    The sink to write the keys to.
      * @return
      *      - ER_OK if successful
      *      - An error status otherwise
      */
     QStatus Push(qcc::Sink& sink);
 
-    /**
-     * Indicates if this is a shared key store.
-     *
-     * @return  Returns true if the key store is shared between multiple applications.
-     */
-    bool IsShared() { return shared; }
-
     /**
      * Search for associated keys with the given key
      * @param key  The header key
@@ -473,11 +502,6 @@ class KeyStore {
      */
     KeyStore();
 
-    /**
-     * Internal function to erase expired keys
-     */
-    size_t EraseExpiredKeys();
-
     /**
      * Internal Load function
      */
@@ -494,22 +518,49 @@ class KeyStore {
     void MarkGuidSet();
 
     /**
-     * synchronized method to invoke the listener load request
-     */
-    QStatus SendLoadRequest(bool waitFor);
-
-    /**
-     * Delete the key from the internal structure.  The lock must be acquired prior to calling this method.
+     * Similar to DelKey, but exclusive lock must be held before calling.
+     * This method only modifies in-memory keys (does not call StoreInternal).
      *
      * @param key  The unique identifier for the key
-     * return ER_OK
      */
-    QStatus DeleteKey(const Key& key);
+    void DelKeyInternal(const Key& key);
 
     /**
-     * synchronized method to invoke the listener store request
+     * Requests the key store listener to store the contents of the key store.
+     *
+     * @return ER_OK if successful; otherwise, error code.
      */
-    QStatus SendStoreRequest(bool waitFor);
+    QStatus StoreInternal(std::vector<Key>& expiredKeys);
+
+    /**
+     * Helper function for acquiring exclusive lock.
+     *
+     * @remark Best practice is to call `AcquireExclusiveLock(MUTEX_CONTEXT)`
+     *
+     * @see MUTEX_CONTEXT
+     *
+     * @param file the name of the file this lock was called from
+     * @param line the line number of the file this lock was called from
+     * @return
+     *      - ER_OK if successful
+     *      - An error status otherwise
+     */
+    QStatus AcquireExclusiveLock(const char* file = NULL, uint32_t line = 0);
+
+    /**
+     * Helper function for releasing exclusive lock.
+     *
+     * @remark Best practice is to call `ReleaseExclusiveLock(MUTEX_CONTEXT)`
+     *
+     * @see MUTEX_CONTEXT
+     *
+     * @param file the name of the file this lock was called from
+     * @param line the line number of the file this lock was called from
+     * @return
+     *      - ER_OK if successful
+     *      - An error status otherwise
+     */
+    void ReleaseExclusiveLock(const char* file = NULL, uint32_t line = 0);
 
     /**
      * The application that owns this key store. If the key store is shared this will be the name
@@ -531,7 +582,8 @@ class KeyStore {
      */
     class KeyRecord {
       public:
-        KeyRecord() : revision(0) { }
+        KeyRecord() : persisted(false), revision(0) { }
+        bool persisted;          /// currently persisted in storage
         uint32_t revision;       ///< Revision number when this key was added
         qcc::KeyBlob keyBlob;        ///< The key blob for the key
         uint8_t accessRights[4]; ///< Access rights associated with this record (see PeerState)
@@ -542,11 +594,23 @@ class KeyStore {
      */
     typedef std::map<Key, KeyRecord> KeyMap;
 
+    /**
+     * Internal function to load the persistent keys
+     */
+    QStatus LoadPersistentKeys();
+
     /**
      * In memory copy of the key store
      */
     KeyMap* keys;
 
+    /**
+     * The hash map for the persistent keys to be used by the Pull method
+     * Once the pull is complete, the persistent keys will be merged with the
+     * keys.
+     */
+    KeyMap* persistentKeys;
+
     /**
      * GUID for keys that have been deleted
      */
@@ -583,9 +647,9 @@ class KeyStore {
     uint32_t revision;
 
     /**
-     * Indicates if the key store is shared between multiple applications.
+     * Revision number for the persistent data store
      */
-    bool shared;
+    uint32_t persistentRevision;
 
     /**
      * Event for synchronizing store requests
@@ -632,6 +696,15 @@ class KeyStore {
      */
     uint8_t guidSetRefCount;
 
+    /**
+     * If we're holding the exclusive lock, we only need to refresh once. This enum keeps track of that.
+     */
+    enum ExclusiveLockRefreshState {
+        ExclusiveLockNotHeld,     /**< Lock is not held. */
+        ExclusiveLockHeld_Dirty,  /**< Lock is held but keystore contents have not been synchronized. */
+        ExclusiveLockHeld_Clean   /**< Lock is held and keystore contents have been synchronized. */
+    } exclusiveLockRefreshState;
+
 };
 
 class KeyStoreKeyEventListener {
@@ -656,6 +729,20 @@ class KeyStoreListenerFactory {
 
 };
 
+/*
+ * Calculates the path to the default key store file corresponding to the
+ * application and fname parameters, and then deletes that file.
+ *
+ * @remark The parameters of this method correspond to the parameters of the DefaultKeyStoreListener constructor
+ *
+ * @param application the appName parameter used when constructing a BusAttachment using this key store file
+ * @param fname key store file used by the application, or nullptr if using the default file name
+ * @return
+ *      - ER_OK if the file was not present, of if it has been deleted successfully
+ *      - An error status otherwise
+ */
+QStatus DeleteDefaultKeyStoreFile(const qcc::String& application, const char* fname = nullptr);
+
 }
 
 #endif
