@@ -54,13 +54,24 @@ namespace ajn {
 
 static const uint32_t LOCAL_ENDPOINT_CONCURRENCY = 4;
 
+#if !defined(LOCAL_ENDPOINT_MAXALARMS)
+/**
+ * This ifdef is here to override the dispatcher's 'maxAlarms' value. This mechanism is designed
+ * to prevent a possible deadlock in apps. See ASACORE-2810 for details.
+ *
+ * Note that this is a temporary solution as 'maxAlarms' is expected to be removed by ASACORE-2650.
+ */
+static const uint32_t LOCAL_ENDPOINT_MAXALARMS = 10;
+#endif
+
 class _LocalEndpoint::Dispatcher : public qcc::Timer, public qcc::AlarmListener {
   public:
     Dispatcher(_LocalEndpoint* endpoint, uint32_t concurrency = LOCAL_ENDPOINT_CONCURRENCY) :
-        Timer("lepDisp" + U32ToString(qcc::IncrementAndFetch(&dispatcherCnt)), true, concurrency, true, 10),
+        Timer("lepDisp" + U32ToString(qcc::IncrementAndFetch(&dispatcherCnt)), true, concurrency, true, LOCAL_ENDPOINT_MAXALARMS),
         AlarmListener(), endpoint(endpoint), pendingWork(),
         needDeferredCallbacks(false), needObserverWork(false),
-        needCachedPropertyReplyWork(false)
+        needCachedPropertyReplyWork(false),
+        workLock(LOCK_LEVEL_LOCALTRANSPORT_LOCALENDPOINT_DISPATCHER_WORKLOCK)
     {
         uint32_t zero = 0;
         qcc::AlarmListener* listener = static_cast<qcc::AlarmListener*>(this);
@@ -191,13 +202,14 @@ _LocalEndpoint::_LocalEndpoint(BusAttachment& bus, uint32_t concurrency) :
     running(false),
     isRegistered(false),
     bus(&bus),
-    objectsLock(),
-    replyMapLock(),
+    objectsLock(LOCK_LEVEL_LOCALTRANSPORT_LOCALENDPOINT_OBJECTSLOCK),
+    replyMapLock(LOCK_LEVEL_LOCALTRANSPORT_LOCALENDPOINT_REPLYMAPLOCK),
     replyTimer("replyTimer", true),
     dbusObj(NULL),
     alljoynObj(NULL),
     alljoynDebugObj(NULL),
-    peerObj(NULL)
+    peerObj(NULL),
+    handlerThreadsLock(LOCK_LEVEL_LOCALTRANSPORT_LOCALENDPOINT_HANDLERTHREADSLOCK)
 {
 }
 
@@ -397,7 +409,7 @@ QStatus _LocalEndpoint::PeerInterface(Message& message)
             return status;
         }
         status = message->ReplyMsg(message, NULL, 0);
-        assert(ER_OK == status);
+        QCC_ASSERT(ER_OK == status);
         BusEndpoint busEndpoint = BusEndpoint::wrap(this);
         return bus->GetInternal().GetRouter().PushMessage(message, busEndpoint);
     }
@@ -412,7 +424,7 @@ QStatus _LocalEndpoint::PeerInterface(Message& message)
         replyArg.v_string.str = guidStr.c_str();
         replyArg.v_string.len = guidStr.size();
         status = message->ReplyMsg(message, &replyArg, 1);
-        assert(ER_OK == status);
+        QCC_ASSERT(ER_OK == status);
         BusEndpoint busEndpoint = BusEndpoint::wrap(this);
         return bus->GetInternal().GetRouter().PushMessage(message, busEndpoint);
     }
@@ -451,7 +463,6 @@ bool _LocalEndpoint::IsReentrantCall()
         return false;
     }
     return dispatcher->IsHoldingReentrantLock();
-
 }
 
 void _LocalEndpoint::Dispatcher::TriggerDeferredCallbacks()
@@ -670,9 +681,9 @@ QStatus _LocalEndpoint::DoPushMessage(Message& message)
             break;
         }
 
-        handlerThreadsLock.Lock();
+        handlerThreadsLock.Lock(MUTEX_CONTEXT);
         handlerThreadsDone.Broadcast();
-        handlerThreadsLock.Unlock();
+        handlerThreadsLock.Unlock(MUTEX_CONTEXT);
     }
     return status;
 }
@@ -882,7 +893,7 @@ QStatus _LocalEndpoint::GetAnnouncedObjectDescription(MsgArg& objectDescriptionA
         }
     }
     // If argCount and announcedObjectsCount don't match something has gone wrong
-    assert(argCount == announcedObjectsCount);
+    QCC_ASSERT(argCount == announcedObjectsCount);
 
     status = objectDescriptionArg.Set("a(oas)", announcedObjectsCount, announceObjectsArg);
     objectDescriptionArg.Stabilize();
@@ -971,7 +982,7 @@ _LocalEndpoint::ReplyContext* _LocalEndpoint::RemoveReplyHandler(uint32_t serial
     if (iter != replyMap.end()) {
         rc = iter->second;
         replyMap.erase(iter);
-        assert(rc->serial == serial);
+        QCC_ASSERT(rc->serial == serial);
     }
     return rc;
 }
@@ -980,13 +991,13 @@ bool _LocalEndpoint::PauseReplyHandlerTimeout(Message& methodCallMsg)
 {
     bool paused = false;
     if (methodCallMsg->GetType() == MESSAGE_METHOD_CALL) {
-        replyMapLock.Lock();
+        replyMapLock.Lock(MUTEX_CONTEXT);
         map<uint32_t, ReplyContext*>::iterator iter = replyMap.find(methodCallMsg->GetCallSerial());
         if (iter != replyMap.end()) {
             ReplyContext*rc = iter->second;
             paused = replyTimer.RemoveAlarm(rc->alarm);
         }
-        replyMapLock.Unlock();
+        replyMapLock.Unlock(MUTEX_CONTEXT);
     }
     return paused;
 }
@@ -995,7 +1006,7 @@ bool _LocalEndpoint::ResumeReplyHandlerTimeout(Message& methodCallMsg)
 {
     bool resumed = false;
     if (methodCallMsg->GetType() == MESSAGE_METHOD_CALL) {
-        replyMapLock.Lock();
+        replyMapLock.Lock(MUTEX_CONTEXT);
         map<uint32_t, ReplyContext*>::iterator iter = replyMap.find(methodCallMsg->GetCallSerial());
         if (iter != replyMap.end()) {
             ReplyContext*rc = iter->second;
@@ -1006,7 +1017,7 @@ bool _LocalEndpoint::ResumeReplyHandlerTimeout(Message& methodCallMsg)
                 QCC_LogError(status, ("Failed to resume reply handler timeout for %s", methodCallMsg->Description().c_str()));
             }
         }
-        replyMapLock.Unlock();
+        replyMapLock.Unlock(MUTEX_CONTEXT);
     }
     return resumed;
 }
@@ -1039,12 +1050,13 @@ bool _LocalEndpoint::OkToUnregisterHandlerObj(MessageReceiver* receiver)
     ActiveHandlers::const_iterator ahit = activeHandlers.find(receiver);
     if ((ahit != activeHandlers.end()) && (ahit->second.find(Thread::GetThread()) != ahit->second.end())) {
         QCC_LogError(ER_DEADLOCK, ("Attempt to unregister MessageReceiver from said MessageReceiver's message handler -- MessageReceiver not unregistered!"));
+        QCC_ASSERT(!"Attempt to unregister MessageReceiver from said MessageReceiver's message handler");
         handlerThreadsLock.Unlock(MUTEX_CONTEXT);
         return false;
     }
     unregisteringObjects.insert(receiver);
     while (ahit != activeHandlers.end()) {
-        handlerThreadsDone.Wait(handlerThreadsLock);
+        QCC_VERIFY(handlerThreadsDone.Wait(handlerThreadsLock) == ER_OK);
         ahit = activeHandlers.find(receiver);
     }
     handlerThreadsLock.Unlock(MUTEX_CONTEXT);
@@ -1148,7 +1160,6 @@ void _LocalEndpoint::AlarmTriggered(const Alarm& alarm, QStatus reason)
     }
     uint32_t serial = rc->serial;
     Message msg(*bus);
-    QStatus status = ER_OK;
 
     /*
      * Clear the encrypted flag so the error response doesn't get rejected.
@@ -1156,7 +1167,10 @@ void _LocalEndpoint::AlarmTriggered(const Alarm& alarm, QStatus reason)
     rc->callFlags &= ~ALLJOYN_FLAG_ENCRYPTED;
     replyMapLock.Unlock(MUTEX_CONTEXT);
 
-    if (running) {
+    QStatus status = ER_OK;
+    bool attemptDispatch = running;
+
+    if (attemptDispatch) {
         QCC_DbgPrintf(("Timed out waiting for METHOD_REPLY with serial %d", serial));
         if (reason == ER_TIMER_EXITING) {
             msg->ErrorMsg("org.alljoyn.Bus.Exiting", serial);
@@ -1167,17 +1181,17 @@ void _LocalEndpoint::AlarmTriggered(const Alarm& alarm, QStatus reason)
          * Forward the message via the dispatcher so we conform to our concurrency model.
          */
         status = dispatcher->DispatchMessage(msg);
-
-    } else {
-        msg->ErrorMsg("org.alljoyn.Bus.Exiting", serial);
-        HandleMethodReply(msg);
     }
+
     /*
-     * If the dispatch failed or we are no longer running handle the reply on this thread.
+     * If the dispatch failed or we are no longer running, handle the reply on this thread.
      */
-    if (status != ER_OK) {
+    if ((status != ER_OK) || !attemptDispatch) {
         msg->ErrorMsg("org.alljoyn.Bus.Exiting", serial);
         HandleMethodReply(msg);
+        handlerThreadsLock.Lock(MUTEX_CONTEXT);
+        handlerThreadsDone.Broadcast();
+        handlerThreadsLock.Unlock(MUTEX_CONTEXT);
     }
 }
 
@@ -1222,19 +1236,19 @@ QStatus _LocalEndpoint::HandleMethodCall(Message& message)
     if (status == ER_OK) {
         /* Call the method handler */
         if (entry) {
-            handlerThreadsLock.Lock();
+            handlerThreadsLock.Lock(MUTEX_CONTEXT);
             bool unregistering = unregisteringObjects.find(entry->object) != unregisteringObjects.end();
             if (!unregistering) {
                 activeHandlers[entry->object].insert(Thread::GetThread());
-                handlerThreadsLock.Unlock();
+                handlerThreadsLock.Unlock(MUTEX_CONTEXT);
                 entry->object->CallMethodHandler(entry->handler, entry->member, message, entry->context);
-                handlerThreadsLock.Lock();
+                handlerThreadsLock.Lock(MUTEX_CONTEXT);
                 activeHandlers[entry->object].erase(Thread::GetThread());
                 if (activeHandlers[entry->object].empty()) {
                     activeHandlers.erase(entry->object);
                 }
             }
-            handlerThreadsLock.Unlock();
+            handlerThreadsLock.Unlock(MUTEX_CONTEXT);
         }
     } else if (message->GetType() == MESSAGE_METHOD_CALL && !(message->GetFlags() & ALLJOYN_FLAG_NO_REPLY_EXPECTED)) {
         /* We are rejecting a method call that expects a response so reply with an error message. */
@@ -1279,7 +1293,7 @@ QStatus _LocalEndpoint::HandleMethodCall(Message& message)
             break;
         }
         QStatus result = message->ErrorMsg(message, errStr.c_str(), errMsg.c_str());
-        assert(ER_OK == result);
+        QCC_ASSERT(ER_OK == result);
         QCC_UNUSED(result);
         BusEndpoint busEndpoint = BusEndpoint::wrap(this);
         status = bus->GetInternal().GetRouter().PushMessage(message, busEndpoint);
@@ -1342,19 +1356,19 @@ QStatus _LocalEndpoint::HandleSignal(Message& message)
         list<SignalTable::Entry>::const_iterator callit;
         for (callit = callList.begin(); callit != callList.end(); ++callit) {
             MessageReceiver* object = callit->object;
-            handlerThreadsLock.Lock();
+            handlerThreadsLock.Lock(MUTEX_CONTEXT);
             bool unregistering = unregisteringObjects.find(object) != unregisteringObjects.end();
             if (!unregistering) {
                 activeHandlers[object].insert(Thread::GetThread());
-                handlerThreadsLock.Unlock();
+                handlerThreadsLock.Unlock(MUTEX_CONTEXT);
                 (object->*callit->handler)(callit->member, message->GetObjectPath(), message);
-                handlerThreadsLock.Lock();
+                handlerThreadsLock.Lock(MUTEX_CONTEXT);
                 activeHandlers[object].erase(Thread::GetThread());
                 if (activeHandlers[object].empty()) {
                     activeHandlers.erase(object);
                 }
             }
-            handlerThreadsLock.Unlock();
+            handlerThreadsLock.Unlock(MUTEX_CONTEXT);
         }
     }
     return status;
@@ -1364,9 +1378,9 @@ QStatus _LocalEndpoint::HandleMethodReply(Message& message)
 {
     QStatus status = ER_OK;
 
-    replyMapLock.Lock();
+    replyMapLock.Lock(MUTEX_CONTEXT);
     ReplyContext* rc = RemoveReplyHandler(message->GetReplySerial());
-    replyMapLock.Unlock();
+    replyMapLock.Unlock(MUTEX_CONTEXT);
     if (rc) {
         if ((rc->callFlags & ALLJOYN_FLAG_ENCRYPTED) && !message->IsEncrypted()) {
             /*
@@ -1405,19 +1419,19 @@ QStatus _LocalEndpoint::HandleMethodReply(Message& message)
             status = ER_OK;
         }
 
-        handlerThreadsLock.Lock();
+        handlerThreadsLock.Lock(MUTEX_CONTEXT);
         bool unregistering = unregisteringObjects.find(rc->receiver) != unregisteringObjects.end();
         if (!unregistering) {
             activeHandlers[rc->receiver].insert(Thread::GetThread());
-            handlerThreadsLock.Unlock();
+            handlerThreadsLock.Unlock(MUTEX_CONTEXT);
             ((rc->receiver)->*(rc->handler))(message, rc->context);
-            handlerThreadsLock.Lock();
+            handlerThreadsLock.Lock(MUTEX_CONTEXT);
             activeHandlers[rc->receiver].erase(Thread::GetThread());
             if (activeHandlers[rc->receiver].empty()) {
                 activeHandlers.erase(rc->receiver);
             }
         }
-        handlerThreadsLock.Unlock();
+        handlerThreadsLock.Unlock(MUTEX_CONTEXT);
 
         delete rc;
     } else {
