@@ -32,7 +32,6 @@
 #include <qcc/GUID.h>
 #include <qcc/Event.h>
 #include <qcc/Mutex.h>
-#include <qcc/StringMapKey.h>
 #include <qcc/Timer.h>
 #include <qcc/Util.h>
 
@@ -456,7 +455,8 @@ class _LocalEndpoint : public _BusEndpoint, public qcc::AlarmListener, public Me
      */
     struct Hash {
         inline size_t operator()(const char* s) const {
-            return qcc::hash_string(s);
+            std::hash<std::string> hash_fn;
+            return hash_fn(std::string(s));
         }
     };
 
