@@ -592,11 +592,16 @@ QStatus _Message::UnmarshalArgs(PeerStateTable* peerStateTable,
     }
 
     bool permissionCheckMet = true;
+    bool authenticated = false;
     if (msgHeader.flags & ALLJOYN_FLAG_ENCRYPTED) {
         bool broadcast = (hdrFields.field[ALLJOYN_HDR_FIELD_DESTINATION].typeId == ALLJOYN_INVALID);
         size_t hdrLen = bodyPtr - (uint8_t*)msgBuf;
         PeerState peerState = peerStateTable->GetPeerState(GetSender());
         KeyBlob key;
+        /* A broadcast message is encrypted but not authenticated since any
+         * peer that connects with the sender has access to the same group key
+         */
+        authenticated = !broadcast;
         status = peerState->GetKey(key, broadcast ? PEER_GROUP_KEY : PEER_SESSION_KEY);
         if (status != ER_OK) {
             QCC_LogError(status, ("Unable to decrypt (broadcast %d) message from sender %s", broadcast, GetSender()));
@@ -622,7 +627,7 @@ QStatus _Message::UnmarshalArgs(PeerStateTable* peerStateTable,
             permissionCheckMet = true;
             Message msg(*this);
             /* decrypting a message means receiving */
-            status = bus->GetInternal().GetPermissionManager().AuthorizeMessage(false, msg, peerState);
+            status = bus->GetInternal().GetPermissionManager().AuthorizeMessage(false, msg, peerState, authenticated);
             QCC_DbgHLPrintf(("_Message::UnmarshalArgs decrypt permission authorization returns status 0x%x\n", status));
             if (status != ER_OK) {
                 goto ExitUnmarshalArgs;
@@ -639,7 +644,7 @@ QStatus _Message::UnmarshalArgs(PeerStateTable* peerStateTable,
         } else {
             authVersion = (int32_t)AUTH_FALLBACK_VERSION;
         }
-        assert(0 <= authVersion);
+        QCC_ASSERT(0 <= authVersion);
 
         QCC_DbgHLPrintf(("Decrypting messge from %s", GetSender()));
         /*
@@ -708,7 +713,7 @@ ExitUnmarshalArgs:
             Message msg(*this);
             /* decrypting a message means receiving */
             PeerState peerState = bus->GetInternal().GetPeerStateTable()->GetPeerState(GetSender());
-            status = bus->GetInternal().GetPermissionManager().AuthorizeMessage(false, msg, peerState);
+            status = bus->GetInternal().GetPermissionManager().AuthorizeMessage(false, msg, peerState, authenticated);
             QCC_DbgHLPrintf(("_Message::UnmarshalArgs decrypt permission authorization returns status 0x%x\n", status));
         }
     } else {
@@ -934,7 +939,7 @@ QStatus _Message::InterpretHeader()
      * message reducing the places where we need to check for bufEOD when unmarshaling the body.
      */
     bufSize = sizeof(msgHeader) + ((pktSize + 7) & ~7) + sizeof(uint64_t);
-    assert(_msgBuf == nullptr);
+    QCC_ASSERT(_msgBuf == nullptr);
     _msgBuf = new uint8_t[bufSize + 7];
     msgBuf = (uint64_t*)((uintptr_t)(_msgBuf + 7) & ~7); /* Align to 8 byte boundary */
     /*
@@ -1090,8 +1095,6 @@ QStatus _Message::LoadBytes(uint8_t* buf, size_t buflen)
 
 QStatus _Message::ReadNonBlocking(RemoteEndpoint& endpoint, bool checkSender, bool pedantic)
 {
-
-
     QStatus status = ER_OK;
     while ((status == ER_OK) && (readState != MESSAGE_COMPLETE)) {
         status = PullBytes(endpoint, checkSender, pedantic, 0); /* timeout zero */
@@ -1144,13 +1147,18 @@ QStatus _Message::Unmarshal(qcc::String& endpointName, bool handlePassing, bool
 {
     QCC_UNUSED(timeout);
 
-    QStatus status;
-
-    uint8_t* endOfHdr;
-    MsgArg* senderField = &hdrFields.field[ALLJOYN_HDR_FIELD_SENDER];
     if (!bus->IsStarted()) {
         return ER_BUS_BUS_NOT_STARTED;
     }
+
+    return Unmarshal(bus->GetInternal().GetPeerStateTable(), endpointName, handlePassing, checkSender, pedantic);
+}
+
+QStatus _Message::Unmarshal(PeerStateTable* peerStateTable, qcc::String& endpointName, bool handlePassing, bool checkSender, bool pedantic)
+{
+    QStatus status;
+    uint8_t* endOfHdr;
+    MsgArg* senderField = &hdrFields.field[ALLJOYN_HDR_FIELD_SENDER];
     bufPos = (uint8_t*)msgBuf + sizeof(msgHeader);
     endOfHdr = bufPos + msgHeader.headerLen;
     rcvEndpointName = endpointName;
@@ -1264,8 +1272,7 @@ QStatus _Message::Unmarshal(qcc::String& endpointName, bool handlePassing, bool
      * session.
      */
     if (senderField->typeId != ALLJOYN_INVALID) {
-        PeerState peerState = bus->GetInternal().GetPeerStateTable()->GetPeerState(senderField->v_string.str,
-                                                                                   (msgHeader.flags & ALLJOYN_FLAG_SESSIONLESS) == 0);
+        PeerState peerState = peerStateTable->GetPeerState(senderField->v_string.str, (msgHeader.flags & ALLJOYN_FLAG_SESSIONLESS) == 0);
         bool unreliable = hdrFields.field[ALLJOYN_HDR_FIELD_TIME_TO_LIVE].typeId != ALLJOYN_INVALID;
         bool secure = (msgHeader.flags & ALLJOYN_FLAG_ENCRYPTED) != 0;
         if ((msgHeader.flags & ALLJOYN_FLAG_SESSIONLESS) == 0) {
