@@ -28,8 +28,8 @@
 #include <qcc/platform.h>
 
 #include <map>
+#include <unordered_map>
 #include <limits>
-#include <assert.h>
 
 #include <alljoyn/Message.h>
 
@@ -42,12 +42,42 @@
 #include <qcc/time.h>
 #include <qcc/CertificateECC.h>
 #include <qcc/Crypto.h>
+#include <qcc/LockLevel.h>
 
 #include <alljoyn/Status.h>
 #include <alljoyn/PermissionPolicy.h>
 
+#include "ConversationHash.h"
+
 namespace ajn {
 
+/* the key exchange is in the 16 MSB.
+   The PIN-based key exchange mechanism was removed in 15.04:
+        AUTH_KEYX_PIN           0x00040000
+   The RSA-based auth mechanism was removed in 15.04:
+        AUTH_KEYX_RSA           0x00200000
+ */
+#define AUTH_KEYX_ANONYMOUS     0x00010000
+#define AUTH_KEYX_EXTERNAL      0x00020000
+#define AUTH_KEYX_SRP           0x00080000
+#define AUTH_KEYX_SRP_LOGON     0x00100000
+#define AUTH_KEYX_ECDHE         0x00400000
+#define AUTH_KEYX_GSSAPI        0x00800000
+
+/*the key authentication suite is in the 16 LSB */
+
+#define AUTH_SUITE_ANONYMOUS    AUTH_KEYX_ANONYMOUS
+#define AUTH_SUITE_EXTERNAL     AUTH_KEYX_EXTERNAL
+#define AUTH_SUITE_SRP_KEYX     AUTH_KEYX_SRP               /* Depreceated, will be removed in a future release. */
+#define AUTH_SUITE_SRP_LOGON    AUTH_KEYX_SRP_LOGON         /* Depreceated, will be removed in a future release. */
+
+#define AUTH_SUITE_ECDHE_NULL   (AUTH_KEYX_ECDHE | 0x0001)
+#define AUTH_SUITE_ECDHE_PSK    (AUTH_KEYX_ECDHE | 0x0002)  /* Depreceated, will be removed in a future release. */
+#define AUTH_SUITE_ECDHE_ECDSA  (AUTH_KEYX_ECDHE | 0x0004)
+#define AUTH_SUITE_ECDHE_SPEKE  (AUTH_KEYX_ECDHE | 0x0008)
+
+#define AUTH_SUITE_GSSAPI       AUTH_KEYX_GSSAPI
+
 /* Conversation hash-related constants */
 
 /*
@@ -90,8 +120,12 @@ class _PeerState {
 
   public:
 
-    PermissionPolicy::Rule* manifest;
-    size_t manifestSize;
+    /**
+     * Key Exchange mode masks
+     */
+    const static uint8_t KEY_EXCHANGE_NONE = 0x00;       /* no key exchange */
+    const static uint8_t KEY_EXCHANGE_INITIATOR = 0x01;  /* in a key exchange as an initiator */
+    const static uint8_t KEY_EXCHANGE_RESPONDER = 0x02;  /* in a key exchange as as responder */
 
     struct GuildMetadata {
         std::vector<qcc::CertificateX509*> certChain;
@@ -116,19 +150,26 @@ class _PeerState {
      * Default constructor
      */
     _PeerState() :
-        manifest(NULL),
-        manifestSize(0),
         guildArgsSentCount(0),
         isLocalPeer(false),
         clockOffset((std::numeric_limits<int32_t>::max)()),
         firstClockAdjust(true),
         lastDriftAdjustTime(0),
-        expectedSerial(0),
         isSecure(false),
         authEvent(NULL),
-        hashUtil(NULL)
+        prevSerial(0),
+        flagWindow(0),
+        initiatorHash(NULL),
+        responderHash(NULL),
+        initiatorHashLock(qcc::LOCK_LEVEL_PEERSTATE_INITIATORHASHLOCK),
+        /* This lock is held while calling into app's KeyStoreListener, that might acquire other locks having an unspecified level */
+        responderHashLock(qcc::LOCK_LEVEL_CHECKING_DISABLED),
+        keyExchangeMode(KEY_EXCHANGE_NONE),
+        m_authSuite(0),
+        m_manifests(),
+        m_manifestsSent(),
+        m_haveExchangedManifests(false)
     {
-        ::memset(window, 0, sizeof(window));
         ::memset(authorizations, 0, sizeof(authorizations));
     }
 
@@ -136,7 +177,7 @@ class _PeerState {
      * Get the (estimated) timestamp for this remote peer converted to local host time. The estimate
      * is updated based on the timestamp recently received.
      *
-     * @param remoteTime  The timestamp received in a message from the remote peer.
+     * @param[in] remoteTime  The timestamp received in a message from the remote peer.
      *
      * @return   The estimated timestamp for the remote peer.
      */
@@ -148,9 +189,9 @@ class _PeerState {
      * have additional checks for replay attacks. Unreliable messages are checked for in-order
      * arrival.
      *
-     * @param serial      The serial number being checked.
-     * @param secure      The message was flagged as secure
-     * @param unreliable  The message is flagged as unreliable.
+     * @param[in] serial      The serial number being checked.
+     * @param[in] secure      The message was flagged as secure
+     * @param[in] unreliable  The message is flagged as unreliable.
      *
      * @return  Returns true if the serial number is valid.
      */
@@ -181,8 +222,8 @@ class _PeerState {
     /**
      * Sets the session key for this peer
      *
-     * @param key        The session key to set.
-     * @param keyType    Indicate if this is the unicast or broadcast key.
+     * @param[in] key        The session key to set.
+     * @param[in] keyType    Indicate if this is the unicast or broadcast key.
      */
     void SetKey(const qcc::KeyBlob& key, PeerKeyType keyType) {
         keys[keyType] = key;
@@ -192,7 +233,7 @@ class _PeerState {
     /**
      * Gets the session key for this peer.
      *
-     * @param key    [out]Returns the session key.
+     * @param[in] key    [out]Returns the session key.
      *
      * @return  - ER_OK if there is a session key set for this peer.
      *          - ER_BUS_KEY_UNAVAILABLE if no session key has been set for this peer.
@@ -219,6 +260,7 @@ class _PeerState {
         keys[PEER_SESSION_KEY].Erase();
         keys[PEER_GROUP_KEY].Erase();
         isSecure = false;
+        m_authSuite = 0;
     }
 
     /**
@@ -242,10 +284,16 @@ class _PeerState {
      * is being authenticated and is used to prevent multiple threads from attempting to
      * simultaneously authenticate the same peer.
      *
-     * @param event  The event to set or NULL if the event is being cleared.
+     * @param[in] event  The event to set or NULL if the event is being cleared.
      */
     void SetAuthEvent(qcc::Event* event) { authEvent = event; }
 
+    /**
+     * Notify all the waiting threads on auth event.
+     */
+
+    void NotifyAuthEvent();
+
     /**
      * Tests if this peer is the local peer.
      *
@@ -258,7 +306,7 @@ class _PeerState {
      *
      * @return Size of the serial number validation window.
      */
-    size_t SerialWindowSize() { return sizeof(window) / sizeof(window[0]); }
+    size_t SerialWindowSize() { return 64; }
 
     static const uint8_t ALLOW_SECURE_TX = 0x01; /* Transmit authorization */
     static const uint8_t ALLOW_SECURE_RX = 0x02; /* Receive authorization */
@@ -267,8 +315,8 @@ class _PeerState {
      * Check if the peer is authorized to send or or receive a message of the specified
      * type.
      *
-     * @param msgType  The type of message that is being authorized.
-     * @param access   The access type being checked
+     * @param[in] msgType  The type of message that is being authorized.
+     * @param[in] access   The access type being checked
      *
      * @return Return true if the message type is authorized.
      */
@@ -283,8 +331,8 @@ class _PeerState {
     /**
      * Set or clear an authorization.
      *
-     * @param msgType  The type of message that is being authorized.
-     * @param access   The access type to authorize, zero to clear.
+     * @param[in] msgType  The type of message that is being authorized.
+     * @param[in] access   The access type to authorize, zero to clear.
      */
     void SetAuthorization(AllJoynMessageType msgType, uint8_t access) {
         if (msgType != MESSAGE_INVALID) {
@@ -299,16 +347,16 @@ class _PeerState {
 
     /**
      * Set the guild metadata indexed by the serial number and the issuer.
-     * @param serial the membership certificate serial number
-     * @param issuerAki the membership certificate issuer authority key id
-     * @param guild the guild metadata
+     * @param[in] serial the membership certificate serial number
+     * @param[in] issuerAki the membership certificate issuer authority key id
+     * @param[in] guild the guild metadata
      */
-    void SetGuildMetadata(const qcc::String& serial, const qcc::String& issuerAki,        GuildMetadata* guild);
+    void SetGuildMetadata(const qcc::String& serial, const qcc::String& issuerAki, GuildMetadata* guild);
 
     /**
      * Retrieve the guild metadata indexed by the serial number and the issuer.
-     * @param serial the membership certificate serial number
-     * @param issuerAki the membership certificate issuer authority key id
+     * @param[in] serial the membership certificate serial number
+     * @param[in] issuerAki the membership certificate issuer authority key id
      * @return the guild metadata
      */
     GuildMetadata* GetGuildMetadata(const qcc::String& serial, const qcc::String& issuerAki);
@@ -358,21 +406,23 @@ class _PeerState {
      * InitializeConversationHash must first be called before calling this method.
      * The hash lock must be acquired via AcquireConversationHashLock prior to calling this method.
      * The hash lock must be released via ReleaseConversationHashLock after calling this method.
+     * @param[in] initiator key exchange started as initiator
      * @param[in] conversationVersion The minimum auth version required for this to be included in the hash.
      * @param[in] byte Byte with which to update the hash.
      */
-    void UpdateHash(uint32_t conversationVersion, uint8_t byte);
+    void UpdateHash(bool initiator, uint32_t conversationVersion, uint8_t byte);
 
     /**
      * Update the conversation hash with a byte array.
      * InitializeConversationHash must first be called before calling this method.
      * The hash lock must be acquired via AcquireConversationHashLock prior to calling this method.
      * The hash lock must be released via ReleaseConversationHashLock after calling this method.
+     * @param[in] initiator key exchange started as initiator
      * @param[in] conversationVersion The minimum auth version required for this to be included in the hash.
      * @param[in] buf Data with which to update the hash.
      * @param[in] bufSize Size of buf.
      */
-    void UpdateHash(uint32_t conversationVersion, const uint8_t* buf, size_t bufSize);
+    void UpdateHash(bool initiator, uint32_t conversationVersion, const uint8_t* buf, size_t bufSize);
 
     /**
      * Update the conversation hash with a string. String will be converted to its
@@ -380,10 +430,11 @@ class _PeerState {
      * InitializeConversationHash must first be called before calling this method.
      * The hash lock must be acquired via AcquireConversationHashLock prior to calling this method.
      * The hash lock must be released via ReleaseConversationHashLock after calling this method.
+     * @param[in] initiator key exchange started as initiator
      * @param[in] conversationVersion The minimum auth version required for this to be included in the hash.
      * @param[in] str String with data with which to update the hash.
      */
-    void UpdateHash(uint32_t conversationVersion, const qcc::String& str);
+    void UpdateHash(bool initiator, uint32_t conversationVersion, const qcc::String& str);
 
     /**
      * Update the conversation hash with a Message. This extracts the raw message buffer
@@ -391,44 +442,49 @@ class _PeerState {
      * InitializeConversationHash must first be called before calling this method.
      * The hash lock must be acquired via AcquireConversationHashLock prior to calling this method.
      * The hash lock must be released via ReleaseConversationHashLock after calling this method.
+     * @param[in] initiator key exchange started as initiator
      * @param[in] conversationVersion The minimum auth version required for this to be included in the hash.
      * @param[in] msg A Message object whose arguments will be added to the hash.
      */
-    void UpdateHash(uint32_t conversationVersion, const Message& msg);
+    void UpdateHash(bool initiator, uint32_t conversationVersion, const Message& msg);
 
     /**
      * Initialize the conversation hash to start a new conversation. Any previous
      * conversation hash is lost. This must be called before any calls to UpdateHash or GetDigest.
      * The hash lock must be acquired via AcquireConversationHashLock prior to calling this method.
      * The hash lock must be released via ReleaseConversationHashLock after calling this method.
+     * @param[in] initiator key exchange started as initiator
      */
-    void InitializeConversationHash();
+    void InitializeConversationHash(bool initiator);
 
     /**
      * Returns true if the conversation hash has been initalized, false otherwise.
+     * @param[in] initiator key exchange started as initiator
      * @see void InitializeConversationHash()
      */
-    bool IsConversationHashInitialized();
+    bool IsConversationHashInitialized(bool initiator);
 
     /**
      * Free the conversation hash when it's no longer needed. After this, any new calls
      * to UpdateHash or GetDigest must be preceded by a call to InitializeConversationHash.
      * The hash lock must be acquired via AcquireConversationHashLock prior to calling this method.
      * The hash lock must be released via ReleaseConversationHashLock after calling this method.
+     * @param[in] initiator key exchange started as initiator
      * @see void InitializeConversationHash()
      */
-    void FreeConversationHash();
+    void FreeConversationHash(bool initiator);
 
     /**
      * Get the current conversation hash digest.
      * InitializeConversationHash must first be called before calling this method.
      * The hash lock must be acquired via AcquireConversationHashLock prior to calling this method.
      * The hash lock must be released via ReleaseConversationHashLock after calling this method.
+     * @param[in] initiator key exchange started as initiator
      * @param[out] digest A buffer of appropriate size to receive the digest. Currently
      *                    only SHA-256 is used, and so 32 bytes will be returned.
      * @param[in] keepAlive Whether or not to keep the hash alive for continuing hash.
      */
-    void GetDigest(uint8_t* digest, bool keepAlive = false);
+    void GetDigest(bool initiator, uint8_t* digest, bool keepAlive = false);
 
     /**
      * Acquire the lock to conversation hash with the intention to call
@@ -438,15 +494,152 @@ class _PeerState {
      * ReplyMethod wishes to aquire an early lock to the conversation hash to
      * prevent any other thread from updating the conversation hash while it
      * is waiting for the MethodRely() to return.
+     * @param[in] initiator key exchange started as initiator
      */
 
-    void AcquireConversationHashLock();
+    void AcquireConversationHashLock(bool initiator);
 
     /**
      * release the conversation hash lock.
+     * @param[in] initiator key exchange started as initiator
      */
 
-    void ReleaseConversationHashLock();
+    void ReleaseConversationHashLock(bool initiator);
+
+    /**
+     * Enable or disable "sensitive mode," where byte arrays that get hashed aren't
+     * logged verbatim. When enabled, calling the overload of Update that takes a byte array will
+     * log the size of the data, but then log secret data was hashed without showing the data.
+     *
+     * Logging for Update for a single byte or GetDigest is unaffected.
+     * @param[in] initiator key exchange started as initiator
+     * @param[in] mode true to enable sensitive mode; false to disable it.
+     */
+    void SetConversationHashSensitiveMode(bool initiator, bool mode);
+
+    /**
+     * Get the key exchange mode.  A peer can be in a key exchange as an initiator or a responder or both.
+     * @return the key exchange mode
+     */
+    uint8_t GetKeyExchangeMode() const;
+
+    /**
+     * Set the key exchange mode. A peer can be in a key exchange as an
+     * initiator or a responder or both.
+     * @param[in] mode the key exchange mode.
+     */
+    void SetKeyExchangeMode(uint8_t mode);
+
+    /**
+     * Enable the key exchange mode mask. A peer can be in a key exchange as
+     * an initiator or a responder or both.
+     * @param[in] mask the mask to enable in the key exchange mode.
+     */
+    void AddKeyExchangeModeMask(uint8_t mask);
+
+    /**
+     * Clear the key exchange mode mask. A peer can be in a key exchange as
+     * an initiator or a responder or both.
+     * @param[in] mask the mask to clear in the key exchange mode.
+     */
+    void ClearKeyExchangeModeMask(uint8_t mask);
+
+    /**
+     * Is the peer in this key exchange mode?
+     * @param[in] mask the key exchange code mask
+     * @return true if so; false, otherwise.
+     */
+    bool IsInKeyExchangeMode(uint8_t mask) const;
+
+    /**
+     * Get the auth suite used to authenticate this peer.
+     * @return One of the AUTH_SUITE_* values, or zero if none was used.
+     */
+    uint32_t GetAuthSuite() const;
+
+    /**
+     * Set the auth suite used to authenticate this peer as a uint32_t.
+     * @param[in] authSuite An AUTH_SUITE_* value reflecting the suite used to authenticate this peer.
+     * @return #ER_OK if the auth suite was successfully set.
+     *         #ER_BAD_ARG_1 if authSuite does not correspond to a valid auth suite.
+     */
+    QStatus SetAuthSuite(uint32_t authSuite);
+
+    /**
+     * Set the auth suite used to authenticate this peer as a string.
+     * @param[in] authSuite A string describing an auth suite compliant with the authMechanisms
+     *                      argument to BusAttachment::EnablePeerSecurity.
+     * @return #ER_OK if the auth suite was successfully set.
+     *         #ER_BAD_ARG_1 if authSuite does not parse as a valid suite.
+     * @see BusAttachment::EnablePeerSecurity
+     */
+    QStatus SetAuthSuite(const qcc::String& authSuite);
+
+    /**
+     * Store a manifest received from the remote peer.
+     * This manifest should be cryptographically verified before calling this.
+     *
+     * @param[in] manifest The verified manifest to be associated with this peer.
+     * @return - #ER_OK if the manifest was successfully added
+     *         - error code otherwise
+     */
+    QStatus StoreManifest(const Manifest& manifest);
+
+    /**
+     * Clear out all previously-received manifests from this peer.
+     *
+     * @return - #ER_OK if the manifest list was successfully cleared
+     *         - error code otherwise
+     */
+    QStatus ClearManifests();
+
+    /**
+     * Get a vector of the peer's verified manifests.
+     *
+     * @return Vector of Manifest objects
+     */
+    const std::vector<Manifest> GetManifests() const;
+
+    /**
+     * Store a manifest we sent to the remote peer.
+     *
+     * @param[in] manifest The manifest we sent to this peer.
+     * @return - #ER_OK if the manifest was successfully added
+     *         - error code otherwise
+     */
+    QStatus StoreSentManifest(const Manifest& manifest);
+
+    /**
+     * Determine if a manifest has been sent to the remote peer.
+     *
+     * @param[in] manifest The manifest to check for.
+     * @return true if the manifest has been sent to this peer, false if not.
+     */
+    bool IsManifestSent(const Manifest& manifest) const;
+
+    /**
+     * Clear out all previously-sent manifests to this peer.
+     *
+     * @return - #ER_OK if the sent manifest list was successfully cleared
+     *         - error code otherwise
+     */
+    QStatus ClearSentManifests();
+
+    /**
+     * Get the field that indicates if we have exchanged manifests with
+     * this peer before.
+     *
+     * @return true if a manifest exchange has happened at least once, false otherwise
+     */
+    bool GetHaveExchangedManifests() const;
+
+    /**
+     * Set the field that indicates if we have exchanged manifests with
+     * this peer before.
+     *
+     * @param[in] haveExchangedManifests  The value to set
+     */
+    void SetHaveExchangedManifests(bool haveExchangedManifests);
 
     /*
      * Destructor
@@ -464,6 +657,18 @@ class _PeerState {
      */
     _PeerState(const _PeerState& other);
 
+    /**
+     * Get the conversation hash
+     * @param[in] initiator key exchange started as initiator
+     */
+    ConversationHash* GetConversationHash(bool initiator) const;
+
+    /**
+     * Get the conversation hash lock
+     * @param[in] initiator key exchange started as initiator
+     */
+    qcc::Mutex& GetConversationHashLock(bool initiator);
+
     /**
      * True if this peer state is for the local peer.
      */
@@ -485,11 +690,6 @@ class _PeerState {
      */
     uint32_t lastDriftAdjustTime;
 
-    /**
-     * The next serial number expected.
-     */
-    uint32_t expectedSerial;
-
     /**
      * Set to true if this peer has keys.
      */
@@ -521,20 +721,66 @@ class _PeerState {
     qcc::KeyBlob keys[2];
 
     /**
-     * Serial number window. Used by IsValidSerial() to detect replay attacks. The size of the
-     * window defines that largest tolerable gap between consecutive serial numbers.
+     * The previous serial number seen from this peer.
+     * Used by IsValidSerial() to detect replay attacks.
      */
-    uint32_t window[128];
+    uint32_t prevSerial;
 
     /**
-     * The conversation hash.
+     * This contains flags indicating if the serial number was received
+     * for each of the last 64 serial numbers from this peer.
+     * Bit 0 indicates whether the prevSerial number was seen from this peer.
+     * i.e. value of 0 indicates no messages were received from this peer.
+     * Bit 1 indicates whether prevSerial - 1 was received from this peer and so on.
+     * Used by IsValidSerial() to detect replay attacks.
      */
-    qcc::Crypto_Hash* hashUtil;
+    uint64_t flagWindow;
 
     /**
-     * Mutex to protect the conversation hash
+     * The initiator conversation hash.
      */
-    qcc::Mutex hashLock;
+    ConversationHash* initiatorHash;
+
+    /**
+     * The responder conversation hash.
+     */
+    ConversationHash* responderHash;
+
+    /**
+     * Mutex to protect the initiator conversation hash
+     */
+    qcc::Mutex initiatorHashLock;
+
+    /**
+     * Mutex to protect the responder conversation hash
+     */
+    qcc::Mutex responderHashLock;
+
+    /**
+     * The key exchange mode
+     */
+    uint8_t keyExchangeMode;
+
+    /**
+     * The auth suite used to authenticate this peer.
+     */
+    uint32_t m_authSuite;
+
+    /**
+     * Manifests received from the peer.
+     */
+    std::vector<Manifest> m_manifests;
+
+    /**
+     * Manifests we've sent to this peer.
+     */
+    std::unordered_map<std::string, Manifest> m_manifestsSent;
+
+    /**
+     * Have we exchanged manifests with this peer at least once?
+     */
+    bool m_haveExchangedManifests;
+
 };
 
 
@@ -553,8 +799,8 @@ class PeerStateTable {
     /**
      * Get the peer state for given a bus name.
      *
-     * @param busName         The bus name for a remote connection
-     * @param createIfUnknown true to create a PeerState if the peer is unknown
+     * @param[in] busName         The bus name for a remote connection
+     * @param[in] createIfUnknown true to create a PeerState if the peer is unknown
      *
      * @return  The peer state.
      */
@@ -563,7 +809,7 @@ class PeerStateTable {
     /**
      * Fnd out if the bus name is for a known peer.
      *
-     * @param busName   The bus name for a remote connection
+     * @param[in] busName   The bus name for a remote connection
      *
      * @return  Returns true if the peer is known.
      */
@@ -577,8 +823,8 @@ class PeerStateTable {
     /**
      * Get the peer state looking the peer state up by a unique name or a known alias for the peer.
      *
-     * @param uniqueName  The bus name for a remote connection
-     * @param aliasName   An alias bus name for a remote connection
+     * @param[in] uniqueName  The bus name for a remote connection
+     * @param[in] aliasName   An alias bus name for a remote connection
      *
      * @return  The peer state.
      */
@@ -587,8 +833,8 @@ class PeerStateTable {
     /**
      * Are two bus names known to refer to the same peer.
      *
-     * @param name1  The first bus name
-     * @param name1  The second bus name
+     * @param[in] name1  The first bus name
+     * @param[in] name2  The second bus name
      *
      * @return  Returns true if the two bus names are known to refer to the same peer.
      */
@@ -599,7 +845,7 @@ class PeerStateTable {
     /**
      * Delete peer state for a busName that is no longer in use
      *
-     * @param busName  The bus name that may was been previously associated with peer state.
+     * @param[in] busName  The bus name that may was been previously associated with peer state.
      */
     void DelPeerState(const qcc::String& busName);
 
@@ -607,7 +853,7 @@ class PeerStateTable {
      * Gets the group (broadcast) key for the local peer. This is used to encrypt
      * broadcast messages sent by this peer.
      *
-     * @param key   [out]Returns the broadcast key.
+     * @param[in] key   [out]Returns the broadcast key.
      */
     void GetGroupKey(qcc::KeyBlob& key);
 
