@@ -115,6 +115,7 @@ static bool IsActionAllowed(uint8_t allowedActions, uint8_t requestedAction)
 
 static bool IsRuleMatched(const PermissionPolicy::Rule& rule, const Request& request, uint8_t requiredAuth, bool scanForDenied, bool& denied, bool strictGetAllProperties)
 {
+    QCC_DbgTrace(("%s: Checking match against rule:\n%s", __FUNCTION__, rule.ToString().c_str()));
     if (rule.GetMembersSize() == 0) {
         return false;
     }
@@ -233,11 +234,15 @@ static bool IsPolicyAclMatched(const PermissionPolicy::Acl& acl, const Request&
     bool strictGetAllProperties = request.outgoing;
     const PermissionPolicy::Rule* rules = acl.GetRules();
     bool allowed = false;
-    for (size_t cnt = 0; cnt < acl.GetRulesSize(); cnt++) {
+    size_t rulesSize = acl.GetRulesSize();
+    QCC_DbgTrace(("%s: Checking if request matches against %u rules.", __FUNCTION__, rulesSize));
+    for (size_t cnt = 0; cnt < rulesSize; cnt++) {
         if (IsRuleMatched(rules[cnt], request, requiredAuth, scanForDenied, denied, strictGetAllProperties)) {
+            QCC_DbgTrace(("%s: Match found, rule allows access. Continuing search for explicit deny.", __FUNCTION__));
             allowed = true; /* track it */
         } else if (denied) {
             /* skip the remainder of the search */
+            QCC_DbgTrace(("%s: Match found, rule denies access. Stopping search.", __FUNCTION__));
             return allowed;
         }
     }
@@ -293,34 +298,49 @@ static void GenRight(const Request& request, Right& right)
 }
 
 /**
- * Enforce the peer's manifest
+ * Enforce the peer's manifests.
+ *
+ * If one manifest allows the access and no manifest explicitly denies it, access is allowed.
+ * If any manifest denies the access, access is denied, no matter what.
  */
-
 static bool IsAuthorizedByPeerManifest(const Request& request, const Right& right, PeerState& peerState)
 {
-    /* no manifest then default not allowed */
-    if ((peerState->manifest == NULL) || (peerState->manifestSize == 0)) {
-        return false;
-    }
-
-    bool strictGetAllProperties = request.outgoing;
-    for (size_t cnt = 0; cnt < peerState->manifestSize; cnt++) {
-        /* validate the peer manifest to make sure it was granted the same thing */
-        bool denied = false;
-        if (IsRuleMatched(peerState->manifest[cnt], request, right.authByPolicy, false, denied, strictGetAllProperties)) {
-            return true;
-        } else if (denied) {
-            /* skip the remainder of the search */
-            return false;
+    QCC_DbgTrace(("%s", __FUNCTION__));
+    QCC_DbgTrace(("request: outgoing %s, propertyRequest %s, isSetProperty %s, objPath %s, iName %s, mbrName %s",
+                  (request.outgoing) ? "true" : "false",
+                  (request.propertyRequest) ? "true" : "false",
+                  (request.isSetProperty) ? "true" : "false",
+                  request.objPath, request.iName, request.mbrName));
+    QCC_DbgTrace(("right.authByPolicy %u, peer GUID %s", right.authByPolicy, peerState->GetGuid().ToString().c_str()));
+    bool allowed = false;
+    for (Manifest peerManifest : peerState->GetManifests()) {
+        bool strictGetAllProperties = request.outgoing;
+        for (size_t cnt = 0; cnt < peerManifest->GetRules().size(); cnt++) {
+            /* validate the peer manifest to make sure it was granted the same thing */
+            bool denied = false;
+            if (IsRuleMatched(peerManifest->GetRules()[cnt], request, right.authByPolicy, false, denied, strictGetAllProperties)) {
+                /* One manifest allows it. Note this for now, but keep looking for any manifests with explicit denials. */
+                QCC_DbgTrace(("Request allowed by manifest"));
+                allowed = true;
+            } else if (denied) {
+                QCC_DbgTrace(("Request specifically denied by manifest"));
+                return false;
+            }
         }
     }
-    return false;
+
+    if (!allowed) {
+        QCC_DbgTrace(("Request was not authorized by any manifest rules"));
+    }
+
+    return allowed;
 }
 
 static bool IsPeerQualifiedForAcl(const PermissionPolicy::Acl& acl, PeerState& peerState, bool trustedPeer, const qcc::ECCPublicKey* peerPublicKey, const std::vector<ECCPublicKey>& issuerChain, bool& qualifiedPeerWithPublicKey)
 {
     qualifiedPeerWithPublicKey = false;
     const PermissionPolicy::Peer* peers = acl.GetPeers();
+    QCC_DbgTrace(("%s: Checking if peer is qualified for ACL:\n%s", __FUNCTION__, acl.ToString().c_str()));
     for (size_t idx = 0; idx < acl.GetPeersSize(); idx++) {
         if (peers[idx].GetType() == PermissionPolicy::Peer::PEER_ALL) {
             return true;
@@ -341,13 +361,16 @@ static bool IsPeerQualifiedForAcl(const PermissionPolicy::Acl& acl, PeerState& p
             }
         }
         if ((peers[idx].GetType() == PermissionPolicy::Peer::PEER_FROM_CERTIFICATE_AUTHORITY) && peers[idx].GetKeyInfo()) {
+            QCC_DbgTrace(("%s: Checking peer's issuer chain (size: %u).", __FUNCTION__, issuerChain.size()));
             for (std::vector<ECCPublicKey>::const_iterator it = issuerChain.begin(); it != issuerChain.end(); it++) {
+                QCC_DbgTrace(("%s: Validating against peer's issuer public key: %s", __FUNCTION__, it->ToString().c_str()));
                 if (*peers[idx].GetKeyInfo()->GetPublicKey() == *it) {
                     return true;
                 }
             }
         }
         if (peers[idx].GetType() == PermissionPolicy::Peer::PEER_WITH_MEMBERSHIP) {
+            QCC_DbgTrace(("%s: Checking peer's memberships (certificates size: %u).", __FUNCTION__, peerState->guildMap.size()));
             for (_PeerState::GuildMap::iterator it = peerState->guildMap.begin(); it != peerState->guildMap.end(); it++) {
                 _PeerState::GuildMetadata* metadata = it->second;
                 if (metadata->certChain.size() > 0) {
@@ -355,6 +378,7 @@ static bool IsPeerQualifiedForAcl(const PermissionPolicy::Acl& acl, PeerState& p
                         continue;
                     }
                     MembershipCertificate* membershipCert = (MembershipCertificate*) metadata->certChain[0];
+                    QCC_DbgTrace(("%s: Membership certificate found:\n%s", __FUNCTION__, membershipCert->ToString().c_str()));
                     if (peers[idx].GetSecurityGroupId() == membershipCert->GetGuild()) {
                         return true;
                     }
@@ -378,10 +402,16 @@ static bool IsPeerAuthorized(const Request& request, const PermissionPolicy* pol
     denied = false;
     bool qualifiedPeerWithPublicKey = false;
     const PermissionPolicy::Acl* acls = policy->GetAcls();
-    for (size_t cnt = 0; cnt < policy->GetAclsSize(); cnt++) {
+    size_t aclsSize = policy->GetAclsSize();
+    QCC_DbgTrace(("%s: Authorizing peer (public key: %s) against %u ACLs", __FUNCTION__,
+                  nullptr == peerPublicKey ? "null" : peerPublicKey->ToString().c_str(),
+                  aclsSize));
+    for (size_t cnt = 0; cnt < aclsSize; cnt++) {
         if (!IsPeerQualifiedForAcl(acls[cnt], peerState, trustedPeer, peerPublicKey, issuerChain, qualifiedPeerWithPublicKey)) {
+            QCC_DbgTrace(("%s: Peer did not qualify for ACL numer %u.", __FUNCTION__, cnt));
             continue;
         }
+        QCC_DbgTrace(("%s: Peer qualified for ACL number %u.", __FUNCTION__, cnt));
         if (IsPolicyAclMatched(acls[cnt], request, requiredAuth, qualifiedPeerWithPublicKey, denied)) {
             allowed = true;   /* track it */
         }
@@ -402,7 +432,7 @@ static bool IsPeerAuthorized(const Request& request, const PermissionPolicy* pol
  * 5. all peers
  */
 
-static bool IsAuthorized(const Request& request, const PermissionPolicy* policy, PeerState& peerState, PermissionMgmtObj* permissionMgmtObj)
+static bool IsAuthorized(const Request& request, const PermissionPolicy* policy, PeerState& peerState, PermissionMgmtObj* permissionMgmtObj, bool authenticated = true)
 {
     Right right;
     GenRight(request, right);
@@ -411,7 +441,7 @@ static bool IsAuthorized(const Request& request, const PermissionPolicy* policy,
     bool denied = false;
     bool enforceManifest = true;
 
-    QCC_DbgPrintf(("IsAuthorized with required permission %d\n", right.authByPolicy));
+    QCC_DbgPrintf(("IsAuthorized with required permission %d, iName %s, mbrName %s\n", right.authByPolicy, request.iName, request.mbrName));
 
     if (right.authByPolicy) {
         if (policy == NULL) {
@@ -425,26 +455,33 @@ static bool IsAuthorized(const Request& request, const PermissionPolicy* policy,
         std::vector<ECCPublicKey> issuerPublicKeys;
         const ECCPublicKey* trustedPeerPublicKey = NULL;
         bool trustedPeer = false;
-        if (peerState->IsLocalPeer()) {
-            if (ER_OK == permissionMgmtObj->GetPublicKey(publicKeyInfo)) {
-                trustedPeerPublicKey = publicKeyInfo.GetPublicKey();
-                trustedPeer = true;
-                enforceManifest = false;
-            }
-        } else {
-            bool publicKeyFound = false;
-            qcc::String authMechanism;
-            if (ER_OK == permissionMgmtObj->GetConnectedPeerAuthMetadata(peerState->GetGuid(), authMechanism, publicKeyFound, &peerPublicKey, NULL, issuerPublicKeys)) {
-                /* trusted peer */
-                if (publicKeyFound) {
-                    trustedPeerPublicKey = &peerPublicKey;
-                    trustedPeer = true;
-                } else if ((authMechanism == KeyExchangerECDHE_PSK::AuthName()) ||
-                           (authMechanism == AuthMechSRP::AuthName()) ||
-                           (authMechanism == AuthMechLogon::AuthName())) {
+        if (authenticated) {
+            if (peerState->IsLocalPeer()) {
+                if (ER_OK == permissionMgmtObj->GetPublicKey(publicKeyInfo)) {
+                    trustedPeerPublicKey = publicKeyInfo.GetPublicKey();
                     trustedPeer = true;
                     enforceManifest = false;
-                } else {
+                }
+            } else {
+                bool publicKeyFound = false;
+                qcc::String authMechanism;
+                QStatus status = permissionMgmtObj->GetConnectedPeerAuthMetadata(peerState->GetGuid(), authMechanism, publicKeyFound, &peerPublicKey, NULL, issuerPublicKeys);
+                if (ER_OK == status) {
+                    /* trusted peer */
+                    if (publicKeyFound) {
+                        trustedPeerPublicKey = &peerPublicKey;
+                        trustedPeer = true;
+                    } else if ((authMechanism == KeyExchangerECDHE_PSK::AuthName()) ||
+                               (authMechanism == AuthMechSRP::AuthName()) ||
+                               (authMechanism == AuthMechLogon::AuthName())) {
+                        trustedPeer = true;
+                        enforceManifest = false;
+                    } else {
+                        enforceManifest = false;
+                    }
+                } else if (ER_BUS_KEY_UNAVAILABLE == status) {
+                    /* assuming the peer secret just expires so it is not a trusted
+                     * peer */
                     enforceManifest = false;
                 }
             }
@@ -461,7 +498,7 @@ static bool IsAuthorized(const Request& request, const PermissionPolicy* policy,
             }
         }
 #endif
-        QCC_DbgPrintf(("Peer's public key: %s Authorized: %d Denied: %d", trustedPeerPublicKey ? trustedPeerPublicKey->ToString().c_str() : "N/A", authorized, denied));
+        QCC_DbgPrintf(("Peer's trusted peer: %d public key: %s Authorized: %d Denied: %d Manifest required: %d", trustedPeer, trustedPeerPublicKey ? trustedPeerPublicKey->ToString().c_str() : "N/A", authorized, denied, enforceManifest));
         if (denied || !authorized) {
             return false;
         }
@@ -582,7 +619,17 @@ static QStatus ParsePropertiesMessage(Request& request, Message& msg)
         request.isSetProperty = (strncmp(mbrName, "Set", 3) == 0);
         QCC_DbgPrintf(("PermissionManager::ParsePropertiesMessage %s %s.%s", mbrName, propIName, propName));
     } else if (strncmp(mbrName, "PropertiesChanged", 17) == 0) {
-        status = msg->GetArg(0)->Get("s", &propIName);
+        const MsgArg* args;
+        size_t numArgs;
+        if (request.outgoing) {
+            msg->GetRefArgs(numArgs, args);
+        } else {
+            msg->GetArgs(numArgs, args);
+        }
+        if (numArgs < 1) {
+            return ER_INVALID_DATA;
+        }
+        status = args[0].Get("s", &propIName);
         if (status != ER_OK) {
             return status;
         }
@@ -597,7 +644,34 @@ static QStatus ParsePropertiesMessage(Request& request, Message& msg)
     return ER_OK;
 }
 
-bool PermissionManager::AuthorizePermissionMgmt(bool outgoing, const char* iName, const char* mbrName, bool& authorized)
+QStatus PermissionManager::ParsePropertiesMessageHeaders(Message& msg, String& interfaceName, String& memberName)
+{
+    Request request(msg, true);
+
+    QStatus status = ParsePropertiesMessage(request, msg);
+    if (ER_OK != status) {
+        return status;
+    }
+    QCC_ASSERT(request.propertyRequest);
+
+    if (nullptr == request.iName) {
+        interfaceName.clear();
+    } else {
+        interfaceName.assign(request.iName);
+    }
+
+    if (nullptr == request.mbrName) {
+        memberName.clear();
+    } else {
+        memberName.assign(request.mbrName);
+    }
+
+    return ER_OK;
+}
+
+
+
+bool PermissionManager::AuthorizePermissionMgmt(bool outgoing, const char* iName, const char* mbrName, bool& authorized, PeerState& peerState)
 {
     authorized = false;
     if (outgoing) {
@@ -616,6 +690,41 @@ bool PermissionManager::AuthorizePermissionMgmt(bool outgoing, const char* iName
         if (strncmp(mbrName, "Claim", 5) == 0) {
             /* only allowed when there is no trust anchor */
             authorized = (!permissionMgmtObj->HasTrustAnchors());
+            /* If this isn't a self-claim, make sure the auth mechanism used is acceptable per
+             * the claim capabilities.
+             */
+            if (authorized && !peerState->IsLocalPeer()) {
+                PermissionConfigurator::ClaimCapabilities capabilities;
+                QStatus status = permissionMgmtObj->GetClaimCapabilities(capabilities);
+                if (ER_OK != status) {
+                    QCC_LogError(status, ("Could not query our claim capabilities"));
+                    authorized = false;
+                } else {
+                    switch (peerState->GetAuthSuite()) {
+                    case AUTH_SUITE_ECDHE_NULL:
+                        authorized = ((capabilities& PermissionConfigurator::CAPABLE_ECDHE_NULL) == PermissionConfigurator::CAPABLE_ECDHE_NULL);
+                        break;
+
+                    case AUTH_SUITE_ECDHE_PSK:
+                        authorized = ((capabilities& PermissionConfigurator::CAPABLE_ECDHE_PSK) == PermissionConfigurator::CAPABLE_ECDHE_PSK);
+                        break;
+
+                    case AUTH_SUITE_ECDHE_SPEKE:
+                        authorized = ((capabilities& PermissionConfigurator::CAPABLE_ECDHE_SPEKE) == PermissionConfigurator::CAPABLE_ECDHE_SPEKE);
+                        break;
+
+                    case AUTH_SUITE_ECDHE_ECDSA:
+                        authorized = ((capabilities& PermissionConfigurator::CAPABLE_ECDHE_ECDSA) == PermissionConfigurator::CAPABLE_ECDHE_ECDSA);
+                        break;
+
+                    default:
+                        /* No other suites supported for claiming. */
+                        QCC_DbgPrintf(("%s Claiming is not supported with this suite (%#x)", __FUNCTION__, peerState->GetAuthSuite()));
+                        authorized = false;
+                        break;
+                    }
+                }
+            }
             return true;  /* handled */
         }
     } else if (strcmp(iName, org::alljoyn::Bus::Security::ManagedApplication::InterfaceName) == 0) {
@@ -654,7 +763,7 @@ bool PermissionManager::AuthorizePermissionMgmt(bool outgoing, const char* iName
     return false;  /* not handled */
 }
 
-QStatus PermissionManager::AuthorizeMessage(bool outgoing, Message& msg, PeerState& peerState)
+QStatus PermissionManager::AuthorizeMessage(bool outgoing, Message& msg, PeerState& peerState, bool authenticated)
 {
     QStatus status = ER_PERMISSION_DENIED;
     bool authorized = false;
@@ -680,15 +789,17 @@ QStatus PermissionManager::AuthorizeMessage(bool outgoing, Message& msg, PeerSta
         request.mbrName = msg->GetMemberName();
     }
     if (!permissionMgmtObj) {
+        QCC_DbgPrintf(("%s No permission management object", __FUNCTION__));
         return ER_PERMISSION_DENIED;
     }
     bool isPermissionMgmt = IsPermissionMgmtInterface(request.iName);
     if (isPermissionMgmt) {
         bool allowed = false;
-        if (AuthorizePermissionMgmt(outgoing, request.iName, request.mbrName, allowed)) {
+        if (AuthorizePermissionMgmt(outgoing, request.iName, request.mbrName, allowed, peerState)) {
             if (allowed) {
                 return ER_OK;
             } else {
+                QCC_DbgPrintf(("%s AuthorizePermissionMgmt check failed", __FUNCTION__));
                 return ER_PERMISSION_DENIED;
             }
         }
@@ -705,7 +816,7 @@ QStatus PermissionManager::AuthorizeMessage(bool outgoing, Message& msg, PeerSta
     QCC_DbgPrintf(("PermissionManager::AuthorizeMessage with outgoing: %d msg %s", outgoing, msg->ToString().c_str()));
     QCC_DbgPrintf(("PermissionManager::AuthorizeMessage: local policy %s", GetPolicy() ? GetPolicy()->ToString().c_str() : "NULL"));
 
-    authorized = IsAuthorized(request, GetPolicy(), peerState, permissionMgmtObj);
+    authorized = IsAuthorized(request, GetPolicy(), peerState, permissionMgmtObj, authenticated);
     if (!authorized) {
         QCC_DbgPrintf(("PermissionManager::AuthorizeMessage IsAuthorized returns ER_PERMISSION_DENIED\n"));
         return ER_PERMISSION_DENIED;
