@@ -26,6 +26,8 @@
 #error Only include PermissionMgmtObj.h in C++ code.
 #endif
 
+#include <memory>
+
 #include <alljoyn/BusObject.h>
 #include <alljoyn/AllJoynStd.h>
 #include <alljoyn/BusAttachment.h>
@@ -33,11 +35,13 @@
 #include <qcc/KeyBlob.h>
 #include <qcc/CryptoECC.h>
 #include <qcc/CertificateECC.h>
+#include <qcc/LockLevel.h>
 #include <alljoyn/PermissionPolicy.h>
 #include "CredentialAccessor.h"
 #include "ProtectedAuthListener.h"
 #include "PeerState.h"
 #include "KeyStore.h"
+#include "BusUtil.h"
 
 namespace ajn {
 
@@ -106,6 +110,25 @@ class PermissionMgmtObj : public BusObject {
      */
     static const char* ERROR_CERTIFICATE_NOT_FOUND;
 
+    /**
+     * Error name Management Already Started.  Error raised when the app being
+     * managed detects that Security Manager called StartManagement two times,
+     * without calling EndManagement in between these two calls. This error
+     * typically means that the first management session has been interrupted
+     * abruptly - e.g. by losing network connectivity between the Security Manager
+     * and the app being managed during the management session.
+     */
+    static const char* ERROR_MANAGEMENT_ALREADY_STARTED;
+
+    /**
+     * Error name Management Not Started.  Error raised when the app being
+     * managed detects that Security Manager called EndManagement without
+     * a matching StartManagement. This error typically means that the previous
+     * management session has been interrupted abruptly - e.g. by restarting
+     * the app being managed during the management session.
+     */
+    static const char* ERROR_MANAGEMENT_NOT_STARTED;
+
     /**
      * For the SendMemberships call, the app sends one cert chain at time since
      * thin client peer may not be able to handle large amount of data.  The app
@@ -176,7 +199,46 @@ class PermissionMgmtObj : public BusObject {
     /**
      * The list of trust anchors
      */
-    typedef std::vector<TrustAnchor*> TrustAnchorList;
+    class TrustAnchorList : public std::vector<std::shared_ptr<TrustAnchor> > {
+      public:
+        TrustAnchorList() : lock(qcc::LOCK_LEVEL_PERMISSIONMGMTOBJ_LOCK)
+        {
+        }
+
+        TrustAnchorList(const TrustAnchorList& other) : std::vector<std::shared_ptr<TrustAnchor> >(other)
+        {
+        }
+
+        TrustAnchorList& operator=(const TrustAnchorList& other)
+        {
+            std::vector<std::shared_ptr<TrustAnchor> >::operator=(other);
+            return *this;
+        }
+
+        QStatus Lock()
+        {
+            return lock.Lock();
+        }
+
+        QStatus Lock(const char* file, uint32_t line)
+        {
+            return lock.Lock(file, line);
+        }
+
+        QStatus Unlock()
+        {
+            return lock.Unlock();
+        }
+
+        QStatus Unlock(const char* file, uint32_t line)
+        {
+            return lock.Unlock(file, line);
+        }
+
+      private:
+        /* Use a member variable instead of inheriting from qcc::Mutex because copying qcc::Mutex objects is not supported */
+        qcc::Mutex lock;
+    };
 
     /**
      * Constructor
@@ -207,7 +269,6 @@ class PermissionMgmtObj : public BusObject {
      *         - ER_OK if successful.
      *         - an error status otherwise.
      */
-
     QStatus GenerateSendMemberships(std::vector<std::vector<MsgArg*> >& args, const qcc::GUID128& remotePeerGuid);
 
     /**
@@ -227,38 +288,28 @@ class PermissionMgmtObj : public BusObject {
 
     /**
      * Parse the message received from the org.alljoyn.bus.Peer.Authentication's
-     * SendManifest method.
+     * SendManifests method.
      * @param msg the message
      * @param peerState the peer state
      * @return ER_OK if successful; otherwise, an error code.
      */
-    QStatus ParseSendManifest(Message& msg, PeerState& peerState);
+    QStatus ParseSendManifests(Message& msg, PeerState& peerState);
 
     /**
      * Is there any trust anchor installed?
      * @return true if there is at least one trust anchors installed; false, otherwise.
      */
-    bool HasTrustAnchors()
-    {
-        return !trustAnchors.empty();
-    }
+    bool HasTrustAnchors();
 
     /**
      * Retrieve the list of trust anchors.
      * @return the list of trust anchors
      */
-
     const TrustAnchorList& GetTrustAnchors()
     {
         return trustAnchors;
     }
 
-    /**
-     * Helper function to release the allocated memory for the trust anchor list.
-     * @param list the list to be clear of allocated memory.
-     */
-    static void ClearTrustAnchorList(TrustAnchorList& list);
-
     /**
      * Help function to store DSA keys in the key store.
      * @param ca the credential accesor object
@@ -266,7 +317,6 @@ class PermissionMgmtObj : public BusObject {
      * @param publicKey the DSA public key
      * @return ER_OK if successful; otherwise, error code.
      */
-
     static QStatus StoreDSAKeys(CredentialAccessor* ca, const qcc::ECCPrivateKey* privateKey, const qcc::ECCPublicKey* publicKey);
 
     /**
@@ -280,7 +330,7 @@ class PermissionMgmtObj : public BusObject {
      * Retrieve the claimable state of the application.
      * @return the claimable state
      */
-    PermissionConfigurator::ApplicationState GetApplicationState();
+    PermissionConfigurator::ApplicationState GetApplicationState() const;
 
     /**
      * Set the application state.  The state can't be changed from CLAIMED to
@@ -307,13 +357,12 @@ class PermissionMgmtObj : public BusObject {
      * @param[out] publicKeyFound the flag indicating whether the peer has an ECC public key
      * @param[out] publicKey the buffer to hold the ECC public key.  Pass NULL
      *                       to skip.
-     * @param[out] manifestDigest the buffer to hold the manifest digest. Pass
-     *                            NULL to skip.
+     * @param[out] identityCertificateThumbprint buffer to receive the SHA-256 thumbprint of the identity certificate
      * @param[out] issuerPublicKeys the vector for the list of issuer public
      *                               keys.
      * @return ER_OK if successful; otherwise, error code.
      */
-    QStatus GetConnectedPeerAuthMetadata(const qcc::GUID128& guid, qcc::String& authMechanism, bool& publicKeyFound, qcc::ECCPublicKey* publicKey, uint8_t* manifestDigest, std::vector<qcc::ECCPublicKey>& issuerPublicKeys);
+    QStatus GetConnectedPeerAuthMetadata(const qcc::GUID128& guid, qcc::String& authMechanism, bool& publicKeyFound, qcc::ECCPublicKey* publicKey, uint8_t* identityCertificateThumbprint, std::vector<qcc::ECCPublicKey>& issuerPublicKeys);
 
     /**
      * Get the connected peer ECC public key if the connection uses the
@@ -321,13 +370,11 @@ class PermissionMgmtObj : public BusObject {
      * @param guid the peer guid
      * @param[out] publicKey the buffer to hold the ECC public key.  Pass NULL
      *                       to skip.
-     * @param[out] manifestDigest the buffer to hold the manifest digest. Pass
-     *                            NULL to skip.
      * @param[out] issuerPublicKeys the vector for the list of issuer public
      *                               keys.
      * @return ER_OK if successful; otherwise, error code.
      */
-    QStatus GetConnectedPeerPublicKey(const qcc::GUID128& guid, qcc::ECCPublicKey* publicKey, uint8_t* manifestDigest, std::vector<qcc::ECCPublicKey>& issuerPublicKeys);
+    QStatus GetConnectedPeerPublicKey(const qcc::GUID128& guid, qcc::ECCPublicKey* publicKey, std::vector<qcc::ECCPublicKey>& issuerPublicKeys);
 
     /**
      * Get the connected peer ECC public key if the connection uses the
@@ -355,7 +402,8 @@ class PermissionMgmtObj : public BusObject {
     QStatus StoreIdentityCertChain(MsgArg& certArg);
     QStatus StorePolicy(PermissionPolicy& policy, bool defaultPolicy = false);
     QStatus StoreMembership(const MsgArg& certArg);
-    QStatus StoreManifest(MsgArg& manifestArg);
+    QStatus StoreManifests(MsgArg& signedManifestsArg, bool append);
+
     /**
      * Generate the SHA-256 digest for the manifest data.
      * @param bus the bus attachment
@@ -370,18 +418,10 @@ class PermissionMgmtObj : public BusObject {
 
     /**
      * Retrieve the manifest from persistence store.
-     * @param manifest[out] The variable to hold the manifest.
-     *                      Set to NULL to find out the size of the manifest.
-     * @param count[in,out] the number of rules in the manifest.
-     *                      If manifest is NULL count will be set to the number
-     *                      of rules in the manifest. If manifest is not NULL
-     *                      count will return the number of rule placed in the
-     *                      manifest. If count is smaller than the number of
-     *                      rules found in the manifest ER_BUFFER_TOO_SMALL will
-     *                      be returned.
+     * @param[out] manifests The variable to hold the manifests.
      * @return ER_OK for success; error code otherwise.
      */
-    QStatus RetrieveManifest(PermissionPolicy::Rule* manifest, size_t* count);
+    QStatus RetrieveManifests(std::vector<Manifest>& manifests);
     /**
      * Reply to a method call with an error message.
      *
@@ -474,7 +514,7 @@ class PermissionMgmtObj : public BusObject {
      *  - #ER_OK if successful
      *  - an error status indicating failure
      */
-    QStatus GetClaimCapabilities(PermissionConfigurator::ClaimCapabilities& claimCapabilities);
+    QStatus GetClaimCapabilities(PermissionConfigurator::ClaimCapabilities& claimCapabilities) const;
 
 
     /**
@@ -485,7 +525,7 @@ class PermissionMgmtObj : public BusObject {
      *  - #ER_OK if successful
      *  - an error status indicating failure
      */
-    QStatus GetClaimCapabilityAdditionalInfo(PermissionConfigurator::ClaimCapabilityAdditionalInfo& additionalInfo);
+    QStatus GetClaimCapabilityAdditionalInfo(PermissionConfigurator::ClaimCapabilityAdditionalInfo& additionalInfo) const;
 
     /**
      * Store a membership certificate chain.
@@ -514,6 +554,20 @@ class PermissionMgmtObj : public BusObject {
         return ready;
     }
 
+    /**
+     * Send any needed manifests in advance of a message about to be sent.
+     *
+     * @param[in] remotePeerObj The ProxyBusObject being asked to send the message, and that we can use to send manifests.
+     * @param[in] msg The message about to be sent. If nullptr, all manifests will be sent.
+     *
+     * @remarks remotePeerObj and msg cannot both be nullptr.
+     *
+     * @return #ER_OK if all needed manifests were sent, possibly none.
+     * @return #ER_BAD_ARG_1 if both remotePeerObj and msgPtr are nullptr
+     *         - other error indicating failure
+     */
+    QStatus SendManifests(const ProxyBusObject* remotePeerObj, Message* msg);
+
   protected:
     void Claim(const InterfaceDescription::Member* member, Message& msg);
     BusAttachment& bus;
@@ -530,6 +584,9 @@ class PermissionMgmtObj : public BusObject {
     void ResetPolicy(const InterfaceDescription::Member* member, Message& msg);
     void InstallMembership(const InterfaceDescription::Member* member, Message& msg);
     void RemoveMembership(const InterfaceDescription::Member* member, Message& msg);
+    void StartManagement(const InterfaceDescription::Member* member, Message& msg);
+    void EndManagement(const InterfaceDescription::Member* member, Message& msg);
+    void InstallManifests(const InterfaceDescription::Member* member, Message& msg);
     QStatus GetMembershipSummaries(MsgArg& arg);
     QStatus GetManifestTemplate(MsgArg& arg);
     QStatus GetManifestTemplateDigest(MsgArg& arg);
@@ -557,7 +614,7 @@ class PermissionMgmtObj : public BusObject {
         uint16_t claimCapabilities;
         uint16_t claimCapabilityAdditionalInfo;
 
-        Configuration() : version(1), applicationStateSet(0), applicationState(PermissionConfigurator::NOT_CLAIMABLE), claimCapabilities(PermissionConfigurator::CAPABLE_ECDHE_NULL), claimCapabilityAdditionalInfo(0)
+        Configuration() : version(1), applicationStateSet(0), applicationState(PermissionConfigurator::NOT_CLAIMABLE), claimCapabilities(PermissionConfigurator::CLAIM_CAPABILITIES_DEFAULT), claimCapabilityAdditionalInfo(0)
         {
         }
     };
@@ -611,9 +668,7 @@ class PermissionMgmtObj : public BusObject {
     bool IsRelevantMembershipCert(std::vector<MsgArg*>& membershipChain, std::vector<qcc::ECCPublicKey> peerIssuers);
     QStatus LookForManifestTemplate(bool& exist);
 
-    /**
-     * Bind to an exclusive port for PermissionMgmt object.
-     */
+    /* Bind to an exclusive port for PermissionMgmt object */
     QStatus BindPort();
 
     CredentialAccessor* ca;
@@ -622,6 +677,9 @@ class PermissionMgmtObj : public BusObject {
     PortListener* portListener;
     MessageEncryptionNotification* callbackToClearSecrets;
     bool ready;
+
+    /* Has bool semantic but using volatile int32_t for atomic compare-and-exchange */
+    volatile int32_t managementStarted;
 };
 
 }
