@@ -22,7 +22,6 @@
 
 #include <qcc/platform.h>
 
-#include <assert.h>
 #include <map>
 #include <set>
 #include <vector>
@@ -33,7 +32,6 @@
 #include <qcc/ManagedObj.h>
 #include <qcc/Mutex.h>
 #include <qcc/String.h>
-#include <qcc/StringMapKey.h>
 #include <qcc/StringSource.h>
 #include <qcc/Util.h>
 #include <qcc/XmlElement.h>
@@ -93,7 +91,7 @@ struct _PropertiesChangedCB {
 
     ProxyBusObject::PropertiesChangedListener& listener;
     void* context;
-    set<StringMapKey> properties;  // Properties to monitor - empty set == all properties.
+    set<std::string> properties;  // Properties to monitor - empty set == all properties.
     bool isRegistered;
     int32_t numRunning;
     _PropertiesChangedCB& operator=(const _PropertiesChangedCB&) { return *this; }
@@ -103,7 +101,7 @@ typedef ManagedObj<_PropertiesChangedCB> PropertiesChangedCB;
 
 class CachedProps {
     qcc::Mutex lock;
-    typedef std::map<qcc::StringMapKey, MsgArg> ValueMap;
+    typedef std::map<std::string, MsgArg> ValueMap;
     ValueMap values;
     const InterfaceDescription* description;
     bool isFullyCacheable;
@@ -116,15 +114,13 @@ class CachedProps {
 
   public:
     CachedProps() :
-        lock(), values(), description(NULL),
-        isFullyCacheable(false),
-        numProperties(0), lastMessageSerial(0),
-        enabled(false) { }
+        lock(LOCK_LEVEL_PROXYBUSOBJECT_CACHEDPROPS_LOCK), values(), description(NULL),
+        isFullyCacheable(false), numProperties(0), lastMessageSerial(0), enabled(false) { }
 
     CachedProps(const InterfaceDescription*intf) :
-        lock(), values(), description(intf),
-        isFullyCacheable(false), lastMessageSerial(0),
-        enabled(false) {
+        lock(LOCK_LEVEL_PROXYBUSOBJECT_CACHEDPROPS_LOCK), values(), description(intf),
+        isFullyCacheable(false), lastMessageSerial(0), enabled(false)
+    {
         numProperties = description->GetProperties();
         if (numProperties > 0) {
             isFullyCacheable = true;
@@ -141,10 +137,9 @@ class CachedProps {
     }
 
     CachedProps(const CachedProps& other) :
-        lock(), values(other.values), description(other.description),
-        isFullyCacheable(other.isFullyCacheable),
-        numProperties(other.numProperties), lastMessageSerial(other.lastMessageSerial),
-        enabled(other.enabled) { }
+        lock(LOCK_LEVEL_PROXYBUSOBJECT_CACHEDPROPS_LOCK), values(other.values), description(other.description),
+        isFullyCacheable(other.isFullyCacheable), numProperties(other.numProperties),
+        lastMessageSerial(other.lastMessageSerial), enabled(other.enabled) { }
 
     CachedProps& operator=(const CachedProps& other) {
         if (&other != this) {
@@ -208,43 +203,46 @@ class ProxyBusObject::Internal : public MessageReceiver, public BusAttachment::A
         hasProperties(false),
         isSecure(false),
         cacheProperties(false),
+        lock(LOCK_LEVEL_PROXYBUSOBJECT_INTERNAL_LOCK),
         registeredPropChangedHandler(false),
         handlerThreads()
     {
         QCC_DbgPrintf(("Creating empty PBO internal: %p", this));
     }
     Internal(BusAttachment& bus,
-             String path,
-             String service,
+             const char* objPath,
+             const char* service,
              SessionId sessionId,
              bool isSecure) :
         bus(&bus),
-        path(path),
-        serviceName(service),
-        uniqueName((serviceName[0] == ':') ? service : ""),
+        path(objPath ? objPath : ""),
+        serviceName(service ? service : ""),
+        uniqueName((!serviceName.empty() && (serviceName[0] == ':')) ? serviceName : ""),
         sessionId(sessionId),
         hasProperties(false),
         isSecure(isSecure),
         cacheProperties(false),
+        lock(LOCK_LEVEL_PROXYBUSOBJECT_INTERNAL_LOCK),
         registeredPropChangedHandler(false),
         handlerThreads()
     {
         QCC_DbgPrintf(("Creating PBO internal: %p   path=%s   serviceName=%s   uniqueName=%s", this, path.c_str(), serviceName.c_str(), uniqueName.c_str()));
     }
     Internal(BusAttachment& bus,
-             String path,
-             String serviceName,
-             String uniqueName,
+             const char* objPath,
+             const char* service,
+             const char* unique,
              SessionId sessionId,
              bool isSecure) :
         bus(&bus),
-        path(path),
-        serviceName(serviceName),
-        uniqueName(uniqueName),
+        path(objPath ? objPath : ""),
+        serviceName(service ? service : ""),
+        uniqueName(unique ? unique : ""),
         sessionId(sessionId),
         hasProperties(false),
         isSecure(isSecure),
         cacheProperties(false),
+        lock(LOCK_LEVEL_PROXYBUSOBJECT_INTERNAL_LOCK),
         registeredPropChangedHandler(false),
         handlerThreads()
     {
@@ -318,10 +316,10 @@ class ProxyBusObject::Internal : public MessageReceiver, public BusAttachment::A
     map<qcc::Thread*, _PropertiesChangedCB*> handlerThreads;   /**< Thread actively calling PropertiesChangedListeners */
 
     /** The interfaces this object implements */
-    map<qcc::StringMapKey, const InterfaceDescription*> ifaces;
+    map<std::string, const InterfaceDescription*> ifaces;
 
     /** The property caches for the various interfaces */
-    mutable map<qcc::StringMapKey, CachedProps> caches;
+    mutable map<std::string, CachedProps> caches;
 
     /** Names of child objects of this object */
     vector<ProxyBusObject> children;
@@ -331,11 +329,11 @@ class ProxyBusObject::Internal : public MessageReceiver, public BusAttachment::A
     mutable Condition syncMethodComplete;
 
     /** Match rule book keeping */
-    typedef map<StringMapKey, MatchRuleInfo> MatchRuleBookKeeping;
+    typedef map<std::string, MatchRuleInfo> MatchRuleBookKeeping;
     MatchRuleBookKeeping matchRuleBookKeeping;
 
     /** Property changed handlers */
-    multimap<StringMapKey, PropertiesChangedCB> propertiesChangedCBs;
+    multimap<std::string, PropertiesChangedCB> propertiesChangedCBs;
 };
 
 ProxyBusObject::Internal::~Internal()
@@ -452,7 +450,7 @@ QStatus ProxyBusObject::GetAllProperties(const char* iface, MsgArg& value, uint3
         bool cached = false;
         internal->lock.Lock(MUTEX_CONTEXT);
         if (internal->cacheProperties) {
-            map<qcc::StringMapKey, CachedProps>::iterator it = internal->caches.find(iface);
+            map<std::string, CachedProps>::iterator it = internal->caches.find(iface);
             if (it != internal->caches.end()) {
                 cached = it->second.GetAll(value);
             }
@@ -478,14 +476,14 @@ QStatus ProxyBusObject::GetAllProperties(const char* iface, MsgArg& value, uint3
             status = ER_BUS_NO_SUCH_INTERFACE;
         } else {
             const InterfaceDescription::Member* getAllProperties = propIface->GetMember("GetAll");
-            assert(getAllProperties);
+            QCC_ASSERT(getAllProperties);
             status = MethodCall(*getAllProperties, &arg, 1, reply, timeout, flags);
             if (ER_OK == status) {
                 value = *(reply->GetArg(0));
                 /* use the retrieved property values to update the cache, if applicable */
                 internal->lock.Lock(MUTEX_CONTEXT);
                 if (internal->cacheProperties) {
-                    map<qcc::StringMapKey, CachedProps>::iterator it = internal->caches.find(iface);
+                    map<std::string, CachedProps>::iterator it = internal->caches.find(iface);
                     if (it != internal->caches.end()) {
                         it->second.SetAll(value, reply->GetCallSerial());
                     }
@@ -508,7 +506,7 @@ void ProxyBusObject::GetAllPropsMethodCB(Message& message, void* context)
         /* use the retrieved property values to update the cache, if applicable */
         internal->lock.Lock(MUTEX_CONTEXT);
         if (internal->cacheProperties) {
-            map<qcc::StringMapKey, CachedProps>::iterator it = internal->caches.find(iface);
+            map<std::string, CachedProps>::iterator it = internal->caches.find(iface);
             if (it != internal->caches.end()) {
                 it->second.SetAll(*message->GetArg(0), message->GetCallSerial());
             }
@@ -549,7 +547,7 @@ QStatus ProxyBusObject::GetAllPropertiesAsync(const char* iface,
         MsgArg value;
         internal->lock.Lock(MUTEX_CONTEXT);
         if (internal->cacheProperties) {
-            map<qcc::StringMapKey, CachedProps>::iterator it = internal->caches.find(iface);
+            map<std::string, CachedProps>::iterator it = internal->caches.find(iface);
             if (it != internal->caches.end()) {
                 cached = it->second.GetAll(value);
             }
@@ -577,7 +575,7 @@ QStatus ProxyBusObject::GetAllPropertiesAsync(const char* iface,
             std::pair<void*, qcc::String>* wrappedContext = new std::pair<void*, qcc::String>(context, iface);
             CBContext<Listener::GetAllPropertiesCB>* ctx = new CBContext<Listener::GetAllPropertiesCB>(listener, callback, wrappedContext);
             const InterfaceDescription::Member* getAllProperties = propIface->GetMember("GetAll");
-            assert(getAllProperties);
+            QCC_ASSERT(getAllProperties);
             status = MethodCallAsync(*getAllProperties,
                                      this,
                                      static_cast<MessageReceiver::ReplyHandler>(&ProxyBusObject::GetAllPropsMethodCB),
@@ -606,7 +604,7 @@ QStatus ProxyBusObject::GetProperty(const char* iface, const char* property, Msg
         bool cached = false;
         internal->lock.Lock(MUTEX_CONTEXT);
         if (internal->cacheProperties) {
-            map<qcc::StringMapKey, CachedProps>::iterator it = internal->caches.find(iface);
+            map<std::string, CachedProps>::iterator it = internal->caches.find(iface);
             if (it != internal->caches.end()) {
                 cached = it->second.Get(property, value);
             }
@@ -634,14 +632,14 @@ QStatus ProxyBusObject::GetProperty(const char* iface, const char* property, Msg
             status = ER_BUS_NO_SUCH_INTERFACE;
         } else {
             const InterfaceDescription::Member* getProperty = propIface->GetMember("Get");
-            assert(getProperty);
+            QCC_ASSERT(getProperty);
             status = MethodCall(*getProperty, inArgs, numArgs, reply, timeout, flags);
             if (ER_OK == status) {
                 value = *(reply->GetArg(0));
                 /* use the retrieved property value to update the cache, if applicable */
                 internal->lock.Lock(MUTEX_CONTEXT);
                 if (internal->cacheProperties) {
-                    map<qcc::StringMapKey, CachedProps>::iterator it = internal->caches.find(iface);
+                    map<std::string, CachedProps>::iterator it = internal->caches.find(iface);
                     if (it != internal->caches.end()) {
                         it->second.Set(property, value, reply->GetCallSerial());
                     }
@@ -667,7 +665,7 @@ void ProxyBusObject::GetPropMethodCB(Message& message, void* context)
         /* use the retrieved property value to update the cache, if applicable */
         internal->lock.Lock(MUTEX_CONTEXT);
         if (internal->cacheProperties) {
-            map<qcc::StringMapKey, CachedProps>::iterator it = internal->caches.find(iface);
+            map<std::string, CachedProps>::iterator it = internal->caches.find(iface);
             if (it != internal->caches.end()) {
                 it->second.Set(property, *message->GetArg(0), message->GetCallSerial());
             }
@@ -709,7 +707,7 @@ QStatus ProxyBusObject::GetPropertyAsync(const char* iface,
         MsgArg value;
         internal->lock.Lock(MUTEX_CONTEXT);
         if (internal->cacheProperties) {
-            map<qcc::StringMapKey, CachedProps>::iterator it = internal->caches.find(iface);
+            map<std::string, CachedProps>::iterator it = internal->caches.find(iface);
             if (it != internal->caches.end()) {
                 cached = it->second.Get(property, value);
             }
@@ -737,7 +735,7 @@ QStatus ProxyBusObject::GetPropertyAsync(const char* iface,
             std::pair<void*, std::pair<qcc::String, qcc::String> >* wrappedContext = new std::pair<void*, std::pair<qcc::String, qcc::String> >(context, std::make_pair(qcc::String(iface), qcc::String(property)));
             CBContext<Listener::GetPropertyCB>* ctx = new CBContext<Listener::GetPropertyCB>(listener, callback, wrappedContext);
             const InterfaceDescription::Member* getProperty = propIface->GetMember("Get");
-            assert(getProperty);
+            QCC_ASSERT(getProperty);
             status = MethodCallAsync(*getProperty,
                                      this,
                                      static_cast<MessageReceiver::ReplyHandler>(&ProxyBusObject::GetPropMethodCB),
@@ -778,7 +776,7 @@ QStatus ProxyBusObject::SetProperty(const char* iface, const char* property, Msg
             status = ER_BUS_NO_SUCH_INTERFACE;
         } else {
             const InterfaceDescription::Member* setProperty = propIface->GetMember("Set");
-            assert(setProperty);
+            QCC_ASSERT(setProperty);
             status = MethodCall(*setProperty,
                                 inArgs,
                                 numArgs,
@@ -835,11 +833,11 @@ QStatus ProxyBusObject::RegisterPropertiesChangedListener(const char* iface,
     bool replace = false;
     String ifaceStr = iface;
     PropertiesChangedCB ctx(listener, properties, propertiesSize, context);
-    pair<StringMapKey, PropertiesChangedCB> cbItem(ifaceStr, ctx);
+    pair<std::string, PropertiesChangedCB> cbItem(ifaceStr, ctx);
     internal->lock.Lock(MUTEX_CONTEXT);
     // remove old version first
-    multimap<StringMapKey, PropertiesChangedCB>::iterator it = internal->propertiesChangedCBs.lower_bound(iface);
-    multimap<StringMapKey, PropertiesChangedCB>::iterator end = internal->propertiesChangedCBs.upper_bound(iface);
+    multimap<std::string, PropertiesChangedCB>::iterator it = internal->propertiesChangedCBs.lower_bound(iface);
+    multimap<std::string, PropertiesChangedCB>::iterator end = internal->propertiesChangedCBs.upper_bound(iface);
     while (it != end) {
         PropertiesChangedCB propChangedCb = it->second;
         if (&propChangedCb->listener == &listener) {
@@ -883,8 +881,8 @@ QStatus ProxyBusObject::UnregisterPropertiesChangedListener(const char* iface,
         return ER_DEADLOCK;
     }
 
-    multimap<StringMapKey, PropertiesChangedCB>::iterator it = internal->propertiesChangedCBs.lower_bound(iface);
-    multimap<StringMapKey, PropertiesChangedCB>::iterator end = internal->propertiesChangedCBs.upper_bound(iface);
+    multimap<std::string, PropertiesChangedCB>::iterator it = internal->propertiesChangedCBs.lower_bound(iface);
+    multimap<std::string, PropertiesChangedCB>::iterator end = internal->propertiesChangedCBs.upper_bound(iface);
     while (it != end) {
         PropertiesChangedCB ctx = it->second;
         if (&ctx->listener == &listener) {
@@ -944,7 +942,7 @@ void ProxyBusObject::Internal::PropertiesChangedHandler(const InterfaceDescripti
     lock.Lock(MUTEX_CONTEXT);
     /* first, update caches */
     if (cacheProperties) {
-        map<StringMapKey, CachedProps>::iterator it = caches.find(ifaceName);
+        map<std::string, CachedProps>::iterator it = caches.find(ifaceName);
         if (it != caches.end()) {
             it->second.PropertiesChanged(changedProps, numChangedProps, invalidProps, numInvalidProps, message->GetCallSerial());
         }
@@ -952,8 +950,8 @@ void ProxyBusObject::Internal::PropertiesChangedHandler(const InterfaceDescripti
 
     /* then, alert listeners */
     handlerThreads[Thread::GetThread()] = nullptr;
-    multimap<StringMapKey, PropertiesChangedCB>::iterator it = propertiesChangedCBs.lower_bound(ifaceName);
-    multimap<StringMapKey, PropertiesChangedCB>::iterator end = propertiesChangedCBs.upper_bound(ifaceName);
+    multimap<std::string, PropertiesChangedCB>::iterator it = propertiesChangedCBs.lower_bound(ifaceName);
+    multimap<std::string, PropertiesChangedCB>::iterator end = propertiesChangedCBs.upper_bound(ifaceName);
     list<PropertiesChangedCB> handlers;
     while (it != end) {
         if (it->second->isRegistered) {
@@ -1074,7 +1072,7 @@ QStatus ProxyBusObject::SetPropertyAsync(const char* iface,
         } else {
             CBContext<Listener::SetPropertyCB>* ctx = new CBContext<Listener::SetPropertyCB>(listener, callback, context);
             const InterfaceDescription::Member* setProperty = propIface->GetMember("Set");
-            assert(setProperty);
+            QCC_ASSERT(setProperty);
             status = MethodCallAsync(*setProperty,
                                      this,
                                      static_cast<MessageReceiver::ReplyHandler>(&ProxyBusObject::SetPropMethodCB),
@@ -1097,7 +1095,7 @@ size_t ProxyBusObject::GetInterfaces(const InterfaceDescription** ifaces, size_t
     size_t count = internal->ifaces.size();
     if (ifaces) {
         count = min(count, numIfaces);
-        map<qcc::StringMapKey, const InterfaceDescription*>::const_iterator it = internal->ifaces.begin();
+        map<std::string, const InterfaceDescription*>::const_iterator it = internal->ifaces.begin();
         for (size_t i = 0; i < count && it != internal->ifaces.end(); ++i, ++it) {
             ifaces[i] = it->second;
         }
@@ -1108,9 +1106,9 @@ size_t ProxyBusObject::GetInterfaces(const InterfaceDescription** ifaces, size_t
 
 const InterfaceDescription* ProxyBusObject::GetInterface(const char* ifaceName) const
 {
-    StringMapKey key = ifaceName;
+    std::string key = ifaceName;
     internal->lock.Lock(MUTEX_CONTEXT);
-    map<StringMapKey, const InterfaceDescription*>::const_iterator it = internal->ifaces.find(key);
+    map<std::string, const InterfaceDescription*>::const_iterator it = internal->ifaces.find(key);
     const InterfaceDescription* ret = (it == internal->ifaces.end()) ? NULL : it->second;
     internal->lock.Unlock(MUTEX_CONTEXT);
     return ret;
@@ -1118,12 +1116,12 @@ const InterfaceDescription* ProxyBusObject::GetInterface(const char* ifaceName)
 
 
 QStatus ProxyBusObject::AddInterface(const InterfaceDescription& iface) {
-    StringMapKey key(qcc::String(iface.GetName()));
-    pair<StringMapKey, const InterfaceDescription*> item(key, &iface);
+    std::string key(qcc::String(iface.GetName()));
+    pair<std::string, const InterfaceDescription*> item(key, &iface);
     bool addRule = false;
 
     internal->lock.Lock(MUTEX_CONTEXT);
-    pair<map<StringMapKey, const InterfaceDescription*>::const_iterator, bool> ret = internal->ifaces.insert(item);
+    pair<map<std::string, const InterfaceDescription*>::const_iterator, bool> ret = internal->ifaces.insert(item);
     QStatus status = ret.second ? ER_OK : ER_BUS_IFACE_ALREADY_EXISTS;
 
     if ((status == ER_OK) && internal->cacheProperties && iface.HasCacheableProperties()) {
@@ -1133,7 +1131,7 @@ QStatus ProxyBusObject::AddInterface(const InterfaceDescription& iface) {
 
     if ((status == ER_OK) && !internal->hasProperties) {
         const InterfaceDescription* propIntf = internal->bus->GetInterface(::ajn::org::freedesktop::DBus::Properties::InterfaceName);
-        assert(propIntf);
+        QCC_ASSERT(propIntf);
         if (iface == *propIntf) {
             internal->hasProperties = true;
         } else if (iface.GetProperties() > 0) {
@@ -1169,7 +1167,7 @@ void ProxyBusObject::Internal::AddMatchCB(QStatus status, void* context)
     lock.Lock(MUTEX_CONTEXT);
     info->addingRef = false;
     /* enable property caches */
-    map<StringMapKey, CachedProps>::iterator it = caches.find(info->ifaceName);
+    map<std::string, CachedProps>::iterator it = caches.find(info->ifaceName);
     if (it != caches.end()) {
         it->second.Enable();
     }
@@ -1194,11 +1192,11 @@ void ProxyBusObject::Internal::AddPropertiesChangedRule(const char* intf, bool b
          * Setup placeholder.  Other threads that call this function with the
          * same interface after us will block until our AddMatch call completes.
          */
-        r = matchRuleBookKeeping.insert(pair<StringMapKey, MatchRuleInfo>(qcc::String(intf), true));
+        r = matchRuleBookKeeping.insert(pair<std::string, MatchRuleInfo>(qcc::String(intf), true));
         it = r.first;
     }
 
-    assert(it != matchRuleBookKeeping.end());
+    QCC_ASSERT(it != matchRuleBookKeeping.end());
     ++it->second.refCount;
 
     if (!registeredPropChangedHandler) {
@@ -1210,7 +1208,7 @@ void ProxyBusObject::Internal::AddPropertiesChangedRule(const char* intf, bool b
     if (registerHandler) {
         QCC_DbgPrintf(("Registering signal handler"));
         const InterfaceDescription* propIntf = bus->GetInterface(::ajn::org::freedesktop::DBus::Properties::InterfaceName);
-        assert(propIntf);
+        QCC_ASSERT(propIntf);
         bus->RegisterSignalHandler(this,
                                    static_cast<MessageReceiver::SignalHandler>(&Internal::PropertiesChangedHandler),
                                    propIntf->GetMember("PropertiesChanged"),
@@ -1235,7 +1233,7 @@ void ProxyBusObject::Internal::AddPropertiesChangedRule(const char* intf, bool b
      * If we don't have the match rule yet, PBO::Internal::AddMatchCB will enable the
      * cache for us. */
     if (!it->second.adding) {
-        map<StringMapKey, CachedProps>::iterator cit = caches.find(intf);
+        map<std::string, CachedProps>::iterator cit = caches.find(intf);
         if (cit != caches.end()) {
             cit->second.Enable();
         }
@@ -1305,7 +1303,7 @@ void ProxyBusObject::EnablePropertyCaching()
     ifcNames.reserve(internal->ifaces.size());
     if (!internal->cacheProperties) {
         internal->cacheProperties = true;
-        map<StringMapKey, const InterfaceDescription*>::const_iterator it = internal->ifaces.begin();
+        map<std::string, const InterfaceDescription*>::const_iterator it = internal->ifaces.begin();
         for (; it != internal->ifaces.end(); ++it) {
             if (it->second->HasCacheableProperties()) {
                 internal->caches.insert(std::make_pair(qcc::String(it->first.c_str()), CachedProps(it->second)));
@@ -1356,7 +1354,7 @@ size_t ProxyBusObject::GetManagedChildren(void* children, size_t numChildren)
 ProxyBusObject* ProxyBusObject::GetChild(const char* inPath)
 {
     /* Add a trailing slash to this path */
-    qcc::String pathSlash = (internal->path == "/") ? internal->path : internal->path + '/';
+    qcc::String pathSlash = (internal->path == "/") ? internal->path : internal->path + "/";
 
     /* Create absolute version of inPath */
     qcc::String inPathStr = ('/' == inPath[0]) ? inPath : pathSlash + inPath;
@@ -1406,7 +1404,7 @@ QStatus ProxyBusObject::AddChild(const ProxyBusObject& child)
     qcc::String childPath = child.GetPath();
 
     /* Sanity check to make sure path is possible */
-    if (((internal->path.size() > 1) && (0 != childPath.find(internal->path + '/'))) ||
+    if (((internal->path.size() > 1) && (0 != childPath.find(internal->path + "/"))) ||
         ((internal->path.size() == 1) && (childPath[0] != '/')) ||
         (childPath[childPath.length() - 1] == '/')) {
         return ER_BUS_BAD_CHILD_PATH;
@@ -1454,7 +1452,7 @@ QStatus ProxyBusObject::RemoveChild(const char* inPath)
     QStatus status;
 
     /* Add a trailing slash to this path */
-    qcc::String pathSlash = (internal->path == "/") ? internal->path : internal->path + '/';
+    qcc::String pathSlash = (internal->path == "/") ? internal->path : internal->path + "/";
 
     /* Create absolute version of inPath */
     qcc::String childPath = ('/' == inPath[0]) ? inPath : pathSlash + inPath;
@@ -1578,7 +1576,7 @@ QStatus ProxyBusObject::MethodCallAsync(const char* ifaceName,
                                         uint8_t flags) const
 {
     internal->lock.Lock(MUTEX_CONTEXT);
-    map<StringMapKey, const InterfaceDescription*>::const_iterator it = internal->ifaces.find(StringMapKey(ifaceName));
+    map<std::string, const InterfaceDescription*>::const_iterator it = internal->ifaces.find(std::string(ifaceName));
     if (it == internal->ifaces.end()) {
         internal->lock.Unlock(MUTEX_CONTEXT);
         return ER_BUS_NO_SUCH_INTERFACE;
@@ -1642,6 +1640,15 @@ QStatus ProxyBusObject::MethodCall(const InterfaceDescription::Member& method,
     if (NULL != callMsg) {
         *callMsg = msg;
     }
+    /*
+     * See if we need to send any manifests in advance of this message.
+     */
+    status = internal->bus->GetInternal().GetPermissionManager().GetPermissionMgmtObj()->SendManifests(this, &msg);
+    if (status != ER_OK) {
+        QCC_LogError(status, ("Failed to SendManifests"));
+        goto MethodCallExit;
+    }
+
     if (flags & ALLJOYN_FLAG_NO_REPLY_EXPECTED) {
         /*
          * Push the message to the router and we are done
@@ -1763,7 +1770,7 @@ QStatus ProxyBusObject::MethodCall(const char* ifaceName,
                                    uint8_t flags) const
 {
     internal->lock.Lock(MUTEX_CONTEXT);
-    map<StringMapKey, const InterfaceDescription*>::const_iterator it = internal->ifaces.find(StringMapKey(ifaceName));
+    map<std::string, const InterfaceDescription*>::const_iterator it = internal->ifaces.find(std::string(ifaceName));
     if (it == internal->ifaces.end()) {
         internal->lock.Unlock(MUTEX_CONTEXT);
         return ER_BUS_NO_SUCH_INTERFACE;
@@ -1829,14 +1836,14 @@ QStatus ProxyBusObject::IntrospectRemoteObject(uint32_t timeout)
     const InterfaceDescription* introIntf = GetInterface(org::freedesktop::DBus::Introspectable::InterfaceName);
     if (!introIntf) {
         introIntf = internal->bus->GetInterface(org::freedesktop::DBus::Introspectable::InterfaceName);
-        assert(introIntf);
+        QCC_ASSERT(introIntf);
         AddInterface(*introIntf);
     }
 
     /* Attempt to retrieve introspection from the remote object using sync call */
     Message reply(*internal->bus);
     const InterfaceDescription::Member* introMember = introIntf->GetMember("Introspect");
-    assert(introMember);
+    QCC_ASSERT(introMember);
     QStatus status = MethodCall(*introMember, NULL, 0, reply, timeout);
 
     /* Parse the XML reply */
@@ -1862,13 +1869,13 @@ QStatus ProxyBusObject::IntrospectRemoteObjectAsync(ProxyBusObject::Listener* li
     const InterfaceDescription* introIntf = GetInterface(org::freedesktop::DBus::Introspectable::InterfaceName);
     if (!introIntf) {
         introIntf = internal->bus->GetInterface(org::freedesktop::DBus::Introspectable::InterfaceName);
-        assert(introIntf);
+        QCC_ASSERT(introIntf);
         AddInterface(*introIntf);
     }
 
     /* Attempt to retrieve introspection from the remote object using async call */
     const InterfaceDescription::Member* introMember = introIntf->GetMember("Introspect");
-    assert(introMember);
+    QCC_ASSERT(introMember);
     CBContext<Listener::IntrospectCB>* ctx = new CBContext<Listener::IntrospectCB>(listener, callback, context);
     QStatus status = MethodCallAsync(*introMember,
                                      this,
@@ -1886,20 +1893,22 @@ QStatus ProxyBusObject::IntrospectRemoteObjectAsync(ProxyBusObject::Listener* li
 void ProxyBusObject::IntrospectMethodCB(Message& msg, void* context)
 {
     QStatus status;
-    if (NULL != msg->GetArg(0)) {
-        QCC_DbgPrintf(("Introspection XML: %s", msg->GetArg(0)->v_string.str));
-    }
     CBContext<Listener::IntrospectCB>* ctx = reinterpret_cast<CBContext<Listener::IntrospectCB>*>(context);
 
     if (msg->GetType() == MESSAGE_METHOD_RET) {
         /* Parse the XML reply to update this ProxyBusObject instance (plus any new interfaces) */
-        qcc::String ident = msg->GetSender();
-        if (internal->uniqueName.empty()) {
-            internal->uniqueName = ident;
+        char* xml;
+        status = msg->GetArgs("s", &xml);
+        if (ER_OK == status) {
+            QCC_DbgPrintf(("Introspection XML: %s", xml));
+            qcc::String ident = msg->GetSender();
+            if (internal->uniqueName.empty()) {
+                internal->uniqueName = ident;
+            }
+            ident += " : ";
+            ident += msg->GetObjectPath();
+            status = ParseXml(xml, ident.c_str());
         }
-        ident += " : ";
-        ident += msg->GetObjectPath();
-        status = ParseXml(msg->GetArg(0)->v_string.str, ident.c_str());
     } else if (msg->GetErrorName() != NULL && ::strcmp("org.freedesktop.DBus.Error.ServiceUnknown", msg->GetErrorName()) == 0) {
         status = ER_BUS_NO_SUCH_SERVICE;
     } else {
