@@ -19,8 +19,12 @@
 #include <qcc/Debug.h>
 #include <qcc/String.h>
 #include <qcc/KeyInfoECC.h>
+#include <qcc/Util.h>
 #include "PermissionMgmtObj.h"
 #include "KeyInfoHelper.h"
+#include "XmlManifestConverter.h"
+#include "XmlRulesConverter.h"
+#include "XmlPoliciesConverter.h"
 
 #define QCC_MODULE "ALLJOYN_SECURITY"
 
@@ -33,7 +37,7 @@ SecurityApplicationProxy::SecurityApplicationProxy(BusAttachment& bus, const cha
     {
         /* Add org.alljoyn.Bus.Security.Application interface */
         const InterfaceDescription* intf = bus.GetInterface(org::alljoyn::Bus::Security::Application::InterfaceName);
-        assert(intf);
+        QCC_ASSERT(intf);
         status = AddInterface(*intf);
         if (status != ER_OK) {
             QCC_LogError(status, ("Failed to add the %s interface to the %s", org::alljoyn::Bus::Security::Application::InterfaceName, __FUNCTION__));
@@ -42,7 +46,7 @@ SecurityApplicationProxy::SecurityApplicationProxy(BusAttachment& bus, const cha
     {
         /* Add org.alljoyn.Bus.Security.ClaimableApplication interface */
         const InterfaceDescription* intf = bus.GetInterface(org::alljoyn::Bus::Security::ClaimableApplication::InterfaceName);
-        assert(intf);
+        QCC_ASSERT(intf);
         status = AddInterface(*intf);
         if (status != ER_OK) {
             QCC_LogError(status, ("Failed to add the %s interface to the %s", org::alljoyn::Bus::Security::ClaimableApplication::InterfaceName, __FUNCTION__));
@@ -51,7 +55,7 @@ SecurityApplicationProxy::SecurityApplicationProxy(BusAttachment& bus, const cha
     {
         /* Add org.alljoyn.Bus.Security.ManagedApplication interface */
         const InterfaceDescription* intf = bus.GetInterface(org::alljoyn::Bus::Security::ManagedApplication::InterfaceName);
-        assert(intf);
+        QCC_ASSERT(intf);
         status = AddInterface(*intf);
         if (status != ER_OK) {
             QCC_LogError(status, ("Failed to add the %s interface to the %s", org::alljoyn::Bus::Security::ManagedApplication::InterfaceName, __FUNCTION__));
@@ -86,6 +90,10 @@ static bool GetStatusBasedOnErrorName(Message& reply, QStatus& status)
         status = ER_CERTIFICATE_NOT_FOUND;
     } else if (strcmp(reply->GetErrorName(), PermissionMgmtObj::ERROR_DUPLICATE_CERTIFICATE) == 0) {
         status = ER_DUPLICATE_CERTIFICATE;
+    } else if (strcmp(reply->GetErrorName(), PermissionMgmtObj::ERROR_MANAGEMENT_ALREADY_STARTED) == 0) {
+        status = ER_MANAGEMENT_ALREADY_STARTED;
+    } else if (strcmp(reply->GetErrorName(), PermissionMgmtObj::ERROR_MANAGEMENT_NOT_STARTED) == 0) {
+        status = ER_MANAGEMENT_NOT_STARTED;
     } else if (strcmp(reply->GetErrorName(), org::alljoyn::Bus::ErrorName) == 0 && reply->GetArg(1)) {
         status = static_cast<QStatus>(reply->GetArg(1)->v_uint16);
     } else {
@@ -236,17 +244,20 @@ QStatus SecurityApplicationProxy::GetClaimCapabilityAdditionalInfo(PermissionCon
 QStatus SecurityApplicationProxy::Claim(const qcc::KeyInfoNISTP256& certificateAuthority,
                                         const qcc::GUID128& adminGroupId,
                                         const qcc::KeyInfoNISTP256& adminGroup,
-                                        const qcc::IdentityCertificate* identityCertChain, size_t identityCertChainSize,
-                                        const PermissionPolicy::Rule* manifest, size_t manifestSize)
+                                        const qcc::CertificateX509* identityCertChain, size_t identityCertChainCount,
+                                        const Manifest* manifests, size_t manifestCount)
 {
     QCC_DbgTrace(("SecurityApplicationProxy::%s", __FUNCTION__));
     QStatus status = ER_OK;
 
     Message reply(GetBusAttachment());
 
-    if ((identityCertChain == NULL) || (manifest == NULL)) {
+    if ((identityCertChain == NULL) || (manifestCount < 1)) {
         return ER_INVALID_DATA;
     }
+    if (identityCertChain[0].GetType() != qcc::CertificateX509::IDENTITY_CERTIFICATE) {
+        QCC_DbgHLPrintf(("Leaf certificate in identity chain is not of type IDENTITY_CERTIFICATE. This is not recommended."));
+    }
     MsgArg inputs[7];
     qcc::KeyInfoNISTP256 caKeyInfo(certificateAuthority);
     KeyInfoHelper::KeyInfoNISTP256PubKeyToMsgArg(caKeyInfo, inputs[0]);
@@ -260,40 +271,43 @@ QStatus SecurityApplicationProxy::Claim(const qcc::KeyInfoNISTP256& certificateA
     KeyInfoHelper::KeyInfoNISTP256PubKeyToMsgArg(adminGroupKeyInfo, inputs[3]);
     KeyInfoHelper::KeyInfoKeyIdToMsgArg(adminGroupKeyInfo, inputs[4]);
 
-    MsgArg* identityArgs = NULL;
-    if (identityCertChainSize == 0) {
+    std::unique_ptr<MsgArg[]> identityArgs;
+    if (identityCertChainCount == 0) {
         status = inputs[5].Set("a(yay)", 0, NULL);
         if (ER_OK != status) {
-            goto Exit;
+            return status;
         }
     } else {
-        identityArgs = new MsgArg[identityCertChainSize];
-        for (size_t cnt = 0; cnt < identityCertChainSize; cnt++) {
+        identityArgs.reset(new (std::nothrow) MsgArg[identityCertChainCount]);
+        if (nullptr == identityArgs.get()) {
+            return ER_OUT_OF_MEMORY;
+        }
+        for (size_t cnt = 0; cnt < identityCertChainCount; cnt++) {
             qcc::String der;
             status = identityCertChain[cnt].EncodeCertificateDER(der);
             if (ER_OK != status) {
-                goto Exit;
+                return status;
             }
             status = identityArgs[cnt].Set("(yay)", qcc::CertificateX509::ENCODING_X509_DER, der.size(), der.data());
             if (ER_OK != status) {
-                goto Exit;
+                return status;
             }
             identityArgs[cnt].Stabilize();
         }
-        status = inputs[5].Set("a(yay)", identityCertChainSize, identityArgs);
+        status = inputs[5].Set("a(yay)", identityCertChainCount, identityArgs.get());
         if (ER_OK != status) {
-            goto Exit;
+            return status;
         }
     }
-    if (manifestSize == 0) {
-        status = inputs[6].Set("a(ssa(syy))", 0, NULL);
+    if (manifestCount == 0) {
+        status = inputs[6].Set(_Manifest::s_MsgArgArraySignature, 0, NULL);
         if (ER_OK != status) {
-            goto Exit;
+            return status;
         }
     } else {
-        status = PermissionPolicy::GenerateRules(manifest, manifestSize, inputs[6]);
+        status = _Manifest::GetArrayMsgArg(manifests, manifestCount, inputs[6]);
         if (ER_OK != status) {
-            goto Exit;
+            return status;
         }
     }
 
@@ -304,8 +318,28 @@ QStatus SecurityApplicationProxy::Claim(const qcc::KeyInfoNISTP256& certificateA
         }
     }
 
-Exit:
-    delete [] identityArgs;
+    return status;
+}
+
+QStatus SecurityApplicationProxy::Claim(const qcc::KeyInfoNISTP256& certificateAuthority,
+                                        const qcc::GUID128& adminGroupId,
+                                        const qcc::KeyInfoNISTP256& adminGroup,
+                                        const qcc::CertificateX509* identityCertChain, size_t identityCertChainCount,
+                                        AJ_PCSTR* manifestsXmls, size_t manifestsCount)
+{
+    QCC_DbgTrace(("SecurityApplicationProxy::%s", __FUNCTION__));
+
+    std::vector<Manifest> manifests;
+    QStatus status = ExtractManifests(manifestsXmls, manifestsCount, manifests);
+
+    if (ER_OK == status) {
+        status = Claim(certificateAuthority,
+                       adminGroupId,
+                       adminGroup,
+                       identityCertChain, identityCertChainCount,
+                       manifests.data(), manifests.size());
+    }
+
     return status;
 }
 
@@ -338,37 +372,51 @@ QStatus SecurityApplicationProxy::Reset()
     return status;
 }
 
-QStatus SecurityApplicationProxy::UpdateIdentity(const qcc::IdentityCertificate* identityCertificateChain, size_t identityCertificateChainSize,
-                                                 const PermissionPolicy::Rule* manifest, size_t manifestSize)
+QStatus SecurityApplicationProxy::UpdateIdentity(const qcc::CertificateX509* identityCertificateChain, size_t identityCertificateChainCount,
+                                                 const Manifest* manifests, size_t manifestCount)
 {
     QCC_DbgTrace(("SecurityApplicationProxy::%s", __FUNCTION__));
 
-    if ((identityCertificateChain == NULL) || (identityCertificateChainSize == 0) || (manifest == NULL) || (manifestSize == 0)) {
+    if ((identityCertificateChain == NULL) || (identityCertificateChainCount == 0) || (manifestCount < 1)) {
         return ER_INVALID_DATA;
     }
+    if (identityCertificateChain[0].GetType() != qcc::CertificateX509::IDENTITY_CERTIFICATE) {
+        QCC_DbgHLPrintf(("Leaf certificate in identity chain is not of type IDENTITY_CERTIFICATE. This is not recommended."));
+    }
     QStatus status = ER_OK;
     Message reply(GetBusAttachment());
     MsgArg inputs[2];
-    MsgArg* certArgs = new MsgArg[identityCertificateChainSize];
-    for (size_t cnt = 0; cnt < identityCertificateChainSize; cnt++) {
+    std::unique_ptr<MsgArg[]> certArgs(new (std::nothrow) MsgArg[identityCertificateChainCount]);
+    if (nullptr == certArgs.get()) {
+        return ER_OUT_OF_MEMORY;
+    }
+    for (size_t cnt = 0; cnt < identityCertificateChainCount; cnt++) {
         qcc::String der;
         status = identityCertificateChain[cnt].EncodeCertificateDER(der);
         if (ER_OK != status) {
-            goto Exit;
+            return status;
         }
         status = certArgs[cnt].Set("(yay)", qcc::CertificateX509::ENCODING_X509_DER, der.size(), der.data());
         if (ER_OK != status) {
-            goto Exit;
+            return status;
         }
         certArgs[cnt].Stabilize();
     }
-    status = inputs[0].Set("a(yay)", identityCertificateChainSize, certArgs);
+    status = inputs[0].Set("a(yay)", identityCertificateChainCount, certArgs.get());
     if (ER_OK != status) {
-        goto Exit;
+        return status;
     }
-    status = PermissionPolicy::GenerateRules(manifest, manifestSize, inputs[1]);
-    if (ER_OK != status) {
-        goto Exit;
+
+    if (manifestCount == 0) {
+        status = inputs[1].Set(_Manifest::s_MsgArgArraySignature, 0, NULL);
+        if (ER_OK != status) {
+            return status;
+        }
+    } else {
+        status = _Manifest::GetArrayMsgArg(manifests, manifestCount, inputs[1]);
+        if (ER_OK != status) {
+            return status;
+        }
     }
     status = MethodCall(org::alljoyn::Bus::Security::ManagedApplication::InterfaceName, "UpdateIdentity", inputs, 2, reply);
 
@@ -378,8 +426,22 @@ QStatus SecurityApplicationProxy::UpdateIdentity(const qcc::IdentityCertificate*
         }
     }
 
-Exit:
-    delete [] certArgs;
+    return status;
+}
+
+QStatus SecurityApplicationProxy::UpdateIdentity(const qcc::CertificateX509* identityCertificateChain, size_t identityCertificateChainCount,
+                                                 AJ_PCSTR* manifestsXmls, size_t manifestsCount)
+{
+    QCC_DbgTrace(("SecurityApplicationProxy::%s", __FUNCTION__));
+
+    std::vector<Manifest> manifests;
+    QStatus status = ExtractManifests(manifestsXmls, manifestsCount, manifests);
+
+    if (ER_OK == status) {
+        status = UpdateIdentity(identityCertificateChain, identityCertificateChainCount,
+                                manifests.data(), manifests.size());
+    }
+
     return status;
 }
 
@@ -404,6 +466,21 @@ QStatus SecurityApplicationProxy::UpdatePolicy(const PermissionPolicy& policy)
     return status;
 }
 
+QStatus SecurityApplicationProxy::UpdatePolicy(AJ_PCSTR policyXml)
+{
+    QCC_DbgTrace(("SecurityApplicationProxy::%s", __FUNCTION__));
+    QStatus status;
+    PermissionPolicy policy;
+
+    status = XmlPoliciesConverter::FromXml(policyXml, policy);
+
+    if (ER_OK == status) {
+        status = UpdatePolicy(policy);
+    }
+
+    return status;
+}
+
 QStatus SecurityApplicationProxy::ResetPolicy()
 {
     QCC_DbgTrace(("SecurityApplicationProxy::%s", __FUNCTION__));
@@ -419,15 +496,19 @@ QStatus SecurityApplicationProxy::ResetPolicy()
     return status;
 }
 
-QStatus SecurityApplicationProxy::InstallMembership(const qcc::MembershipCertificate* certificateChain, size_t certificateChainSize)
+QStatus SecurityApplicationProxy::InstallMembership(const qcc::CertificateX509* certificateChain, size_t certificateChainCount)
 {
     QCC_DbgTrace(("SecurityApplicationProxy::%s", __FUNCTION__));
     QStatus status = ER_OK;
     Message reply(GetBusAttachment());
 
+    if ((certificateChainCount >= 0) && (certificateChain[0].GetType() != qcc::CertificateX509::MEMBERSHIP_CERTIFICATE)) {
+        QCC_DbgHLPrintf(("Leaf certificate in membership chain is not of type MEMBERSHIP_CERTIFICATE. This is not recommended."));
+    }
+
     MsgArg inputs[1];
-    MsgArg* certArgs = new MsgArg[certificateChainSize];
-    for (size_t cnt = 0; cnt < certificateChainSize; cnt++) {
+    MsgArg* certArgs = new MsgArg[certificateChainCount];
+    for (size_t cnt = 0; cnt < certificateChainCount; cnt++) {
         qcc::String der;
         status = certificateChain[cnt].EncodeCertificateDER(der);
         if (ER_OK != status) {
@@ -439,7 +520,7 @@ QStatus SecurityApplicationProxy::InstallMembership(const qcc::MembershipCertifi
         }
         certArgs[cnt].Stabilize();
     }
-    status = inputs[0].Set("a(yay)", certificateChainSize, certArgs);
+    status = inputs[0].Set("a(yay)", certificateChainCount, certArgs);
     if (ER_OK != status) {
         goto Exit;
     }
@@ -489,6 +570,31 @@ Exit:
     return status;
 }
 
+QStatus SecurityApplicationProxy::InstallManifests(const Manifest* manifests, size_t manifestCount)
+{
+    QCC_DbgTrace(("SecurityApplicationProxy::%s", __FUNCTION__));
+    QStatus status = ER_OK;
+
+    Message reply(GetBusAttachment());
+
+    MsgArg inputs[1];
+
+    status = _Manifest::GetArrayMsgArg(manifests, manifestCount, inputs[0]);
+    if (ER_OK != status) {
+        QCC_LogError(status, ("Could not get MsgArg for manifests"));
+        return status;
+    }
+
+    status = MethodCall(org::alljoyn::Bus::Security::ManagedApplication::InterfaceName, "InstallManifests", inputs, ArraySize(inputs), reply);
+    if (ER_OK != status) {
+        if (!GetStatusBasedOnErrorName(reply, status)) {
+            QCC_LogError(status, ("SecurityApplicationProxy::%s error %s", __FUNCTION__, reply->GetErrorDescription().c_str()));
+        }
+    }
+
+    return status;
+}
+
 QStatus SecurityApplicationProxy::GetManagedApplicationVersion(uint16_t& version)
 {
     QCC_DbgTrace(("SecurityApplicationProxy::%s", __FUNCTION__));
@@ -503,7 +609,7 @@ QStatus SecurityApplicationProxy::GetManagedApplicationVersion(uint16_t& version
     return status;
 }
 
-QStatus SecurityApplicationProxy::MsgArgToIdentityCertChain(const MsgArg& arg, qcc::IdentityCertificate* certs, size_t expectedSize)
+QStatus SecurityApplicationProxy::MsgArgToIdentityCertChain(const MsgArg& arg, qcc::CertificateX509* certs, size_t expectedSize)
 {
     size_t certChainSize = 0;
     MsgArg* certArgs;
@@ -618,13 +724,15 @@ QStatus SecurityApplicationProxy::GetIdentity(MsgArg& identityCertificate)
     return status;
 }
 
-QStatus SecurityApplicationProxy::GetManifest(MsgArg& manifest)
+QStatus SecurityApplicationProxy::GetManifests(std::vector<Manifest>& manifests)
 {
     QCC_DbgTrace(("SecurityApplicationProxy::%s", __FUNCTION__));
     QStatus status = ER_OK;
 
+    manifests.clear();
+
     MsgArg arg;
-    status = GetProperty(org::alljoyn::Bus::Security::ManagedApplication::InterfaceName, "Manifest", arg);
+    status = GetProperty(org::alljoyn::Bus::Security::ManagedApplication::InterfaceName, "Manifests", arg);
     if (ER_OK == status) {
         /* GetProperty returns a variant wrapper */
         MsgArg* resultArg;
@@ -632,13 +740,53 @@ QStatus SecurityApplicationProxy::GetManifest(MsgArg& manifest)
         if (ER_OK != status) {
             return status;
         }
-        manifest = *resultArg;
-        manifest.Stabilize();
+        MsgArg* signedManifestArg;
+        size_t signedManifestCount;
+        status = resultArg->Get(_Manifest::s_MsgArgArraySignature, &signedManifestCount, &signedManifestArg);
+        if (ER_OK != status) {
+            return status;
+        }
+        /* Potentially save a reallocation for each push_back later by reserving now. */
+        manifests.reserve(signedManifestCount);
+        for (size_t i = 0; i < signedManifestCount; i++) {
+            Manifest manifest;
+            status = manifest->SetFromMsgArg(signedManifestArg[i]);
+            if (ER_OK != status) {
+                manifests.clear();
+                return status;
+            }
+            manifests.push_back(std::move(manifest));
+        }
     }
 
     return status;
 }
 
+QStatus SecurityApplicationProxy::GetManifestTemplate(AJ_PSTR* manifestTemplateXml)
+{
+    QCC_DbgTrace(("SecurityApplicationProxy::%s", __FUNCTION__));
+
+    QStatus status;
+    MsgArg argManifestTemplate;
+    PermissionPolicy::Rule* rules = nullptr;
+    size_t rulesCount = 0;
+
+    *manifestTemplateXml = nullptr;
+    status = GetManifestTemplate(argManifestTemplate);
+
+    if (ER_OK == status) {
+        status = PermissionPolicy::ParseRules(argManifestTemplate, &rules, &rulesCount);
+    }
+
+    if (ER_OK == status) {
+        status = XmlRulesConverter::RulesToXml(rules, rulesCount, manifestTemplateXml);
+    }
+
+    delete[] rules;
+
+    return status;
+}
+
 QStatus SecurityApplicationProxy::GetIdentityCertificateId(qcc::String& serial, qcc::KeyInfoNISTP256& issuerKeyInfo)
 {
     QCC_DbgTrace(("SecurityApplicationProxy::%s", __FUNCTION__));
@@ -752,4 +900,51 @@ QStatus SecurityApplicationProxy::GetMembershipSummaries(MsgArg& membershipSumma
     return status;
 }
 
+QStatus SecurityApplicationProxy::StartManagement()
+{
+    QCC_DbgTrace(("SecurityApplicationProxy::%s", __FUNCTION__));
+    Message reply(GetBusAttachment());
+    QStatus status = MethodCall(org::alljoyn::Bus::Security::ManagedApplication::InterfaceName, "StartManagement", nullptr, 0, reply);
+    if (ER_OK != status) {
+        if (!GetStatusBasedOnErrorName(reply, status)) {
+            QCC_LogError(status, ("SecurityApplicationProxy::%s error %s", __FUNCTION__, reply->GetErrorDescription().c_str()));
+        }
+    }
+    return status;
+}
+
+QStatus SecurityApplicationProxy::EndManagement()
+{
+    QCC_DbgTrace(("SecurityApplicationProxy::%s", __FUNCTION__));
+    Message reply(GetBusAttachment());
+    QStatus status = MethodCall(org::alljoyn::Bus::Security::ManagedApplication::InterfaceName, "EndManagement", nullptr, 0, reply);
+    if (ER_OK != status) {
+        if (!GetStatusBasedOnErrorName(reply, status)) {
+            QCC_LogError(status, ("SecurityApplicationProxy::%s error %s", __FUNCTION__, reply->GetErrorDescription().c_str()));
+        }
+    }
+    return status;
+}
+
+QStatus SecurityApplicationProxy::ExtractManifests(AJ_PCSTR* manifestsXmls, size_t manifestsCount, std::vector<Manifest>& manifests)
+{
+    QCC_DbgTrace(("SecurityApplicationProxy::%s", __FUNCTION__));
+
+    QStatus status = ER_OK;
+
+    for (size_t index = 0; index < manifestsCount; index++) {
+        Manifest manifest;
+
+        status = XmlManifestConverter::XmlToManifest(manifestsXmls[index], manifest);
+
+        if (ER_OK != status) {
+            break;
+        }
+
+        manifests.push_back(manifest);
+    }
+
+    return status;
+}
+
 } // end namespace ajn
