@@ -22,7 +22,6 @@
 
 #include <qcc/platform.h>
 
-#include <assert.h>
 
 #include <qcc/Debug.h>
 #include <qcc/String.h>
@@ -62,25 +61,13 @@ const qcc::String& GetSecureAnnotation(const XmlElement* elem)
     return qcc::String::Empty;
 }
 
-bool GetDescription(const XmlElement* elem, qcc::String& description)
-{
-    vector<XmlElement*>::const_iterator it = elem->GetChildren().begin();
-    for (; it != elem->GetChildren().end(); it++) {
-        if ((*it)->GetName().compare("description")) {
-            description = (*it)->GetContent();
-            return true;
-        }
-    }
-
-    return false;
-}
-
 QStatus XmlHelper::ParseInterface(const XmlElement* elem, ProxyBusObject* obj)
 {
     QStatus status = ER_OK;
     InterfaceSecurityPolicy secPolicy;
+    const qcc::String docString("org.alljoyn.Bus.DocString");
 
-    assert(elem->GetName() == "interface");
+    QCC_ASSERT(elem->GetName() == "interface");
 
     qcc::String ifName = elem->GetAttribute("name");
     if (!IsLegalInterfaceName(ifName.c_str())) {
@@ -141,12 +128,24 @@ QStatus XmlHelper::ParseInterface(const XmlElement* elem, ProxyBusObject* obj)
                 bool isSessionlessSignal = false;
                 bool isUnicastSignal = false;
                 bool isGlobalBroadcastSignal = false;
-
-                /* Iterate over member children */
                 const vector<XmlElement*>& argChildren = ifChildElem->GetChildren();
                 vector<XmlElement*>::const_iterator argIt = argChildren.begin();
                 map<qcc::String, qcc::String> argDescriptions;
+                map<pair<qcc::String, qcc::String>, qcc::String> argAnnotations;
                 qcc::String memberDescription;
+
+                if (isSignal) {
+                    const qcc::String& sessioncastStr = ifChildElem->GetAttribute("sessioncast");
+                    isSessioncastSignal = (sessioncastStr == "true");
+                    const qcc::String& sessionlessStr = ifChildElem->GetAttribute("sessionless");
+                    isSessionlessSignal = (sessionlessStr == "true");
+                    const qcc::String& unicastStr = ifChildElem->GetAttribute("unicast");
+                    isUnicastSignal = (unicastStr == "true");
+                    const qcc::String& globalBroadcastStr = ifChildElem->GetAttribute("globalbroadcast");
+                    isGlobalBroadcastSignal = (globalBroadcastStr == "true");
+                }
+
+                /* Iterate over member children */
                 while ((ER_OK == status) && (argIt != argChildren.end())) {
                     const XmlElement* argElem = *argIt++;
                     if (argElem->GetName() == "arg") {
@@ -167,9 +166,26 @@ QStatus XmlHelper::ParseInterface(const XmlElement* elem, ProxyBusObject* obj)
                             isArgNamesEmpty = false;
                             argNames += nameAtt;
 
-                            qcc::String description;
-                            if (GetDescription(argElem, description)) {
-                                argDescriptions.insert(pair<qcc::String, qcc::String>(nameAtt, description));
+                            /* Add argument annotations and descriptions */
+                            vector<XmlElement*>::const_iterator it = argElem->GetChildren().begin();
+                            for (; it != argElem->GetChildren().end(); it++) {
+                                if ((*it)->GetName() == "annotation") {
+                                    const qcc::String& argNameAtt = (*it)->GetAttribute("name");
+                                    const qcc::String& argValueAtt = (*it)->GetAttribute("value");
+                                    pair<qcc::String, qcc::String> item(nameAtt, argNameAtt);
+                                    argAnnotations[item] = argValueAtt;
+                                    if ((*it)->GetAttribute("name").compare_std(0, docString.length(), docString) == 0) {
+                                        argDescriptions.insert(pair<qcc::String, qcc::String>(nameAtt, (*it)->GetAttribute("value")));
+                                    }
+                                } else if ((*it)->GetName() == "description") {
+                                    const qcc::String& desc = (*it)->GetContent();
+                                    argDescriptions.insert(pair<qcc::String, qcc::String>(nameAtt, desc));
+                                    const qcc::String& lang = (*it)->GetAttribute("language");
+                                    if (!lang.empty()) {
+                                        const pair<qcc::String, qcc::String> item(nameAtt, docString + "." + lang);
+                                        argAnnotations[item] = desc;
+                                    }
+                                }
                             }
                         }
 
@@ -181,21 +197,34 @@ QStatus XmlHelper::ParseInterface(const XmlElement* elem, ProxyBusObject* obj)
                     } else if (argElem->GetName() == "annotation") {
                         const qcc::String& nameAtt = argElem->GetAttribute("name");
                         const qcc::String& valueAtt = argElem->GetAttribute("value");
+
+                        /* Treat annotation DocString as description */
+                        if (nameAtt.compare_std(0, docString.length(), docString) == 0) {
+                            memberDescription = valueAtt;
+                        }
                         annotations[nameAtt] = valueAtt;
+
+                        /* Unified XML signal emission behaviors */
+                        if (isSignal) {
+                            if (nameAtt == "org.alljoyn.Bus.Signal.Sessionless") {
+                                isSessionlessSignal = (valueAtt == "true");
+                            } else if (nameAtt == "org.alljoyn.Bus.Signal.Sessioncast") {
+                                isSessioncastSignal = (valueAtt == "true");
+                            } else if (nameAtt == "org.alljoyn.Bus.Signal.Unicast") {
+                                isUnicastSignal = (valueAtt == "true");
+                            } else if (nameAtt == "org.alljoyn.Bus.Signal.GlobalBroadcast") {
+                                isGlobalBroadcastSignal = (valueAtt == "true");
+                            }
+                        }
                     } else if (argElem->GetName() == "description") {
                         memberDescription = argElem->GetContent();
+                        /* Add description to annotation */
+                        const qcc::String& lang = argElem->GetAttribute("language");
+                        if (!lang.empty()) {
+                            const qcc::String nameAtt = docString + "." + lang;
+                            annotations[nameAtt] = memberDescription;
+                        }
                     }
-
-                }
-                if (isSignal) {
-                    const qcc::String& sessioncastStr = ifChildElem->GetAttribute("sessioncast");
-                    isSessioncastSignal = (sessioncastStr == "true");
-                    const qcc::String& sessionlessStr = ifChildElem->GetAttribute("sessionless");
-                    isSessionlessSignal = (sessionlessStr == "true");
-                    const qcc::String& unicastStr = ifChildElem->GetAttribute("unicast");
-                    isUnicastSignal = (unicastStr == "true");
-                    const qcc::String& globalBroadcastStr = ifChildElem->GetAttribute("globalbroadcast");
-                    isGlobalBroadcastSignal = (globalBroadcastStr == "true");
                 }
 
                 /* Add the member */
@@ -231,6 +260,10 @@ QStatus XmlHelper::ParseInterface(const XmlElement* elem, ProxyBusObject* obj)
                     for (std::map<String, String>::const_iterator it = argDescriptions.begin(); it != argDescriptions.end(); it++) {
                         intf.SetArgDescription(memberName.c_str(), it->first.c_str(), it->second.c_str());
                     }
+
+                    for (std::map<std::pair<qcc::String, qcc::String>, qcc::String>::const_iterator it = argAnnotations.begin(); it != argAnnotations.end(); it++) {
+                        intf.AddArgAnnotation(memberName.c_str(), it->first.first.c_str(), it->first.second, it->second);
+                    }
                 }
             } else {
                 status = ER_BUS_BAD_MEMBER_NAME;
@@ -258,26 +291,48 @@ QStatus XmlHelper::ParseInterface(const XmlElement* elem, ProxyBusObject* obj)
                 }
                 status = intf.AddProperty(memberName.c_str(), sig.c_str(), access);
 
-                // add Property annotations
+                // add Property annotations and descriptions
                 const vector<XmlElement*>& argChildren = ifChildElem->GetChildren();
                 vector<XmlElement*>::const_iterator argIt = argChildren.begin();
                 while ((ER_OK == status) && (argIt != argChildren.end())) {
                     const XmlElement* argElem = *argIt++;
-                    status = intf.AddPropertyAnnotation(memberName, argElem->GetAttribute("name"), argElem->GetAttribute("value"));
-                }
-
-                qcc::String description;
-                if (GetDescription(ifChildElem, description)) {
-                    intf.SetPropertyDescription(memberName.c_str(), description.c_str());
+                    if (argElem->GetName() == "annotation") {
+                        if (argElem->GetAttribute("name").compare_std(0, docString.length(), docString) == 0) {
+                            intf.SetPropertyDescription(memberName.c_str(), argElem->GetAttribute("value").c_str());
+                        }
+                        status = intf.AddPropertyAnnotation(memberName, argElem->GetAttribute("name"), argElem->GetAttribute("value"));
+                    } else if (argElem->GetName() == "description") {
+                        const qcc::String& content = argElem->GetContent();
+                        const qcc::String& lang = argElem->GetAttribute("language");
+                        if (!lang.empty()) {
+                            intf.AddPropertyAnnotation(memberName, docString + "." + lang, content);
+                        }
+                        status = intf.SetPropertyDescription(memberName.c_str(), content.c_str());
+                    }
                 }
             }
         } else if (ifChildName == "annotation") {
-            status = intf.AddAnnotation(ifChildElem->GetAttribute("name"), ifChildElem->GetAttribute("value"));
+            const qcc::String& nameAtt = ifChildElem->GetAttribute("name");
+            const qcc::String& valueAtt = ifChildElem->GetAttribute("value");
+
+            /* Treat annotation DocString as description (for example, "org.alljoyn.Bus.DocString.En") */
+            if (nameAtt.compare_std(0, docString.length(), docString) == 0) {
+                if ((nameAtt.length() > 26) && (nameAtt[25] == '.')) {
+                    qcc::String const& language = nameAtt.substr(26);
+                    intf.SetDescriptionLanguage(language.c_str());
+                }
+                intf.SetDescription(valueAtt.c_str());
+            }
+            status = intf.AddAnnotation(nameAtt, valueAtt);
         } else if (ifChildName == "description") {
             qcc::String const& description = ifChildElem->GetContent();
             qcc::String const& language = ifChildElem->GetAttribute("language");
             intf.SetDescriptionLanguage(language.c_str());
             intf.SetDescription(description.c_str());
+            if (!language.empty()) {
+                const qcc::String nameAtt = docString + "." + language;
+                intf.AddAnnotation(nameAtt, description);
+            }
         } else {
             status = ER_FAIL;
             QCC_LogError(status, ("Unknown element \"%s\" found in introspection data from %s", ifChildName.c_str(), ident));
@@ -323,7 +378,7 @@ QStatus XmlHelper::ParseNode(const XmlElement* root, ProxyBusObject* obj)
 {
     QStatus status = ER_OK;
 
-    assert(root->GetName() == "node");
+    QCC_ASSERT(root->GetName() == "node");
 
     if (GetSecureAnnotation(root) == "true") {
         if (obj) {
