@@ -242,7 +242,7 @@ int CDECL_CALL main(int argc, char** argv)
 
             // RULE-6: Official AllSeen Alliance names must start with "org.alljoyn", not
             // "org.allseen" or "org.allseenalliance".
-            if (interfaceName.compare(0, 11, "org.allseen") == 0) {
+            if (interfaceName.compare_std(0, 11, "org.allseen") == 0) {
                 g_warnings++;
                 printf("WARNING-6: interface name '%s' does not start with org.alljoyn\n", interfaceName.c_str());
             }
@@ -255,8 +255,8 @@ int CDECL_CALL main(int argc, char** argv)
         }
 
         const qcc::String secureValue = GetAnnotation(interfaceElement, "org.alljoyn.Bus.Secure");
-        if (!secureValue.empty() && (secureValue != "true") && (secureValue != "false")) {
-            // RULE-59: Secure annotation must be true or false, if present.
+        if (!secureValue.empty() && (secureValue != "true") && (secureValue != "inherit") && (secureValue != "off")) {
+            // RULE-59: Secure annotation must be true, inherit, or off, if present.
             g_errors++;
             printf("ERROR-59: interface '%s' org.alljoyn.Bus.Secure annotation has illegal value '%s'\n",
                    interfaceName.c_str(), secureValue.c_str());
@@ -502,7 +502,7 @@ int CDECL_CALL main(int argc, char** argv)
             }
 
             // RULE-12: The name of a signal should be phrased in terms of past tense.
-            if (signalName.compare(signalName.length() - 2, 2, "ed") != 0) {
+            if (signalName.compare_std(signalName.length() - 2, 2, "ed") != 0) {
                 g_info++;
                 printf("INFO-12: signal name '%s' in interface '%s' should end in a past tense verb\n",
                        signalName.c_str(), interfaceName.c_str());
@@ -613,16 +613,24 @@ int CDECL_CALL main(int argc, char** argv)
                        propertyName.c_str(), interfaceName.c_str());
             }
 
-            // RULE-14: They must consist solely of alphanumeric characters.
-            length = propertyName.find_first_not_of("ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz1234567890");
+            // RULE-14: They must consist solely of alphanumeric characters plus underscore.
+            length = propertyName.find_first_not_of("ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz1234567890_");
             if (length != qcc::String::npos) {
                 g_errors++;
                 printf("ERROR-14: property name '%s' in interface '%s' contains illegal character '%c'\n",
                        propertyName.c_str(), interfaceName.c_str(), propertyName.c_str()[length]);
             }
 
+            // RULE-65: Underscore should not be used either, even though it is legal in D-Bus.
+            length = propertyName.find_first_of('_');
+            if (length != qcc::String::npos) {
+                g_warnings++;
+                printf("WARNING-65: property name '%s' in interface '%s' should use UpperCamelCase, not underscore\n",
+                       propertyName.c_str(), interfaceName.c_str());
+            }
+
             // RULE-15: Property names should be nouns or predicates.
-            if (propertyName.compare(0, 3, "Get") == 0) {
+            if (propertyName.compare_std(0, 3, "Get") == 0) {
                 g_warnings++;
                 printf("WARNING-15: property name '%s' in interface '%s' should be a noun or predicate\n",
                        propertyName.c_str(), interfaceName.c_str());
@@ -704,8 +712,8 @@ int CDECL_CALL main(int argc, char** argv)
             }
         }
 
-        if ((interfaceName.compare(0, 12, "org.alljoyn.") == 0) &&
-            (interfaceName.compare(0, 20, "org.alljoyn.example.") != 0)) {
+        if ((interfaceName.compare_std(0, 12, "org.alljoyn.") == 0) &&
+            (interfaceName.compare_std(0, 20, "org.alljoyn.example.") != 0)) {
             if (versionElement == nullptr) {
                 // RULE-42: Every standardized Interface must include a uint16 (signature ''q'') property
                 // ''Version'' that indicates the implemented version of the Interface.
