@@ -17,7 +17,6 @@
  ******************************************************************************/
 #include <qcc/platform.h>
 #include <qcc/StringUtil.h>
-#include <assert.h>
 #include <signal.h>
 #include <stdio.h>
 #include <vector>
@@ -156,7 +155,7 @@ class MyBusListener : public BusListener, public SessionPortListener, public Ses
 
             g_msgBus->EnableConcurrentCallbacks();
 
-            QStatus status = g_msgBus->LeaveSession(sessionId);
+            status = g_msgBus->LeaveSession(sessionId);
 
             if (status == ER_OK) {
                 if (g_sleepBeforeRejoin) {
@@ -334,7 +333,7 @@ int CDECL_CALL main(int argc, char** argv)
 #endif
     const uint64_t startTime = GetTimestamp64(); // timestamp in milliseconds
     QStatus status = ER_OK;
-    TransportMask transportOpts = TRANSPORT_TCP;
+    TransportMask transportOpts = TRANSPORT_ANY;
     AboutObj* aboutObj = NULL;
     // echo command line to provide distinguishing information within multipoint session
     for (int i = 0; i < argc; i++) {
@@ -496,7 +495,7 @@ int CDECL_CALL main(int argc, char** argv)
                 aboutObj->Announce(SESSION_PORT, g_aboutData);
             } else {
                 /* Request a well-known name */
-                QStatus status = g_msgBus->RequestName(g_wellKnownName.c_str(), DBUS_NAME_FLAG_REPLACE_EXISTING | DBUS_NAME_FLAG_DO_NOT_QUEUE);
+                status = g_msgBus->RequestName(g_wellKnownName.c_str(), DBUS_NAME_FLAG_REPLACE_EXISTING | DBUS_NAME_FLAG_DO_NOT_QUEUE);
                 if (status != ER_OK) {
                     QCC_LogError(status, ("RequestName(%s) failed. ", g_wellKnownName.c_str()));
                     exit(-1);
