@@ -20,7 +20,6 @@
  ******************************************************************************/
 #include <qcc/platform.h>
 
-#include <assert.h>
 #include <signal.h>
 #include <stdio.h>
 #include <vector>
@@ -333,12 +332,12 @@ class LocalTestObject : public BusObject {
 
         static void AddResponse(LocalTestObject& lto, uint32_t delay, Message& msg, MsgArg* args, size_t argCount)
         {
-            Timespec future;
+            Timespec<MonotonicTime> future;
             GetTimeNow(&future);
             future += delay;
             DelayedResponseInfo* respInfo = new DelayedResponseInfo(msg, args, argCount);
             delayedResponseLock.Lock(MUTEX_CONTEXT);
-            delayedResponses.insert(pair<uint64_t, DelayedResponseInfo*>(future.GetAbsoluteMillis(), respInfo));
+            delayedResponses.insert(pair<uint64_t, DelayedResponseInfo*>(future.GetMillis(), respInfo));
             delayedResponseLock.Unlock(MUTEX_CONTEXT);
 
             delayedResponseThreadLock.Lock(MUTEX_CONTEXT);
@@ -369,9 +368,9 @@ class LocalTestObject : public BusObject {
 
             while (!done) {
                 delayedResponseLock.Lock(MUTEX_CONTEXT);
-                Timespec now;
+                Timespec<MonotonicTime> now;
                 GetTimeNow(&now);
-                uint64_t nowms = now.GetAbsoluteMillis();
+                uint64_t nowms = now.GetMillis();
                 multimap<uint64_t, DelayedResponseInfo*>::iterator it = delayedResponses.begin();
                 uint64_t nextms = it->first;
                 delayedResponseLock.Unlock(MUTEX_CONTEXT);
@@ -386,7 +385,7 @@ class LocalTestObject : public BusObject {
 
                 delayedResponseLock.Lock(MUTEX_CONTEXT);
                 GetTimeNow(&now);
-                nowms = now.GetAbsoluteMillis();
+                nowms = now.GetMillis();
                 it = delayedResponses.begin();
                 while ((it != delayedResponses.end()) && (nextms <= nowms)) {
                     Message msg = it->second->msg;
@@ -436,7 +435,7 @@ class LocalTestObject : public BusObject {
 
         /* Add the test interface to this object */
         const InterfaceDescription* regTestIntf = bus.GetInterface(::org::alljoyn::alljoyn_test::InterfaceName);
-        assert(regTestIntf);
+        QCC_ASSERT(regTestIntf);
         if (g_useAboutFeatureDiscovery) {
             AddInterface(*regTestIntf, ANNOUNCED);
         } else {
@@ -444,7 +443,7 @@ class LocalTestObject : public BusObject {
         }
         /* Add the values interface to this object */
         const InterfaceDescription* valuesIntf = bus.GetInterface(::org::alljoyn::alljoyn_test::values::InterfaceName);
-        assert(valuesIntf);
+        QCC_ASSERT(valuesIntf);
         if (g_useAboutFeatureDiscovery) {
             AddInterface(*valuesIntf, ANNOUNCED);
         } else {
@@ -453,7 +452,7 @@ class LocalTestObject : public BusObject {
 
         /* Register the signal handler with the bus */
         const InterfaceDescription::Member* member = regTestIntf->GetMember("my_signal");
-        assert(member);
+        QCC_ASSERT(member);
         status = bus.RegisterSignalHandler(this,
                                            static_cast<MessageReceiver::SignalHandler>(&LocalTestObject::SignalHandler),
                                            member,
