@@ -15,7 +15,6 @@
  ******************************************************************************/
 #include <qcc/platform.h>
 
-#include <assert.h>
 #include <signal.h>
 #include <stdio.h>
 #include <vector>
@@ -23,6 +22,7 @@
 #include <qcc/String.h>
 #include <qcc/Thread.h>
 
+#include <alljoyn/ApplicationStateListener.h>
 #include <alljoyn/BusAttachment.h>
 #include <alljoyn/DBusStd.h>
 #include <alljoyn/AllJoynStd.h>
@@ -32,6 +32,8 @@
 
 #include <alljoyn/Status.h>
 
+#include "BusInternal.h"
+
 /* Header files included for Google Test Framework */
 #include <gtest/gtest.h>
 #include "ajTestCommon.h"
@@ -41,19 +43,33 @@ using namespace std;
 using namespace qcc;
 using namespace ajn;
 
+class TestApplicationStateListener : public ApplicationStateListener {
+  public:
+    virtual void State(const char* busName, const qcc::KeyInfoNISTP256& publicKeyInfo, PermissionConfigurator::ApplicationState state)
+    {
+        QCC_UNUSED(busName);
+        QCC_UNUSED(publicKeyInfo);
+        QCC_UNUSED(state);
+    }
+};
+
 class BusAttachmentTest : public testing::Test {
   public:
     BusAttachment bus;
+    TestApplicationStateListener testListener;
+    TestApplicationStateListener* nullListener;
 
-    BusAttachmentTest() : bus("BusAttachmentTest", false) { };
+    BusAttachmentTest() :
+        bus("BusAttachmentTest", false),
+        nullListener(nullptr)
+    {
+        EXPECT_EQ(ER_OK, DeleteDefaultKeyStoreFile("BusAttachmentTest"));
+    };
 
     virtual void SetUp() {
-        QStatus status = ER_OK;
-        status = bus.Start();
-        ASSERT_EQ(ER_OK, status);
+        ASSERT_EQ(ER_OK, bus.Start());
         ASSERT_FALSE(bus.IsConnected());
-        status = bus.Connect(getConnectArg().c_str());
-        ASSERT_EQ(ER_OK, status);
+        ASSERT_EQ(ER_OK, bus.Connect(getConnectArg().c_str()));
         ASSERT_TRUE(bus.IsConnected());
     }
 
@@ -64,6 +80,75 @@ class BusAttachmentTest : public testing::Test {
 
 };
 
+TEST_F(BusAttachmentTest, shouldReturnErrorWhenRegisteringNullApplicationStateListener)
+{
+    EXPECT_EQ(ER_INVALID_ADDRESS, bus.RegisterApplicationStateListener(*nullListener));
+}
+
+TEST_F(BusAttachmentTest, shouldReturnErrorWhenUnregisteringNullApplicationStateListener)
+{
+    EXPECT_EQ(ER_INVALID_ADDRESS, bus.UnregisterApplicationStateListener(*nullListener));
+}
+
+TEST_F(BusAttachmentTest, shouldNotHaveMatchRuleWithoutRegisteredListener)
+{
+    EXPECT_EQ(ER_BUS_MATCH_RULE_NOT_FOUND, bus.RemoveMatch(BusAttachment::Internal::STATE_MATCH_RULE));
+}
+
+TEST_F(BusAttachmentTest, shouldNotAddMatchRuleWhenFailedToRegisterListener)
+{
+    ASSERT_EQ(ER_INVALID_ADDRESS, bus.RegisterApplicationStateListener(*nullListener));
+
+    EXPECT_EQ(ER_BUS_MATCH_RULE_NOT_FOUND, bus.RemoveMatch(BusAttachment::Internal::STATE_MATCH_RULE));
+}
+
+TEST_F(BusAttachmentTest, shouldReturnErrorWhenUnregisteringUnknownApplicationStateListener)
+{
+    EXPECT_EQ(ER_APPLICATION_STATE_LISTENER_NO_SUCH_LISTENER, bus.UnregisterApplicationStateListener(testListener));
+}
+
+TEST_F(BusAttachmentTest, shouldRegisterApplicationStateListener)
+{
+    EXPECT_EQ(ER_OK, bus.RegisterApplicationStateListener(testListener));
+}
+
+TEST_F(BusAttachmentTest, shouldAddMatchRuleWhenListenerWasRegistered)
+{
+    ASSERT_EQ(ER_OK, bus.RegisterApplicationStateListener(testListener));
+
+    EXPECT_EQ(ER_OK, bus.RemoveMatch(BusAttachment::Internal::STATE_MATCH_RULE));
+}
+
+TEST_F(BusAttachmentTest, shouldReturnErrorWhenRegisteringSameApplicationStateListenerTwice)
+{
+    ASSERT_EQ(ER_OK, bus.RegisterApplicationStateListener(testListener));
+
+    EXPECT_EQ(ER_APPLICATION_STATE_LISTENER_ALREADY_EXISTS, bus.RegisterApplicationStateListener(testListener));
+}
+
+TEST_F(BusAttachmentTest, shouldUnregisterApplicationStateListener)
+{
+    ASSERT_EQ(ER_OK, bus.RegisterApplicationStateListener(testListener));
+
+    EXPECT_EQ(ER_OK, bus.UnregisterApplicationStateListener(testListener));
+}
+
+TEST_F(BusAttachmentTest, shouldRemoveMatchRuleAfterUnregisteringListener)
+{
+    ASSERT_EQ(ER_OK, bus.RegisterApplicationStateListener(testListener));
+    ASSERT_EQ(ER_OK, bus.UnregisterApplicationStateListener(testListener));
+
+    EXPECT_EQ(ER_BUS_MATCH_RULE_NOT_FOUND, bus.RemoveMatch(BusAttachment::Internal::STATE_MATCH_RULE));
+}
+
+TEST_F(BusAttachmentTest, shouldNotRemoveMatchRuleWhenFailedToUnregisterListener)
+{
+    ASSERT_EQ(ER_OK, bus.RegisterApplicationStateListener(testListener));
+    ASSERT_EQ(ER_INVALID_ADDRESS, bus.UnregisterApplicationStateListener(*nullListener));
+
+    EXPECT_EQ(ER_OK, bus.RemoveMatch(BusAttachment::Internal::STATE_MATCH_RULE));
+}
+
 TEST_F(BusAttachmentTest, IsConnected)
 {
     EXPECT_TRUE(bus.IsConnected());
@@ -469,7 +554,7 @@ TEST_F(BusAttachmentTest, JoinLeaveSession) {
     // User defined sessionPort Number
     SessionPort sessionPort = 42;
 
-    // bindSessionPort new SessionPortListener
+    // BindSessionPort new SessionPortListener
     JoinSession_SessionPortListener sessionPortListener(&bus);
     status = bus.BindSessionPort(sessionPort, sessionOpts, sessionPortListener);
     EXPECT_EQ(ER_OK, status);
@@ -494,7 +579,7 @@ TEST_F(BusAttachmentTest, JoinLeaveSession) {
     JoinSession_BusListener busListener(&otherBus);
     otherBus.RegisterBusListener(busListener);
 
-    // find the AdvertisedName
+    // Find the AdvertisedName
     status = otherBus.FindAdvertisedName("org.alljoyn.bus.BusAttachmentTest.advertise");
     EXPECT_EQ(ER_OK, status);
 
@@ -533,6 +618,9 @@ TEST_F(BusAttachmentTest, JoinLeaveSession) {
 
     otherBus.Stop();
     otherBus.Join();
+
+    // Remove the session port listener allocated on the stack, before it gets destroyed
+    EXPECT_EQ(ER_OK, bus.UnbindSessionPort(sessionPort));
 }
 
 TEST_F(BusAttachmentTest, GetDBusProxyObj) {
@@ -646,6 +734,7 @@ TEST_F(BusAttachmentTest, BasicSecureConnection)
 {
     DefaultECDHEAuthListener al;
     BusAttachment otherBus("BusAttachmentOtherBus", false);
+    EXPECT_EQ(ER_OK, DeleteDefaultKeyStoreFile("BusAttachmentOtherBus"));
     ASSERT_EQ(ER_BUS_NOT_CONNECTED, otherBus.SecureConnection(bus.GetUniqueName().c_str()));
     otherBus.Start();
     // Use expect from now onward to make sure we reached the end of the function and do all clean-up
