@@ -69,6 +69,14 @@ class InMemoryKeyStoreListener : public KeyStoreListener {
         return status;
     }
 
+    /**
+     * Copy constructor
+     */
+    InMemoryKeyStoreListener(const InMemoryKeyStoreListener& other)
+    {
+        *this = other;
+    }
+
     /**
      * Assignment operator
      */
@@ -81,15 +89,6 @@ class InMemoryKeyStoreListener : public KeyStoreListener {
         return *this;
     }
 
-    /**
-     * Copy constructor
-     */
-    InMemoryKeyStoreListener(const InMemoryKeyStoreListener& other)
-    {
-        pwd = other.pwd;
-        CopySink(other.sink);
-    }
-
   private:
 
     QStatus CopySink(qcc::String& other)
