@@ -279,7 +279,7 @@ TEST_F(InterfaceTest, AddSignalToInterface_AfterItIsActivated) {
     status = g_msgBus->CreateInterface(myService.getAlljoynDummyInterfaceName2(), regTestIntf);
     ASSERT_EQ(ER_OK, status);
 
-    assert(regTestIntf);
+    QCC_ASSERT(regTestIntf);
     regTestIntf->Activate();
 
     /* Adding a signal to the activated interface -- Error */
@@ -379,7 +379,7 @@ TEST_F(InterfaceTest, AddSamePropertyToInterface_AndActivateItLater) {
     ASSERT_TRUE(myService.getobjectRegistered());
 
     status = g_msgBus->CreateInterface(myService.getAlljoynValuesDummyInterfaceName3(), valuesIntf);
-    assert(valuesIntf);
+    QCC_ASSERT(valuesIntf);
 
     /* Adding properties to the interface */
     status = valuesIntf->AddProperty("int_val", "i", PROP_ACCESS_RW);
@@ -535,3 +535,37 @@ TEST_F(InterfaceTest, ArgNamesTest) {
     ASSERT_TRUE(member != NULL);
     EXPECT_STREQ(",arg1", member->argNames.c_str());
 }
+
+TEST_F(InterfaceTest, ArgAnnotationsTest) {
+    QStatus status = ER_OK;
+
+    ASSERT_EQ(ER_OK, ServiceBusSetup());
+
+    InterfaceDescription* testIntf = NULL;
+    ServiceObject myService(*g_msgBus, "/org/alljoyn/xmlTest");
+
+    ASSERT_EQ(ER_OK, ServiceBusSetup());
+
+    /* Add org.alljoyn.alljoyn_test interface */
+    status = g_msgBus->CreateInterface("org.alljoyn.xmlTest", testIntf);
+    EXPECT_EQ(status, ER_OK);
+    ASSERT_TRUE(testIntf != NULL);
+
+    const char* methodName = "Method1";
+    const char* argName = "arg1";
+    const char* annotationName1 = "org.alljoyn.Bus.DocString.en";
+    const char* annotationName2 = "org.alljoyn.Bus.DocString.de";
+    const char* annotationValue1 = "The first argument";
+    const char* annotationValue2 = "DE: The first argument";
+    testIntf->AddMethod(methodName, NULL, "h", "arg1");
+    testIntf->AddArgAnnotation(methodName, argName, annotationName1, annotationValue1);
+    testIntf->AddArgAnnotation(methodName, argName, annotationName2, annotationValue2);
+    testIntf->AddMemberAnnotation("NoReply", org::freedesktop::DBus::AnnotateNoReply, "true");
+
+    qcc::String value;
+    ASSERT_TRUE(testIntf->GetArgAnnotation(methodName, argName, annotationName1, value));
+    EXPECT_STREQ(annotationValue1, value.c_str());
+    ASSERT_TRUE(testIntf->GetArgAnnotation(methodName, argName, annotationName2, value));
+    EXPECT_STREQ(annotationValue2, value.c_str());
+}
+
