@@ -71,7 +71,7 @@ void TestPermissionConfigurationListener::PolicyChanged()
     policyChangedReceived = true;
 }
 
-QStatus PermissionMgmtTestHelper::CreateIdentityCertChain(BusAttachment& caBus, BusAttachment& issuerBus, const qcc::String& serial, const qcc::String& subject, const ECCPublicKey* subjectPubKey, const qcc::String& alias, uint32_t expiredInSecs, qcc::IdentityCertificate* certChain, size_t chainCount, uint8_t* digest, size_t digestSize)
+QStatus PermissionMgmtTestHelper::CreateIdentityCertChain(BusAttachment& caBus, BusAttachment& issuerBus, const qcc::String& serial, const qcc::String& subject, const ECCPublicKey* subjectPubKey, const qcc::String& alias, uint32_t expiredInSecs, qcc::IdentityCertificate* certChain, size_t chainCount)
 {
     if (chainCount > 3) {
         return ER_INVALID_DATA;
@@ -130,9 +130,6 @@ QStatus PermissionMgmtTestHelper::CreateIdentityCertChain(BusAttachment& caBus,
     certChain[0].SetSubjectCN((const uint8_t*) subject.data(), subject.size());
     certChain[0].SetSubjectPublicKey(subjectPubKey);
     certChain[0].SetAlias(alias);
-    if (digest) {
-        certChain[0].SetDigest(digest, digestSize);
-    }
     certChain[0].SetValidity(&validity);
 
     /* use the issuer bus to sign the cert */
@@ -149,7 +146,61 @@ QStatus PermissionMgmtTestHelper::CreateIdentityCertChain(BusAttachment& caBus,
     return ER_OK;
 }
 
-QStatus PermissionMgmtTestHelper::CreateIdentityCert(BusAttachment& issuerBus, const qcc::String& serial, const qcc::String& subject, const ECCPublicKey* subjectPubKey, const qcc::String& alias, uint32_t expiredInSecs, qcc::IdentityCertificate& cert, uint8_t* digest, size_t digestSize, bool setEmptyAKI)
+QStatus PermissionMgmtTestHelper::CreateAllInclusiveManifest(Manifest& manifest)
+{
+    // All Inclusive manifest
+    const size_t manifestSize = 1;
+    PermissionPolicy::Rule manifestRules[manifestSize];
+    manifestRules[0].SetObjPath("*");
+    manifestRules[0].SetInterfaceName("*");
+    {
+        PermissionPolicy::Rule::Member member[1];
+        member[0].Set("*", PermissionPolicy::Rule::Member::NOT_SPECIFIED,
+                      PermissionPolicy::Rule::Member::ACTION_PROVIDE |
+                      PermissionPolicy::Rule::Member::ACTION_MODIFY |
+                      PermissionPolicy::Rule::Member::ACTION_OBSERVE);
+        manifestRules[0].SetMembers(1, member);
+    }
+    return manifest->SetRules(manifestRules, manifestSize);
+}
+
+QStatus PermissionMgmtTestHelper::SignManifests(BusAttachment& issuerBus, const qcc::CertificateX509& subjectCertificate, std::vector<Manifest>& manifests)
+{
+    for (Manifest manifest : manifests) {
+        QStatus status = SignManifest(issuerBus, subjectCertificate, manifest);
+        if (ER_OK != status) {
+            return status;
+        }
+    }
+
+    return ER_OK;
+}
+
+QStatus PermissionMgmtTestHelper::SignManifest(BusAttachment& issuerBus, const qcc::CertificateX509& subjectCertificate, Manifest& manifest)
+{
+    CredentialAccessor ca(issuerBus);
+    ECCPrivateKey privateKey;
+    QStatus status = ca.GetDSAPrivateKey(privateKey);
+    if (ER_OK != status) {
+        return status;
+    }
+
+    return manifest->ComputeThumbprintAndSign(subjectCertificate, &privateKey);
+}
+
+QStatus PermissionMgmtTestHelper::SignManifest(BusAttachment& issuerBus, const std::vector<uint8_t>& subjectThumbprint, Manifest& manifest)
+{
+    CredentialAccessor ca(issuerBus);
+    ECCPrivateKey privateKey;
+    QStatus status = ca.GetDSAPrivateKey(privateKey);
+    if (ER_OK != status) {
+        return status;
+    }
+
+    return manifest->Sign(subjectThumbprint, &privateKey);
+}
+
+QStatus PermissionMgmtTestHelper::CreateIdentityCert(BusAttachment& issuerBus, const qcc::String& serial, const qcc::String& subject, const ECCPublicKey* subjectPubKey, const qcc::String& alias, uint32_t expiredInSecs, qcc::IdentityCertificate& cert, bool setEmptyAKI)
 {
     qcc::GUID128 issuer(0);
     GetGUID(issuerBus, issuer);
@@ -162,9 +213,6 @@ QStatus PermissionMgmtTestHelper::CreateIdentityCert(BusAttachment& issuerBus, c
     cert.SetSubjectCN((const uint8_t*) subject.data(), subject.size());
     cert.SetSubjectPublicKey(subjectPubKey);
     cert.SetAlias(alias);
-    if (digest) {
-        cert.SetDigest(digest, digestSize);
-    }
     CertificateX509::ValidPeriod validity;
     BuildValidity(validity, expiredInSecs);
     cert.SetValidity(&validity);
@@ -196,10 +244,17 @@ QStatus PermissionMgmtTestHelper::CreateIdentityCert(BusAttachment& issuerBus, c
     return ER_OK;
 }
 
+QStatus PermissionMgmtTestHelper::SetCAFlagOnCert(BusAttachment& issuerBus, qcc::CertificateX509& certificate)
+{
+    certificate.SetCA(true);
+    PermissionConfigurator& pc = issuerBus.GetPermissionConfigurator();
+    return pc.SignCertificate(certificate);
+}
+
 QStatus PermissionMgmtTestHelper::CreateIdentityCert(BusAttachment& issuerBus, const qcc::String& serial, const qcc::String& subject, const ECCPublicKey* subjectPubKey, const qcc::String& alias, uint32_t expiredInSecs, qcc::String& der)
 {
     IdentityCertificate cert;
-    QStatus status = CreateIdentityCert(issuerBus, serial, subject, subjectPubKey, alias, expiredInSecs, cert, NULL, 0);
+    QStatus status = CreateIdentityCert(issuerBus, serial, subject, subjectPubKey, alias, expiredInSecs, cert);
     if (ER_OK != status) {
         return status;
     }
@@ -286,7 +341,7 @@ void BasePermissionMgmtTest::RegisterKeyStoreListeners()
 static void GenerateSecurityGroupKey(BusAttachment& bus, KeyInfoNISTP256& keyInfo)
 {
     PermissionConfigurator& pc = bus.GetPermissionConfigurator();
-    pc.GetSigningPublicKey(keyInfo);
+    EXPECT_EQ(ER_OK, pc.GetSigningPublicKey(keyInfo));
 }
 
 void BasePermissionMgmtTest::SetUp()
@@ -304,9 +359,7 @@ void BasePermissionMgmtTest::SetUp()
     EXPECT_EQ(ER_OK, status) << "  Actual Status: " << QCC_StatusText(status);
     RegisterKeyStoreListeners();
 
-    adminBus.RegisterApplicationStateListener(testASL);
-    status = adminBus.AddApplicationStateRule();
-    EXPECT_EQ(ER_OK, status) << "  Failed to show interest in session-less signal.  Actual Status: " << QCC_StatusText(status);
+    EXPECT_EQ(ER_OK, adminBus.RegisterApplicationStateListener(testASL));
 }
 
 void BasePermissionMgmtTest::TearDown()
@@ -345,10 +398,10 @@ void BasePermissionMgmtTest::PropertiesChanged(ProxyBusObject& obj, const char*
 void BasePermissionMgmtTest::GenerateCAKeys()
 {
     KeyInfoNISTP256 keyInfo;
-    adminBus.GetPermissionConfigurator().GetSigningPublicKey(keyInfo);
-    consumerBus.GetPermissionConfigurator().GetSigningPublicKey(keyInfo);
-    serviceBus.GetPermissionConfigurator().GetSigningPublicKey(keyInfo);
-    remoteControlBus.GetPermissionConfigurator().GetSigningPublicKey(keyInfo);
+    EXPECT_EQ(ER_OK, adminBus.GetPermissionConfigurator().GetSigningPublicKey(keyInfo));
+    EXPECT_EQ(ER_OK, consumerBus.GetPermissionConfigurator().GetSigningPublicKey(keyInfo));
+    EXPECT_EQ(ER_OK, serviceBus.GetPermissionConfigurator().GetSigningPublicKey(keyInfo));
+    EXPECT_EQ(ER_OK, remoteControlBus.GetPermissionConfigurator().GetSigningPublicKey(keyInfo));
     GenerateSecurityGroupKey(adminBus, adminAdminGroupAuthority);
     GenerateSecurityGroupKey(consumerBus, consumerAdminGroupAuthority);
 }
@@ -356,7 +409,9 @@ void BasePermissionMgmtTest::GenerateCAKeys()
 static AuthListener* GenAuthListener(const char* keyExchange) {
     if (strstr(keyExchange, "ECDHE_PSK")) {
         qcc::String psk("38347892FFBEF5B2442AEDE9E53C4B32");
-        return new DefaultECDHEAuthListener(reinterpret_cast<const uint8_t*>(psk.data()), psk.size());
+        DefaultECDHEAuthListener* authListener = new DefaultECDHEAuthListener();
+        PermissionMgmtTestHelper::CallDeprecatedSetPSK(authListener, reinterpret_cast<const uint8_t*>(psk.data()), psk.size());
+        return authListener;
     }
     return new DefaultECDHEAuthListener();
 }
@@ -492,7 +547,7 @@ void BasePermissionMgmtTest::SetApplicationStateSignalReceived(bool flag)
     testASL.signalApplicationStateReceived = flag;
 }
 
-const bool BasePermissionMgmtTest::GetApplicationStateSignalReceived()
+bool BasePermissionMgmtTest::GetApplicationStateSignalReceived() const
 {
     return testASL.signalApplicationStateReceived;
 }
@@ -502,7 +557,7 @@ void BasePermissionMgmtTest::SetFactoryResetReceived(bool flag)
     testPCL.factoryResetReceived = flag;
 }
 
-const bool BasePermissionMgmtTest::GetFactoryResetReceived()
+bool BasePermissionMgmtTest::GetFactoryResetReceived() const
 {
     return testPCL.factoryResetReceived;
 }
@@ -512,7 +567,7 @@ void BasePermissionMgmtTest::SetPolicyChangedReceived(bool flag)
     testPCL.policyChangedReceived = flag;
 }
 
-const bool BasePermissionMgmtTest::GetPolicyChangedReceived()
+bool BasePermissionMgmtTest::GetPolicyChangedReceived() const
 {
     return testPCL.policyChangedReceived;
 }
@@ -522,7 +577,7 @@ void BasePermissionMgmtTest::SetChannelChangedSignalReceived(bool flag)
     channelChangedSignalReceived = flag;
 }
 
-const bool BasePermissionMgmtTest::GetChannelChangedSignalReceived()
+bool BasePermissionMgmtTest::GetChannelChangedSignalReceived() const
 {
     return channelChangedSignalReceived;
 }
@@ -532,7 +587,7 @@ void BasePermissionMgmtTest::SetPropertiesChangedSignalReceived(bool flag)
     propertiesChangedSignalReceived = flag;
 }
 
-const bool BasePermissionMgmtTest::GetPropertiesChangedSignalReceived()
+bool BasePermissionMgmtTest::GetPropertiesChangedSignalReceived() const
 {
     return propertiesChangedSignalReceived;
 }
@@ -554,7 +609,7 @@ void BasePermissionMgmtTest::TVUp(const InterfaceDescription::Member* member, Me
     QCC_UNUSED(member);
     currentTVChannel++;
     MethodReply(msg, ER_OK);
-    TVChannelChanged(member, msg, true);
+    TVChannelChanged(member, msg, SEND_SIGNAL_SESSIONCAST);
 }
 
 void BasePermissionMgmtTest::TVDown(const InterfaceDescription::Member* member, Message& msg)
@@ -564,7 +619,7 @@ void BasePermissionMgmtTest::TVDown(const InterfaceDescription::Member* member,
         currentTVChannel--;
     }
     MethodReply(msg, ER_OK);
-    TVChannelChanged(member, msg, true);
+    TVChannelChanged(member, msg, SEND_SIGNAL_BROADCAST);
 }
 
 void BasePermissionMgmtTest::TVChannel(const InterfaceDescription::Member* member, Message& msg)
@@ -572,17 +627,20 @@ void BasePermissionMgmtTest::TVChannel(const InterfaceDescription::Member* membe
     QCC_UNUSED(member);
     MethodReply(msg, ER_OK);
     /* emit a signal */
-    TVChannelChanged(member, msg, false);
+    TVChannelChanged(member, msg, SEND_SIGNAL_UNICAST);
 }
 
-void BasePermissionMgmtTest::TVChannelChanged(const InterfaceDescription::Member* member, Message& msg, bool sendSessioncastSignal)
+void BasePermissionMgmtTest::TVChannelChanged(const InterfaceDescription::Member* member, Message& msg, SignalSendMethod sendMethod)
 {
     QCC_UNUSED(msg);
     /* emit a signal */
     MsgArg args[1];
     args[0].Set("u", currentTVChannel);
-    if (sendSessioncastSignal) {
+    if (sendMethod == SEND_SIGNAL_SESSIONCAST) {
         Signal(NULL, SESSION_ID_ALL_HOSTED, *member->iface->GetMember("ChannelChanged"), args, 1, 0, 0);
+    } else if (sendMethod == SEND_SIGNAL_BROADCAST) {
+        /* sending a broadcast signal */
+        Signal(NULL, 0, *member->iface->GetMember("ChannelChanged"), args, 1, 0, 0);
     } else {
         Signal(consumerBus.GetUniqueName().c_str(), 0, *member->iface->GetMember("ChannelChanged"), args, 1, 0, 0);
         Signal(remoteControlBus.GetUniqueName().c_str(), 0, *member->iface->GetMember("ChannelChanged"), args, 1, 0, 0);
@@ -1047,3 +1105,31 @@ QStatus BasePermissionMgmtTest::Set(const char* ifcName, const char* propName, M
     }
     return ER_BUS_NO_SUCH_PROPERTY;
 }
+
+void PermissionMgmtTestHelper::CallDeprecatedSetPSK(DefaultECDHEAuthListener* authListener, const uint8_t* pskBytes, size_t pskLength)
+{
+    /*
+     * This function suppresses compiler warnings when calling SetPSK, which is deprecated.
+     * ECHDE_PSK is deprecated as of 16.04, but we still test it, per the Alliance deprecation policy.
+     * ASACORE-2762 tracks removal of the ECDHE_PSK tests (and this function can be removed as a part of that work).
+     * https://jira.allseenalliance.org/browse/ASACORE-2762
+     */
+#ifdef __GNUC__
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
+#endif
+#if defined(QCC_OS_GROUP_WINDOWS)
+#pragma warning(push)
+#pragma warning(disable: 4996)
+#endif
+
+    QCC_VERIFY(ER_OK == authListener->SetPSK(pskBytes, pskLength));
+
+#if defined(QCC_OS_GROUP_WINDOWS)
+#pragma warning(pop)
+#endif
+#ifdef __GNUC__
+#pragma GCC diagnostic pop
+#endif
+
+}
\ No newline at end of file
