@@ -515,7 +515,7 @@ class DaemonRouterTest : public TestParamTuple {
 
     static String ToDot1(const String& name)
     {
-        return name.substr(0, name.find_last_of('.')) + ".1";
+        return name.substr(0, name.find_last_of_std('.')) + ".1";
     }
 
     virtual void SetUp()
@@ -601,7 +601,7 @@ class DaemonRouterTest : public TestParamTuple {
             policyDb->NameOwnerChanged(epInfo->name,
                                        &epInfo->name, SessionOpts::ALL_NAMES,
                                        NULL, SessionOpts::ALL_NAMES);
-            if (epInfo->name.compare(1, 7, "virtual") == 0) {
+            if (epInfo->name.compare_std(1, 7, "virtual") == 0) {
                 String name1 = ToDot1(epInfo->name);
                 policyDb->NameOwnerChanged(name1,
                                            &name1, SessionOpts::ALL_NAMES,
@@ -635,8 +635,8 @@ class DaemonRouterTest : public TestParamTuple {
                 String b2bname = GenUniqueName(ENDPOINT_TYPE_BUS2BUS, epInfo->id, epInfo->allow, epInfo->slsMatchRule);
                 TestEndpointInfo dot1Info(epInfo, true);
                 dot1Info->name = ToDot1(epInfo->name);
-                String b2bName = String(":bus2bus_") + epInfo->name.substr(9, epInfo->name.find_last_of('.') - 9);
-                map<StringMapKey, TestRemoteEndpoint>::iterator it = b2bEps.find(b2bName);
+                String b2bName = String(":bus2bus_") + epInfo->name.substr(9, epInfo->name.find_last_of_std('.') - 9);
+                map<std::string, TestRemoteEndpoint>::iterator it = b2bEps.find(b2bName);
                 assert(it != b2bEps.end());
                 TestRemoteEndpoint& trep = it->second;
                 trep->SetRemoteName(dot1Info->name);
@@ -651,7 +651,7 @@ class DaemonRouterTest : public TestParamTuple {
 
         case ENDPOINT_TYPE_BUS2BUS: {
                 TestRemoteEndpoint ep(epInfo);
-                b2bEps[epInfo->name.substr(0, epInfo->name.find_last_of('.'))] = ep;
+                b2bEps[epInfo->name.substr(0, epInfo->name.find_last_of_std('.'))] = ep;
                 bep = BusEndpoint::cast(ep);
                 break;
             }
@@ -787,7 +787,7 @@ class DaemonRouterTest : public TestParamTuple {
     Rule matchRule1;
     Rule matchRule2;
     list<BusEndpoint> epList;
-    map<StringMapKey, TestRemoteEndpoint> b2bEps;
+    map<std::string, TestRemoteEndpoint> b2bEps;
 
     static ConfigDB* configDb;
     static BusAttachment* bus;
@@ -1170,7 +1170,7 @@ INSTANTIATE_TEST_CASE_P(SendSignalsDirect,
                         Combine(ValuesIn(DaemonRouterTest::GetSrcDirectEpInfoList()),
                                 ValuesIn(DaemonRouterTest::GetDstDirectEpInfoList()),
                                 Values(MESSAGE_SIGNAL),
-                                Values(0, TEST_SESSION_ID),
+                                Values(0U, TEST_SESSION_ID),
                                 Values(MF_NONE, MF_SESSIONLESS, MF_GLOBAL_BROADCAST),
                                 PolicyDBMemberParams(),
                                 Values(SF_NONE)));
@@ -1194,7 +1194,7 @@ INSTANTIATE_TEST_CASE_P(SendSignalsCast,
                         Combine(ValuesIn(DaemonRouterTest::GetSrcEpInfoList()),
                                 Values(DaemonRouterTest::emptyDestInfo),
                                 Values(MESSAGE_SIGNAL),
-                                Values(0, TEST_SESSION_ID),
+                                Values(0U, TEST_SESSION_ID),
                                 Values(MF_NONE, MF_SESSIONLESS, MF_GLOBAL_BROADCAST),
                                 PolicyDBMemberParams(),
                                 Values(SF_NONE, SF_SLS_ONLY, SF_SELF_JOIN)));
@@ -1218,7 +1218,7 @@ INSTANTIATE_TEST_CASE_P(SendMethodCalls,
                         Combine(ValuesIn(DaemonRouterTest::GetSrcDirectEpInfoList()),
                                 ValuesIn(DaemonRouterTest::GetDstDirectEpInfoList()),
                                 Values(MESSAGE_METHOD_CALL),
-                                Values(0, TEST_SESSION_ID),
+                                Values(0U, TEST_SESSION_ID),
                                 Values(MF_NONE, MF_NO_REPLY_EXPECTED),
                                 PolicyDBMemberParams(),
                                 Values(SF_NONE)));
@@ -1244,7 +1244,7 @@ INSTANTIATE_TEST_CASE_P(SendMethodReplies,
                         Combine(ValuesIn(DaemonRouterTest::GetSrcDirectEpInfoList()),
                                 ValuesIn(DaemonRouterTest::GetDstDirectEpInfoList()),
                                 Values(MESSAGE_METHOD_RET),
-                                Values(0, TEST_SESSION_ID),
+                                Values(0U, TEST_SESSION_ID),
                                 Values(MF_NONE),
                                 Values(TEST_MEMBER),
                                 Values(SF_NONE)));
@@ -1267,7 +1267,7 @@ INSTANTIATE_TEST_CASE_P(SendErrors,
                         Combine(ValuesIn(DaemonRouterTest::GetSrcDirectEpInfoList()),
                                 ValuesIn(DaemonRouterTest::GetDstDirectEpInfoList()),
                                 Values(MESSAGE_ERROR),
-                                Values(0, TEST_SESSION_ID),
+                                Values(0U, TEST_SESSION_ID),
                                 Values(MF_NONE),
                                 PolicyDBErrorParams(),
                                 Values(SF_NONE)));
