@@ -1100,7 +1100,7 @@ public class BusAttachment {
     private native void nativeDisconnect();
 
     private native Status enablePeerSecurity(String authMechanisms,
-            AuthListenerInternal busAuthListener, String keyStoreFileName, Boolean isShared);
+            AuthListenerInternal busAuthListener, String keyStoreFileName, boolean isShared);
 
     private native Status registerBusObject(String objPath, BusObject busObj,
             InterfaceDescription[] busInterfaces, boolean secure,
@@ -1719,6 +1719,32 @@ public class BusAttachment {
      */
     public native Status reloadKeyStore();
 
+    /**
+     * Explicitly secure the connection to the remote peer. Peer-to-peer
+     * connections can only be secured if EnablePeerSecurity() was previously called on the bus
+     * attachment. If the peer-to-peer connection is already secure this
+     * function does nothing. Note that peer-to-peer connections are automatically secured when a
+     * method call requiring encryption is sent.
+     *
+     * This call causes messages to be sent on the bus, therefore it cannot be called within AllJoyn
+     * callbacks (method/signal/reply handlers or ObjectRegistered callbacks, etc.)
+     *
+     * @param  name       The unique name of the remote peer or NULL to secure the connections to all peers
+     *                        this BusAttachment is in session with.
+     * @param  forceAuth  If true, forces re-authentication even if the peer connection is already
+     *                        authenticated.
+     *
+     * @return
+     * <ul>
+     * <li>         - OK if the connection was secured or an error status indicating that the </li>
+     * <li>           connection could not be secured. </li>
+     * <li>         - BUS_NO_AUTHENTICATION_MECHANISM if BusAttachment::EnablePeerSecurity() has not been called. </li>
+     * <li>         - AUTH_FAIL if the attempt(s) to authenticate the peer failed. </li>
+     * <li>         - Other error status codes indicating a failure. </li>
+     * </ul>
+     */
+    public native Status secureConnection(String name, boolean forceAuth);
+
     /**
      * Registers a user-defined authentication listener class with a specific
      * default key store.
@@ -1850,11 +1876,31 @@ public class BusAttachment {
      * This method is typically used when a method, signal handler or other
      * AllJoyn callback needs to execute for a long period of time or when the
      * callback needs to make any kind of blocking call.
+     * </p>
      * <p>
      * This method MUST be called prior to making any non-asynchronous AllJoyn
      * remote procedure calls from within an AllJoyn callback. This includes
      * calls such as joinSession(), advertiseName(), cancelAdvertisedName(),
      * findAdvertisedName(), cancelFindAdvertisedName(), setLinkTimeout(), etc.
+     * </p>
+     * <p>
+     * enableConcurrentCallbacks doesn't take effect when a BusAttachment is
+     * created with just one thread. If the BusAttachment is created with just
+     * one thread,
+     * i.e. `BusAttachment busAttachment(appName, RemoteMessage.Receive, 1)` and
+     * the application developer attempts to make a blocking method call in a
+     * callback after invoking EnableConcurrentCallbacks(), the application will
+     * deadlock.
+     * </p>
+     * <p>
+     * For the same reason that enableConcurrentCallbacks cannot be used with
+     * just one thread, the maximum number of concurrent callbacks is limited
+     * to the value specified when creating the BusAttachment. If no concurrency
+     * value was chosen the DEFAULT_CONCURRENCY = 4. It is the application
+     * developers responsibility to make sure the maximum number of concurrent
+     * callbacks is not exceeded. If the maximum number is exceeded the
+     * application will deadlock.
+     * </p>
      */
     public native void enableConcurrentCallbacks();
 
