@@ -75,9 +75,9 @@ public class AboutIconProxyTest extends TestCase{
             Map<String, Variant> aboutData = new HashMap<String, Variant>();
             //nonlocalized values
             aboutData.put("AppId",  new Variant(new byte[] {1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16 }));
-            aboutData.put("DefaultLanguage",  new Variant(new String("en")));
+            aboutData.put("DefaultLanguage",  new Variant("en"));
             aboutData.put("DeviceId",  new Variant(new String("93c06771-c725-48c2-b1ff-6a2a59d445b8")));
-            aboutData.put("ModelNumber", new Variant(new String("A1B2C3")));
+            aboutData.put("ModelNumber", new Variant("A1B2C3"));
             aboutData.put("SupportedLanguages", new Variant(new String[] {"en", "es"}));
             aboutData.put("DateOfManufacture", new Variant(new String("2014-09-23")));
             aboutData.put("SoftwareVersion", new Variant(new String("1.0")));
@@ -88,12 +88,12 @@ public class AboutIconProxyTest extends TestCase{
             // If the language String is null or an empty string we return the
             // default language
             if ((language == null) || (language.length() == 0) || language.equalsIgnoreCase("en")) {
-                aboutData.put("DeviceName", new Variant(new String("A device name")));
-                aboutData.put("AppName", new Variant(new String("An application name")));
-                aboutData.put("Manufacturer", new Variant(new String("A mighty manufacturing company")));
-                aboutData.put("Description", new Variant(new String("Sample showing the about feature in a service application")));
+                aboutData.put("DeviceName", new Variant("A device name"));
+                aboutData.put("AppName", new Variant("An application name"));
+                aboutData.put("Manufacturer", new Variant("A mighty manufacturing company"));
+                aboutData.put("Description", new Variant("Sample showing the about feature in a service application"));
             } else if (language.equalsIgnoreCase("es")) { //Spanish
-                aboutData.put("DeviceName", new Variant(new String("Un nombre de dispositivo")));
+                aboutData.put("DeviceName", new Variant("Un nombre de dispositivo"));
                 aboutData.put("AppName", new Variant(new String("Un nombre de aplicación")));
                 aboutData.put("Manufacturer", new Variant(new String("Una empresa de fabricación de poderosos")));
                 aboutData.put("Description", new Variant(new String("Muestra que muestra la característica de sobre en una aplicación de servicio")));
@@ -107,12 +107,12 @@ public class AboutIconProxyTest extends TestCase{
         public Map<String, Variant> getAnnouncedAboutData() throws ErrorReplyBusException {
             Map<String, Variant> announceData = new HashMap<String, Variant>();
             announceData.put("AppId",  new Variant(new byte[] {1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16 }));
-            announceData.put("DefaultLanguage",  new Variant(new String("en")));
-            announceData.put("DeviceName", new Variant(new String("A device name")));
+            announceData.put("DefaultLanguage",  new Variant("en"));
+            announceData.put("DeviceName", new Variant("A device name"));
             announceData.put("DeviceId",  new Variant(new String("93c06771-c725-48c2-b1ff-6a2a59d445b8")));
-            announceData.put("AppName", new Variant(new String("An application name")));
-            announceData.put("Manufacturer", new Variant(new String("A mighty manufacturing company")));
-            announceData.put("ModelNumber", new Variant(new String("A1B2C3")));
+            announceData.put("AppName", new Variant("An application name"));
+            announceData.put("Manufacturer", new Variant("A mighty manufacturing company"));
+            announceData.put("ModelNumber", new Variant("A1B2C3"));
             return announceData;
         }
 
