@@ -613,8 +613,8 @@ public class BusAttachmentTest extends TestCase {
         Introspectable introspectable = proxyObj.getInterface(Introspectable.class);
         String actual = introspectable.Introspect();
         String expected =
-                "<!DOCTYPE node PUBLIC \"-//freedesktop//DTD D-BUS Object Introspection 1.0//EN\"\n" +
-                        "\"http://standards.freedesktop.org/dbus/introspect-1.0.dtd\">\n" +
+                "<!DOCTYPE node PUBLIC \"-//allseen//DTD ALLJOYN Object Introspection 1.1//EN\"\n" +
+                        "\"http://www.allseen.org/alljoyn/introspect-1.1.dtd\">\n" +
                         "<node>\n" +
                         "  <interface name=\"org.alljoyn.bus.AnnotationInterface\">\n" +
                         "    <method name=\"DeprecatedMethod\">\n" +
