@@ -528,7 +528,7 @@ void ToAny(Plugin& plugin, const NPVariant& value, const qcc::String signature,
                 size_t numElements = propertiesCount;
                 ajn::MsgArg* elements = new ajn::MsgArg[numElements];
 
-                arg.v_array.SetElements(elemSignature.c_str(), numElements, elements);
+                status = arg.v_array.SetElements(elemSignature.c_str(), numElements, elements);
                 if (ER_OK != status) {
                     NPN_MemFree(properties);
                     delete[] elements;
@@ -1421,6 +1421,10 @@ void ToDOMString(Plugin& plugin, const char* str, uint32_t len, NPVariant& varia
         }
     }
     char* val = reinterpret_cast<char*>(NPN_MemAlloc(len + 1));
-    strncpy(val, str, len + 1);
+    if (!str) {
+        val[0] = 0;
+    } else {
+        strncpy(val, str, len + 1);
+    }
     STRINGZ_TO_NPVARIANT(val, variant);
 }
