@@ -92,8 +92,8 @@ AsyncTestCase("BusObjectTest", {
     testToXMLCallback: function(queue) {
         queue.call(function(callbacks) {
             var expectedCustomXML =
-                '<!DOCTYPE node PUBLIC "-//freedesktop//DTD D-BUS Object Introspection 1.0//EN"\n' +
-                '"http://standards.freedesktop.org/dbus/introspect-1.0.dtd">\n' +
+                '<!DOCTYPE node PUBLIC "-//allseen//DTD ALLJOYN Object Introspection 1.1//EN"\n' +
+                '"http://www.allseen.org/alljoyn/introspect-1.1.dtd">\n' +
                 '<node>\n' +
                 '  <interface name="org.alljoyn.bus.samples.simple.SimpleInterface">\n' +
                 '    <method name="Ping">\n' +
@@ -109,8 +109,8 @@ AsyncTestCase("BusObjectTest", {
                 '  </interface>\n' +
                 '</node>\n';
             var expectedBuiltinXML =
-                '<!DOCTYPE node PUBLIC "-//freedesktop//DTD D-BUS Object Introspection 1.0//EN"\n' +
-                '"http://standards.freedesktop.org/dbus/introspect-1.0.dtd">\n' +
+                '<!DOCTYPE node PUBLIC "-//allseen//DTD ALLJOYN Object Introspection 1.1//EN"\n' +
+                '"http://www.allseen.org/alljoyn/introspect-1.1.dtd">\n' +
                 '<node>\n' +
                 '  <interface name="org.alljoyn.bus.samples.simple.SimpleInterface">\n' +
                 '    <method name="Ping">\n' +
