@@ -349,7 +349,7 @@
         //
         // create an interface description, or if that fails, get the interface as it was already created
         //
-        interfaceDescription = [self.clientController.bus createInterfaceWithName:@"org.alljoyn.bus.test.perf.both"];
+        interfaceDescription = [self.clientController.bus createInterfaceWithName:@"org.alljoyn.bus.test.perf.both" withInterfaceSecPolicy:AJN_IFC_SECURITY_OFF];
         
         
         // add the methods to the interface description
