@@ -33,6 +33,8 @@
 
 #include <qcc/CryptoECCMath.h>
 
+#include "Crypto.h"
+
 #include <Status.h>
 #include <bcrypt.h>
 
@@ -206,7 +208,7 @@ QStatus ECCSecret::DerivePreMasterSecret(
 
 Exit:
 
-    SecureZeroMemory(rgbPreMasterSecret, sizeof(rgbPreMasterSecret));
+    ClearMemory(rgbPreMasterSecret, sizeof(rgbPreMasterSecret));
 
     return status;
 
@@ -238,7 +240,6 @@ ECCPublicKey* Crypto_ECC_GetPublicKey(
 {
     QStatus status = ER_OK;
     NTSTATUS ntStatus;
-    errno_t err;
 
     QCC_DbgTrace(("Crypto_ECC_GetPublicKey"));
 
@@ -273,32 +274,7 @@ ECCPublicKey* Crypto_ECC_GetPublicKey(
     x = keyBlob + sizeof(BCRYPT_ECCKEY_BLOB);
     y = x + header->cbKey;
 
-    err = memcpy_s(pEcPubKey->x, sizeof(pEcPubKey->x), x, header->cbKey);
-    if (0 != err) {
-        status = ER_CRYPTO_ERROR;
-        QCC_LogError(status, ("Failed to copy x coordinate out, errno=%d", err));
-        goto Exit;
-    }
-
-    /* Interop key fields may be longer than CNG keys. Pad out the rest with zero. */
-    for (int i = (header->cbKey / sizeof(pEcPubKey->x[0]));
-         i < (sizeof(pEcPubKey->x) / sizeof(pEcPubKey->x[0]));
-         i++) {
-        pEcPubKey->x[i] = 0;
-    }
-
-    err = memcpy_s(pEcPubKey->y, sizeof(pEcPubKey->y), y, header->cbKey);
-    if (0 != err) {
-        status = ER_CRYPTO_ERROR;
-        QCC_LogError(status, ("Failed to copy y coordinate out, errno=%d", err));
-        goto Exit;
-    }
-
-    for (int i = (header->cbKey / sizeof(pEcPubKey->y[0]));
-         i < (sizeof(pEcPubKey->y) / sizeof(pEcPubKey->y[0]));
-         i++) {
-        pEcPubKey->y[i] = 0;
-    }
+    status = pEcPubKey->Import(x, header->cbKey, y, header->cbKey);
 
 Exit:
 
@@ -320,14 +296,13 @@ static QStatus Crypto_ECC_SetPublicKey(
     BCRYPT_ALG_HANDLE hAlg,
     BCRYPT_KEY_HANDLE*phPubKey)
 {
-    errno_t err;
     NTSTATUS ntStatus;
     QStatus status = ER_OK;
     PBYTE keyBlob = NULL;
     PBCRYPT_ECCKEY_BLOB header = NULL;
-    PBYTE x = NULL, y = NULL;
     ULONG keySize;
     ULONG blobSize;
+    size_t exportSize;
 
     BCRYPT_KEY_HANDLE hPubKey = NULL;
 
@@ -376,57 +351,13 @@ static QStatus Crypto_ECC_SetPublicKey(
     }
 
     header->cbKey = keySize;
-    x = keyBlob + sizeof(BCRYPT_ECCKEY_BLOB);
-    y = x + header->cbKey;
+    exportSize = 2 * header->cbKey;
 
-    /* Interop key fields may be larger than CNG's. Make sure the high
-     * order bytes are zero and copy the rest.
-     */
-    assert(header->cbKey <= sizeof(pubKey->x));
-    for (int i = (header->cbKey / sizeof(pubKey->x[0]));
-         i < (sizeof(pubKey->x) / sizeof(pubKey->x[0]));
-         i++) {
-        if (pubKey->x[i] != 0) {
-            status = ER_CRYPTO_ERROR;
-            QCC_LogError(status, ("High order bytes of x are not zero"));
-            goto Exit;
-        }
-    }
-    assert(header->cbKey <= sizeof(pubKey->y));
-    for (int i = (header->cbKey / sizeof(pubKey->y[0]));
-         i < (sizeof(pubKey->y) / sizeof(pubKey->y[0]));
-         i++) {
-        if (pubKey->y[i] != 0) {
-            status = ER_CRYPTO_ERROR;
-            QCC_LogError(status, ("High order bytes of y are not zero"));
-            goto Exit;
-        }
-    }
-
-    err = memcpy_s(x, header->cbKey, pubKey->x, header->cbKey);
-    if (0 != err) {
-        status = ER_CRYPTO_ERROR;
-        QCC_LogError(status, ("Failed to copy x coordinate, errno=%d", err));
+    status = pubKey->Export(keyBlob + sizeof(BCRYPT_ECCKEY_BLOB), &exportSize);
+    if (ER_OK != status) {
         goto Exit;
     }
 
-    err = memcpy_s(y, header->cbKey, pubKey->y, header->cbKey);
-    if (0 != err) {
-        status = ER_CRYPTO_ERROR;
-        QCC_LogError(status, ("Failed to copy y coordinate, errno=%d", err));
-        goto Exit;
-    }
-
-    hPubKey = *phPubKey;
-
-    if (NULL != hPubKey) {
-        ntStatus = BCryptDestroyKey(hPubKey);
-        if (!BCRYPT_SUCCESS(ntStatus)) {
-            QCC_LogError(ER_CRYPTO_ERROR, ("Not fatal: Failed to destroy old public key, ntStatus=%X", ntStatus));
-            /* Try to carry on anyway. */
-        }
-    }
-
     ntStatus = BCryptImportKeyPair(hAlg, NULL, BCRYPT_ECCPUBLIC_BLOB, &hPubKey, keyBlob, blobSize, 0);
     if (!BCRYPT_SUCCESS(ntStatus)) {
         status = ER_CRYPTO_ERROR;
@@ -434,6 +365,14 @@ static QStatus Crypto_ECC_SetPublicKey(
         goto Exit;
     }
 
+    if (NULL != *phPubKey) {
+        ntStatus = BCryptDestroyKey(*phPubKey);
+        if (!BCRYPT_SUCCESS(ntStatus)) {
+            QCC_LogError(ER_CRYPTO_ERROR, ("Not fatal: Failed to destroy old public key, ntStatus=%X", ntStatus));
+            /* Try to carry on anyway. */
+        }
+    }
+
     *phPubKey = hPubKey;
     hPubKey = NULL;
 
@@ -460,7 +399,6 @@ static ECCPrivateKey* Crypto_ECC_GetPrivateKey(
 {
     QStatus status = ER_OK;
     NTSTATUS ntStatus;
-    errno_t err;
 
     PBYTE keyBlob = NULL;
     ULONG blobSize;
@@ -494,23 +432,16 @@ static ECCPrivateKey* Crypto_ECC_GetPrivateKey(
     header = reinterpret_cast<PBCRYPT_ECCKEY_BLOB>(keyBlob);
     d = keyBlob + sizeof(BCRYPT_ECCKEY_BLOB) + (2 * header->cbKey); /* Skip header, X and Y. */
 
-    err = memcpy_s(pEccPrivateKey->d, sizeof(pEccPrivateKey->d), d, header->cbKey);
-    if (0 != err) {
-        status = ER_CRYPTO_ERROR;
-        QCC_LogError(status, ("Failed to copy private key out, errno=%d", err));
+    status = pEccPrivateKey->Import(d, header->cbKey);
+    if (status != ER_OK) {
+        QCC_LogError(status, ("Failed to copy private key out"));
         goto Exit;
     }
 
-    /* Fill out the extra high order bytes with zero. */
-    for (int i = (header->cbKey / sizeof(pEccPrivateKey->d[0]));
-         i < (sizeof(pEccPrivateKey->d) / sizeof(pEccPrivateKey->d[0]));
-         i++) {
-        pEccPrivateKey->d[i] = 0;
-    }
-
 Exit:
 
     if (NULL != keyBlob) {
+        ClearMemory(keyBlob, blobSize);
         free(keyBlob);
     }
 
@@ -529,12 +460,11 @@ static QStatus Crypto_ECC_SetPrivateKey(
     BCRYPT_KEY_HANDLE*phPrivateKey)
 {
     QStatus status = ER_OK;
-    errno_t err;
     NTSTATUS ntStatus;
     PBYTE keyBlob = NULL;
     PBCRYPT_ECCKEY_BLOB header = NULL;
     PBYTE x = NULL, y = NULL, d = NULL;
-    ULONG keySize, blobSize;
+    ULONG keySize, blobSize = 0;
 
     BCRYPT_KEY_HANDLE hPrivateKey = NULL;
 
@@ -593,23 +523,18 @@ static QStatus Crypto_ECC_SetPrivateKey(
      */
     memset(x, 0, header->cbKey);
     memset(y, 0, header->cbKey);
-    /* Interop private keys can have extra bytes. Make sure the high
-     * order bytes are zero and copy the rest.
-     */
-    assert(header->cbKey <= sizeof(privateKey->d));
-    for (int i = (header->cbKey / sizeof(privateKey->d[0]));
-         i < (sizeof(privateKey->d) / sizeof(privateKey->d[0]));
-         i++) {
-        if (privateKey->d[i] != 0) {
-            status = ER_CRYPTO_ERROR;
-            QCC_LogError(status, ("High order bytes of input private key are not zero"));
-            goto Exit;
-        }
-    }
-    err = memcpy_s(d, header->cbKey, privateKey->d, header->cbKey);
-    if (0 != err) {
+
+    size_t keySizeAsSizeT = header->cbKey;
+
+    status = privateKey->Export(d, &keySizeAsSizeT);
+    if (ER_OK != status) {
         status = ER_CRYPTO_ERROR;
-        QCC_LogError(status, ("Failed to copy private key bytes, errno=%d", err));
+        QCC_LogError(status, ("Failed to export private key bytes"));
+        goto Exit;
+    }
+    if (header->cbKey != keySizeAsSizeT) {
+        status = ER_CRYPTO_ERROR;
+        QCC_LogError(status, ("Change in size of key was unexpected after Export; expected %u, got %" PRIuSIZET, header->cbKey, keySize));
         goto Exit;
     }
 
@@ -636,6 +561,7 @@ static QStatus Crypto_ECC_SetPrivateKey(
 Exit:
 
     if (NULL != keyBlob) {
+        ClearMemory(keyBlob, blobSize);
         free(keyBlob);
     }
 
@@ -678,7 +604,7 @@ static QStatus Crypto_ECC_GenerateKeyPair(
 
     QCC_DbgTrace(("Crypto_ECC_GenerateKeyPair"));
 
-    assert(NULL != hAlg);
+    QCC_ASSERT(NULL != hAlg);
 
     switch (CurveType) {
     case Crypto_ECC::ECC_NIST_P256:
@@ -770,57 +696,47 @@ Exit:
     return status;
 }
 
+static QStatus InitializeBCryptProviderHandles(uint8_t curveType)
+{
+    QCC_DbgTrace(("%s", __FUNCTION__));
+
+    /* Make sure the crypto subsystem has been initialized; some common library-only code
+     * might invoke ECC functionality without spinning up the whole AllJoyn subsystem first.
+     */
+    QStatus status = Crypto::Init();
+    if (ER_OK != status) {
+        return status;
+    }
+
+    if (NULL == cngCache.ecdsaHandles[curveType]) {
+        status = cngCache.OpenEcdsaHandle(curveType);
+        if (ER_OK != status) {
+            return status;
+        }
+    }
+
+    if (NULL == cngCache.ecdhHandles[curveType]) {
+        status = cngCache.OpenEcdhHandle(curveType);
+        if (ER_OK != status) {
+            return status;
+        }
+    }
+
+    return ER_OK;
+}
+
 Crypto_ECC::Crypto_ECC()
 {
-    QStatus status = ER_OK;
-    NTSTATUS ntStatus;
-
-    uint8_t CurveType;
-
-    LPCWSTR ecdsaAlgId = NULL;
-    LPCWSTR ecdhAlgId = NULL;
-
     QCC_DbgTrace(("Crypto_ECC::Crypto_ECC"));
 
-    eccState = new ECCState();
+    QStatus status = InitializeBCryptProviderHandles(GetCurveType());
+    if (ER_OK == status) {
+        eccState = new ECCState();
 
-    memset(eccState, 0, sizeof(*eccState));
+        memset(eccState, 0, sizeof(*eccState));
+    } else {
+        QCC_ASSERT(!"InitializeBCryptProviderHandles failed");
 
-    CurveType = GetCurveType();
-
-    switch (CurveType) {
-    case ECC_NIST_P256:
-        ecdsaAlgId = BCRYPT_ECDSA_P256_ALGORITHM;
-        ecdhAlgId = BCRYPT_ECDH_P256_ALGORITHM;
-        break;
-
-    default:
-        status = ER_CRYPTO_ILLEGAL_PARAMETERS;
-        QCC_LogError(status, ("Unrecognized curve type %d", CurveType));
-        goto Exit;
-    }
-
-    if (NULL == cngCache.ecdsaHandles[CurveType]) {
-        ntStatus = BCryptOpenAlgorithmProvider(&cngCache.ecdsaHandles[CurveType], ecdsaAlgId, NULL, 0);
-        if (!BCRYPT_SUCCESS(ntStatus)) {
-            status = ER_CRYPTO_ERROR;
-            QCC_LogError(status, ("Failed to open ECDSA algorithm provider, ntStatus=%X", ntStatus));
-            goto Exit;
-        }
-    }
-
-    if (NULL == cngCache.ecdhHandles[CurveType]) {
-        ntStatus = BCryptOpenAlgorithmProvider(&cngCache.ecdhHandles[CurveType], ecdhAlgId, NULL, 0);
-        if (!BCRYPT_SUCCESS(ntStatus)) {
-            status = ER_CRYPTO_ERROR;
-            QCC_LogError(status, ("Failed to open ECDH algorithm provider, ntStatus=%X", ntStatus));
-            goto Exit;
-        }
-    }
-
-Exit:
-
-    if (ER_OK != status) {
         /*
          *  Don't clean up any of the BCrypt Algorithm provider handles.
          *  If they have been created on the CNG Cache they'll be freed when that is cleaned up.
@@ -831,10 +747,8 @@ Exit:
          * But to keep things clean, make sure eccState isn't pointing at anything partly initialized.
          */
         eccState = NULL;
-
         abort();
     }
-
 }
 
 const ECCPublicKey* Crypto_ECC::GetDHPublicKey() const
@@ -870,10 +784,11 @@ void Crypto_ECC::SetDHPublicKey(const ECCPublicKey* pubKey)
                                      cngCache.ecdhHandles[CurveType],
                                      &eccState->ecdhPublicKey);
 
+    QCC_ASSERT(ER_OK == status);
     if (ER_OK != status) {
+        QCC_ASSERT(!"Crypto_ECC_SetPublicKey failed");
         abort();
     }
-
 }
 
 const ECCPrivateKey* Crypto_ECC::GetDHPrivateKey() const
@@ -908,7 +823,9 @@ void Crypto_ECC::SetDHPrivateKey(const ECCPrivateKey* privateKey)
                                       cngCache.ecdhHandles[CurveType],
                                       &eccState->ecdhPrivateKey);
 
+    QCC_ASSERT(ER_OK == status);
     if (ER_OK != status) {
+        QCC_ASSERT(!"Crypto_ECC_SetPrivateKey failed");
         abort();
     }
 }
@@ -918,7 +835,7 @@ QStatus Crypto_ECC::GenerateDHKeyPair() {
 
     QCC_DbgTrace(("Crypto_ECC::GenerateDHKeyPair"));
 
-    assert(NULL != cngCache.ecdhHandles[CurveType]);
+    QCC_ASSERT(NULL != cngCache.ecdhHandles[CurveType]);
 
     return Crypto_ECC_GenerateKeyPair(CurveType,
                                       cngCache.ecdhHandles[CurveType],
@@ -926,6 +843,24 @@ QStatus Crypto_ECC::GenerateDHKeyPair() {
                                       &eccState->ecdhPrivateKey);
 }
 
+
+QStatus Crypto_ECC::GenerateSPEKEKeyPair(const uint8_t* pw, const size_t pwLen, const GUID128 clientGUID, const GUID128 serviceGUID) {
+
+    /*
+     * CNG does not implement EC-SPEKE, so the builtin key EC-SPEKE key generation is used.
+     * We need to call SetDHPrivateKey and SetDHPublic so that the CNG-specific part of ECCState is properly updated.
+     */
+    QStatus status = Crypto_ECC_GenerateSPEKEKeyPair(&eccState->dhPublicKey, &eccState->dhPrivateKey, pw, pwLen, clientGUID, serviceGUID);
+    if (status != ER_OK) {
+        return status;
+    }
+
+    this->SetDHPublicKey(&eccState->dhPublicKey);
+    this->SetDHPrivateKey(&eccState->dhPrivateKey);
+
+    return status;
+}
+
 QStatus Crypto_ECC::GenerateSharedSecret(const ECCPublicKey* peerPublicKey, ECCSecret* secret)
 {
     QStatus status = ER_FAIL;
@@ -985,7 +920,7 @@ Exit:
         }
     }
 
-    SecureZeroMemory(&eccSecretState, sizeof(ECCSecret::ECCSecretState));
+    ClearMemory(&eccSecretState, sizeof(ECCSecret::ECCSecretState));
 
     return status;
 }
@@ -1025,6 +960,7 @@ void Crypto_ECC::SetDSAPublicKey(const ECCPublicKey* pubKey)
 
     if (ER_OK != status) {
         QCC_LogError(status, ("Crypto_ECC_SetPublicKey failed and aborting."));
+        QCC_ASSERT(!"Crypto_ECC_SetPublicKey failed");
         abort();
     }
 
@@ -1063,6 +999,7 @@ void Crypto_ECC::SetDSAPrivateKey(const ECCPrivateKey* privateKey)
                                       &eccState->ecdsaPrivateKey);
 
     if (ER_OK != status) {
+        QCC_ASSERT(!"Crypto_ECC_SetPrivateKey failed");
         abort();
     }
 
@@ -1074,7 +1011,7 @@ QStatus Crypto_ECC::GenerateDSAKeyPair()
 
     QCC_DbgTrace(("Crypto_ECC::GenerateDSAKeyPair"));
 
-    assert(NULL != cngCache.ecdsaHandles[CurveType]);
+    QCC_ASSERT(NULL != cngCache.ecdsaHandles[CurveType]);
 
     return Crypto_ECC_GenerateKeyPair(CurveType,
                                       cngCache.ecdsaHandles[CurveType],
@@ -1124,7 +1061,7 @@ QStatus Crypto_ECC::DSASignDigest(const uint8_t* digest, uint16_t len, ECCSignat
         goto Exit;
     }
 
-    assert(bytesReceived == cngSignatureSize);
+    QCC_ASSERT(bytesReceived == cngSignatureSize);
 #endif
 
     /* The type of the buffer input to BCryptSignHash is not marked const, but the function
@@ -1138,7 +1075,7 @@ QStatus Crypto_ECC::DSASignDigest(const uint8_t* digest, uint16_t len, ECCSignat
         goto Exit;
     }
 
-    assert(bytesReceived == cngSignatureSize);
+    QCC_ASSERT(bytesReceived == cngSignatureSize);
 
 
     err = memcpy_s(sig->r, sizeof(sig->r), cngSignature, cngSignatureElementSize);
@@ -1367,42 +1304,177 @@ Crypto_ECC::~Crypto_ECC()
         }
     }
 
-    SecureZeroMemory(&eccState, sizeof(eccState));
+    // deleting eccState recursively calls destructor on private key objects
+    // these destructors zero the secrets.
     delete eccState;
 }
 
-const qcc::String ECCPublicKey::ToString() const
+/** ECCPublicKey **/
+
+/**
+ * Empty ECC coordinate
+ */
+static const uint8_t ECC_COORDINATE_EMPTY[ECC_COORDINATE_SZ] = { 0 };
+
+bool ECCPublicKey::empty() const
 {
-    qcc::String s = "x=[";
-    s.append(BytesToHexString(x, ECC_COORDINATE_SZ));
+    return (memcmp(x, ECC_COORDINATE_EMPTY, GetCoordinateSize()) == 0) &&
+           (memcmp(y, ECC_COORDINATE_EMPTY, GetCoordinateSize()) == 0);
+}
+
+const String ECCPublicKey::ToString() const
+{
+    String s = "x=[";
+    s.append(BytesToHexString(x, GetCoordinateSize()));
     s.append("], y=[");
-    s.append(BytesToHexString(y, ECC_COORDINATE_SZ));
+    s.append(BytesToHexString(y, GetCoordinateSize()));
     s.append("]");
     return s;
 }
 
 QStatus ECCPublicKey::Export(uint8_t* data, size_t* size) const
 {
-    if (data == NULL || size == NULL || *size < (ECC_COORDINATE_SZ + ECC_COORDINATE_SZ)) {
+    errno_t err;
+
+    const size_t coordinateSize = GetCoordinateSize();
+
+    if (data == NULL || size == NULL || *size < (coordinateSize + coordinateSize)) {
         return ER_FAIL;
     }
 
-    memcpy(data, x, ECC_COORDINATE_SZ);
-    memcpy(data + ECC_COORDINATE_SZ, y, ECC_COORDINATE_SZ);
-    *size = ECC_COORDINATE_SZ + ECC_COORDINATE_SZ;
+    err = memcpy_s(data, *size, x, coordinateSize);
+    if (0 != err) {
+        QCC_LogError(ER_FAIL, ("Failed to memcpy_s x; err is %d", err));
+        return ER_FAIL;
+    }
+    err = memcpy_s(data + coordinateSize, *size - coordinateSize, y, coordinateSize);
+    if (0 != err) {
+        QCC_LogError(ER_FAIL, ("Failed to memcpy_s y; err is %d", err));
+        return ER_FAIL;
+    }
+    *size = coordinateSize + coordinateSize;
     return ER_OK;
 }
 
-QStatus ECCPublicKey::Import(const uint8_t* data, size_t size)
+static QStatus ECCPublicKey_Validate(ECCPublicKey* pubKey)
 {
-    if ((size != (ECC_COORDINATE_SZ + ECC_COORDINATE_SZ)) || (!data)) {
-        return ER_FAIL;
+    /* Attempt to import the key into a CNG handle. CNG will verify the point lies on
+     * the curve, and so success here means that it does.
+     */
+    QStatus status;
+    uint8_t CurveType = Crypto_ECC::ECC_NIST_P256;
+    BCRYPT_KEY_HANDLE keyHandle = nullptr;
+
+    /* Most of the ECC code assumes it's called with a Crypto_ECC object having been created.
+     * That's not necessarily true with this function, so be sure the BCrypt provider handles
+     * are opened.
+     */
+    status = InitializeBCryptProviderHandles(CurveType);
+    if (ER_OK != status) {
+        return status;
     }
 
-    memcpy(x, data, ECC_COORDINATE_SZ);
-    memcpy(y, data + ECC_COORDINATE_SZ, ECC_COORDINATE_SZ);
+    status = Crypto_ECC_SetPublicKey(CurveType,
+                                     CNG_ECC_ALG_DSA,
+                                     pubKey,
+                                     cngCache.ecdsaHandles[CurveType],
+                                     &keyHandle);
+
+    if (ER_OK != status) {
+        QCC_LogError(status, ("Failed to import ECCPublicKey."));
+    } else {
+        /* Only if the import succeeded does keyHandle need to be freed. */
+        QCC_VERIFY(BCRYPT_SUCCESS(BCryptDestroyKey(keyHandle)));
+    }
+
+    return status;
+}
+
+QStatus ECCPublicKey::Import(const uint8_t* data, const size_t size)
+{
+    if (NULL == data) {
+        return ER_BAD_ARG_1;
+    }
+    if (size != GetSize()) {
+        return ER_BAD_ARG_2;
+    }
+
+    const size_t coordinateSize = GetCoordinateSize();
+
+    return this->Import(data, coordinateSize, data + coordinateSize, coordinateSize);
+}
+
+QStatus ECCPublicKey::Import(const uint8_t* xData, const size_t xSize, const uint8_t* yData, const size_t ySize)
+{
+    if (NULL == xData) {
+        return ER_BAD_ARG_1;
+    }
+    if (GetCoordinateSize() != xSize) {
+        return ER_BAD_ARG_2;
+    }
+    if (NULL == yData) {
+        return ER_BAD_ARG_3;
+    }
+    if (GetCoordinateSize() != ySize) {
+        return ER_BAD_ARG_4;
+    }
+
+    /* Save the old values just in case the import fails, to not leave the object clobbered. */
+    std::vector<uint8_t> xBackup(GetCoordinateSize());
+    std::vector<uint8_t> yBackup(GetCoordinateSize());
+
+    memcpy(xBackup.data(), x, xSize);
+    memcpy(yBackup.data(), y, ySize);
+
+    /* Copy in new data and validate. */
+    QCC_ASSERT(sizeof(x) >= xSize);
+    QCC_ASSERT(sizeof(y) >= ySize);
+    memcpy(x, xData, xSize);
+    memcpy(y, yData, ySize);
+
+    if (ER_OK != ECCPublicKey_Validate(this)) {
+        /* Restore previous values. */
+        memcpy(x, xBackup.data(), xSize);
+        memcpy(y, yBackup.data(), ySize);
+
+        return ER_CORRUPT_KEYBLOB;
+    }
 
     return ER_OK;
 }
 
+/** ECCPrivateKey **/
+
+ECCPrivateKey::~ECCPrivateKey()
+{
+    ClearMemory(d, ECC_COORDINATE_SZ);
+}
+
+QStatus ECCPrivateKey::Export(uint8_t* data, size_t* size) const
+{
+    if (NULL == data) {
+        return ER_BAD_ARG_1;
+    }
+    if (NULL == size) {
+        return ER_BAD_ARG_2;
+    }
+    if (this->GetSize() > *size) {
+        *size = this->GetSize();
+        return ER_BUFFER_TOO_SMALL;
+    }
+
+    *size = this->GetSize();
+    memcpy(data, d, *size);
+
+    return ER_OK;
+}
+
+const String ECCPrivateKey::ToString() const
+{
+    qcc::String s = "d=[";
+    s.append(BytesToHexString(d, GetSize()));
+    s.append("]");
+    return s;
+}
+
 } /* namespace qcc */
