@@ -380,6 +380,10 @@ QStatus Crypto_ECC::GenerateDHKeyPair() {
     return Crypto_ECC_GenerateKeyPair(&eccState->dhPublicKey, &eccState->dhPrivateKey);
 }
 
+QStatus Crypto_ECC::GenerateSPEKEKeyPair(const uint8_t* pw, const size_t pwLen, const GUID128 clientGUID, const GUID128 serviceGUID) {
+    return Crypto_ECC_GenerateSPEKEKeyPair(&eccState->dhPublicKey, &eccState->dhPrivateKey, pw, pwLen, clientGUID, serviceGUID);
+}
+
 QStatus Crypto_ECC::GenerateSharedSecret(const ECCPublicKey* peerPublicKey, ECCSecret* secret)
 {
     bigval_t sec;
@@ -644,6 +648,27 @@ QStatus ECCPublicKey::Export(uint8_t* data, size_t* size) const
     return ER_OK;
 }
 
+static bool EC_point_validation(affine_point_t const* pubkey)
+{
+    bool status;
+    QStatus ajstatus;
+    ec_t curve;
+    ecpoint_t Q;
+
+    ajstatus = ec_getcurve(&curve, NISTP256r1);
+    if (ER_OK != ajstatus) {
+        return false;
+    }
+
+    status = bigval_to_digit256(&(pubkey->x), Q.x);
+    status = status && bigval_to_digit256(&(pubkey->y), Q.y);
+    status = status && ecpoint_validation(&Q, &curve);
+
+    ec_freecurve(&curve);
+
+    return status;
+}
+
 QStatus ECCPublicKey::Import(const uint8_t* data, size_t size)
 {
     if (NULL == data) {
@@ -655,10 +680,7 @@ QStatus ECCPublicKey::Import(const uint8_t* data, size_t size)
 
     const size_t coordinateSize = GetCoordinateSize();
 
-    memcpy(x, data, coordinateSize);
-    memcpy(y, data + coordinateSize, coordinateSize);
-
-    return ER_OK;
+    return this->Import(data, coordinateSize, data + coordinateSize, coordinateSize);
 }
 
 QStatus ECCPublicKey::Import(const uint8_t* xData, const size_t xSize, const uint8_t* yData, const size_t ySize)
@@ -676,6 +698,18 @@ QStatus ECCPublicKey::Import(const uint8_t* xData, const size_t xSize, const uin
         return ER_BAD_ARG_4;
     }
 
+    /* Verify that this public key is valid. */
+    affine_point_t pub;
+
+    pub.infinity = 0;
+    binary_to_bigval(xData, &pub.x, xSize);
+    binary_to_bigval(yData, &pub.y, ySize);
+
+    if (!EC_point_validation(&pub)) {
+        QCC_LogError(ER_CORRUPT_KEYBLOB, ("Failed to import ECCPublicKey."));
+        return ER_CORRUPT_KEYBLOB;
+    }
+
     memcpy(x, xData, xSize);
     memcpy(y, yData, ySize);
 
