@@ -20,7 +20,6 @@
  *    OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  ******************************************************************************/
 
-#include <assert.h>
 #include <ctype.h>
 
 #include <qcc/platform.h>
@@ -211,6 +210,11 @@ QStatus Crypto_Hash::Update(const qcc::String& str)
     return Update((const uint8_t*)str.data(), str.size());
 }
 
+QStatus Crypto_Hash::Update(const vector<uint8_t, SecureAllocator<uint8_t> >& d)
+{
+    return Update(d.data(), d.size());
+}
+
 QStatus Crypto_Hash::GetDigest(uint8_t* digest, bool keepAlive)
 {
     QStatus status = ER_OK;
