@@ -22,7 +22,7 @@
 #include <qcc/platform.h>
 #include <qcc/Mutex.h>
 #include <qcc/Thread.h>
-#include <assert.h>
+#include <qcc/MutexInternal.h>
 
 #include "Crypto.h"
 #include "OpenSsl.h"
@@ -54,8 +54,15 @@ static void LockingCb(int mode, int type, const char* file, int line)
 QStatus Crypto::Init()
 {
     if (!locks) {
-        locks = new Mutex[CRYPTO_num_locks()];
+        int numLocks = CRYPTO_num_locks();
+        locks = new Mutex[numLocks];
         CRYPTO_set_locking_callback(LockingCb);
+
+#ifndef NDEBUG
+        for (int index = 0; index < numLocks; index++) {
+            MutexInternal::SetLevel(locks[index], LOCK_LEVEL_OPENSSL_LOCK);
+        }
+#endif
     }
     return ER_OK;
 }
@@ -87,7 +94,7 @@ OpenSsl_ScopedLock::OpenSsl_ScopedLock()
 
 OpenSsl_ScopedLock::~OpenSsl_ScopedLock()
 {
-    assert(mutex);
+    QCC_ASSERT(mutex);
     mutex->Unlock();
 }
 
