@@ -22,7 +22,6 @@
  *    OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  ******************************************************************************/
 
-#include <assert.h>
 #include <qcc/platform.h>
 #include <qcc/GUID.h>
 #include <qcc/CryptoECC.h>
@@ -122,6 +121,9 @@ class CertificateX509 {
         ENCODING_X509_DER_PEM = 1  ///< X.509 DER PEM format
     } EncodingType;
 
+    /**
+     * Certificate type
+     */
     typedef enum {
         UNRESTRICTED_CERTIFICATE,  ///< Unrestricted certificate
         IDENTITY_CERTIFICATE,      ///< identity certificate
@@ -476,6 +478,10 @@ class CertificateX509 {
      */
     QStatus GenerateAuthorityKeyId(const qcc::ECCPublicKey* issuerPubKey);
 
+    /**
+     * Get the Authority Key Identifier
+     * @return the Authority Key Identifier
+     */
     const qcc::String& GetAuthorityKeyId() const
     {
         return aki;
@@ -546,7 +552,7 @@ class CertificateX509 {
     /**
      * Set the digest of the external data.
      * @param digest the digest of the external data
-     * @param count the size of the digest.
+     * @param size the size of the digest.
      */
     void SetDigest(const uint8_t* digest, size_t size)
     {
@@ -555,7 +561,7 @@ class CertificateX509 {
 
     /**
      * Get the digest of the external data.
-     * @param digest the digest of the external data
+     * @return The digest of the external data
      */
     const uint8_t* GetDigest() const
     {
@@ -632,6 +638,14 @@ class CertificateX509 {
      */
     bool IsSubjectPublicKeyEqual(const ECCPublicKey* publicKey) const;
 
+    /**
+     * Get the SHA-256 thumbprint of this certificate.
+     * @param[out] thumbprint buffer of size Crypto_SHA256::DIGEST_SIZE to receive the thumbprint
+     *
+     * @return #ER_OK if successful, error code otherwise
+     */
+    QStatus GetSHA256Thumbprint(uint8_t* thumbprint) const;
+
     /**
      * Destructor
      */
@@ -640,6 +654,10 @@ class CertificateX509 {
         delete [] serial;
     }
 
+    /**
+     * Get the certificate type.
+     * @return Certificate type
+     */
     const CertificateType GetType() const
     {
         return type;
@@ -672,6 +690,8 @@ class CertificateX509 {
      * Assign operator for CertificateX509
      *
      * @param[in] other    CertificateX509 to assign from
+     * @return Reference to the *this which has just been assigned from other
+     *         CertificateX509
      */
     CertificateX509& operator=(const CertificateX509& other) {
         if (&other != this) {
@@ -720,15 +740,23 @@ class CertificateX509 {
         {
             ouLen = len;
             delete [] this->ou;
-            this->ou = new uint8_t[len];
-            memcpy(this->ou, newOu, len);
+            if (len > 0) {
+                this->ou = new uint8_t[len];
+                memcpy(this->ou, newOu, len);
+            } else {
+                this->ou = NULL;
+            }
         }
         void SetCN(const uint8_t* newCn, size_t len)
         {
             cnLen = len;
             delete [] this->cn;
-            this->cn = new uint8_t[len];
-            memcpy(this->cn, newCn, len);
+            if (len > 0) {
+                this->cn = new uint8_t[len];
+                memcpy(this->cn, newCn, len);
+            } else {
+                this->cn = NULL;
+            }
         }
         ~DistinguishedName()
         {
@@ -736,24 +764,16 @@ class CertificateX509 {
             delete [] cn;
         }
 
-        DistinguishedName(const DistinguishedName& other) :
-            ouLen(other.ouLen), cnLen(other.cnLen) {
-            ou = new uint8_t[ouLen];
-            memcpy(ou, other.ou, ouLen);
-            cn = new uint8_t[cnLen];
-            memcpy(cn, other.cn, cnLen);
+        DistinguishedName(const DistinguishedName& other) : ou(NULL), ouLen(0), cn(NULL), cnLen(0)
+        {
+            SetOU(other.ou, other.ouLen);
+            SetCN(other.cn, other.cnLen);
         }
 
         DistinguishedName& operator=(const DistinguishedName& other) {
             if (&other != this) {
-                ouLen = other.ouLen;
-                cnLen = other.cnLen;
-                delete[] ou;
-                ou = new uint8_t[ouLen];
-                memcpy(ou, other.ou, ouLen);
-                delete[] cn;
-                cn = new uint8_t[cnLen];
-                memcpy(cn, other.cn, cnLen);
+                SetOU(other.ou, other.ouLen);
+                SetCN(other.cn, other.cnLen);
             }
             return *this;
         }
