@@ -24,11 +24,11 @@
 
 #include <qcc/platform.h>
 
-#include <assert.h>
 #include <string.h>
 #include <stdarg.h>
 
 #include <qcc/KeyBlob.h>
+#include <qcc/SecureAllocator.h>
 #include <qcc/Stream.h>
 #include <qcc/String.h>
 
@@ -85,7 +85,7 @@ class Crypto_AES {
          * Null pad end of block
          */
         void Pad(size_t padLen) {
-            assert(padLen <= 16);
+            QCC_ASSERT(padLen <= 16);
             if (padLen > 0) { memset(&data[16 - padLen], 0, padLen); }
         }
     };
@@ -246,8 +246,15 @@ class Crypto_AES {
  * Generic hash calculation interface abstraction class.
  */
 class Crypto_Hash {
+
   public:
 
+    /// Typedef for abstracting the hash algorithm specifier.
+    typedef enum {
+        SHA1,          ///< SHA1 algorithm specifier
+        SHA256         ///< SHA256 algorithm specifier
+    } Algorithm;
+
     /**
      * Default constructor
      */
@@ -291,6 +298,15 @@ class Crypto_Hash {
      */
     QStatus Update(const qcc::String& str);
 
+    /**
+     * Update the digest with the contents of a string
+     *
+     * @param d   The vector<uint8_t> to hash.
+     *
+     * @return  Indication of success or failure.
+     */
+    QStatus Update(const std::vector<uint8_t, SecureAllocator<uint8_t> >& d);
+
     /**
      * Retrieve the digest into the supplied buffer.  It is assumed that buffer is large enough to
      * store the digest. Unless keepAlive is true, after the digest has been computed the hash
@@ -306,12 +322,6 @@ class Crypto_Hash {
 
   protected:
 
-    /// Typedef for abstracting the hash algorithm specifier.
-    typedef enum {
-        SHA1,          ///< SHA1 algorithm specifier
-        SHA256         ///< SHA256 algorithm specifier
-    } Algorithm;
-
     static const size_t SHA1_SIZE = 20;   ///< SHA1 digest size - 20 bytes == 160 bits
     static const size_t SHA256_SIZE = 32; ///< SHA256 digest size - 32 bytes == 256 bits
 
@@ -385,7 +395,7 @@ class Crypto_SHA256 : public Crypto_Hash {
  * @param out     Output data
  * @param outLen  The required length of the output data.
  */
-QStatus Crypto_PseudorandomFunction(const KeyBlob& secret, const char* label, const qcc::String& seed, uint8_t* out, size_t outLen);
+QStatus Crypto_PseudorandomFunction(const KeyBlob& secret, const char* label, const std::vector<uint8_t, SecureAllocator<uint8_t> >& seed, uint8_t* out, size_t outLen);
 
 /**
  *  Secure Remote Password (SRP6) class. This implements the core algorithm for Secure Remote
@@ -706,6 +716,17 @@ class Crypto_ASN1 {
      */
     static QStatus AJ_CALL DecodeBase64(const qcc::String& b64, qcc::String& bin);
 
+    /**
+     * Decode a PEM base-64 ANSI string to a vector of bytes.
+     *
+     * @param b64  The base-64 string to decode.
+     * @param bin  The vector of bytes output of the decoding.
+     *
+     * @return ER_OK if the decode succeeded.
+     *         An error status otherwise.
+     */
+    static QStatus AJ_CALL DecodeBase64(const qcc::String& b64, std::vector<uint8_t>& bin);
+
     /**
      * Encode a binary string as a PEM base-64 ANSI string.
      *
@@ -717,6 +738,17 @@ class Crypto_ASN1 {
      */
     static QStatus AJ_CALL EncodeBase64(const qcc::String& bin, qcc::String& b64);
 
+    /**
+     * Encode a vector of bytes as a PEM base-64 ANSI string.
+     *
+     * @param bin  The vector of bytes to encode.
+     * @param b64  The base-64 output of the decoding.
+     *
+     * @return ER_OK if the encode succeeded.
+     *         An error status otherwise.
+     */
+    static QStatus AJ_CALL EncodeBase64(const std::vector<uint8_t>& bin, qcc::String& b64);
+
     /*
      * Render ASN.1 as a "human" readable string
      */
