@@ -27,6 +27,7 @@
 #include <qcc/String.h>
 #include <qcc/Event.h>
 #include <qcc/Mutex.h>
+#include <qcc/LockOrderChecker.h>
 #include <Status.h>
 
 #include <set>
@@ -49,7 +50,7 @@ typedef void* ThreadReturn;
  *
  * @param ms    Number of milliseconds to sleep.
  */
-QStatus Sleep(uint32_t ms);
+QStatus AJ_CALL Sleep(uint32_t ms);
 
 /** @internal */
 class Thread;
@@ -266,6 +267,13 @@ class Thread {
      */
     void RemoveAuxListener(ThreadListener* listener);
 
+    /**
+     * Get the platform-specific thread ID.
+     *
+     * @return Current thread's ID.
+     */
+    static ThreadId GetCurrentThreadId();
+
   protected:
 
     Event stopEvent;            ///< Event that indicates a stop request when set.
@@ -285,8 +293,8 @@ class Thread {
 
   private:
 
-    static QStatus Init();
-    static QStatus Shutdown();
+    static QStatus StaticInit();
+    static QStatus StaticShutdown();
     friend class StaticGlobals;
 
     /**
@@ -314,22 +322,32 @@ class Thread {
     typedef std::set<ThreadListener*> ThreadListeners;
     ThreadListeners auxListeners;
     Mutex auxListenersLock;
+    volatile int32_t waitCount;
 
 #if defined(QCC_OS_GROUP_POSIX)
-    volatile int32_t waitCount;
     Mutex waitLock;
     bool hasBeenJoined;
-    qcc::Mutex hbjMutex;
+    Mutex hbjMutex;
 #elif defined(QCC_OS_GROUP_WINDOWS)
     ThreadId threadId;          ///< Thread ID used by windows
 #endif
 
+#ifndef NDEBUG
+    LockOrderChecker lockChecker;    ///< Detects locks acquired out of order
+
+    /** Allow the MutexInternal class to use directly the lockChecker member of this class */
+    friend class MutexInternal;
+#endif
+
     /** Lock that protects global list of Threads and their handles */
     static Mutex* threadListLock;
 
     /** Thread list */
     static std::map<ThreadId, Thread*>* threadList;
 
+    /** Set to true during SCL initialization and back to false after shutdown */
+    static bool initialized;
+
     /** Called on thread exit to deallocate external Thread objects */
     static void STDCALL CleanExternalThread(void* thread);
 
