@@ -46,6 +46,18 @@ struct XmlParseContext;
 class XmlElement {
   public:
 
+    /**
+     * Retrieves the root of the XML in form of a XmlElement.
+     *
+     * @param[in]     xml     Input XML.
+     * @param[out]    root    XML root, null if call failed.
+     *                        Must be freed by calling "delete".
+     * @return    ER_OK if parse was successful.
+     *            ER_WOULDBLOCK if parse is partially completed pending more I/O.
+     *            Otherwise error.
+     */
+    static QStatus AJ_CALL GetRoot(AJ_PCSTR xml, XmlElement** root);
+
     /**
      * Create an XmlElement from an XML document fragment.
      * It is the responsibility of the caller to free the pointer returned in root.
@@ -78,6 +90,14 @@ class XmlElement {
      */
     qcc::String Generate(qcc::String* outStr = NULL) const;
 
+    /**
+     * Return a C string representation of the XmlElement
+     *
+     * @return   XML in form of a zero-terminated string.
+     *           Must be destroyed by the caller using "delete[]".
+     */
+    AJ_PSTR ToString() const;
+
     /**
      * Get the element name
      *
@@ -127,6 +147,13 @@ class XmlElement {
      */
     void AddAttribute(const qcc::String& attributeName, const qcc::String& value) { attributes[attributeName] = value; }
 
+    /**
+     * Add a child element. The parent will take over memory managment for the child.
+     *
+     * @param child  The child XmlElement.
+     */
+    void AddChild(XmlElement* child);
+
     /**
      * Get the element map.
      */
@@ -156,7 +183,7 @@ class XmlElement {
      *
      * @param name   Child node name.
      */
-    XmlElement& CreateChild(const qcc::String& name);
+    XmlElement* CreateChild(const qcc::String& name);
 
     /**
      * Get the content.
