@@ -38,6 +38,16 @@
 #error No OS GROUP defined.
 #endif
 
+/**
+ * Platform-independent assertion macro. If this is not defined in the
+ * platform-specific header file or by other preprocessor define, it defaults
+ * to assert.
+ */
+#ifndef QCC_ASSERT
+#include <assert.h>
+#define QCC_ASSERT(expr) assert(expr)
+#endif
+
 #if defined(__GNUC__)
 
 #define GCC_VERSION ((__GNUC__ * 10000) + (__GNUC_MINOR__ * 100) + __GNUC_PATCHLEVEL__)
@@ -62,23 +72,30 @@
 
 #if (__GNUC__ >= 4) || ((__GNUC__ == 3) && (__GNUC_MINOR__ >= 1))
 #define QCC_DEPRECATED(func) func __attribute__((deprecated)) /**< mark a function as deprecated in gcc. */
+#define QCC_DEPRECATED_ON(func, date) QCC_DEPRECATED(func) /**< same as above but with date of the AllJoyn release applying this macro (date format: YY.MM). */
 #define QCC_DEPRECATED_CLASS __attribute__((deprecated)) /**< mark a class as deprecated in qcc. */
+#define QCC_DEPRECATED_CLASS_ON(date) QCC_DEPRECATED_CLASS /**< same as above but with date of the AllJoyn release applying this macro (date format: YY.MM). */
 #else
 #define QCC_DEPRECATED(func) func /**< not all gcc versions support the deprecated attribute. */
+#define QCC_DEPRECATED_ON(func, date) func /**< not all gcc versions support the deprecated attribute. */
 #define QCC_DEPRECATED_CLASS /**< not all gcc versions support the deprecated attribute. */
+#define QCC_DEPRECATED_CLASS_ON(date) /**< not all gcc versions support the deprecated attribute. */
 #endif
 
 
 #elif defined(_MSC_VER)
 
 #define QCC_DEPRECATED(func) __declspec(deprecated) func /**< mark a function as deprecated in msvc. */
+#define QCC_DEPRECATED_ON(func, date) QCC_DEPRECATED(func) /**< same as above but with date of the AllJoyn release applying this macro (date format: YY.MM). */
 #define QCC_DEPRECATED_CLASS __declspec(deprecated) /**< mark a class as deprecated in msvc. */
-
+#define QCC_DEPRECATED_CLASS_ON(date) QCC_DEPRECATED_CLASS /**< same as above but with date of the AllJoyn release applying this macro (date format: YY.MM). */
 
 #else /* Some unknown compiler */
 
-#define QCC_DEPRECATED(func); /**< mark a function as deprecated. */
-#define QCC_DEPRECATED_CLASS /**< mark a class as deprecated. */
+#define QCC_DEPRECATED(func) func
+#define QCC_DEPRECATED_ON(func, date) func
+#define QCC_DEPRECATED_CLASS
+#define QCC_DEPRECATED_CLASS_ON(date)
 
 #endif /* Compiler type */
 
