@@ -30,6 +30,8 @@
 #include <errno.h>
 #include <string>
 
+#include <memory>
+
 #include <qcc/String.h>
 #include <qcc/Stream.h>
 #include <Status.h>
@@ -61,14 +63,21 @@ class FileSource : public Source {
   public:
 
     /**
-     * Create an FileSource
+     * Create an FileSource.
      *
      * @param fileName   Name of file to read/write
      */
     FileSource(qcc::String fileName);
 
     /**
-     * Create an FileSource from stdin
+     * Create an FileSource from existing file descriptor.
+     *
+     * @param fdesc   File descriptor.
+     */
+    FileSource(int fdesc);
+
+    /**
+     * Create an FileSource from stdin.
      */
     FileSource();
 
@@ -124,7 +133,7 @@ class FileSource : public Source {
     bool IsValid() { return 0 <= fd; }
 
     /**
-     * Lock the underlying file for exclusive access
+     * Lock the underlying file for shared access.
      *
      * @param block  If block is true the function will block until file access if permitted.
      *
@@ -145,12 +154,14 @@ class FileSource : public Source {
     bool locked;   /**< true if the sink has been locked for exclusive access */
 };
 
+class FileLock;
 
 /**
  * FileSink is an implementation of Sink used to write to files.
  */
 class FileSink : public Sink {
-
+    /* Required to access fd. */
+    friend class FileLock;
   public:
 
     /**
@@ -169,6 +180,14 @@ class FileSink : public Sink {
      */
     FileSink(qcc::String fileName, Mode mode = WORLD_READABLE);
 
+    /**
+     * Create an read-write FileSink.
+     *
+     * @param fileName     Name of file to use as sink.
+     * @param truncate     Set this to true to truncate the file.
+     */
+    FileSink(qcc::String fileName, bool truncate, Mode mode = WORLD_READABLE);
+
     /**
      * Create an FileSink for stdout
      */
@@ -216,13 +235,20 @@ class FileSink : public Sink {
      */
     bool IsValid() { return 0 <= fd; }
 
+    /**
+     * Truncate file to current offset.
+     *
+     * @return true on success.
+     */
+    bool Truncate();
+
     /**
      * Lock the underlying file for exclusive access
      *
      * @param block  If block is true the function will block until file access if permitted.
      *
      * @return Returns true if the file was locked, false if the file was not locked or if the file
-     *         was not valid, i.e. if IsValid() would returnf false;
+     *         was not valid, i.e. if IsValid() would return false.
      */
     bool Lock(bool block = false);
 
@@ -239,6 +265,46 @@ class FileSink : public Sink {
     bool locked;   /**< true if the sink has been locked for exclusive access */
 };
 
+class FileLocker;
+
+class FileLock {
+    friend class FileLocker;
+  public:
+    FileSource* GetSource();
+    FileSink* GetSink();
+    void Release();
+
+  private:
+    QStatus InitReadOnly(const char* fullFileName);
+    QStatus InitReadWrite(std::shared_ptr<FileSink> sink);
+
+    std::unique_ptr<FileSource> m_source;
+    std::shared_ptr<FileSink> m_sink;
+};
+
+class FileLocker {
+  public:
+    FileLocker(const char* fullFileName);
+    ~FileLocker();
+
+    const char* GetFileName() const;
+
+    bool HasWriteLock() const;
+
+    /* Request read lock on the shared file. Once the caller's FileLock goes out of scope, the read lock is released. */
+    QStatus GetFileLockForRead(FileLock* fileLock);
+    QStatus GetFileLockForWrite(FileLock* fileLock);
+
+    /* Request write lock on the shared file. Caller must call ReleaseWriteLock. */
+    QStatus AcquireWriteLock();
+    void ReleaseWriteLock();
+
+  private:
+    qcc::String m_fileName;
+    std::shared_ptr<FileSink> m_sink;
+    mutable qcc::Mutex m_sinkLock;
+};
+
 }   /* namespace */
 
 #endif
