@@ -24,21 +24,6 @@
 
 #include <windows.h>
 #include <float.h>
-/// @cond ALLJOYN_DEV
-
-#if ((_MSC_VER <= 1800) || defined(DO_SNPRINTF_MAPPING))
-
-/**
- * Map snprintf to _snprintf
- *
- * snprintf does not properly map in Windows; this is needed to ensure calls to
- * snprintf(char *str, size_t size, const char *format, ...) will compile in
- * Windows.
- */
-#define snprintf _snprintf
-
-#endif
-/// @endcond
 
 /**
  * Map stroll to _strtoi64
@@ -92,6 +77,29 @@
 #include <algorithm>
 #endif
 #endif
+
+/**
+ * Map QCC_ASSERT to _ASSERTE if not already defined.
+ *
+ * _ASSERTE allows the developer to break into a debugger instead of aborting.
+ */
+#ifndef QCC_ASSERT
+
+/* <crtdbg.h> keys off of the presence of _DEBUG, not the absence of NDEBUG.
+ * Make sure _DEBUG is defined if NDEBUG is not so _ASSERTE is correctly defined
+ * for both build variants. This must be done before <crtdbg.h> is included.
+ */
+#ifndef NDEBUG
+#ifndef _DEBUG
+#define _DEBUG
+#endif
+#endif
+
+#include <crtdbg.h>
+
+#define QCC_ASSERT(expr) _ASSERTE(expr)
+
+#endif /* QCC_ASSERT */
 /// @endcond
 
 /**
@@ -114,17 +122,4 @@
 #define WHITELISTED_APPLICATION       "WhitelistedApplication"
 ///@}
 
-/**
- * @name _WIN32_WINNT version constants
- * _WIN32_WINNT version constants, defined here just in case the Windows SDK version used to
- * compile AllJoyn doesn't include them already in Include\shared\sdkddkver.h. Compiler would complain
- * if the SDK public header values were different from the values below.
- */
-///@{
-#define _WIN32_WINNT_WIN7     0x0601
-#define _WIN32_WINNT_WIN8     0x0602
-#define _WIN32_WINNT_WINBLUE  0x0603
-#define _WIN32_WINNT_WIN10    0x0A00
-///@}
-
 #endif
