@@ -21,7 +21,6 @@
  ******************************************************************************/
 #include <qcc/platform.h>
 
-#include <assert.h>
 #include <errno.h>
 #include <fcntl.h>
 #include <pthread.h>
@@ -485,13 +484,18 @@ static void CreateMechanism(int* rdFd, int* wrFd)
 #else
     /* TODO: Potential thread safety issue here */
     if (NULL == pipeLock) {
-        pipeLock = new Mutex();
+        /*
+         * Disable LockOrderChecker for the pipeLock lock, because this lock is
+         * being used indirectly by the LockChecker itself when it looks up the
+         * current thread.
+         */
+        pipeLock = new Mutex(LOCK_LEVEL_CHECKING_DISABLED);
         freePipeList = new vector<pair<int, int> >;
         usedPipeList = new vector<pair<int, int> >;
     }
 
     /* Check for something on the free pipe list */
-    pipeLock->Lock();
+    pipeLock->Lock(MUTEX_CONTEXT);
     if (!freePipeList->empty()) {
         pair<int, int> fdPair = freePipeList->back();
         usedPipeList->push_back(fdPair);
@@ -511,7 +515,7 @@ static void CreateMechanism(int* rdFd, int* wrFd)
             QCC_LogError(ER_FAIL, ("Failed to create pipe. (%d) %s", errno, strerror(errno)));
         }
     }
-    pipeLock->Unlock();
+    pipeLock->Unlock(MUTEX_CONTEXT);
 #endif
 }
 
@@ -521,7 +525,7 @@ static void DestroyMechanism(int rdFd, int wrFd)
     close(rdFd);
     close(wrFd);
 #else
-    pipeLock->Lock();
+    pipeLock->Lock(MUTEX_CONTEXT);
 
     /*
      * Delete the pipe (permanently) if the number of pipes on the free list is twice as many as
@@ -572,9 +576,9 @@ static void DestroyMechanism(int rdFd, int wrFd)
                 ret = read(rdFd, buf, sizeof(buf));
             }
         }
-        pipeLock->Unlock();
+        pipeLock->Unlock(MUTEX_CONTEXT);
     } else {
-        pipeLock->Unlock();
+        pipeLock->Unlock(MUTEX_CONTEXT);
     }
 #endif
 }
@@ -666,7 +670,7 @@ static void DestroyMechanism(int readFd, int writeFd)
 {
     QCC_UNUSED(writeFd);
     QCC_DbgTrace(("DestroyMechanism()"));
-    assert(readFd == writeFd && "destroyMechanism(): expect readFd == writeFd for eventfd mechanism");
+    QCC_ASSERT(readFd == writeFd && "destroyMechanism(): expect readFd == writeFd for eventfd mechanism");
     close(readFd);
 }
 
