@@ -48,6 +48,7 @@
 #include <qcc/IPAddress.h>
 #include <qcc/Util.h>
 #include <qcc/Thread.h>
+#include <qcc/PerfCounters.h>
 
 #include <Status.h>
 
@@ -74,7 +75,7 @@ QStatus MakeSockAddr(const char* path,
 {
     size_t pathLen = strlen(path);
     struct sockaddr_un sa;
-    assert((size_t)addrSize >= sizeof(sa));
+    QCC_ASSERT((size_t)addrSize >= sizeof(sa));
     memset(&sa, 0, sizeof(sa));
     sa.sun_family = AF_UNIX;
     memcpy(sa.sun_path, path, (std::min)(pathLen, sizeof(sa.sun_path) - 1));
@@ -103,7 +104,7 @@ QStatus MakeSockAddr(const IPAddress& addr, uint16_t port, uint32_t scopeId,
 {
     if (addr.IsIPv4()) {
         struct sockaddr_in sa;
-        assert((size_t)addrSize >= sizeof(sa));
+        QCC_ASSERT((size_t)addrSize >= sizeof(sa));
         memset(&sa, 0, sizeof(sa));
         sa.sin_family = AF_INET;
         sa.sin_port = htons(port);
@@ -112,7 +113,7 @@ QStatus MakeSockAddr(const IPAddress& addr, uint16_t port, uint32_t scopeId,
         memcpy(addrBuf, &sa, sizeof(sa));
     } else {
         struct sockaddr_in6 sa;
-        assert((size_t)addrSize >= sizeof(sa));
+        QCC_ASSERT((size_t)addrSize >= sizeof(sa));
         memset(&sa, 0, sizeof(sa));
         sa.sin6_family = AF_INET6;
         sa.sin6_port = htons(port);
@@ -459,7 +460,11 @@ QStatus Shutdown(SocketFd sockfd, ShutdownHow how)
 
     ret = shutdown(static_cast<int>(sockfd), static_cast<int>(how));
     if (ret != 0) {
-        status = ER_OS_ERROR;
+        if (errno == ENOTCONN) {
+            status = ER_NOT_CONN;
+        } else {
+            status = ER_OS_ERROR;
+        }
         QCC_LogError(status, ("Shutdown socket (sockfd = %u): %d - %s", sockfd, errno, strerror(errno)));
     }
     return status;
@@ -472,7 +477,7 @@ QStatus Shutdown(SocketFd sockfd)
 
 void Close(SocketFd sockfd)
 {
-    assert(sockfd >= 0);
+    QCC_ASSERT(sockfd >= 0);
     close(static_cast<int>(sockfd));
 }
 
@@ -523,7 +528,6 @@ QStatus GetLocalAddress(SocketFd sockfd, IPAddress& addr, uint16_t& port)
     return status;
 }
 
-
 QStatus Send(SocketFd sockfd, const void* buf, size_t len, size_t& sent)
 {
     QStatus status = ER_OK;
@@ -531,7 +535,8 @@ QStatus Send(SocketFd sockfd, const void* buf, size_t len, size_t& sent)
 
     QCC_DbgTrace(("Send(sockfd = %d, *buf = <>, len = %lu, sent = <>)",
                   sockfd, len));
-    assert(buf != NULL);
+    IncrementPerfCounter(PERF_COUNTER_SOCKET_SEND);
+    QCC_ASSERT(buf != NULL);
 
     QCC_DbgLocalData(buf, len);
 
@@ -562,7 +567,8 @@ QStatus SendTo(SocketFd sockfd, IPAddress& remoteAddr, uint16_t remotePort, uint
 
     QCC_DbgTrace(("SendTo(sockfd = %d, remoteAddr = %s, remotePort = %u, *buf = <>, len = %lu, sent = <>, flags = 0x%x)",
                   sockfd, remoteAddr.ToString().c_str(), remotePort, len, (int)flags));
-    assert(buf != NULL);
+    IncrementPerfCounter(PERF_COUNTER_SOCKET_SENDTO);
+    QCC_ASSERT(buf != NULL);
 
     QCC_DbgLocalData(buf, len);
 
@@ -603,7 +609,8 @@ QStatus Recv(SocketFd sockfd, void* buf, size_t len, size_t& received)
     ssize_t ret;
 
     QCC_DbgTrace(("Recv(sockfd = %d, buf = <>, len = %lu, received = <>)", sockfd, len));
-    assert(buf != NULL);
+    IncrementPerfCounter(PERF_COUNTER_SOCKET_RECV);
+    QCC_ASSERT(buf != NULL);
 
     ret = recv(static_cast<int>(sockfd), buf, len, 0);
     if ((ret == -1) && (errno == EWOULDBLOCK)) {
@@ -621,7 +628,6 @@ QStatus Recv(SocketFd sockfd, void* buf, size_t len, size_t& received)
     return status;
 }
 
-
 QStatus RecvFrom(SocketFd sockfd, IPAddress& remoteAddr, uint16_t& remotePort,
                  void* buf, size_t len, size_t& received)
 {
@@ -633,7 +639,8 @@ QStatus RecvFrom(SocketFd sockfd, IPAddress& remoteAddr, uint16_t& remotePort,
 
     QCC_DbgTrace(("RecvFrom(sockfd = %d, remoteAddr = %s, remotePort = %u, buf = <>, len = %lu, received = <>)",
                   sockfd, remoteAddr.ToString().c_str(), remotePort, len));
-    assert(buf != NULL);
+    IncrementPerfCounter(PERF_COUNTER_SOCKET_RECV_FROM);
+    QCC_ASSERT(buf != NULL);
 
     ret = recvfrom(static_cast<int>(sockfd), buf, len, 0,
                    reinterpret_cast<struct sockaddr*>(&addr), &addrLen);
@@ -659,6 +666,7 @@ QStatus RecvWithAncillaryData(SocketFd sockfd, IPAddress& remoteAddr, uint16_t&
     received = 0;
     interfaceIndex = -1;
     uint16_t localPort;
+    IncrementPerfCounter(PERF_COUNTER_SOCKET_RECV_WITH_ANCILLARY_DATA);
 
     struct iovec iov[] = { { buf, len } };
 
@@ -697,7 +705,7 @@ QStatus RecvWithAncillaryData(SocketFd sockfd, IPAddress& remoteAddr, uint16_t&
         return status;
     }
 
-    assert(buf != NULL);
+    QCC_ASSERT(buf != NULL);
 
     ssize_t ret = recvmsg(static_cast<int>(sockfd), &msg, 0);
 
@@ -744,13 +752,15 @@ QStatus RecvWithFds(SocketFd sockfd, void* buf, size_t len, size_t& received, So
 {
     QStatus status = ER_OK;
 
+    QCC_DbgHLPrintf(("RecvWithFds"));
+    IncrementPerfCounter(PERF_COUNTER_SOCKET_RECV_WITH_FDS);
+
     if (!fdList) {
         return ER_BAD_ARG_5;
     }
     if (!maxFds) {
         return ER_BAD_ARG_6;
     }
-    QCC_DbgHLPrintf(("RecvWithFds"));
 
     recvdFds = 0;
     maxFds = std::min(maxFds, SOCKET_MAX_FILE_DESCRIPTORS);
@@ -804,6 +814,9 @@ QStatus SendWithFds(SocketFd sockfd, const void* buf, size_t len, size_t& sent,
     QCC_UNUSED(pid);
     QStatus status = ER_OK;
 
+    QCC_DbgHLPrintf(("SendWithFds"));
+    IncrementPerfCounter(PERF_COUNTER_SOCKET_SEND_WITH_FDS);
+
     if (!fdList) {
         return ER_BAD_ARG_5;
     }
@@ -811,8 +824,6 @@ QStatus SendWithFds(SocketFd sockfd, const void* buf, size_t len, size_t& sent,
         return ER_BAD_ARG_6;
     }
 
-    QCC_DbgHLPrintf(("SendWithFds"));
-
     struct iovec iov[] = { { const_cast<void*>(buf), len } };
     size_t sz = numFds * sizeof(SocketFd);
     char* cbuf = new char[CMSG_SPACE(sz)];
@@ -1025,11 +1036,11 @@ QStatus MulticastGroupOpInternal(SocketFd sockFd, AddressFamily family, String m
      * We assume that no external API will be trying to call here and so asserts
      * are appropriate when checking for completely bogus parameters.
      */
-    assert(sockFd >= 0);
-    assert(family == AF_INET || family == AF_INET6);
-    assert(multicastGroup.size());
-    assert(iface.size());
-    assert(op == JOIN || op == LEAVE);
+    QCC_ASSERT(sockFd >= 0);
+    QCC_ASSERT(family == AF_INET || family == AF_INET6);
+    QCC_ASSERT(multicastGroup.size());
+    QCC_ASSERT(iface.size());
+    QCC_ASSERT(op == JOIN || op == LEAVE);
     /*
      * Joining a multicast group requires a different approach based on the
      * address family of the socket.  There's no way to get the address family
@@ -1116,9 +1127,9 @@ QStatus SetMulticastInterface(SocketFd sockFd, AddressFamily family, qcc::String
      * We assume that no external API will be trying to call here and so asserts
      * are appropriate when checking for completely bogus parameters.
      */
-    assert(sockFd >= 0);
-    assert(family == AF_INET || family == AF_INET6);
-    assert(iface.size());
+    QCC_ASSERT(sockFd >= 0);
+    QCC_ASSERT(family == AF_INET || family == AF_INET6);
+    QCC_ASSERT(iface.size());
 
     /*
      * Associating the multicast interface with a socket requires a different
@@ -1175,8 +1186,8 @@ QStatus SetMulticastHops(SocketFd sockFd, AddressFamily family, uint32_t hops)
      * We assume that no external API will be trying to call here and so asserts
      * are appropriate when checking for completely bogus parameters.
      */
-    assert(sockFd >= 0);
-    assert(family == AF_INET || family == AF_INET6);
+    QCC_ASSERT(sockFd >= 0);
+    QCC_ASSERT(family == AF_INET || family == AF_INET6);
 
     /*
      * IPv4 and IPv6 are almost the same.  Of course, not quite, though.
@@ -1215,8 +1226,8 @@ QStatus SetRecvPktAncillaryData(SocketFd sockfd, AddressFamily addrFamily, bool
      * We assume that no external API will be trying to call here and so asserts
      * are appropriate when checking for completely bogus parameters.
      */
-    assert(sockfd >= 0);
-    assert(addrFamily == AF_INET || addrFamily == AF_INET6);
+    QCC_ASSERT(sockfd >= 0);
+    QCC_ASSERT(addrFamily == AF_INET || addrFamily == AF_INET6);
 
     QStatus status = ER_OK;
     int arg = recv ? 1 : -0;
