@@ -34,6 +34,7 @@
 #include <qcc/Util.h>
 #include <qcc/Thread.h>
 #include <qcc/StringUtil.h>
+#include <qcc/PerfCounters.h>
 #include <qcc/windows/utility.h>
 
 #include <Status.h>
@@ -407,14 +408,14 @@ QStatus GetLocalAddress(SocketFd sockfd, IPAddress& addr, uint16_t& port)
     return status;
 }
 
-
 QStatus Send(SocketFd sockfd, const void* buf, size_t len, size_t& sent)
 {
     QStatus status = ER_OK;
     size_t ret;
 
-    QCC_DbgTrace(("ERSend(sockfd = %d, *buf = <>, len = %lu, sent = <>)", sockfd, len));
-    assert(buf != NULL);
+    QCC_DbgTrace(("Send(sockfd = %d, *buf = <>, len = %lu, sent = <>)", sockfd, len));
+    IncrementPerfCounter(PERF_COUNTER_SOCKET_SEND);
+    QCC_ASSERT(buf != NULL);
 
     QCC_DbgLocalData(buf, len);
 
@@ -444,7 +445,8 @@ QStatus SendTo(SocketFd sockfd, IPAddress& remoteAddr, uint16_t remotePort, uint
 
     QCC_DbgTrace(("SendTo(sockfd = %d, remoteAddr = %s, remotePort = %u, *buf = <>, len = %lu, sent = <>, flags = 0x%x)",
                   sockfd, remoteAddr.ToString().c_str(), remotePort, len, (int)flags));
-    assert(buf != NULL);
+    IncrementPerfCounter(PERF_COUNTER_SOCKET_SENDTO);
+    QCC_ASSERT(buf != NULL);
 
     QCC_DbgLocalData(buf, len);
 
@@ -478,7 +480,8 @@ QStatus Recv(SocketFd sockfd, void* buf, size_t len, size_t& received)
     size_t ret;
 
     QCC_DbgTrace(("Recv(sockfd = %d, buf = <>, len = %lu, received = <>)", sockfd, len));
-    assert(buf != NULL);
+    IncrementPerfCounter(PERF_COUNTER_SOCKET_RECV);
+    QCC_ASSERT(buf != NULL);
 
     ret = recv(static_cast<SOCKET>(sockfd), static_cast<char*>(buf), len, 0);
     if (ret == SOCKET_ERROR) {
@@ -506,9 +509,9 @@ QStatus RecvWithAncillaryData(SocketFd sockfd, IPAddress& remoteAddr, uint16_t&
     received = 0;
     interfaceIndex = -1;
     uint16_t localPort;
+    IncrementPerfCounter(PERF_COUNTER_SOCKET_RECV_WITH_ANCILLARY_DATA);
 
     WSABUF iov[] = { { (u_long)len, reinterpret_cast<char*>(buf) } };
-
     char cbuf[1024];
 
     WSAMSG msg;
@@ -560,7 +563,7 @@ QStatus RecvWithAncillaryData(SocketFd sockfd, IPAddress& remoteAddr, uint16_t&
         }
     }
 
-    assert(buf != NULL);
+    QCC_ASSERT(buf != NULL);
 
     ret = WSARecvMsg(static_cast<SOCKET>(sockfd), &msg, &recv, NULL, NULL);
 
@@ -613,7 +616,8 @@ QStatus RecvFrom(SocketFd sockfd, IPAddress& remoteAddr, uint16_t& remotePort,
     received = 0;
 
     QCC_DbgTrace(("RecvFrom(sockfd = %d, buf = <>, len = %lu, received = <>)", sockfd, len));
-    assert(buf != NULL);
+    IncrementPerfCounter(PERF_COUNTER_SOCKET_RECV_FROM);
+    QCC_ASSERT(buf != NULL);
 
     ret = recvfrom(static_cast<int>(sockfd), static_cast<char*>(buf), len, 0,
                    reinterpret_cast<sockaddr*>(&fromAddr), &addrLen);
@@ -641,13 +645,15 @@ QStatus RecvWithFds(SocketFd sockfd, void* buf, size_t len, size_t& received, So
 {
     QStatus status = ER_OK;
 
+    QCC_DbgHLPrintf(("RecvWithFds"));
+    IncrementPerfCounter(PERF_COUNTER_SOCKET_RECV_WITH_FDS);
+
     if (!fdList) {
         return ER_BAD_ARG_5;
     }
     if (!maxFds) {
         return ER_BAD_ARG_6;
     }
-    QCC_DbgHLPrintf(("RecvWithFds"));
 
     recvdFds = 0;
     maxFds = std::min(maxFds, SOCKET_MAX_FILE_DESCRIPTORS);
@@ -731,6 +737,9 @@ QStatus SendWithFds(SocketFd sockfd, const void* buf, size_t len, size_t& sent,
 {
     QStatus status = ER_OK;
 
+    QCC_DbgHLPrintf(("SendWithFds"));
+    IncrementPerfCounter(PERF_COUNTER_SOCKET_SEND_WITH_FDS);
+
     if (!fdList) {
         return ER_BAD_ARG_5;
     }
@@ -738,8 +747,6 @@ QStatus SendWithFds(SocketFd sockfd, const void* buf, size_t len, size_t& sent,
         return ER_BAD_ARG_6;
     }
 
-    QCC_DbgHLPrintf(("SendWithFds"));
-
     /*
      * We send the file descriptor count as OOB data.
      */
@@ -977,8 +984,6 @@ QStatus SetReuseAddress(SocketFd sockfd, bool reuse)
     return status;
 }
 
-
-
 QStatus SetReusePort(SocketFd sockfd, bool reuse)
 {
     QStatus status = ER_OK;
@@ -1004,11 +1009,11 @@ QStatus MulticastGroupOpInternal(SocketFd sockFd, AddressFamily family, String m
      * We assume that no external API will be trying to call here and so asserts
      * are appropriate when checking for completely bogus parameters.
      */
-    assert(sockFd != INVALID_SOCKET);
-    assert(family == AF_INET || family == AF_INET6);
-    assert(multicastGroup.size());
-    assert(iface.size());
-    assert(op == JOIN || op == LEAVE);
+    QCC_ASSERT(sockFd != INVALID_SOCKET);
+    QCC_ASSERT(family == AF_INET || family == AF_INET6);
+    QCC_ASSERT(multicastGroup.size());
+    QCC_ASSERT(iface.size());
+    QCC_ASSERT(op == JOIN || op == LEAVE);
     /*
      * Joining a multicast group requires a different approach based on the
      * address family of the socket.  There's no way to get the address family
@@ -1088,9 +1093,9 @@ QStatus SetMulticastInterface(SocketFd sockFd, AddressFamily family, qcc::String
      * We assume that no external API will be trying to call here and so asserts
      * are appropriate when checking for completely bogus parameters.
      */
-    assert(sockFd != INVALID_SOCKET);
-    assert(family == AF_INET || family == AF_INET6);
-    assert(iface.size());
+    QCC_ASSERT(sockFd != INVALID_SOCKET);
+    QCC_ASSERT(family == AF_INET || family == AF_INET6);
+    QCC_ASSERT(iface.size());
 
     /*
      * Associating the multicast interface with a socket requires a different
@@ -1144,8 +1149,8 @@ QStatus SetMulticastHops(SocketFd sockFd, AddressFamily family, uint32_t hops)
      * We assume that no external API will be trying to call here and so asserts
      * are appropriate when checking for completely bogus parameters.
      */
-    assert(sockFd != INVALID_SOCKET);
-    assert(family == AF_INET || family == AF_INET6);
+    QCC_ASSERT(sockFd != INVALID_SOCKET);
+    QCC_ASSERT(family == AF_INET || family == AF_INET6);
 
     /*
      * IPv4 and IPv6 are almost the same.  Of course, not quite, though.
@@ -1184,8 +1189,8 @@ QStatus SetRecvPktAncillaryData(SocketFd sockfd, AddressFamily addrFamily, bool
      * We assume that no external API will be trying to call here and so asserts
      * are appropriate when checking for completely bogus parameters.
      */
-    assert(sockfd != INVALID_SOCKET);
-    assert(addrFamily == AF_INET || addrFamily == AF_INET6);
+    QCC_ASSERT(sockfd != INVALID_SOCKET);
+    QCC_ASSERT(addrFamily == AF_INET || addrFamily == AF_INET6);
 
     QStatus status = ER_OK;
     int arg = recv ? 1 : -0;
