@@ -23,7 +23,6 @@
 #include <qcc/platform.h>
 
 #include <algorithm>
-#include <assert.h>
 #include <process.h>
 #include <map>
 
@@ -32,6 +31,8 @@
 #include <qcc/Debug.h>
 #include <qcc/Thread.h>
 #include <qcc/Mutex.h>
+#include <qcc/LockLevel.h>
+#include <qcc/PerfCounters.h>
 
 #include <Status.h>
 
@@ -42,11 +43,9 @@ using namespace std;
 
 namespace qcc {
 
-#ifndef NDEBUG
 static volatile int32_t started = 0;
 static volatile int32_t running = 0;
 static volatile int32_t stopped = 0;
-#endif
 
 /** Maximum number of milliseconds to wait between calls to select to check for thread death */
 static const uint32_t MAX_SELECT_WAIT_MS = 10000;
@@ -55,8 +54,8 @@ static const uint32_t MAX_SELECT_WAIT_MS = 10000;
 Mutex* Thread::threadListLock = NULL;
 map<ThreadId, Thread*>* Thread::threadList = NULL;
 
-static DWORD cleanExternalThreadKey;
-static bool initialized = false;
+static DWORD cleanExternalThreadKey = FLS_OUT_OF_INDEXES;
+bool Thread::initialized = false;
 
 void Thread::CleanExternalThread(void* t)
 {
@@ -76,10 +75,11 @@ void Thread::CleanExternalThread(void* t)
     threadListLock->Unlock();
 }
 
-QStatus Thread::Init()
+QStatus Thread::StaticInit()
 {
     if (!initialized) {
-        Thread::threadListLock = new Mutex();
+        /* Disable LockChecker for the threadListLock, thus allowing LockChecker to call GetThread() */
+        Thread::threadListLock = new Mutex(LOCK_LEVEL_CHECKING_DISABLED);
         Thread::threadList = new map<ThreadId, Thread*>();
         cleanExternalThreadKey = FlsAlloc(Thread::CleanExternalThread);
         if (cleanExternalThreadKey == FLS_OUT_OF_INDEXES) {
@@ -93,20 +93,31 @@ QStatus Thread::Init()
     return ER_OK;
 }
 
-QStatus Thread::Shutdown()
+QStatus Thread::StaticShutdown()
 {
     if (initialized) {
-        // Note that FlsFree will call the callback function for all
-        // fibers with a valid key in the Fls slot.
-        FlsFree(cleanExternalThreadKey);
+        if (cleanExternalThreadKey != FLS_OUT_OF_INDEXES) {
+            /*
+             * Free all External Thread objects, by triggering their FLS callbacks.
+             * Note that the Thread object corresponding to the current thread gets
+             * freed here too!
+             */
+            QCC_VERIFY(FlsFree(cleanExternalThreadKey));
+            cleanExternalThreadKey = FLS_OUT_OF_INDEXES;
+        }
+
+        /* A common root cause of a failed assertion here is the app forgetting to call BusAttachment::Join() */
+        QCC_ASSERT(Thread::threadList->size() == 0);
         delete Thread::threadList;
+        Thread::threadList = nullptr;
         delete Thread::threadListLock;
+        Thread::threadListLock = nullptr;
         initialized = false;
     }
     return ER_OK;
 }
 
-QStatus Sleep(uint32_t ms) {
+QStatus AJ_CALL Sleep(uint32_t ms) {
     ::sleep(ms);
     return ER_OK;
 }
@@ -184,9 +195,12 @@ Thread::Thread(qcc::String name, Thread::ThreadFunction func, bool isExternal) :
     platformContext(NULL),
     alertCode(0),
     auxListeners(),
-    auxListenersLock(),
+    auxListenersLock(LOCK_LEVEL_THREAD_AUXLISTENERSLOCK),
+    waitCount(0),
     threadId(isExternal ? GetCurrentThreadId() : 0)
 {
+    IncrementPerfCounter(PERF_COUNTER_THREAD_CREATED);
+
     /* qcc::String is not thread safe.  Don't use it here. */
     funcName[0] = '\0';
     strncpy(funcName, name.c_str(), sizeof(funcName));
@@ -203,7 +217,7 @@ Thread::Thread(qcc::String name, Thread::ThreadFunction func, bool isExternal) :
             if (ret == 0) {
                 QCC_LogError(ER_OS_ERROR, ("Setting TLS key: %s", GetLastError()));
             }
-            assert(ret != 0);
+            QCC_ASSERT(ret != 0);
         }
         threadListLock->Unlock();
     }
@@ -213,16 +227,14 @@ Thread::Thread(qcc::String name, Thread::ThreadFunction func, bool isExternal) :
 Thread::~Thread(void)
 {
     if (!isExternal) {
-        if (IsRunning()) {
-            Stop();
-            Join();
-        } else if (handle) {
-            CloseHandle(handle);
-            handle = 0;
-            QCC_DEBUG_ONLY(IncrementAndFetch(&stopped));
-        }
+        Stop();
+        Join();
     }
     QCC_DbgHLPrintf(("Thread::~Thread() [%s,%x] started:%d running:%d stopped:%d", GetName(), this, started, running, stopped));
+    IncrementPerfCounter(PERF_COUNTER_THREAD_DESTROYED);
+
+    /* To avoid the waitCount assertion, don't call Join() while deleting the thread object */
+    QCC_ASSERT(waitCount == 0);
 }
 
 
@@ -230,9 +242,9 @@ ThreadInternalReturn STDCALL Thread::RunInternal(void* arg)
 {
     Thread* thread(reinterpret_cast<Thread*>(arg));
 
-    assert(thread != NULL);
-    assert(thread->state == STARTED);
-    assert(!thread->isExternal);
+    QCC_ASSERT(thread != NULL);
+    QCC_ASSERT(thread->state == STARTED);
+    QCC_ASSERT(!thread->isExternal);
 
     QCC_DEBUG_ONLY(IncrementAndFetch(&started));
 
@@ -377,52 +389,47 @@ QStatus Thread::Alert(uint32_t threadAlertCode)
 
 QStatus Thread::Join(void)
 {
-
-    assert(!isExternal);
+    QCC_ASSERT(!isExternal);
 
     QStatus status = ER_OK;
     bool self = (threadId == GetCurrentThreadId());
 
     QCC_DbgTrace(("Thread::Join() [%s run: %s]", funcName, IsRunning() ? "true" : "false"));
 
-    /*
-     * Nothing to join if the thread is dead
-     */
+    IncrementAndFetch(&waitCount);
+
     if (state == DEAD) {
+        /*
+         * Nothing to join if the thread is dead
+         */
         QCC_DbgPrintf(("Thread::Join() thread is dead [%s]", funcName));
-        isStopping = false;
-        return ER_OK;
+    } else {
+        QCC_DbgPrintf(("[%s - %x] %s thread %x [%s - %x]",
+                       self ? funcName : GetThread()->funcName,
+                       self ? threadId : GetThread()->threadId,
+                       self ? "Closing" : "Joining",
+                       threadId, funcName, threadId));
+
+        if (!self) {
+            QCC_VERIFY(WaitForSingleObject(handle, INFINITE) == WAIT_OBJECT_0);
+        }
+        state = DEAD;
     }
 
-    QCC_DbgPrintf(("[%s - %x] %s thread %x [%s - %x]",
-                   self ? funcName : GetThread()->funcName,
-                   self ? threadId : GetThread()->threadId,
-                   self ? "Closing" : "Joining",
-                   threadId, funcName, threadId));
-    /*
-     * make local copy of handle so it is not deleted if two threads are in
-     * Join at the same time.
-     */
-    HANDLE goner = handle;
-    if (goner) {
-        DWORD ret;
-        handle = 0;
-        if (self) {
-            ret = WAIT_OBJECT_0;
-        } else {
-            ret = WaitForSingleObject(goner, INFINITE);
+    if (DecrementAndFetch(&waitCount) == 0) {
+        /*
+         * make local copy of handle so it is not closed multiple times if two threads are in
+         * Join at the same time.
+         */
+        HANDLE goner = InterlockedExchangePointer(&handle, nullptr);
+        if (goner != nullptr) {
+            QCC_VERIFY(CloseHandle(goner));
+            QCC_DEBUG_ONLY(IncrementAndFetch(&stopped));
         }
-        if (ret != WAIT_OBJECT_0) {
-            status = ER_OS_ERROR;
-            QCC_LogError(status, ("Joining thread: %d", ret));
-        }
-        CloseHandle(goner);
-        QCC_DEBUG_ONLY(IncrementAndFetch(&stopped));
     }
+
     QCC_DbgPrintf(("%s thread %s", self ? "Closed" : "Joined", funcName));
     isStopping = false;
-    /* once the state is changed to DEAD, we must not touch any member of this class anymore */
-    state = DEAD;
     return status;
 }
 
@@ -445,8 +452,13 @@ void Thread::RemoveAuxListener(ThreadListener* listener)
 
 ThreadReturn STDCALL Thread::Run(void* arg)
 {
-    assert(NULL != function);
+    QCC_ASSERT(NULL != function);
     return (*function)(arg);
 }
 
+ThreadId Thread::GetCurrentThreadId()
+{
+    return ::GetCurrentThreadId();
+}
+
 }    /* namespace */
