@@ -20,8 +20,6 @@
  *    OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  ******************************************************************************/
 
-#include <assert.h>
-
 #include <qcc/platform.h>
 #include <qcc/Debug.h>
 #include <qcc/Crypto.h>
@@ -150,6 +148,26 @@ QStatus AJ_CALL Crypto_ASN1::DecodeBase64(const qcc::String& b64in, qcc::String&
     return status;
 }
 
+QStatus AJ_CALL Crypto_ASN1::DecodeBase64(const qcc::String& b64, std::vector<uint8_t>& bin)
+{
+    size_t decodedBytes;
+    qcc::String binValue;
+    QStatus status = DecodeBase64(b64, binValue);
+
+    if (ER_OK == status) {
+        status = (binValue.size() % 2 == 0) ? ER_OK : ER_FAIL;
+    }
+
+    if (ER_OK == status) {
+        bin.resize(binValue.size() / 2);
+        decodedBytes = HexStringToBytes(binValue, bin.data(), bin.size());
+
+        status = (decodedBytes == bin.size()) ? ER_OK : ER_FAIL;
+    }
+
+    return status;
+}
+
 QStatus Crypto_ASN1::EncodeV(const char*& syntax, qcc::String& asn, va_list* argpIn)
 {
     va_list& argp = *argpIn;
@@ -435,7 +453,7 @@ QStatus Crypto_ASN1::DecodeV(const char*& syntax, const uint8_t* asn, size_t asn
             if ((tag != ASN_BITS) || !DecodeLen(asn, eod, len)) {
                 status = ER_FAIL;
             } else {
-                if (len == 0) {
+                if (len < 2) {
                     status = ER_FAIL;
                     continue;
                 }
@@ -536,10 +554,10 @@ QStatus Crypto_ASN1::DecodeV(const char*& syntax, const uint8_t* asn, size_t asn
                     continue;
                 }
                 if ((ASN_CONTEXT_SPECIFIC != (tag & ASN_CONTEXT_SPECIFIC)) ||
-                    ((uint8_t) (tag & 0x1F) != v) || !DecodeLen(asn, eod, len)) {
+                    ((uint8_t) (tag & 0x1F) != v) ||
+                    !DecodeLen(asn, eod, len)) {
                     status = ER_FAIL;
                 } else if (ASN_CONSTRUCTED_ENCODING == (tag & ASN_CONSTRUCTED_ENCODING)) {
-                    qcc::String seq;
                     status = DecodeV(syntax, asn, len, &argp);
                     if (*syntax++ != ')') {
                         status = ER_FAIL;
@@ -548,17 +566,17 @@ QStatus Crypto_ASN1::DecodeV(const char*& syntax, const uint8_t* asn, size_t asn
                     }
                 } else {
                     /* primitive content */
-                    while (*syntax && (*syntax != ')')) {
-                        syntax++;
-                    }
-                    if (*syntax == ')') {
-                        /* skip to the end of the c(...) block */
-                        status = ER_OK;
-                        syntax++;
-                        break; /* no more parsing */
-                    } else {
-                        status = ER_FAIL;
+                    if ((*syntax++ == '.') && (*syntax++ == ')')) {
+                        /*
+                         * Output len bytes as a String, by jumping to the common code path
+                         * at the end of the switch/case statement.
+                         */
+                        break;
                     }
+
+                    QCC_LogError(status, ("Mismatched tag %#x and syntax character '%c'", (uint32_t)tag, *(syntax - 1)));
+                    status = ER_FAIL;
+                    continue;
                 }
             }
             continue;
@@ -609,7 +627,7 @@ QStatus Crypto_ASN1::DecodeV(const char*& syntax, const uint8_t* asn, size_t asn
                 if (val) {
                     val->assign((char*)start, len);
                 }
-                asn = eod; /* every thing is consumed */
+                asn = eod; /* everything has been consumed */
                 continue;
             }
 
@@ -618,23 +636,26 @@ QStatus Crypto_ASN1::DecodeV(const char*& syntax, const uint8_t* asn, size_t asn
             QCC_LogError(status, ("Invalid syntax character \'%c\'", *(syntax - 1)));
         }
         // Shared code for all cases that fall through here
-        if (status == ER_OK) {
+        if ((status == ER_OK) && (len > 0)) {
             val = va_arg(argp, qcc::String*);
             val->assign((char*)asn, len);
             asn += len;
         }
     }
-    // Consume wildcard if we ran out of data
-    if (*syntax == '*') {
-        ++syntax;
-    } else if (*syntax == '/') {
-        // Optional arg was not present
-        val = va_arg(argp, qcc::String*);
-        val->clear();
-        if (syntax[1]) {
-            syntax += 2;
-        } else {
-            status = ER_BAD_ARG_1;
+
+    if (status == ER_OK) {
+        // Consume wildcard if we ran out of data
+        if (*syntax == '*') {
+            ++syntax;
+        } else if (*syntax == '/') {
+            // Optional arg was not present
+            val = va_arg(argp, qcc::String*);
+            val->clear();
+            if (syntax[1]) {
+                syntax += 2;
+            } else {
+                status = ER_BAD_ARG_1;
+            }
         }
     }
     return status;
@@ -753,6 +774,12 @@ qcc::String Crypto_ASN1::DecodeOID(const uint8_t* p, size_t len)
     return oid;
 }
 
+QStatus AJ_CALL Crypto_ASN1::EncodeBase64(const std::vector<uint8_t>& bin, qcc::String& b64)
+{
+    qcc::String binValue = BytesToHexString(bin.data(), bin.size(), true);
+    return EncodeBase64(binValue, b64);
+}
+
 qcc::String AJ_CALL Crypto_ASN1::ToString(const uint8_t* asn, size_t len, size_t indent)
 {
     qcc::String dump;
