@@ -41,6 +41,19 @@ namespace qcc {
  */
 #define X509_VERSION_3  2
 
+/*
+ * This is the largest supported PEM input. 100KB will easily accomodate
+ * cert chains of length 100 with the current ECC parameters, since each
+ * certificate is < 1KB in PEM format.
+ */
+#define MAX_PEM_INPUT_LEN_BYTES (100 * 1024)
+
+/*
+ * This is the smallest supported PEM input. 51 is the number of
+ * bytes required to have an empty public key.
+ */
+#define MIN_PEM_INPUT_LEN_BYTES 51
+
 /**
  * The X.509 OIDs
  */
@@ -165,7 +178,7 @@ QStatus AJ_CALL CertificateX509::DecodePrivateKeyPEM(const String& encoded, ECCP
     if (ER_OK != status) {
         return status;
     }
-    uint32_t ver;
+    uint32_t ver = 0;
     qcc::String prv;
     qcc::String oid;
     qcc::String rem;
@@ -216,7 +229,7 @@ QStatus AJ_CALL CertificateX509::EncodePublicKeyPEM(const ECCPublicKey* publicKe
     }
 
     // Uncompressed points only
-    qcc::String key(0x4);
+    qcc::String key(1, '\4');
     key += qcc::String((const char*)publicKeyBytes, publicKeySize);
     delete[] publicKeyBytes;
     status = Crypto_ASN1::Encode(der, "((oo)b)", &oid1, &oid2, &key, 8 * key.size());
@@ -251,7 +264,7 @@ QStatus AJ_CALL CertificateX509::DecodePublicKeyPEM(const String& encoded, ECCPu
     qcc::String oid1;
     qcc::String oid2;
     qcc::String key;
-    size_t keylen;
+    size_t keylen = 0;
     status = Crypto_ASN1::Decode(der, "((oo)b)", &oid1, &oid2, &key, &keylen);
     if (ER_OK != status) {
         return status;
@@ -545,7 +558,7 @@ QStatus CertificateX509::DecodeCertificatePub(const qcc::String& pub)
     qcc::String oid1;
     qcc::String oid2;
     qcc::String key;
-    size_t keylen;
+    size_t keylen = 0;
 
     status = Crypto_ASN1::Decode(pub, "(oo)b", &oid1, &oid2, &key, &keylen);
     if (ER_OK != status) {
@@ -586,7 +599,7 @@ QStatus CertificateX509::EncodeCertificatePub(qcc::String& pub) const
     }
 
     // Uncompressed points only
-    qcc::String key(0x4);
+    qcc::String key(1, '\4');
     key += qcc::String((const char*) publicKeyBytes, publicKeySize);
     status = Crypto_ASN1::Encode(pub, "(oo)b", &oid1, &oid2, &key, 8 * key.size());
 
@@ -763,7 +776,7 @@ QStatus CertificateX509::EncodeCertificateExt(qcc::String& ext) const
         break;
 
     default: /* Shouldn't ever happen. */
-        assert(false);
+        QCC_ASSERT(false);
         QCC_LogError(ER_FAIL, ("Encountered unknown type %u", type));
         return ER_FAIL;
     }
@@ -834,7 +847,7 @@ QStatus CertificateX509::EncodeCertificateExt(qcc::String& ext) const
 QStatus CertificateX509::DecodeCertificateTBS()
 {
     QStatus status = ER_OK;
-    uint32_t x509Version;
+    uint32_t x509Version = 0;
     qcc::String oid;
     qcc::String iss;
     qcc::String sub;
@@ -919,7 +932,7 @@ QStatus CertificateX509::EncodeCertificateTBS()
         return status;
     }
     serialStr.assign((const char*)serial, serialLen);
-    tbs.clear(tbs.size()); //empty the tbs string.
+    tbs.clear(); //empty the tbs string.
     status = Crypto_ASN1::Encode(tbs, "(c(i)l(o)(R)(R)(R)(R)R)",
                                  0, x509Version, &serialStr, &oid, &iss, &time, &sub, &pub, &ext);
 
@@ -976,14 +989,14 @@ QStatus CertificateX509::DecodeCertificateDER(const qcc::String& der)
     qcc::String oid;
     qcc::String sig;
     qcc::String tmp;
-    size_t siglen;
+    size_t siglen = 0;
 
     status = Crypto_ASN1::Decode(der, "((.)(o)b)", &tmp, &oid, &sig, &siglen);
     if (ER_OK != status) {
         return status;
     }
     // Put the sequence back on the TBS
-    tbs.clear(tbs.size());
+    tbs.clear();
     status = Crypto_ASN1::Encode(tbs, "(R)", &tmp);
     if (ER_OK != status) {
         return status;
@@ -1021,6 +1034,55 @@ QStatus CertificateX509::EncodeCertificateDER(qcc::String& der) const
     return status;
 }
 
+QStatus static ValidatePEMInput(const char* pemData, size_t pemLength)
+{
+    /*
+     * PEM Format Input Validation
+     * 1.  Character validation
+     *   - base64 valid characters 0-9 A-Z, a-z, +, /, =
+     *   - Valid PEM headers also include - and ' '
+     *   - Newline characters are valid as well
+     * 2. Length validation
+     *   - Min length must be at least as large as the smallest PEM
+     *     header/footer pair (begin/end public key).
+     *   - Max length is set to 100KB: we support up to 10 certificates at
+     *     most 1KB each.
+     *   - Each line should be <= 64 bytes by the PEM standard, but AJ has been
+     *     allowing large lines so we can't enforce this without the
+     *     possibility of breaking existing apps.
+     */
+
+    /* Validate length */
+    if (pemLength < MIN_PEM_INPUT_LEN_BYTES) {
+        QCC_LogError(ER_INVALID_DATA, ("PEM input validation failed; input too small."));
+        return ER_INVALID_DATA;
+    }
+
+    if (pemLength > MAX_PEM_INPUT_LEN_BYTES) {
+        QCC_LogError(ER_INVALID_DATA, ("PEM input validation failed; input too large."));
+        return ER_INVALID_DATA;
+    }
+
+    /* Check that all characters are valid. */
+    for (size_t i = 0; i < pemLength; i++) {
+        unsigned char c = pemData[i];
+        if (((c < 'A') || (c > 'Z'))  &&
+            ((c < 'a') || (c > 'z')) &&
+            ((c < 47)  || (c > 57))  &&      /* 47-57 is [/,0-9] */
+            (c != '-')  &&
+            (c != '+')  &&
+            (c != '=')  &&
+            (c != ' ')  &&
+            (c != 0x0D) &&      /* CR */
+            (c != 0x0A)) {      /* LF */
+            QCC_LogError(ER_INVALID_DATA, ("PEM input validation failed; invalid character('%#X') in input.", c));
+            return ER_INVALID_DATA;
+        }
+    }
+
+    return ER_OK;
+}
+
 QStatus CertificateX509::DecodeCertificatePEM(const qcc::String& pem)
 {
     QStatus status;
@@ -1030,6 +1092,11 @@ QStatus CertificateX509::DecodeCertificatePEM(const qcc::String& pem)
     qcc::String tag1 = CERTIFICATE_PEM_BEGIN_TAG;
     qcc::String tag2 = CERTIFICATE_PEM_END_TAG;
 
+    status = ValidatePEMInput(pem.data(), pem.size());
+    if (status != ER_OK) {
+        return status;
+    }
+
     pos = pem.find(tag1);
     if (pos == qcc::String::npos) {
         QCC_LogError(ER_INVALID_DATA, ("Error decoding certificate data from PEM. Only support tag -----BEGIN CERTIFICATE-----, tag -----END CERTIFICATE-----, and data"));
@@ -1280,6 +1347,11 @@ bool CertificateX509::IsDNEqual(const CertificateX509& other) const
 
 bool CertificateX509::IsDNEqual(const uint8_t* cn, const size_t cnLength, const uint8_t* ou, const size_t ouLength) const
 {
+    QCC_ASSERT((cnLength > 0) || (nullptr == cn));
+    QCC_ASSERT((ouLength > 0) || (nullptr == ou));
+    QCC_ASSERT((this->GetSubjectCNLength() > 0) || (nullptr == this->GetSubjectCN()));
+    QCC_ASSERT((this->GetSubjectOULength() > 0) || (nullptr == this->GetSubjectOU()));
+
     /* Check lengths are equal. */
     if ((this->GetSubjectCNLength() != cnLength) || (this->GetSubjectOULength() != ouLength)) {
         return false;
@@ -1343,4 +1415,25 @@ bool CertificateX509::ValidateCertificateTypeInCertChain(const CertificateX509*
     return true;
 }
 
+QStatus CertificateX509::GetSHA256Thumbprint(uint8_t* thumbprint) const
+{
+    String encodedPem;
+    QStatus status = this->EncodeCertificateDER(encodedPem);
+    if (ER_OK != status) {
+        return status;
+    }
+
+    Crypto_SHA256 hash;
+    status = hash.Init();
+    if (ER_OK != status) {
+        return status;
+    }
+    status = hash.Update((const uint8_t*)encodedPem.data(), encodedPem.size());
+    if (ER_OK != status) {
+        return status;
+    }
+
+    return hash.GetDigest(thumbprint, false);
+}
+
 }
