@@ -20,7 +20,6 @@
  *    OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  ******************************************************************************/
 
-#include <assert.h>
 
 #include <qcc/platform.h>
 #include <qcc/Debug.h>
@@ -38,7 +37,7 @@ using namespace qcc;
 
 namespace qcc {
 
-QStatus Crypto_PseudorandomFunction(const KeyBlob& secret, const char* label, const qcc::String& seed, uint8_t* out, size_t outLen)
+QStatus Crypto_PseudorandomFunction(const KeyBlob& secret, const char* label, const vector<uint8_t, SecureAllocator<uint8_t> >& seed, uint8_t* out, size_t outLen)
 {
     if (!label) {
         return ER_BAD_ARG_2;
@@ -62,7 +61,7 @@ QStatus Crypto_PseudorandomFunction(const KeyBlob& secret, const char* label, co
             hash.Update(digest, sizeof(digest));
         }
         hash.Update((const uint8_t*)label, strlen(label));
-        hash.Update((const uint8_t*)seed.data(), seed.size());
+        hash.Update(seed);
         hash.GetDigest(digest);
         len =  (std::min)(sizeof(digest), outLen);
         memcpy(out, digest, len);
@@ -76,8 +75,8 @@ int Crypto_Compare(const void* buf1, const void* buf2, size_t count)
 {
     uint8_t different = 0;
 
-    assert(buf1 != NULL);
-    assert(buf2 != NULL);
+    QCC_ASSERT(buf1 != NULL);
+    QCC_ASSERT(buf2 != NULL);
 
     /* This loop uses the same number of cycles for any two buffers of size count. */
     for (size_t i = 0; i < count; i++) {
