@@ -17,6 +17,7 @@
  *    OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  ******************************************************************************/
 
+#include <stdlib.h>
 #include <qcc/Util.h>
 #include <qcc/CryptoECC.h>
 #include <qcc/CryptoECCp256.h>
@@ -108,8 +109,8 @@ void ec_freecurve(ec_t* curve)
 /* Convert affine point Q = (x,y) to Jacobian P = (X:Y:1), where X=x, Y=y */
 void ec_affine_tojacobian(const ecpoint_t* Q, ecpoint_jacobian_t* P)
 {
-    assert(Q != NULL);
-    assert(P != NULL);
+    QCC_ASSERT(Q != NULL);
+    QCC_ASSERT(P != NULL);
 
     fpcopy_p256(Q->x, P->X);
     fpcopy_p256(Q->y, P->Y);
@@ -120,7 +121,7 @@ void ec_affine_tojacobian(const ecpoint_t* Q, ecpoint_jacobian_t* P)
 /* Set P to the generator of the curve.  */
 void ec_get_generator(ecpoint_t* P, ec_t* curve)
 {
-    assert(curve != NULL);
+    QCC_ASSERT(curve != NULL);
 
     memcpy(P->x, curve->generator.x, sizeof(digit256_t));
     memcpy(P->y, curve->generator.y, sizeof(digit256_t));
@@ -129,7 +130,7 @@ void ec_get_generator(ecpoint_t* P, ec_t* curve)
 /* Set the jacobian point P to zero (0,0,0). */
 static void ecpoint_jacobian_zero(ecpoint_jacobian_t* P)
 {
-    assert(P != NULL);
+    QCC_ASSERT(P != NULL);
     fpzero_p256(P->X);
     fpzero_p256(P->Y);
     fpzero_p256(P->Z);
@@ -138,8 +139,8 @@ static void ecpoint_jacobian_zero(ecpoint_jacobian_t* P)
 /* Copy Jacobian points, set src = dst. */
 static void ecpoint_jacobian_copy(ecpoint_jacobian_t* src, ecpoint_jacobian_t* dst)
 {
-    assert(src != NULL);
-    assert(dst != NULL);
+    QCC_ASSERT(src != NULL);
+    QCC_ASSERT(dst != NULL);
 
     fpcopy_p256(src->X, dst->X);
     fpcopy_p256(src->Y, dst->Y);
@@ -148,7 +149,7 @@ static void ecpoint_jacobian_copy(ecpoint_jacobian_t* src, ecpoint_jacobian_t* d
 
 static void ecpoint_chudnovsky_zero(ecpoint_chudnovsky_t* P)
 {
-    assert(P != NULL);
+    QCC_ASSERT(P != NULL);
     fpzero_p256(P->X);
     fpzero_p256(P->Y);
     fpzero_p256(P->Z);
@@ -705,9 +706,11 @@ void fixed_window_recode(digit256_t scalar, unsigned int nbit, unsigned int w, i
     digit_t temp, res, borrow;
 
     cwords = NBITS_TO_NDIGITS(nbit);            /* Number of computer words to represent scalar */
+    QCC_ASSERT(w >= 2);
     t = (nbit + (w - 2)) / (w - 1);             /* Fixed length of the fixed window representation */
     mask = (1 << w) - 1;                        /* w-bit mask */
-    val = (digit_t)(1 << (w - 1));              /* 2^(w-1)  */
+    QCC_ASSERT(w <= 32);                        /* Safe to convert to 64 bit digit_t value before shifting */
+    val = ((digit_t)1) << (w - 1);              /* 2^(w-1)  */
 
     for (i = 0; i <= (t - 1); i++) {
         temp = (scalar[0] & mask) - val;        /* ki = (k mod 2^w) - 2^(w-1)  */
@@ -843,4 +846,4 @@ QStatus ec_scalarmul(const ecpoint_t* P, digit256_t k, ecpoint_t* Q, ec_t* curve
     return status;
 }
 
-}
\ No newline at end of file
+}
