@@ -21,17 +21,17 @@
  ******************************************************************************/
 #include <qcc/IODispatch.h>
 #include <qcc/StringUtil.h>
+#include <qcc/LockLevel.h>
 #define QCC_MODULE "IODISPATCH"
 
 using namespace qcc;
 using namespace std;
 
 volatile int32_t IODispatch::iodispatchCnt = 0;
-volatile int32_t IODispatch::activeStreamsCnt = 0;
-volatile uint64_t IODispatch::stopStreamTimestamp = 0;
 
 IODispatch::IODispatch(const char* name, uint32_t concurrency) :
-    timer((String(name) + U32ToString(IncrementAndFetch(&iodispatchCnt)).c_str()), true, concurrency, false, 96),
+    timer((String(name) + U32ToString(IncrementAndFetch(&iodispatchCnt))), true, concurrency, false, 96),
+    lock(LOCK_LEVEL_IODISPATCH_LOCK),
     reload(false),
     isRunning(false),
     numAlarmsInProgress(0),
@@ -49,7 +49,7 @@ IODispatch::~IODispatch()
      * so, there should be no dispatch entries.
      * Just a sanity check.
      */
-    assert(dispatchEntries.size() == 0);
+    QCC_ASSERT(dispatchEntries.size() == 0);
 }
 
 QStatus IODispatch::Start(void* arg, ThreadListener* listener)
@@ -119,8 +119,8 @@ QStatus IODispatch::StartStream(Stream* stream, IOReadListener* readListener, IO
     if (dispatchEntries.find(stream) != dispatchEntries.end()) {
         lock.Unlock();
         return ER_INVALID_STREAM;
-
     }
+
     dispatchEntries[stream] = IODispatchEntry(stream, readListener, writeListener, exitListener, readEnable, writeEnable);
     dispatchEntries[stream].readCtxt = new CallbackContext(stream, IO_READ);
     dispatchEntries[stream].writeCtxt = new CallbackContext(stream, IO_WRITE);
@@ -131,11 +131,9 @@ QStatus IODispatch::StartStream(Stream* stream, IOReadListener* readListener, IO
     /* Set reload to false and alert the IODispatch::Run thread */
     reload = false;
     lock.Unlock();
-
-    UpdateIdleInformation(true);
     Thread::Alert();
 
-    /* Dont need to wait for the IODispatch::Run thread to reload
+    /* Don't need to wait for the IODispatch::Run thread to reload
      * the set of file descriptors since we are adding a new stream.
      */
     return ER_OK;
@@ -159,7 +157,6 @@ QStatus IODispatch::StopStream(Stream* stream)
     IODispatchEntry dispatchEntry = it->second;
 
     /* Disable further read and writes on this stream */
-    StoppingState previousState = it->second.stopping_state;
     it->second.stopping_state = IO_STOPPING;
 
     /* Set reload to false and alert the IODispatch::Run thread */
@@ -181,7 +178,8 @@ QStatus IODispatch::StopStream(Stream* stream)
          * added the exit alarm for this stream. The exit alarm makes the exit callback
          * which ensures that the RemoteEndpoint can be joined.
          */
-        if (it->second.stopping_state == IO_STOPPING) {
+        it = dispatchEntries.find(stream);
+        if (it != dispatchEntries.end() && it->second.stopping_state == IO_STOPPING) {
             /* Add the exit alarm since it has not added by the main IODispatch::Run thread. */
             it->second.stopping_state = IO_STOPPED;
             /* We dont need to keep track of the exit alarm, since we never remove
@@ -200,10 +198,6 @@ QStatus IODispatch::StopStream(Stream* stream)
         }
     }
 
-    if (previousState == IO_RUNNING) {
-        UpdateIdleInformation(false);
-    }
-
     return ER_OK;
 }
 
@@ -237,7 +231,7 @@ void IODispatch::AlarmTriggered(const Alarm& alarm, QStatus reason)
     /* Only correct values of type are IO_READ, IO_READ_TIMEOUT,
      * IO_WRITE, IO_WRITE_TIMEOUT, IO_EXIT
      */
-    assert(ctxt->type >= IO_READ && ctxt->type <= IO_EXIT);
+    QCC_ASSERT(ctxt->type >= IO_READ && ctxt->type <= IO_EXIT);
 
     if (!isRunning && ctxt->type != IO_EXIT) {
         /* If IODispatch is being shut down, only service exit alarms.
@@ -252,7 +246,7 @@ void IODispatch::AlarmTriggered(const Alarm& alarm, QStatus reason)
         /* If stream is not found(should never happen since the exit alarm ensures that
          * read and write alarms are removed before deleting the entry from the map)
          */
-        assert(false);
+        QCC_ASSERT(false);
         /*
          * asserts are compiled out so me must return to prevent segfaults in
          * release code.
@@ -350,7 +344,7 @@ void IODispatch::AlarmTriggered(const Alarm& alarm, QStatus reason)
             /* This should never happen - it means that the entry was deleted
              * while the exit callback was being made
              */
-            assert(false);
+            QCC_ASSERT(false);
             /*
              * since asserts are compiled out or release code we must add code
              * to handle this if it occurs.
@@ -879,37 +873,3 @@ bool IODispatch::IsTimerCallbackThread() const
 {
     return timer.IsTimerCallbackThread();
 }
-
-void IODispatch::UpdateIdleInformation(bool isStarting)
-{
-    if (isStarting) {
-        QCC_VERIFY(IncrementAndFetch(&activeStreamsCnt) > 0);
-    } else {
-        stopStreamTimestamp = GetTimestamp64();
-        QCC_VERIFY(DecrementAndFetch(&activeStreamsCnt) >= 0);
-    }
-}
-
-bool AJ_CALL IODispatch::IsIdle(uint64_t minTime)
-{
-    /* The dispatcher is considered idle if there are no connected leaf nodes
-     * and no leaf node has disconnected during the minTime period.
-     * Note that the dispatcher idle state can transition while this method is
-     * running, so the caller of this method has to be mindful about that race
-     * condition.
-     */
-    if (activeStreamsCnt == 0) {
-        uint64_t currentTimestamp = GetTimestamp64();
-        uint64_t previousTimestamp = stopStreamTimestamp;
-
-        if (currentTimestamp >= previousTimestamp) {
-            currentTimestamp -= previousTimestamp;
-
-            if (currentTimestamp >= minTime) {
-                return true;
-            }
-        }
-    }
-
-    return false;
-}
