@@ -27,6 +27,7 @@
 #include <qcc/String.h>
 #include <qcc/Thread.h>
 #include <qcc/Util.h>
+#include <qcc/PerfCounters.h>
 #ifdef QCC_OS_GROUP_WINDOWS
 #include <qcc/windows/utility.h>
 #include <qcc/windows/NamedPipeWrapper.h>
@@ -38,6 +39,9 @@
 
 namespace qcc {
 
+/* Counters easily found from a debugger, incremented for frequent SCL actions */
+volatile uint32_t s_PerfCounters[PERF_COUNTER_COUNT];
+
 class StaticGlobals {
   public:
     static QStatus Init()
@@ -58,7 +62,7 @@ class StaticGlobals {
         String::Init();
         DebugControl::Init();
         LoggerSetting::Init();
-        QStatus status = Thread::Init();
+        QStatus status = Thread::StaticInit();
         if (status != ER_OK) {
             Shutdown();
             return status;
@@ -74,7 +78,7 @@ class StaticGlobals {
     static QStatus Shutdown()
     {
         Crypto::Shutdown();
-        Thread::Shutdown();
+        Thread::StaticShutdown();
         LoggerSetting::Shutdown();
         DebugControl::Shutdown();
         String::Shutdown();
