@@ -28,6 +28,7 @@
 
 #include <qcc/String.h>
 #include <qcc/StringUtil.h>
+#include <qcc/SecureAllocator.h>
 
 using namespace std;
 using namespace qcc;
@@ -92,7 +93,8 @@ qcc::String AJ_CALL qcc::HexStringToByteString(const qcc::String& hex, char sepa
     } else {
         len = hex.length() / 2;
     }
-    qcc::String result(0, '\0', len);
+    qcc::String result;
+    result.reserve(len);
     qcc::String::const_iterator it = hex.begin();
     for (size_t i = 0; i < len; i++) {
         if (separator && (i != 0)) {
@@ -236,7 +238,7 @@ uint32_t AJ_CALL qcc::StringToU32(const qcc::String& inStr, unsigned int base, u
     bool isBad = true;
     qcc::String::const_iterator it = inStr.begin();
     if (base == 0) {
-        if (*it == '0') {
+        if ((it != inStr.end()) && (*it == '0')) {
             ++it;
             if (it == inStr.end()) {
                 return 0;
@@ -250,9 +252,9 @@ uint32_t AJ_CALL qcc::StringToU32(const qcc::String& inStr, unsigned int base, u
             base = 10;
         }
     } else if (base == 16) {
-        if (*it == '0') {
+        if ((it != inStr.end()) && (*it == '0')) {
             ++it;
-            if ((*it == 'x') || (*it == 'X')) {
+            if ((it != inStr.end()) && ((*it == 'x') || (*it == 'X'))) {
                 ++it;
             }
         }
@@ -307,7 +309,7 @@ uint64_t AJ_CALL qcc::StringToU64(const qcc::String& inStr, unsigned int base, u
     bool isBad = true;
     qcc::String::const_iterator it = inStr.begin();
     if (base == 0) {
-        if (*it == '0') {
+        if ((it != inStr.end()) && (*it == '0')) {
             ++it;
             if (it == inStr.end()) {
                 return 0;
@@ -321,9 +323,9 @@ uint64_t AJ_CALL qcc::StringToU64(const qcc::String& inStr, unsigned int base, u
             base = 10;
         }
     } else if (base == 16) {
-        if (*it == '0') {
+        if ((it != inStr.end()) && (*it == '0')) {
             ++it;
-            if ((*it == 'x') || (*it == 'X')) {
+            if ((it != inStr.end()) && ((*it == 'x') || (*it == 'X'))) {
                 ++it;
             }
         }
@@ -373,7 +375,7 @@ double AJ_CALL qcc::StringToDouble(const qcc::String& inStr)
         double val = 0.0;
         bool neg = false;
         qcc::String::const_iterator it = inStr.begin();
-        if (*it == '-') {
+        if ((it != inStr.end()) && (*it == '-')) {
             neg = true;
             ++it;
         }
@@ -386,7 +388,7 @@ double AJ_CALL qcc::StringToDouble(const qcc::String& inStr)
             val += static_cast<double>(v);
             ++it;
         }
-        if (*it == '.') {
+        if ((it != inStr.end()) && (*it == '.')) {
             double divisor = 1.0;
             ++it;
             while ((it != inStr.end()) && ((*it != 'e') && (*it != 'E'))) {
@@ -401,13 +403,13 @@ double AJ_CALL qcc::StringToDouble(const qcc::String& inStr)
             }
             val /= divisor;
         }
-        if ((*it == 'e') || (*it == 'E')) {
+        if ((it != inStr.end() && ((*it == 'e') || (*it == 'E')))) {
             ++it;
-            qcc::String exponentString = qcc::String(it, inStr.end() - it);
+            qcc::String exponentString = qcc::String(it, inStr.end());
 
             // verify that the exponent portion is sane
             qcc::String::const_iterator expStrIter = exponentString.begin();
-            if (*expStrIter == '-') {
+            if ((expStrIter != exponentString.end()) && (*expStrIter == '-')) {
                 ++expStrIter;
             }
             while (expStrIter != exponentString.end()) {
@@ -439,7 +441,7 @@ double AJ_CALL qcc::StringToDouble(const qcc::String& inStr)
 
 qcc::String AJ_CALL qcc::LineBreak(const qcc::String& inStr, size_t maxLen, size_t indent)
 {
-    qcc::String indentStr(' ', indent);
+    qcc::String indentStr(indent, ' ');
     qcc::String outStr;
     outStr.reserve(inStr.size() + maxLen + (inStr.size() / maxLen) * (indent + 1));
     size_t pos = 0;
@@ -499,6 +501,14 @@ qcc::String AJ_CALL qcc::StringVectorToString(const vector<qcc::String>* list, c
     return out;
 }
 
+
+void AJ_CALL qcc::AppendStringToSecureVector(const qcc::String& str, vector<uint8_t, SecureAllocator<uint8_t> >& v)
+{
+    auto begin = reinterpret_cast<const uint8_t*>(str.data());
+    auto end = begin + str.size();
+    v.insert(v.end(), begin, end);
+}
+
 bool AJ_CALL qcc::IsDecimalDigit(char c)
 {
     if (c >= '0' && c <= '9') {
