@@ -26,6 +26,8 @@
 #include <qcc/Thread.h>
 #include <qcc/Timer.h>
 #include <qcc/StringUtil.h>
+#include <qcc/LockLevel.h>
+#include <qcc/PerfCounters.h>
 #include <Status.h>
 #include <algorithm>
 
@@ -45,7 +47,7 @@ namespace qcc {
 class TimerThread : public Thread {
   public:
 
-    enum {
+    volatile enum {
         STOPPED,        /**< Thread must be started via Start() */
         STARTING,       /**< Thread has been Started but is not ready to service requests */
         IDLE,           /**< Thrad is sleeping. Waiting to be alerted via Alert() */
@@ -263,13 +265,14 @@ class TimerImpl : public ThreadListener {
     Alarm* currentAlarm;
     bool expireOnExit;
     std::vector<TimerThread*> timerThreads;
-    bool isRunning;
+    volatile bool isRunning;
     int32_t controllerIdx;
-    qcc::Timespec yieldControllerTime;
+    qcc::Timespec<qcc::MonotonicTime> yieldControllerTime;
     bool preventReentrancy;
     Mutex reentrancyLock;
     qcc::String nameStr;
     const uint32_t maxAlarms;
+    uint32_t numLimitableAlarms;               /**< Number of alarms currently in the alarm queue that count towards the limit */
     std::deque<qcc::Thread*> addWaitQueue;     /**< Threads waiting for alarms set to become not-full */
 
 };
@@ -277,14 +280,17 @@ class TimerImpl : public ThreadListener {
 }
 
 TimerImpl::TimerImpl(String name, bool expireOnExit, uint32_t concurrency, bool preventReentrancy, uint32_t maxAlarms) :
+    lock(LOCK_LEVEL_TIMERIMPL_LOCK),
     currentAlarm(NULL),
     expireOnExit(expireOnExit),
     timerThreads(concurrency),
     isRunning(false),
     controllerIdx(0),
     preventReentrancy(preventReentrancy),
+    reentrancyLock(LOCK_LEVEL_TIMERIMPL_REENTRANCYLOCK),
     nameStr(name),
-    maxAlarms(maxAlarms)
+    maxAlarms(maxAlarms),
+    numLimitableAlarms(0)
 {
     /* TimerImpl thread objects will be created when required */
 }
@@ -304,7 +310,7 @@ TimerImpl::~TimerImpl()
 QStatus TimerImpl::Start()
 {
     QStatus status = ER_OK;
-    lock.Lock();
+    lock.Lock(MUTEX_CONTEXT);
     if (!isRunning) {
         controllerIdx = 0;
         isRunning = true;
@@ -321,22 +327,22 @@ QStatus TimerImpl::Start()
                     status = ER_FAIL;
                     break;
                 } else {
-                    lock.Unlock();
+                    lock.Unlock(MUTEX_CONTEXT);
                     Sleep(2);
-                    lock.Lock();
+                    lock.Lock(MUTEX_CONTEXT);
                 }
             }
         }
         isRunning = (status == ER_OK);
     }
-    lock.Unlock();
+    lock.Unlock(MUTEX_CONTEXT);
     return status;
 }
 
 QStatus TimerImpl::Stop()
 {
     QStatus status = ER_OK;
-    lock.Lock();
+    lock.Lock(MUTEX_CONTEXT);
     isRunning = false;
     for (size_t i = 0; i < timerThreads.size(); ++i) {
         if (timerThreads[i] != NULL) {
@@ -349,37 +355,37 @@ QStatus TimerImpl::Stop()
     while (it != addWaitQueue.end()) {
         (*it++)->Alert(TIMER_IS_DEAD_ALERTCODE);
     }
-    lock.Unlock();
+    lock.Unlock(MUTEX_CONTEXT);
     return status;
 }
 
 QStatus TimerImpl::Join()
 {
     QStatus status = ER_OK;
-    lock.Lock();
+    lock.Lock(MUTEX_CONTEXT);
     for (size_t i = 0; i < timerThreads.size(); ++i) {
         if (timerThreads[i] != NULL) {
-            lock.Unlock();
+            lock.Unlock(MUTEX_CONTEXT);
             QStatus tStatus = timerThreads[i]->Join();
-            lock.Lock();
+            lock.Lock(MUTEX_CONTEXT);
             status = (status == ER_OK) ? tStatus : status;
         }
 
 
     }
-    lock.Unlock();
+    lock.Unlock(MUTEX_CONTEXT);
     return status;
 }
 
 QStatus TimerImpl::AddAlarm(const Alarm& alarm)
 {
     QStatus status = ER_OK;
-    lock.Lock();
+    lock.Lock(MUTEX_CONTEXT);
     if (isRunning) {
         /* Don't allow an infinite number of alarms to exist on this timer */
-        while (maxAlarms && (alarms.size() >= maxAlarms) && isRunning) {
+        while (maxAlarms && alarm->limitable && (numLimitableAlarms >= maxAlarms) && isRunning) {
             Thread* thread = Thread::GetThread();
-            assert(thread);
+            QCC_ASSERT(thread);
             addWaitQueue.push_front(thread);
             lock.Unlock(MUTEX_CONTEXT);
             QStatus status1 = Event::Wait(Event::neverSet, Event::WAIT_FOREVER);
@@ -405,7 +411,9 @@ QStatus TimerImpl::AddAlarm(const Alarm& alarm)
             /* Insert the alarm and alert the TimerImpl thread if necessary */
             bool alertThread = alarms.empty() || (alarm < *alarms.begin());
             alarms.insert(alarm);
-
+            if (alarm->limitable) {
+                numLimitableAlarms++;
+            }
             if (alertThread && (controllerIdx >= 0)) {
                 TimerThread* tt = timerThreads[controllerIdx];
                 if (tt->state == TimerThread::IDLE) {
@@ -420,25 +428,27 @@ QStatus TimerImpl::AddAlarm(const Alarm& alarm)
         status = ER_TIMER_EXITING;
     }
 
-    lock.Unlock();
+    lock.Unlock(MUTEX_CONTEXT);
     return status;
 }
 
 QStatus TimerImpl::AddAlarmNonBlocking(const Alarm& alarm)
 {
     QStatus status = ER_OK;
-    lock.Lock();
+    lock.Lock(MUTEX_CONTEXT);
     if (isRunning) {
         /* Don't allow an infinite number of alarms to exist on this timer */
-        if (maxAlarms && (alarms.size() >= maxAlarms)) {
-            lock.Unlock();
+        if (maxAlarms && (alarm->limitable && numLimitableAlarms >= maxAlarms)) {
+            lock.Unlock(MUTEX_CONTEXT);
             return ER_TIMER_FULL;
         }
 
         /* Insert the alarm and alert the TimerImpl thread if necessary */
         bool alertThread = alarms.empty() || (alarm < *alarms.begin());
         alarms.insert(alarm);
-
+        if (alarm->limitable) {
+            numLimitableAlarms++;
+        }
         if (alertThread && (controllerIdx >= 0)) {
             TimerThread* tt = timerThreads[controllerIdx];
             if (tt->state == TimerThread::IDLE) {
@@ -449,20 +459,23 @@ QStatus TimerImpl::AddAlarmNonBlocking(const Alarm& alarm)
         status = ER_TIMER_EXITING;
     }
 
-    lock.Unlock();
+    lock.Unlock(MUTEX_CONTEXT);
     return status;
 }
 
 bool TimerImpl::RemoveAlarm(const Alarm& alarm, bool blockIfTriggered)
 {
     bool foundAlarm = false;
-    lock.Lock();
+    lock.Lock(MUTEX_CONTEXT);
     if (isRunning || expireOnExit) {
         if (alarm->periodMs) {
             set<Alarm>::iterator it = alarms.begin();
             while (it != alarms.end()) {
                 if ((*it)->id == alarm->id) {
                     foundAlarm = true;
+                    if ((*it)->limitable) {
+                        numLimitableAlarms--;
+                    }
                     alarms.erase(it);
                     break;
                 }
@@ -472,6 +485,9 @@ bool TimerImpl::RemoveAlarm(const Alarm& alarm, bool blockIfTriggered)
             set<Alarm>::iterator it = alarms.find(alarm);
             if (it != alarms.end()) {
                 foundAlarm = true;
+                if ((*it)->limitable) {
+                    numLimitableAlarms--;
+                }
                 alarms.erase(it);
             }
         }
@@ -486,9 +502,9 @@ bool TimerImpl::RemoveAlarm(const Alarm& alarm, bool blockIfTriggered)
                 }
                 const Alarm* curAlarm = timerThreads[i]->GetCurrentAlarm();
                 while (curAlarm && (*curAlarm == alarm)) {
-                    lock.Unlock();
+                    lock.Unlock(MUTEX_CONTEXT);
                     qcc::Sleep(2);
-                    lock.Lock();
+                    lock.Lock(MUTEX_CONTEXT);
                     if (timerThreads[i] == NULL) {
                         break;
                     }
@@ -497,20 +513,23 @@ bool TimerImpl::RemoveAlarm(const Alarm& alarm, bool blockIfTriggered)
             }
         }
     }
-    lock.Unlock();
+    lock.Unlock(MUTEX_CONTEXT);
     return foundAlarm;
 }
 
 bool TimerImpl::ForceRemoveAlarm(const Alarm& alarm, bool blockIfTriggered)
 {
     bool foundAlarm = false;
-    lock.Lock();
+    lock.Lock(MUTEX_CONTEXT);
     if (isRunning || expireOnExit) {
         if (alarm->periodMs) {
             set<Alarm>::iterator it = alarms.begin();
             while (it != alarms.end()) {
                 if ((*it)->id == alarm->id) {
                     foundAlarm = true;
+                    if ((*it)->limitable) {
+                        numLimitableAlarms--;
+                    }
                     alarms.erase(it);
                     break;
                 }
@@ -520,6 +539,9 @@ bool TimerImpl::ForceRemoveAlarm(const Alarm& alarm, bool blockIfTriggered)
             set<Alarm>::iterator it = alarms.find(alarm);
             if (it != alarms.end()) {
                 foundAlarm = true;
+                if ((*it)->limitable) {
+                    numLimitableAlarms--;
+                }
                 alarms.erase(it);
             }
         }
@@ -535,9 +557,9 @@ bool TimerImpl::ForceRemoveAlarm(const Alarm& alarm, bool blockIfTriggered)
                 const Alarm* curAlarm = timerThreads[i]->GetCurrentAlarm();
                 while (curAlarm && (*curAlarm == alarm)) {
                     timerThreads[i]->Alert(FORCEREMOVEALARM_ALERTCODE);
-                    lock.Unlock();
+                    lock.Unlock(MUTEX_CONTEXT);
                     qcc::Sleep(2);
-                    lock.Lock();
+                    lock.Lock(MUTEX_CONTEXT);
                     if (timerThreads[i] == NULL) {
                         break;
                     }
@@ -546,17 +568,20 @@ bool TimerImpl::ForceRemoveAlarm(const Alarm& alarm, bool blockIfTriggered)
             }
         }
     }
-    lock.Unlock();
+    lock.Unlock(MUTEX_CONTEXT);
     return foundAlarm;
 }
 
 QStatus TimerImpl::ReplaceAlarm(const Alarm& origAlarm, const Alarm& newAlarm, bool blockIfTriggered)
 {
     QStatus status = ER_NO_SUCH_ALARM;
-    lock.Lock();
+    lock.Lock(MUTEX_CONTEXT);
     if (isRunning) {
         set<Alarm>::iterator it = alarms.find(origAlarm);
         if (it != alarms.end()) {
+            if ((*it)->limitable) {
+                numLimitableAlarms--;
+            }
             alarms.erase(it);
             status = AddAlarm(newAlarm);
         } else if (blockIfTriggered) {
@@ -570,9 +595,9 @@ QStatus TimerImpl::ReplaceAlarm(const Alarm& origAlarm, const Alarm& newAlarm, b
                 }
                 const Alarm* curAlarm = timerThreads[i]->GetCurrentAlarm();
                 while (curAlarm && (*curAlarm == origAlarm)) {
-                    lock.Unlock();
+                    lock.Unlock(MUTEX_CONTEXT);
                     qcc::Sleep(2);
-                    lock.Lock();
+                    lock.Lock(MUTEX_CONTEXT);
                     if (timerThreads[i] == NULL) {
                         break;
                     }
@@ -581,18 +606,21 @@ QStatus TimerImpl::ReplaceAlarm(const Alarm& origAlarm, const Alarm& newAlarm, b
             }
         }
     }
-    lock.Unlock();
+    lock.Unlock(MUTEX_CONTEXT);
     return status;
 }
 
 bool TimerImpl::RemoveAlarm(const AlarmListener& listener, Alarm& alarm)
 {
     bool removedOne = false;
-    lock.Lock();
+    lock.Lock(MUTEX_CONTEXT);
     if (isRunning || expireOnExit) {
         for (set<Alarm>::iterator it = alarms.begin(); it != alarms.end(); ++it) {
             if ((*it)->listener == &listener) {
                 alarm = *it;
+                if ((*it)->limitable) {
+                    numLimitableAlarms--;
+                }
                 alarms.erase(it);
                 removedOne = true;
                 break;
@@ -610,9 +638,9 @@ bool TimerImpl::RemoveAlarm(const AlarmListener& listener, Alarm& alarm)
                 }
                 const Alarm* curAlarm = timerThreads[i]->GetCurrentAlarm();
                 while (curAlarm && ((*curAlarm)->listener == &listener)) {
-                    lock.Unlock();
+                    lock.Unlock(MUTEX_CONTEXT);
                     qcc::Sleep(5);
-                    lock.Lock();
+                    lock.Lock(MUTEX_CONTEXT);
                     if (timerThreads[i] == NULL) {
                         break;
                     }
@@ -621,7 +649,7 @@ bool TimerImpl::RemoveAlarm(const AlarmListener& listener, Alarm& alarm)
             }
         }
     }
-    lock.Unlock();
+    lock.Unlock(MUTEX_CONTEXT);
     return removedOne;
 }
 
@@ -635,11 +663,11 @@ void TimerImpl::RemoveAlarmsWithListener(const AlarmListener& listener)
 bool TimerImpl::HasAlarm(const Alarm& alarm) const
 {
     bool ret = false;
-    lock.Lock();
+    lock.Lock(MUTEX_CONTEXT);
     if (isRunning) {
         ret = alarms.count(alarm) != 0;
     }
-    lock.Unlock();
+    lock.Unlock(MUTEX_CONTEXT);
     return ret;
 }
 
@@ -651,12 +679,12 @@ const qcc::String& TimerImpl::GetName() const
 QStatus TimerThread::Start(void* arg, ThreadListener* listener)
 {
     QStatus status = ER_OK;
-    timer->lock.Lock();
+    timer->lock.Lock(MUTEX_CONTEXT);
     if (timer->isRunning) {
         state = TimerThread::STARTING;
         status = Thread::Start(arg, listener);
     }
-    timer->lock.Unlock();
+    timer->lock.Unlock(MUTEX_CONTEXT);
     return status;
 }
 
@@ -678,11 +706,11 @@ ThreadReturn STDCALL TimerThread::Run(void* arg)
     /*
      * Enter the main loop with the timer lock held.
      */
-    timer->lock.Lock();
+    timer->lock.Lock(MUTEX_CONTEXT);
 
     while (!IsStopping()) {
         QCC_DbgPrintf(("TimerThread::Run(): Looping."));
-        Timespec now;
+        Timespec<MonotonicTime> now;
         GetTimeNow(&now);
         bool isController = (timer->controllerIdx == index);
 
@@ -734,10 +762,10 @@ ThreadReturn STDCALL TimerThread::Run(void* arg)
                     for (size_t i = 0; i < timer->timerThreads.size(); ++i) {
                         if (i != static_cast<size_t>(index) && timer->timerThreads[i] != NULL) {
 
-                            while ((timer->timerThreads[i]->state != TimerThread::STOPPED || timer->timerThreads[i]->IsRunning()) && timer->isRunning && status == ER_TIMEOUT && delay > WORKER_IDLE_TIMEOUT_MS) {
-                                timer->lock.Unlock();
+                            while ((timer->timerThreads[i]->state != TimerThread::STOPPED) && timer->isRunning && status == ER_TIMEOUT && delay > WORKER_IDLE_TIMEOUT_MS) {
+                                timer->lock.Unlock(MUTEX_CONTEXT);
                                 status = Event::Wait(Event::neverSet, WORKER_IDLE_TIMEOUT_MS);
-                                timer->lock.Lock();
+                                timer->lock.Lock(MUTEX_CONTEXT);
                                 GetTimeNow(&now);
                                 delay = topAlarm->alarmTime - now;
                             }
@@ -745,7 +773,7 @@ ThreadReturn STDCALL TimerThread::Run(void* arg)
                             if (status == ER_ALERTED_THREAD || status == ER_STOPPING_THREAD || !timer->isRunning || delay <= WORKER_IDLE_TIMEOUT_MS) {
                                 break;
                             }
-                            if (timer->timerThreads[i]->state == TimerThread::STOPPED && !timer->timerThreads[i]->IsRunning()) {
+                            if (timer->timerThreads[i]->state == TimerThread::STOPPED) {
                                 delete timer->timerThreads[i];
                                 timer->timerThreads[i] = NULL;
                                 QCC_DbgPrintf(("TimerThread::Run(): Deleted unused worker thread %d", i));
@@ -755,10 +783,10 @@ ThreadReturn STDCALL TimerThread::Run(void* arg)
 
                 }
                 if ((status == ER_TIMEOUT) && (delay > 0)) {
-                    timer->lock.Unlock();
+                    timer->lock.Unlock(MUTEX_CONTEXT);
                     Event evt(static_cast<uint32_t>(delay), 0);
                     Event::Wait(evt);
-                    timer->lock.Lock();
+                    timer->lock.Lock(MUTEX_CONTEXT);
                 }
                 stopEvent.ResetEvent();
             } else if (isController || (delay <= 0)) {
@@ -785,12 +813,12 @@ ThreadReturn STDCALL TimerThread::Run(void* arg)
 
                 state = RUNNING;
                 stopEvent.ResetEvent();
-                timer->lock.Unlock();
+                timer->lock.Unlock(MUTEX_CONTEXT);
 
                 /* Get the reentrancy lock if necessary */
                 hasTimerLock = timer->preventReentrancy;
                 if (hasTimerLock) {
-                    timer->reentrancyLock.Lock();
+                    timer->reentrancyLock.Lock(MUTEX_CONTEXT);
                 }
 
                 /*
@@ -799,7 +827,7 @@ ThreadReturn STDCALL TimerThread::Run(void* arg)
                  * either case, we are going to handle the alarm at the head of
                  * the list.
                  */
-                timer->lock.Lock();
+                timer->lock.Lock(MUTEX_CONTEXT);
 
 
                 TimerThread* tt = NULL;
@@ -849,7 +877,7 @@ ThreadReturn STDCALL TimerThread::Run(void* arg)
                                     tt = timer->timerThreads[i];
                                     QCC_DbgPrintf(("TimerThread::Run(): Found idle worker at index %d", i));
                                     break;
-                                } else if (timer->timerThreads[i]->state == TimerThread::STOPPED && !timer->timerThreads[i]->IsRunning()) {
+                                } else if (timer->timerThreads[i]->state == TimerThread::STOPPED) {
                                     tt = timer->timerThreads[i];
                                     QCC_DbgPrintf(("TimerThread::Run(): Found stopped worker at index %d", i));
                                 }
@@ -864,9 +892,9 @@ ThreadReturn STDCALL TimerThread::Run(void* arg)
                              */
                             break;
                         }
-                        timer->lock.Unlock();
+                        timer->lock.Unlock(MUTEX_CONTEXT);
                         Sleep(2);
-                        timer->lock.Lock();
+                        timer->lock.Lock(MUTEX_CONTEXT);
                     }
 
 
@@ -897,6 +925,7 @@ ThreadReturn STDCALL TimerThread::Run(void* arg)
                                 }
                             } else if (tt->state == TimerThread::STOPPED) {
                                 QCC_DbgPrintf(("TimerThread::Run(): Start()ing stopped timer thread (tt)"));
+                                tt->Join();
                                 QStatus status = tt->Start(NULL, timer);
                                 if (status != ER_OK) {
                                     QCC_LogError(status, ("Error starting timer thread %s", tt->GetName()));
@@ -923,6 +952,9 @@ ThreadReturn STDCALL TimerThread::Run(void* arg)
                 set<Alarm>::iterator it = timer->alarms.find(topAlarm);
                 if (it != timer->alarms.end()) {
                     Alarm top = *it;
+                    if (top->limitable) {
+                        timer->numLimitableAlarms--;
+                    }
                     timer->alarms.erase(it);
                     currentAlarm = &top;
                     if (0 < timer->addWaitQueue.size()) {
@@ -933,15 +965,16 @@ ThreadReturn STDCALL TimerThread::Run(void* arg)
                             QCC_LogError(status, ("Failed to alert thread blocked on full tx queue"));
                         }
                     }
-                    timer->lock.Unlock();
+                    timer->lock.Unlock(MUTEX_CONTEXT);
 
                     QCC_DbgPrintf(("TimerThread::Run(): ******** AlarmTriggered()"));
+                    IncrementPerfCounter(PERF_COUNTER_ALARM_TRIGGERED);
                     (top->listener->AlarmTriggered)(top, ER_OK);
                     if (hasTimerLock) {
                         hasTimerLock = false;
-                        timer->reentrancyLock.Unlock();
+                        timer->reentrancyLock.Unlock(MUTEX_CONTEXT);
                     }
-                    timer->lock.Lock();
+                    timer->lock.Lock(MUTEX_CONTEXT);
                     /* If ForceRemoveAlarm has been called for this alarm, we need to reset the alert code.
                      * Note that this must be atomic with setting the currentAlarm to NULL.
                      */
@@ -962,7 +995,7 @@ ThreadReturn STDCALL TimerThread::Run(void* arg)
                 } else {
                     if (hasTimerLock) {
                         hasTimerLock = false;
-                        timer->reentrancyLock.Unlock();
+                        timer->reentrancyLock.Unlock(MUTEX_CONTEXT);
                     }
                 }
             } else {
@@ -973,9 +1006,9 @@ ThreadReturn STDCALL TimerThread::Run(void* arg)
                  */
                 state = IDLE;
                 QCC_DbgPrintf(("TimerThread::Run(): Worker with nothing to do"));
-                timer->lock.Unlock();
+                timer->lock.Unlock(MUTEX_CONTEXT);
                 QStatus status = Event::Wait(Event::neverSet, WORKER_IDLE_TIMEOUT_MS);
-                timer->lock.Lock();
+                timer->lock.Lock(MUTEX_CONTEXT);
                 if (status == ER_TIMEOUT && timer->controllerIdx != -1) {
                     QCC_DbgPrintf(("TimerThread::Run(): Worker with nothing to do stopping"));
                     state = STOPPING;
@@ -1003,15 +1036,15 @@ ThreadReturn STDCALL TimerThread::Run(void* arg)
                 for (size_t i = 0; i < timer->timerThreads.size(); ++i) {
                     if (i != static_cast<size_t>(index) && timer->timerThreads[i] != NULL) {
 
-                        while ((timer->timerThreads[i]->state != TimerThread::STOPPED || timer->timerThreads[i]->IsRunning()) && timer->isRunning && status == ER_TIMEOUT) {
-                            timer->lock.Unlock();
+                        while ((timer->timerThreads[i]->state != TimerThread::STOPPED) && timer->isRunning && status == ER_TIMEOUT) {
+                            timer->lock.Unlock(MUTEX_CONTEXT);
                             status = Event::Wait(Event::neverSet, WORKER_IDLE_TIMEOUT_MS);
-                            timer->lock.Lock();
+                            timer->lock.Lock(MUTEX_CONTEXT);
                         }
                         if (status == ER_ALERTED_THREAD || status == ER_STOPPING_THREAD || !timer->isRunning) {
                             break;
                         }
-                        if (timer->timerThreads[i]->state == TimerThread::STOPPED && !timer->timerThreads[i]->IsRunning()) {
+                        if (timer->timerThreads[i]->state == TimerThread::STOPPED) {
                             delete timer->timerThreads[i];
                             timer->timerThreads[i] = NULL;
                             QCC_DbgPrintf(("TimerThread::Run(): Deleted unused worker thread %d", i));
@@ -1025,17 +1058,17 @@ ThreadReturn STDCALL TimerThread::Run(void* arg)
                     /* The controller has successfully deleted objects of all other worker threads.
                      * and has not been alerted/stopped.
                      */
-                    timer->lock.Unlock();
+                    timer->lock.Unlock(MUTEX_CONTEXT);
                     Event::Wait(Event::neverSet);
-                    timer->lock.Lock();
+                    timer->lock.Lock(MUTEX_CONTEXT);
                 }
                 stopEvent.ResetEvent();
             } else {
                 QCC_DbgPrintf(("TimerThread::Run(): non-Controller idling"));
                 state = IDLE;
-                timer->lock.Unlock();
+                timer->lock.Unlock(MUTEX_CONTEXT);
                 QStatus status = Event::Wait(Event::neverSet, WORKER_IDLE_TIMEOUT_MS);
-                timer->lock.Lock();
+                timer->lock.Lock(MUTEX_CONTEXT);
                 if (status == ER_TIMEOUT && timer->controllerIdx != -1) {
                     QCC_DbgPrintf(("TimerThread::Run(): non-Controller stopping"));
                     state = STOPPING;
@@ -1050,14 +1083,14 @@ ThreadReturn STDCALL TimerThread::Run(void* arg)
      * We entered the main loop with the lock taken, so we need to give it here.
      */
     state = STOPPING;
-    timer->lock.Unlock();
+    timer->lock.Unlock(MUTEX_CONTEXT);
     return (ThreadReturn)0;
 }
 
 void TimerImpl::ThreadExit(Thread* thread)
 {
     TimerThread* tt = static_cast<TimerThread*>(thread);
-    lock.Lock();
+    lock.Lock(MUTEX_CONTEXT);
     if ((!isRunning) && expireOnExit) {
         /* Call all alarms */
         while (!alarms.empty()) {
@@ -1066,31 +1099,35 @@ void TimerImpl::ThreadExit(Thread* thread)
              */
             set<Alarm>::iterator it = alarms.begin();
             Alarm alarm = *it;
+            if (alarm->limitable) {
+                numLimitableAlarms--;
+            }
             alarms.erase(it);
             tt->SetCurrentAlarm(&alarm);
-            lock.Unlock();
+            lock.Unlock(MUTEX_CONTEXT);
             tt->hasTimerLock = preventReentrancy;
             if (tt->hasTimerLock) {
-                reentrancyLock.Lock();
+                reentrancyLock.Lock(MUTEX_CONTEXT);
             }
+
+            IncrementPerfCounter(PERF_COUNTER_ALARM_TRIGGERED);
             alarm->listener->AlarmTriggered(alarm, ER_TIMER_EXITING);
             if (tt->hasTimerLock) {
                 tt->hasTimerLock = false;
-                reentrancyLock.Unlock();
+                reentrancyLock.Unlock(MUTEX_CONTEXT);
             }
-            lock.Lock();
+            lock.Lock(MUTEX_CONTEXT);
             tt->SetCurrentAlarm(NULL);
         }
     }
     tt->state = TimerThread::STOPPED;
-    lock.Unlock();
-    tt->Join();
+    lock.Unlock(MUTEX_CONTEXT);
 }
 
 void TimerImpl::EnableReentrancy()
 {
     Thread* thread = Thread::GetThread();
-    lock.Lock();
+    lock.Lock(MUTEX_CONTEXT);
     bool allowed = false;
     for (uint32_t i = 0; i < timerThreads.size(); ++i) {
         if ((timerThreads[i] != NULL) && (static_cast<Thread*>(timerThreads[i]) == thread)) {
@@ -1098,12 +1135,12 @@ void TimerImpl::EnableReentrancy()
             break;
         }
     }
-    lock.Unlock();
+    lock.Unlock(MUTEX_CONTEXT);
     if (allowed) {
         TimerThread* tt = static_cast<TimerThread*>(thread);
         if (tt->hasTimerLock) {
             tt->hasTimerLock = false;
-            reentrancyLock.Unlock();
+            reentrancyLock.Unlock(MUTEX_CONTEXT);
         }
     } else {
         QCC_LogError(ER_TIMER_NOT_ALLOWED, ("Invalid call to TimerImpl::EnableReentrancy from thread %s", Thread::GetThreadName()));
@@ -1113,21 +1150,21 @@ void TimerImpl::EnableReentrancy()
 bool TimerImpl::IsTimerCallbackThread() const
 {
     bool result = false;
-    lock.Lock();
+    lock.Lock(MUTEX_CONTEXT);
     for (size_t i = 0; i < timerThreads.size(); ++i) {
         if ((timerThreads[i] != NULL) && (timerThreads[i] == Thread::GetThread())) {
             result = true;
             break;
         }
     }
-    lock.Unlock();
+    lock.Unlock(MUTEX_CONTEXT);
     return result;
 }
 
 bool TimerImpl::ThreadHoldsLock() const
 {
     Thread* thread = Thread::GetThread();
-    lock.Lock();
+    lock.Lock(MUTEX_CONTEXT);
     bool allowed = false;
     for (uint32_t i = 0; i < timerThreads.size(); ++i) {
         if ((timerThreads[i] != NULL) && (static_cast<Thread*>(timerThreads[i]) == thread)) {
@@ -1135,7 +1172,7 @@ bool TimerImpl::ThreadHoldsLock() const
             break;
         }
     }
-    lock.Unlock();
+    lock.Unlock(MUTEX_CONTEXT);
     if (allowed) {
         TimerThread* tt = static_cast<TimerThread*>(thread);
         return tt->hasTimerLock;
