@@ -27,6 +27,7 @@
 
 #include <qcc/Debug.h>
 #include <qcc/String.h>
+#include <qcc/StringSource.h>
 #include <qcc/StringUtil.h>
 #include <qcc/XmlElement.h>
 
@@ -179,7 +180,10 @@ qcc::String AJ_CALL XmlElement::UnescapeXml(const qcc::String& str) {
 /* Finalize and return true if element ends the root Xml element */
 void XmlElement::FinalizeElement(XmlParseContext& ctx)
 {
-    /* Sanity check */
+    /* Sanity checks */
+    if (ctx.curElem == nullptr) {
+        return;
+    }
     qcc::String cookedContent = Trim(UnescapeXml(ctx.rawContent));
     if ((0 < cookedContent.size()) && (0 < ctx.curElem->GetChildren().size())) {
         QCC_DbgPrintf(("XML Element <%s> has both children and content", ctx.curElem->GetName().c_str()));
@@ -194,6 +198,25 @@ void XmlElement::FinalizeElement(XmlParseContext& ctx)
     ctx.curElem = ctx.curElem->GetParent();
 }
 
+QStatus AJ_CALL XmlElement::GetRoot(AJ_PCSTR xml, XmlElement** root)
+{
+    QCC_ASSERT(nullptr != xml);
+    QCC_ASSERT(nullptr != root);
+
+    QStatus status;
+    StringSource source(xml);
+    XmlParseContext context(source);
+    *root = nullptr;
+
+    status = XmlElement::Parse(context);
+
+    if (ER_OK == status) {
+        *root = context.DetachRoot();
+    }
+
+    return status;
+}
+
 QStatus AJ_CALL XmlElement::Parse(XmlParseContext& ctx)
 {
     bool done = false;
@@ -242,7 +265,7 @@ QStatus AJ_CALL XmlElement::Parse(XmlParseContext& ctx)
                             ctx.curElem = ctx.root;
                             ctx.curElem->SetName(ctx.elemName);
                         } else {
-                            ctx.curElem = &(ctx.curElem->CreateChild(ctx.elemName));
+                            ctx.curElem = ctx.curElem->CreateChild(ctx.elemName);
                         }
                     } else {
                         FinalizeElement(ctx);
@@ -257,7 +280,7 @@ QStatus AJ_CALL XmlElement::Parse(XmlParseContext& ctx)
                         ctx.curElem = ctx.root;
                         ctx.curElem->SetName(ctx.elemName);
                     } else {
-                        ctx.curElem = &(ctx.curElem->CreateChild(ctx.elemName));
+                        ctx.curElem = ctx.curElem->CreateChild(ctx.elemName);
                     }
                     ctx.isEndTag = true;
                 } else {
@@ -407,11 +430,19 @@ qcc::String XmlElement::Generate(qcc::String* outStr) const
     return *outStr;
 }
 
-XmlElement& XmlElement::CreateChild(const qcc::String& elementName)
+AJ_PSTR XmlElement::ToString() const
+{
+    String tempXml = Generate();
+    AJ_PSTR xml = new char[tempXml.size() + 1];
+    strcpy(xml, tempXml.c_str());
+    return xml;
+}
+
+XmlElement* XmlElement::CreateChild(const qcc::String& elementName)
 {
     QCC_DbgTrace(("XmlElement::CreateChild(\"%s\")", elementName.c_str()));
     new XmlElement(elementName, this, true);
-    return *children.back();
+    return children.back();
 }
 
 std::vector<const XmlElement*> XmlElement::GetChildren(const qcc::String& elementName) const
@@ -454,6 +485,12 @@ const qcc::String& XmlElement::GetAttribute(const qcc::String& attName) const
     }
 }
 
+void XmlElement::AddChild(XmlElement* child)
+{
+    child->parentOwned = true;
+    children.push_back(child);
+}
+
 std::vector<const XmlElement*> XmlElement::GetPath(const qcc::String& inPath) const
 {
     std::vector<const XmlElement*> matches;
